searchData={"items":[{"type":"module","title":"SnmpKit.SnmpLib.ASN1","doc":"Comprehensive ASN.1 BER (Basic Encoding Rules) encoding and decoding utilities.\n\nProvides low-level ASN.1 operations that are used by the PDU module and can be\nused for other ASN.1 encoding/decoding needs. This module offers improved\nperformance and better error handling compared to basic implementations.","ref":"SnmpKit.SnmpLib.ASN1.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.ASN1","doc":"- Complete BER encoding/decoding support\n- Optimized length handling for large values\n- Comprehensive error reporting\n- Support for constructed and primitive types\n- Memory-efficient implementations\n- Validation and constraint checking\n- RFC-compliant OID multibyte encoding (values ≥ 128)","ref":"SnmpKit.SnmpLib.ASN1.html#module-features"},{"type":"module","title":"ASN.1 Types Supported - SnmpKit.SnmpLib.ASN1","doc":"- **INTEGER**: Signed integers with arbitrary precision\n- **OCTET STRING**: Binary data of any length\n- **NULL**: Null value representation\n- **OBJECT IDENTIFIER**: Hierarchical object identifiers with multibyte subidentifiers\n- **SEQUENCE**: Constructed type for complex structures\n- **Custom Tags**: Application-specific and context-specific tags","ref":"SnmpKit.SnmpLib.ASN1.html#module-asn-1-types-supported"},{"type":"module","title":"Important: OID Encoding - SnmpKit.SnmpLib.ASN1","doc":"OID subidentifiers use 7-bit encoding where values ≥ 128 require multibyte encoding:\n\n- Values 0-127: Single byte\n- Values 128+: Multibyte with continuation bits\n- Example: 200 → `[0x81, 0x48]` (not single byte `0xC8`)","ref":"SnmpKit.SnmpLib.ASN1.html#module-important-oid-encoding"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.ASN1","doc":"# Integer encoding/decoding\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_integer(42)\n    iex> {:ok, {value, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_integer(encoded)\n    iex> value\n    42\n\n    # OCTET STRING encoding/decoding\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_octet_string(\"Hello\")\n    iex> {:ok, {value, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_octet_string(encoded)\n    iex> value\n    \"Hello\"\n\n    # OID encoding/decoding with multibyte values\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_oid([1, 3, 6, 1, 4, 1, 200])\n    iex> {:ok, {oid, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_oid(encoded)\n    iex> oid\n    [1, 3, 6, 1, 4, 1, 200]\n\n    # Length encoding for large values\n    iex> encoded_length = SnmpKit.SnmpLib.ASN1.encode_length(1000)\n    iex> {:ok, {length, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_length(encoded_length)\n    iex> length\n    1000\n\n    # NULL encoding\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_null()\n    iex> {:ok, {value, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_null(encoded)\n    iex> value\n    :null","ref":"SnmpKit.SnmpLib.ASN1.html#module-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.calculate_ber_length/1","doc":"Calculates the total length of a BER-encoded structure.","ref":"SnmpKit.SnmpLib.ASN1.html#calculate_ber_length/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.calculate_ber_length/1","doc":"- `data`: BER-encoded binary data","ref":"SnmpKit.SnmpLib.ASN1.html#calculate_ber_length/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.calculate_ber_length/1","doc":"- `{:ok, total_length}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.ASN1.html#calculate_ber_length/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.calculate_ber_length/1","doc":"{:ok, 10} = SnmpKit.SnmpLib.ASN1.calculate_ber_length(ber_data)","ref":"SnmpKit.SnmpLib.ASN1.html#calculate_ber_length/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.ASN1.content/0","doc":"","ref":"SnmpKit.SnmpLib.ASN1.html#t:content/0"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_integer/1","doc":"Decodes an ASN.1 INTEGER value.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_integer/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.decode_integer/1","doc":"- `data`: Binary data starting with an INTEGER TLV","ref":"SnmpKit.SnmpLib.ASN1.html#decode_integer/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.decode_integer/1","doc":"- `{:ok, {value, remaining_data}}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.ASN1.html#decode_integer/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_integer/1","doc":"{:ok, {42, remaining}} = SnmpKit.SnmpLib.ASN1.decode_integer(<<2, 1, 42, 99, 100>>)\n    # Returns {42, <<99, 100>>}","ref":"SnmpKit.SnmpLib.ASN1.html#decode_integer/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_length/1","doc":"Decodes an ASN.1 length field.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_length/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.decode_length/1","doc":"- `data`: Binary data starting with a length field","ref":"SnmpKit.SnmpLib.ASN1.html#decode_length/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.decode_length/1","doc":"- `{:ok, {length, remaining_data}}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.ASN1.html#decode_length/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_length/1","doc":"{:ok, {42, remaining}} = SnmpKit.SnmpLib.ASN1.decode_length(<<42, 1, 2, 3>>)\n    {:ok, {300, remaining}} = SnmpKit.SnmpLib.ASN1.decode_length(<<0x82, 1, 44, 1, 2, 3>>)","ref":"SnmpKit.SnmpLib.ASN1.html#decode_length/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_null/1","doc":"Decodes an ASN.1 NULL value.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_null/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_null/1","doc":"{:ok, {:null, remaining}} = SnmpKit.SnmpLib.ASN1.decode_null(<<5, 0, 1, 2, 3>>)\n    # Returns {:null, <<1, 2, 3>>}","ref":"SnmpKit.SnmpLib.ASN1.html#decode_null/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_octet_string/1","doc":"Decodes an ASN.1 OCTET STRING value.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_octet_string/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_octet_string/1","doc":"{:ok, {\"Hello\", remaining}} = SnmpKit.SnmpLib.ASN1.decode_octet_string(encoded_data)","ref":"SnmpKit.SnmpLib.ASN1.html#decode_octet_string/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_oid/1","doc":"Decodes an ASN.1 OBJECT IDENTIFIER value.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_oid/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_oid/1","doc":"{:ok, {[1, 3, 6, 1], remaining}} = SnmpKit.SnmpLib.ASN1.decode_oid(encoded_data)","ref":"SnmpKit.SnmpLib.ASN1.html#decode_oid/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_sequence/1","doc":"Decodes an ASN.1 SEQUENCE value.\n\nReturns the content of the sequence without further parsing.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_sequence/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_sequence/1","doc":"{:ok, {sequence_content, remaining}} = SnmpKit.SnmpLib.ASN1.decode_sequence(encoded_data)","ref":"SnmpKit.SnmpLib.ASN1.html#decode_sequence/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.decode_tlv/1","doc":"Decodes a generic TLV structure.","ref":"SnmpKit.SnmpLib.ASN1.html#decode_tlv/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.decode_tlv/1","doc":"- `data`: Binary data starting with a TLV","ref":"SnmpKit.SnmpLib.ASN1.html#decode_tlv/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.decode_tlv/1","doc":"- `{:ok, {tag, content, remaining}}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.ASN1.html#decode_tlv/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.decode_tlv/1","doc":"{:ok, {tag, content, remaining}} = SnmpKit.SnmpLib.ASN1.decode_tlv(binary_data)","ref":"SnmpKit.SnmpLib.ASN1.html#decode_tlv/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_custom_tlv/2","doc":"Encodes a custom TLV (Tag-Length-Value) structure.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_custom_tlv/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_custom_tlv/2","doc":"- `tag`: ASN.1 tag value\n- `content`: Binary content","ref":"SnmpKit.SnmpLib.ASN1.html#encode_custom_tlv/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_custom_tlv/2","doc":"{:ok, tlv} = SnmpKit.SnmpLib.ASN1.encode_custom_tlv(0xA0, <<\"custom_content\">>)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_custom_tlv/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_integer/1","doc":"Encodes an ASN.1 INTEGER value using BER (Basic Encoding Rules).\n\nSupports arbitrary precision integers using two's complement representation.\nThe encoded result includes the ASN.1 tag (0x02), length, and content bytes.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_integer/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_integer/1","doc":"- `value`: Integer value to encode (any size)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_integer/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.encode_integer/1","doc":"- `{:ok, encoded_bytes}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.ASN1.html#encode_integer/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_integer/1","doc":"# Positive integers\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_integer(42)\n    iex> encoded\n    <<2, 1, 42>>\n\n    # Negative integers (two's complement)\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_integer(-1)\n    iex> encoded\n    <<2, 1, 255>>\n\n    # Zero\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_integer(0)\n    iex> encoded\n    <<2, 1, 0>>\n\n    # Large integers\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_integer(32767)\n    iex> byte_size(encoded) > 3\n    true","ref":"SnmpKit.SnmpLib.ASN1.html#encode_integer/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_length/1","doc":"Encodes an ASN.1 length field.\n\nSupports both short form (< 128) and long form encoding.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_length/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_length/1","doc":"- `length`: Length value to encode","ref":"SnmpKit.SnmpLib.ASN1.html#encode_length/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.encode_length/1","doc":"- Binary representation of the length","ref":"SnmpKit.SnmpLib.ASN1.html#encode_length/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_length/1","doc":"<<42>> = SnmpKit.SnmpLib.ASN1.encode_length(42)\n    <<0x81, 200>> = SnmpKit.SnmpLib.ASN1.encode_length(200)\n    <<0x82, 1, 44>> = SnmpKit.SnmpLib.ASN1.encode_length(300)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_length/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_null/0","doc":"Encodes an ASN.1 NULL value.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_null/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_null/0","doc":"{:ok, <<5, 0>>} = SnmpKit.SnmpLib.ASN1.encode_null()","ref":"SnmpKit.SnmpLib.ASN1.html#encode_null/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_octet_string/1","doc":"Encodes an ASN.1 OCTET STRING value.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_octet_string/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_octet_string/1","doc":"- `value`: Binary data to encode","ref":"SnmpKit.SnmpLib.ASN1.html#encode_octet_string/1-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_octet_string/1","doc":"{:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_octet_string(\"Hello\")\n    {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_octet_string(<<1, 2, 3, 4>>)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_octet_string/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_oid/1","doc":"Encodes an ASN.1 OBJECT IDENTIFIER (OID) value using BER encoding.\n\nOIDs are hierarchical identifiers where each component is encoded using\n7-bit subidentifiers. Values ≥ 128 require multibyte encoding with\ncontinuation bits, which is correctly handled by this implementation.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_oid/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_oid/1","doc":"- `oid_list`: List of non-negative integers representing the OID (minimum 2 components)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_oid/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.encode_oid/1","doc":"- `{:ok, encoded_bytes}` on success\n- `{:error, reason}` on failure (invalid OID format)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_oid/1-returns"},{"type":"function","title":"Encoding Rules - SnmpKit.SnmpLib.ASN1.encode_oid/1","doc":"- First two components are combined: `first * 40 + second`\n- Remaining components use 7-bit encoding with continuation bits\n- Values 0-127: single byte\n- Values 128+: multibyte with high bit indicating continuation","ref":"SnmpKit.SnmpLib.ASN1.html#encode_oid/1-encoding-rules"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_oid/1","doc":"# Standard SNMP OID (sysDescr.0)\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_oid([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    iex> {:ok, {decoded, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_oid(encoded)\n    iex> decoded\n    [1, 3, 6, 1, 2, 1, 1, 1, 0]\n\n    # OID with multibyte values (≥ 128)\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.ASN1.encode_oid([1, 3, 6, 1, 4, 1, 200])\n    iex> {:ok, {decoded, <<>>}} = SnmpKit.SnmpLib.ASN1.decode_oid(encoded)\n    iex> decoded\n    [1, 3, 6, 1, 4, 1, 200]\n\n    # Invalid OIDs\n    iex> SnmpKit.SnmpLib.ASN1.encode_oid([])\n    {:error, :invalid_oid}\n\n    iex> SnmpKit.SnmpLib.ASN1.encode_oid([1])\n    {:error, :invalid_oid}","ref":"SnmpKit.SnmpLib.ASN1.html#encode_oid/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.encode_sequence/1","doc":"Encodes an ASN.1 SEQUENCE with the given content.","ref":"SnmpKit.SnmpLib.ASN1.html#encode_sequence/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.encode_sequence/1","doc":"- `content`: Pre-encoded content for the sequence","ref":"SnmpKit.SnmpLib.ASN1.html#encode_sequence/1-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.encode_sequence/1","doc":"content = encode_integer_content(42) <> encode_octet_string_content(\"test\")\n    {:ok, sequence} = SnmpKit.SnmpLib.ASN1.encode_sequence(content)","ref":"SnmpKit.SnmpLib.ASN1.html#encode_sequence/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.ASN1.length/0","doc":"","ref":"SnmpKit.SnmpLib.ASN1.html#t:length/0"},{"type":"type","title":"SnmpKit.SnmpLib.ASN1.oid/0","doc":"","ref":"SnmpKit.SnmpLib.ASN1.html#t:oid/0"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.parse_tag/1","doc":"Parses an ASN.1 tag byte.\n\nReturns information about the tag including class, constructed bit, and tag number.","ref":"SnmpKit.SnmpLib.ASN1.html#parse_tag/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.parse_tag/1","doc":"- `tag_byte`: Single byte representing the tag","ref":"SnmpKit.SnmpLib.ASN1.html#parse_tag/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.parse_tag/1","doc":"- Map with tag information","ref":"SnmpKit.SnmpLib.ASN1.html#parse_tag/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.parse_tag/1","doc":"info = SnmpKit.SnmpLib.ASN1.parse_tag(0x30)\n    # Returns %{class: :universal, constructed: true, tag_number: 16}","ref":"SnmpKit.SnmpLib.ASN1.html#parse_tag/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.ASN1.tag/0","doc":"","ref":"SnmpKit.SnmpLib.ASN1.html#t:tag/0"},{"type":"type","title":"SnmpKit.SnmpLib.ASN1.tlv/0","doc":"","ref":"SnmpKit.SnmpLib.ASN1.html#t:tlv/0"},{"type":"function","title":"SnmpKit.SnmpLib.ASN1.validate_ber_structure/1","doc":"Validates the structure of BER-encoded data.\n\nPerforms basic validation without full decoding.","ref":"SnmpKit.SnmpLib.ASN1.html#validate_ber_structure/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ASN1.validate_ber_structure/1","doc":"- `data`: BER-encoded binary data","ref":"SnmpKit.SnmpLib.ASN1.html#validate_ber_structure/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ASN1.validate_ber_structure/1","doc":"- `:ok` if structure is valid\n- `{:error, reason}` if structure is invalid","ref":"SnmpKit.SnmpLib.ASN1.html#validate_ber_structure/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ASN1.validate_ber_structure/1","doc":":ok = SnmpKit.SnmpLib.ASN1.validate_ber_structure(valid_ber_data)\n    {:error, :invalid_length} = SnmpKit.SnmpLib.ASN1.validate_ber_structure(malformed_data)","ref":"SnmpKit.SnmpLib.ASN1.html#validate_ber_structure/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Cache","doc":"Intelligent caching system for SNMP operations with adaptive strategies.\n\nThis module provides sophisticated caching capabilities designed to optimize\nSNMP polling performance in high-throughput environments. Based on patterns\nproven in the DDumb project for managing thousands of concurrent device polls.","ref":"SnmpKit.SnmpLib.Cache.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Cache","doc":"- **Multi-Level Caching**: L1 (in-memory), L2 (ETS), L3 (persistent storage)\n- **Adaptive TTL**: Dynamic cache expiration based on data volatility\n- **Smart Invalidation**: Automatic cache invalidation based on data patterns\n- **Compression**: Efficient storage of large SNMP responses\n- **Hot/Cold Data Management**: Automatic promotion of frequently accessed data\n- **Cache Warming**: Proactive loading of expected data","ref":"SnmpKit.SnmpLib.Cache.html#module-features"},{"type":"module","title":"Caching Strategies - SnmpKit.SnmpLib.Cache","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#module-caching-strategies"},{"type":"module","title":"Time-Based Caching - SnmpKit.SnmpLib.Cache","doc":"Standard TTL-based caching for static or slowly changing data.","ref":"SnmpKit.SnmpLib.Cache.html#module-time-based-caching"},{"type":"module","title":"Volatility-Based Caching - SnmpKit.SnmpLib.Cache","doc":"Dynamic TTL adjustment based on observed change frequency.","ref":"SnmpKit.SnmpLib.Cache.html#module-volatility-based-caching"},{"type":"module","title":"Dependency-Based Caching - SnmpKit.SnmpLib.Cache","doc":"Cache invalidation based on related data changes.","ref":"SnmpKit.SnmpLib.Cache.html#module-dependency-based-caching"},{"type":"module","title":"Predictive Caching - SnmpKit.SnmpLib.Cache","doc":"Pre-loading data based on access patterns and time of day.","ref":"SnmpKit.SnmpLib.Cache.html#module-predictive-caching"},{"type":"module","title":"Performance Benefits - SnmpKit.SnmpLib.Cache","doc":"- **50-80% reduction** in redundant SNMP queries\n- **Improved response times** for frequently accessed data\n- **Reduced network load** on monitored devices\n- **Better scalability** for large device inventories","ref":"SnmpKit.SnmpLib.Cache.html#module-performance-benefits"},{"type":"module","title":"Usage Patterns - SnmpKit.SnmpLib.Cache","doc":"# Cache SNMP response data\n    SnmpKit.SnmpLib.Cache.put(\"device_123:sysDescr\", response_data, ttl: 300_000)\n\n    # Retrieve cached data\n    case SnmpKit.SnmpLib.Cache.get(\"device_123:sysDescr\") do\n      {:ok, data} -> data\n      :miss -> perform_snmp_query()\n    end\n\n    # Cache with adaptive TTL\n    SnmpKit.SnmpLib.Cache.put_adaptive(\"device_123:ifTable\", interface_data,\n      base_ttl: 60_000,\n      volatility: :medium\n    )\n\n    # Warm cache for predictable access\n    SnmpKit.SnmpLib.Cache.warm_cache(\"device_123\", [:sysDescr, :sysUpTime, :ifTable])\n\n    # Invalidate related caches\n    SnmpKit.SnmpLib.Cache.invalidate_pattern(\"device_123:*\")","ref":"SnmpKit.SnmpLib.Cache.html#module-usage-patterns"},{"type":"module","title":"Cache Key Patterns - SnmpKit.SnmpLib.Cache","doc":"- `device_id:oid` - Single OID values\n- `device_id:table:index` - Table row data\n- `device_id:walk:base_oid` - Walk results\n- `device_id:bulk:oids` - Bulk query results\n- `global:topology` - Cross-device topology data","ref":"SnmpKit.SnmpLib.Cache.html#module-cache-key-patterns"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_key/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_key/0"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_stats/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_stats/0"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_strategy/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_strategy/0"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_ttl/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_ttl/0"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.cache_value/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:cache_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.Cache.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.clear/0","doc":"Clears all cached data.","ref":"SnmpKit.SnmpLib.Cache.html#clear/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.clear/0","doc":":ok = SnmpKit.SnmpLib.Cache.clear()","ref":"SnmpKit.SnmpLib.Cache.html#clear/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.delete/1","doc":"Removes a specific key from the cache.","ref":"SnmpKit.SnmpLib.Cache.html#delete/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.delete/1","doc":":ok = SnmpKit.SnmpLib.Cache.delete(\"device_1:sysDescr\")","ref":"SnmpKit.SnmpLib.Cache.html#delete/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.get/1","doc":"Retrieves a value from the cache.","ref":"SnmpKit.SnmpLib.Cache.html#get/1"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Cache.get/1","doc":"- `{:ok, value}`: Cache hit with the stored value\n- `:miss`: Cache miss, value not found or expired","ref":"SnmpKit.SnmpLib.Cache.html#get/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.get/1","doc":"case SnmpKit.SnmpLib.Cache.get(\"device_1:sysDescr\") do\n      {:ok, description} ->\n        Logger.debug(\"Cache hit for system description\")\n        description\n      :miss ->\n        Logger.debug(\"Cache miss, performing SNMP query\")\n        perform_snmp_get(device, [1,3,6,1,2,1,1,1,0])\n    end","ref":"SnmpKit.SnmpLib.Cache.html#get/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.get_stats/0","doc":"Gets comprehensive cache performance statistics.","ref":"SnmpKit.SnmpLib.Cache.html#get_stats/0"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Cache.get_stats/0","doc":"Statistics including hit rates, memory usage, and performance metrics.","ref":"SnmpKit.SnmpLib.Cache.html#get_stats/0-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.get_stats/0","doc":"cache_stats = SnmpKit.SnmpLib.Cache.get_stats()\n    IO.puts \"Cache hit rate: \" <> Float.to_string(Float.round(cache_stats.hit_rate * 100, 2)) <> \"%\"\n    IO.puts \"Memory usage: \" <> Float.to_string(Float.round(cache_stats.memory_usage_mb, 2)) <> \" MB\"","ref":"SnmpKit.SnmpLib.Cache.html#get_stats/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.invalidate_by_tag/1","doc":"Invalidates cache entries by tag.","ref":"SnmpKit.SnmpLib.Cache.html#invalidate_by_tag/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.invalidate_by_tag/1","doc":"SnmpKit.SnmpLib.Cache.invalidate_by_tag(:routing_data)","ref":"SnmpKit.SnmpLib.Cache.html#invalidate_by_tag/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.invalidate_pattern/1","doc":"Invalidates multiple cache entries matching a pattern.\n\nSupports wildcards (*) for pattern matching.","ref":"SnmpKit.SnmpLib.Cache.html#invalidate_pattern/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.invalidate_pattern/1","doc":"# Invalidate all data for a device\n    SnmpKit.SnmpLib.Cache.invalidate_pattern(\"device_1:*\")\n\n    # Invalidate all interface data\n    SnmpKit.SnmpLib.Cache.invalidate_pattern(\"*:ifTable\")\n\n    # Invalidate by tag\n    SnmpKit.SnmpLib.Cache.invalidate_by_tag(:interface_data)","ref":"SnmpKit.SnmpLib.Cache.html#invalidate_pattern/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.put/3","doc":"Stores a value in the cache with specified options.","ref":"SnmpKit.SnmpLib.Cache.html#put/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Cache.put/3","doc":"- `key`: Unique cache key\n- `value`: Data to cache\n- `opts`: Caching options","ref":"SnmpKit.SnmpLib.Cache.html#put/3-parameters"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Cache.put/3","doc":"- `ttl`: Time-to-live in milliseconds (default: 300,000)\n- `strategy`: Caching strategy (default: :time_based)\n- `volatility`: Data change frequency (default: :medium)\n- `compress`: Force compression for this entry (default: auto)\n- `dependencies`: Keys that invalidate this entry when changed\n- `tags`: Metadata tags for grouping and invalidation","ref":"SnmpKit.SnmpLib.Cache.html#put/3-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.put/3","doc":"# Simple time-based caching\n    :ok = SnmpKit.SnmpLib.Cache.put(\"device_1:sysDescr\", \"Cisco Router\", ttl: 600_000)\n\n    # Adaptive caching based on volatility\n    :ok = SnmpKit.SnmpLib.Cache.put(\"device_1:ifTable\", interface_data,\n      strategy: :volatility_based,\n      volatility: :high,\n      tags: [:interface_data]\n    )\n\n    # Dependency-based caching\n    :ok = SnmpKit.SnmpLib.Cache.put(\"device_1:route_summary\", summary_data,\n      dependencies: [\"device_1:routeTable\", \"device_1:arpTable\"]\n    )","ref":"SnmpKit.SnmpLib.Cache.html#put/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.put_adaptive/4","doc":"Stores a value with adaptive TTL based on observed volatility.\n\nThe cache automatically adjusts TTL based on how frequently the data changes.","ref":"SnmpKit.SnmpLib.Cache.html#put_adaptive/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Cache.put_adaptive/4","doc":"- `key`: Cache key\n- `value`: Data to cache\n- `base_ttl`: Starting TTL value\n- `volatility`: Expected change frequency","ref":"SnmpKit.SnmpLib.Cache.html#put_adaptive/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.put_adaptive/4","doc":"# Interface counters change frequently\n    SnmpKit.SnmpLib.Cache.put_adaptive(\"device_1:ifInOctets\", counter_data,\n      base_ttl: 30_000,\n      volatility: :high\n    )\n\n    # System description rarely changes\n    SnmpKit.SnmpLib.Cache.put_adaptive(\"device_1:sysDescr\", description,\n      base_ttl: 3_600_000,\n      volatility: :low\n    )","ref":"SnmpKit.SnmpLib.Cache.html#put_adaptive/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.start_link/1","doc":"Starts the cache manager with specified configuration.","ref":"SnmpKit.SnmpLib.Cache.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Cache.start_link/1","doc":"- `max_size`: Maximum number of cache entries (default: 100,000)\n- `cleanup_interval`: Cleanup frequency in milliseconds (default: 60,000)\n- `compression_enabled`: Enable compression for large values (default: true)\n- `adaptive_ttl_enabled`: Enable adaptive TTL based on volatility (default: true)\n- `predictive_enabled`: Enable predictive caching (default: false)","ref":"SnmpKit.SnmpLib.Cache.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.start_link/1","doc":"# Start with defaults\n    {:ok, _pid} = SnmpKit.SnmpLib.Cache.start_link()\n\n    # Start with custom configuration\n    {:ok, _pid} = SnmpKit.SnmpLib.Cache.start_link(\n      max_size: 50_000,\n      compression_enabled: true,\n      predictive_enabled: true\n    )","ref":"SnmpKit.SnmpLib.Cache.html#start_link/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Cache.volatility/0","doc":"","ref":"SnmpKit.SnmpLib.Cache.html#t:volatility/0"},{"type":"function","title":"SnmpKit.SnmpLib.Cache.warm_cache/3","doc":"Pre-loads cache with expected data to improve response times.","ref":"SnmpKit.SnmpLib.Cache.html#warm_cache/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Cache.warm_cache/3","doc":"- `device_id`: Target device identifier\n- `oids`: List of OIDs to pre-load\n- `strategy`: Warming strategy (:immediate, :scheduled, :predictive)","ref":"SnmpKit.SnmpLib.Cache.html#warm_cache/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Cache.warm_cache/3","doc":"# Immediate cache warming\n    SnmpKit.SnmpLib.Cache.warm_cache(\"device_1\",\n      [\"1.3.6.1.2.1.1.1.0\", \"1.3.6.1.2.1.1.3.0\"],\n      strategy: :immediate\n    )\n\n    # Predictive warming based on historical access\n    SnmpKit.SnmpLib.Cache.warm_cache(\"device_1\", :auto,\n      strategy: :predictive\n    )","ref":"SnmpKit.SnmpLib.Cache.html#warm_cache/3-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Config","doc":"Configuration management system for production SNMP deployments.\n\nThis module provides a flexible, environment-aware configuration system designed\nfor real-world SNMP management applications. Based on patterns proven in large-scale\ndeployments like the DDumb project managing 1000+ devices.","ref":"SnmpKit.SnmpLib.Config.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Config","doc":"- **Environment-Aware**: Automatic detection of dev/test/prod environments\n- **Layered Configuration**: Application, environment, and runtime overrides\n- **Dynamic Updates**: Hot-reload configuration without service restart\n- **Validation**: Schema validation for all configuration values\n- **Secrets Management**: Secure handling of sensitive configuration data\n- **Multi-Tenant Support**: Per-deployment configuration isolation","ref":"SnmpKit.SnmpLib.Config.html#module-features"},{"type":"module","title":"Configuration Sources (in order of precedence) - SnmpKit.SnmpLib.Config","doc":"1. Runtime environment variables\n2. Configuration files (config/*.exs)\n3. Application defaults\n4. Module defaults","ref":"SnmpKit.SnmpLib.Config.html#module-configuration-sources-in-order-of-precedence"},{"type":"module","title":"Usage Patterns - SnmpKit.SnmpLib.Config","doc":"# Get configuration for a specific component\n    pool_config = SnmpKit.SnmpLib.Config.get(:pool, :default_settings)\n\n    # Get configuration with fallback\n    timeout = SnmpKit.SnmpLib.Config.get(:snmp, :timeout, 5000)\n\n    # Update configuration at runtime\n    SnmpKit.SnmpLib.Config.put(:pool, :max_size, 50)\n\n    # Load configuration from file\n    SnmpKit.SnmpLib.Config.load_from_file(\"/etc/snmp_lib/production.exs\")\n\n    # Validate current configuration\n    {:ok, _} = SnmpKit.SnmpLib.Config.validate()","ref":"SnmpKit.SnmpLib.Config.html#module-usage-patterns"},{"type":"module","title":"Environment Detection - SnmpKit.SnmpLib.Config","doc":"The configuration system automatically detects the current environment:\n- `:dev` - Development with verbose logging and relaxed timeouts\n- `:test` - Testing with simulated backends and fast timeouts\n- `:prod` - Production with optimized settings and monitoring\n- `:staging` - Pre-production environment for integration testing","ref":"SnmpKit.SnmpLib.Config.html#module-environment-detection"},{"type":"module","title":"Configuration Schema - SnmpKit.SnmpLib.Config","doc":"All configuration follows a validated schema to prevent runtime errors:\n\n    %{\n      snmp: %{\n        default_version: :v2c,\n        default_timeout: 5_000,\n        default_retries: 3,\n        default_community: \"public\"\n      },\n      pool: %{\n        default_size: 10,\n        max_overflow: 5,\n        strategy: :fifo,\n        health_check_interval: 30_000\n      },\n      monitoring: %{\n        metrics_enabled: true,\n        prometheus_port: 9090,\n        dashboard_enabled: true,\n        alert_thresholds: %{...}\n      }\n    }","ref":"SnmpKit.SnmpLib.Config.html#module-configuration-schema"},{"type":"function","title":"SnmpKit.SnmpLib.Config.all/0","doc":"Gets all configuration as a nested map.","ref":"SnmpKit.SnmpLib.Config.html#all/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.all/0","doc":"config = SnmpKit.SnmpLib.Config.all()\n    IO.inspect(config.snmp.default_timeout)","ref":"SnmpKit.SnmpLib.Config.html#all/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.Config.html#child_spec/1"},{"type":"type","title":"SnmpKit.SnmpLib.Config.config_key/0","doc":"","ref":"SnmpKit.SnmpLib.Config.html#t:config_key/0"},{"type":"type","title":"SnmpKit.SnmpLib.Config.config_section/0","doc":"","ref":"SnmpKit.SnmpLib.Config.html#t:config_section/0"},{"type":"type","title":"SnmpKit.SnmpLib.Config.config_value/0","doc":"","ref":"SnmpKit.SnmpLib.Config.html#t:config_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.Config.environment/0","doc":"Gets the current environment.","ref":"SnmpKit.SnmpLib.Config.html#environment/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.environment/0","doc":"env = SnmpKit.SnmpLib.Config.environment()  # :prod","ref":"SnmpKit.SnmpLib.Config.html#environment/0-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Config.environment/0","doc":"","ref":"SnmpKit.SnmpLib.Config.html#t:environment/0"},{"type":"function","title":"SnmpKit.SnmpLib.Config.get/3","doc":"Gets a configuration value by key path with optional default.","ref":"SnmpKit.SnmpLib.Config.html#get/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Config.get/3","doc":"- `section`: Configuration section (:snmp, :pool, :monitoring, etc.)\n- `key`: Specific configuration key or nested key path\n- `default`: Value to return if key is not found","ref":"SnmpKit.SnmpLib.Config.html#get/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.get/3","doc":"# Get a simple value\n    timeout = SnmpKit.SnmpLib.Config.get(:snmp, :default_timeout)\n\n    # Get nested value\n    threshold = SnmpKit.SnmpLib.Config.get(:monitoring, [:alert_thresholds, :error_rate])\n\n    # Get with default\n    community = SnmpKit.SnmpLib.Config.get(:snmp, :community, \"public\")","ref":"SnmpKit.SnmpLib.Config.html#get/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.load_from_file/1","doc":"Loads configuration from a file and merges with current config.","ref":"SnmpKit.SnmpLib.Config.html#load_from_file/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.load_from_file/1","doc":":ok = SnmpKit.SnmpLib.Config.load_from_file(\"/etc/snmp_lib/production.exs\")","ref":"SnmpKit.SnmpLib.Config.html#load_from_file/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.merge_opts/1","doc":"Merges user-provided options with default SNMP configuration values.\n\nThis function provides the default SNMP configuration options that are commonly\nused across SNMP operations, then merges them with user-provided options. User\noptions take precedence over defaults.","ref":"SnmpKit.SnmpLib.Config.html#merge_opts/1"},{"type":"function","title":"Default Values - SnmpKit.SnmpLib.Config.merge_opts/1","doc":"- `community`: \"public\"\n- `timeout`: 5000 (milliseconds)\n- `retries`: 3\n- `port`: 161\n- `version`: :v2c\n- `mib_paths`: []","ref":"SnmpKit.SnmpLib.Config.html#merge_opts/1-default-values"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Config.merge_opts/1","doc":"- `opts`: Keyword list of user-provided options that override defaults","ref":"SnmpKit.SnmpLib.Config.html#merge_opts/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Config.merge_opts/1","doc":"Keyword list with defaults merged with user options, where user options take precedence.","ref":"SnmpKit.SnmpLib.Config.html#merge_opts/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.merge_opts/1","doc":"iex> SnmpKit.SnmpLib.Config.merge_opts([])\n    [community: \"public\", timeout: 5000, retries: 3, port: 161, version: :v2c, mib_paths: []]\n\n    iex> result = SnmpKit.SnmpLib.Config.merge_opts([timeout: 10000])\n    iex> result[:community]\n    \"public\"\n    iex> result[:timeout]\n    10000\n    iex> result[:retries]\n    3\n\n    iex> result = SnmpKit.SnmpLib.Config.merge_opts([community: \"private\", port: 162])\n    iex> result[:community]\n    \"private\"\n    iex> result[:port]\n    162\n    iex> result[:timeout]\n    5000","ref":"SnmpKit.SnmpLib.Config.html#merge_opts/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.put/3","doc":"Sets a configuration value at runtime.\n\nChanges are applied immediately and optionally persisted.","ref":"SnmpKit.SnmpLib.Config.html#put/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.put/3","doc":":ok = SnmpKit.SnmpLib.Config.put(:pool, :default_size, 20)\n    :ok = SnmpKit.SnmpLib.Config.put(:monitoring, [:alert_thresholds, :error_rate], 0.10)","ref":"SnmpKit.SnmpLib.Config.html#put/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.reload/0","doc":"Reloads configuration from environment and files.","ref":"SnmpKit.SnmpLib.Config.html#reload/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.reload/0","doc":":ok = SnmpKit.SnmpLib.Config.reload()","ref":"SnmpKit.SnmpLib.Config.html#reload/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.start_link/1","doc":"Starts the configuration manager with initial configuration.","ref":"SnmpKit.SnmpLib.Config.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Config.start_link/1","doc":"- `config_file`: Path to configuration file to load on startup\n- `environment`: Override automatic environment detection\n- `validate_on_start`: Validate configuration on startup (default: true)","ref":"SnmpKit.SnmpLib.Config.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.start_link/1","doc":"{:ok, _pid} = SnmpKit.SnmpLib.Config.start_link()\n    {:ok, _pid} = SnmpKit.SnmpLib.Config.start_link(config_file: \"/etc/snmp_lib/prod.exs\")","ref":"SnmpKit.SnmpLib.Config.html#start_link/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.validate/0","doc":"Validates the current configuration against the schema.","ref":"SnmpKit.SnmpLib.Config.html#validate/0"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Config.validate/0","doc":"- `{:ok, config}`: Configuration is valid\n- `{:error, errors}`: List of validation errors","ref":"SnmpKit.SnmpLib.Config.html#validate/0-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.validate/0","doc":"case SnmpKit.SnmpLib.Config.validate() do\n      {:ok, _config} -> Logger.info(\"Configuration is valid\")\n      {:error, validation_errors} -> Logger.error(\"Configuration errors: \" <> inspect(validation_errors))\n    end","ref":"SnmpKit.SnmpLib.Config.html#validate/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Config.watch/2","doc":"Registers a callback function to be called when configuration changes.","ref":"SnmpKit.SnmpLib.Config.html#watch/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Config.watch/2","doc":"SnmpKit.SnmpLib.Config.watch(:pool, fn old_config, new_config ->\n      Logger.info(\"Pool configuration changed\")\n      SnmpKit.SnmpLib.Pool.reload_config(new_config)\n    end)","ref":"SnmpKit.SnmpLib.Config.html#watch/2-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Dashboard","doc":"Real-time monitoring dashboard and metrics aggregation for SNMP operations.\n\nThis module provides a comprehensive monitoring and visualization system for\nproduction SNMP deployments. Based on patterns proven in large-scale monitoring\nsystems managing thousands of network devices.","ref":"SnmpKit.SnmpLib.Dashboard.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Dashboard","doc":"- **Real-Time Metrics**: Live updates of performance and health metrics\n- **Historical Analytics**: Trend analysis and capacity planning data\n- **Alert Management**: Configurable thresholds and notification routing\n- **Performance Insights**: Detailed breakdown of operation performance\n- **Device Health**: Per-device status monitoring and diagnostics\n- **Resource Utilization**: Pool, memory, and system resource tracking","ref":"SnmpKit.SnmpLib.Dashboard.html#module-features"},{"type":"module","title":"Metrics Categories - SnmpKit.SnmpLib.Dashboard","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#module-metrics-categories"},{"type":"module","title":"Performance Metrics - SnmpKit.SnmpLib.Dashboard","doc":"- Request/response times (min, max, average, percentiles)\n- Throughput (operations per second)\n- Error rates and failure classifications\n- Connection pool utilization","ref":"SnmpKit.SnmpLib.Dashboard.html#module-performance-metrics"},{"type":"module","title":"Health Metrics - SnmpKit.SnmpLib.Dashboard","doc":"- Device availability and reachability\n- Circuit breaker states\n- Retry counts and backoff status\n- Resource exhaustion indicators","ref":"SnmpKit.SnmpLib.Dashboard.html#module-health-metrics"},{"type":"module","title":"System Metrics - SnmpKit.SnmpLib.Dashboard","doc":"- Memory usage and garbage collection\n- Process counts and supervision tree health\n- Network socket utilization\n- Queue depths and processing delays","ref":"SnmpKit.SnmpLib.Dashboard.html#module-system-metrics"},{"type":"module","title":"Dashboard Views - SnmpKit.SnmpLib.Dashboard","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#module-dashboard-views"},{"type":"module","title":"Overview Dashboard - SnmpKit.SnmpLib.Dashboard","doc":"Global health and performance summary with key indicators.","ref":"SnmpKit.SnmpLib.Dashboard.html#module-overview-dashboard"},{"type":"module","title":"Device Dashboard - SnmpKit.SnmpLib.Dashboard","doc":"Per-device detailed metrics and troubleshooting information.","ref":"SnmpKit.SnmpLib.Dashboard.html#module-device-dashboard"},{"type":"module","title":"Pool Dashboard - SnmpKit.SnmpLib.Dashboard","doc":"Connection pool health, utilization, and performance metrics.","ref":"SnmpKit.SnmpLib.Dashboard.html#module-pool-dashboard"},{"type":"module","title":"Alerts Dashboard - SnmpKit.SnmpLib.Dashboard","doc":"Active alerts, acknowledgments, and escalation status.","ref":"SnmpKit.SnmpLib.Dashboard.html#module-alerts-dashboard"},{"type":"module","title":"Usage Patterns - SnmpKit.SnmpLib.Dashboard","doc":"# Start the dashboard server\n    {:ok, _pid} = SnmpKit.SnmpLib.Dashboard.start_link(port: 4000)\n\n    # Record custom metrics\n    SnmpKit.SnmpLib.Dashboard.record_metric(:custom_operation, %{\n      duration: 150,\n      device: \"192.168.1.1\",\n      status: :success\n    })\n\n    # Create custom alert\n    SnmpKit.SnmpLib.Dashboard.create_alert(:high_error_rate, %{\n      device: \"192.168.1.100\",\n      error_rate: 0.15,\n      threshold: 0.10\n    })\n\n    # Export metrics for external systems\n    prometheus_data = SnmpKit.SnmpLib.Dashboard.export_prometheus()","ref":"SnmpKit.SnmpLib.Dashboard.html#module-usage-patterns"},{"type":"module","title":"Integration with External Systems - SnmpKit.SnmpLib.Dashboard","doc":"- **Prometheus**: Native metrics export in Prometheus format\n- **Grafana**: Pre-built dashboards and alerting rules\n- **PagerDuty**: Alert escalation and incident management\n- **Slack/Teams**: Notification integration for team alerting","ref":"SnmpKit.SnmpLib.Dashboard.html#module-integration-with-external-systems"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.acknowledge_alert/2","doc":"Acknowledges an alert to stop notifications.","ref":"SnmpKit.SnmpLib.Dashboard.html#acknowledge_alert/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.acknowledge_alert/2","doc":":ok = SnmpKit.SnmpLib.Dashboard.acknowledge_alert(:device_unreachable, \"192.168.1.1\")","ref":"SnmpKit.SnmpLib.Dashboard.html#acknowledge_alert/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Dashboard.alert_level/0","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#t:alert_level/0"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.Dashboard.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.create_alert/3","doc":"Creates an alert for monitoring and notification systems.","ref":"SnmpKit.SnmpLib.Dashboard.html#create_alert/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Dashboard.create_alert/3","doc":"- `alert_name`: Unique identifier for the alert type\n- `level`: Alert severity level (:info, :warning, :critical)\n- `details`: Alert metadata and context information","ref":"SnmpKit.SnmpLib.Dashboard.html#create_alert/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.create_alert/3","doc":"# Create device unreachable alert\n    SnmpKit.SnmpLib.Dashboard.create_alert(:device_unreachable, :critical, %{\n      device: \"192.168.1.1\",\n      last_seen: DateTime.utc_now(),\n      consecutive_failures: 5\n    })\n\n    # Create performance degradation warning\n    SnmpKit.SnmpLib.Dashboard.create_alert(:slow_response, :warning, %{\n      device: \"192.168.1.1\",\n      avg_response_time: 5000,\n      threshold: 2000\n    })","ref":"SnmpKit.SnmpLib.Dashboard.html#create_alert/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Dashboard.dashboard_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#t:dashboard_opts/0"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.export_prometheus/0","doc":"Exports metrics in Prometheus format for external monitoring.","ref":"SnmpKit.SnmpLib.Dashboard.html#export_prometheus/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.export_prometheus/0","doc":"prometheus_data = SnmpKit.SnmpLib.Dashboard.export_prometheus()\n    File.write!(\"/tmp/snmp_metrics.prom\", prometheus_data)","ref":"SnmpKit.SnmpLib.Dashboard.html#export_prometheus/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.get_active_alerts/1","doc":"Gets all active alerts with optional filtering.","ref":"SnmpKit.SnmpLib.Dashboard.html#get_active_alerts/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.get_active_alerts/1","doc":"all_alerts = SnmpKit.SnmpLib.Dashboard.get_active_alerts()\n    critical_alerts = SnmpKit.SnmpLib.Dashboard.get_active_alerts(level: :critical)","ref":"SnmpKit.SnmpLib.Dashboard.html#get_active_alerts/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.get_device_metrics/1","doc":"Gets detailed metrics for a specific device.","ref":"SnmpKit.SnmpLib.Dashboard.html#get_device_metrics/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.get_device_metrics/1","doc":"device_metrics = SnmpKit.SnmpLib.Dashboard.get_device_metrics(\"192.168.1.1\")\n    IO.inspect device_metrics.response_times","ref":"SnmpKit.SnmpLib.Dashboard.html#get_device_metrics/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.get_metrics_summary/0","doc":"Gets current performance metrics summary.","ref":"SnmpKit.SnmpLib.Dashboard.html#get_metrics_summary/0"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Dashboard.get_metrics_summary/0","doc":"A map containing aggregated metrics:\n- `total_operations`: Total SNMP operations performed\n- `success_rate`: Percentage of successful operations\n- `avg_response_time`: Average response time in milliseconds\n- `active_devices`: Number of devices being monitored\n- `pool_utilization`: Connection pool usage percentage\n- `error_rates`: Breakdown of error types and frequencies","ref":"SnmpKit.SnmpLib.Dashboard.html#get_metrics_summary/0-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.get_metrics_summary/0","doc":"metrics = SnmpKit.SnmpLib.Dashboard.get_metrics_summary()\n    IO.puts \"Success rate: \" <> Float.to_string(metrics.success_rate * 100) <> \"%\"","ref":"SnmpKit.SnmpLib.Dashboard.html#get_metrics_summary/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.get_timeseries/3","doc":"Gets historical time series data for a metric.","ref":"SnmpKit.SnmpLib.Dashboard.html#get_timeseries/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Dashboard.get_timeseries/3","doc":"- `metric_name`: Name of the metric to retrieve\n- `duration`: Time window in milliseconds (default: 1 hour)\n- `tags`: Optional tag filters","ref":"SnmpKit.SnmpLib.Dashboard.html#get_timeseries/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.get_timeseries/3","doc":"# Get last hour of response times\n    timeseries = SnmpKit.SnmpLib.Dashboard.get_timeseries(:snmp_response_time)\n\n    # Get last 24 hours for specific device\n    device_data = SnmpKit.SnmpLib.Dashboard.get_timeseries(\n      :snmp_response_time,\n      24 * 60 * 60 * 1000,\n      %{device: \"192.168.1.1\"}\n    )","ref":"SnmpKit.SnmpLib.Dashboard.html#get_timeseries/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Dashboard.metric_name/0","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#t:metric_name/0"},{"type":"type","title":"SnmpKit.SnmpLib.Dashboard.metric_tags/0","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#t:metric_tags/0"},{"type":"type","title":"SnmpKit.SnmpLib.Dashboard.metric_value/0","doc":"","ref":"SnmpKit.SnmpLib.Dashboard.html#t:metric_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.record_metric/3","doc":"Records a metric data point for monitoring and visualization.","ref":"SnmpKit.SnmpLib.Dashboard.html#record_metric/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Dashboard.record_metric/3","doc":"- `metric_name`: Unique identifier for the metric type\n- `value`: Numeric value for the metric\n- `tags`: Optional metadata for filtering and grouping","ref":"SnmpKit.SnmpLib.Dashboard.html#record_metric/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.record_metric/3","doc":"# Record response time metric\n    SnmpKit.SnmpLib.Dashboard.record_metric(:snmp_response_time, 125, %{\n      device: \"192.168.1.1\",\n      operation: \"get\",\n      community: \"public\"\n    })\n\n    # Record error count\n    SnmpKit.SnmpLib.Dashboard.record_metric(:snmp_errors, 1, %{\n      device: \"192.168.1.1\",\n      error_type: \"timeout\"\n    })\n\n    # Record pool utilization\n    SnmpKit.SnmpLib.Dashboard.record_metric(:pool_utilization, 0.75, %{\n      pool_name: \"main_pool\"\n    })","ref":"SnmpKit.SnmpLib.Dashboard.html#record_metric/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Dashboard.start_link/1","doc":"Starts the dashboard server with monitoring and web interface.","ref":"SnmpKit.SnmpLib.Dashboard.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Dashboard.start_link/1","doc":"- `port`: Web dashboard port (default: 4000)\n- `update_interval`: Metrics update frequency in milliseconds (default: 5000)\n- `retention_days`: How long to keep historical data (default: 7)\n- `prometheus_enabled`: Enable Prometheus metrics endpoint (default: false)\n- `grafana_integration`: Enable Grafana dashboard integration (default: false)","ref":"SnmpKit.SnmpLib.Dashboard.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Dashboard.start_link/1","doc":"# Start with defaults\n    {:ok, _pid} = SnmpKit.SnmpLib.Dashboard.start_link()\n\n    # Start with custom configuration\n    {:ok, _pid} = SnmpKit.SnmpLib.Dashboard.start_link(\n      port: 8080,\n      prometheus_enabled: true,\n      retention_days: 14\n    )","ref":"SnmpKit.SnmpLib.Dashboard.html#start_link/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Error","doc":"Standard SNMP error handling and error code utilities.\n\nProvides standardized error codes, error handling utilities, and error response\ngeneration for SNMP operations. This module centralizes all SNMP-specific error\nhandling to ensure consistent error reporting across the library.","ref":"SnmpKit.SnmpLib.Error.html"},{"type":"module","title":"SNMP Error Codes - SnmpKit.SnmpLib.Error","doc":"Standard SNMP error status values as defined in RFC 1157 and RFC 3416:\n\n- `no_error` (0) - No error occurred\n- `too_big` (1) - Response message would be too large\n- `no_such_name` (2) - Requested OID does not exist\n- `bad_value` (3) - Invalid value for SET operation\n- `read_only` (4) - Attempted to set read-only variable\n- `gen_err` (5) - General error","ref":"SnmpKit.SnmpLib.Error.html#module-snmp-error-codes"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Error","doc":"","ref":"SnmpKit.SnmpLib.Error.html#module-usage-examples"},{"type":"module","title":"Basic Error Handling - SnmpKit.SnmpLib.Error","doc":"# Check if an error is retriable\n    if SnmpKit.SnmpLib.Error.retriable_error?(error_code) do\n      retry_operation()\n    end\n\n    # Format error for logging\n    error_msg = SnmpKit.SnmpLib.Error.format_error(3, 2, varbinds)\n    Logger.error(error_msg)","ref":"SnmpKit.SnmpLib.Error.html#module-basic-error-handling"},{"type":"module","title":"Error Response Generation - SnmpKit.SnmpLib.Error","doc":"# Create error response for invalid request\n    {:ok, error_response} = SnmpKit.SnmpLib.Error.create_error_response(\n      request_pdu,\n      :no_such_name,\n      error_index\n    )","ref":"SnmpKit.SnmpLib.Error.html#module-error-response-generation"},{"type":"function","title":"SnmpKit.SnmpLib.Error.all_error_atoms/0","doc":"Returns a list of all standard SNMP error atoms.","ref":"SnmpKit.SnmpLib.Error.html#all_error_atoms/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.all_error_atoms/0","doc":"iex> atoms = SnmpKit.SnmpLib.Error.all_error_atoms()\n    iex> :no_error in atoms\n    true\n    iex> :gen_err in atoms\n    true","ref":"SnmpKit.SnmpLib.Error.html#all_error_atoms/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.all_error_codes/0","doc":"Returns a list of all standard SNMP error codes.","ref":"SnmpKit.SnmpLib.Error.html#all_error_codes/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.all_error_codes/0","doc":"iex> codes = SnmpKit.SnmpLib.Error.all_error_codes()\n    iex> 0 in codes\n    true\n    iex> 5 in codes\n    true","ref":"SnmpKit.SnmpLib.Error.html#all_error_codes/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.bad_value/0","doc":"Returns the numeric code for 'bad value' error status.\n\nThe value provided in a SET operation is invalid for the variable.","ref":"SnmpKit.SnmpLib.Error.html#bad_value/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.bad_value/0","doc":"iex> SnmpKit.SnmpLib.Error.bad_value()\n    3","ref":"SnmpKit.SnmpLib.Error.html#bad_value/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.create_error_response/3","doc":"Creates an SNMP error response PDU.\n\nGenerates a properly formatted error response based on the original request\nand the error condition that occurred.","ref":"SnmpKit.SnmpLib.Error.html#create_error_response/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Error.create_error_response/3","doc":"- `request_pdu`: Original request PDU\n- `error_status`: Error status code or atom\n- `error_index`: Index of the varbind that caused the error (1-based)","ref":"SnmpKit.SnmpLib.Error.html#create_error_response/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Error.create_error_response/3","doc":"- `{:ok, error_pdu}`: Successfully created error response\n- `{:error, reason}`: Failed to create error response","ref":"SnmpKit.SnmpLib.Error.html#create_error_response/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.create_error_response/3","doc":"request = %{type: :get_request, request_id: 123, varbinds: [...]}\n    {:ok, error_response} = SnmpKit.SnmpLib.Error.create_error_response(\n      request,\n      :no_such_name,\n      1\n    )","ref":"SnmpKit.SnmpLib.Error.html#create_error_response/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.error_atom/1","doc":"Converts error status code to atom representation.","ref":"SnmpKit.SnmpLib.Error.html#error_atom/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.error_atom/1","doc":"iex> SnmpKit.SnmpLib.Error.error_atom(2)\n    :no_such_name\n\n    iex> SnmpKit.SnmpLib.Error.error_atom(999)\n    :unknown_error","ref":"SnmpKit.SnmpLib.Error.html#error_atom/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.error_code/1","doc":"Converts error atom or name to numeric code.","ref":"SnmpKit.SnmpLib.Error.html#error_code/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.error_code/1","doc":"iex> SnmpKit.SnmpLib.Error.error_code(:no_such_name)\n    2\n\n    iex> SnmpKit.SnmpLib.Error.error_code(\"bad_value\")\n    3","ref":"SnmpKit.SnmpLib.Error.html#error_code/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Error.error_index/0","doc":"","ref":"SnmpKit.SnmpLib.Error.html#t:error_index/0"},{"type":"function","title":"SnmpKit.SnmpLib.Error.error_name/1","doc":"Returns the human-readable name for an error status code.","ref":"SnmpKit.SnmpLib.Error.html#error_name/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Error.error_name/1","doc":"- `code`: Numeric error status code or atom","ref":"SnmpKit.SnmpLib.Error.html#error_name/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Error.error_name/1","doc":"- String name of the error status\n- \"unknown_error\" for unrecognized codes","ref":"SnmpKit.SnmpLib.Error.html#error_name/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.error_name/1","doc":"iex> SnmpKit.SnmpLib.Error.error_name(0)\n    \"no_error\"\n\n    iex> SnmpKit.SnmpLib.Error.error_name(:too_big)\n    \"too_big\"\n\n    iex> SnmpKit.SnmpLib.Error.error_name(999)\n    \"unknown_error\"","ref":"SnmpKit.SnmpLib.Error.html#error_name/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.error_severity/1","doc":"Categorizes error by severity level.","ref":"SnmpKit.SnmpLib.Error.html#error_severity/1"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Error.error_severity/1","doc":"- `:info` - No error\n- `:warning` - Retriable errors\n- `:error` - Non-retriable errors","ref":"SnmpKit.SnmpLib.Error.html#error_severity/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.error_severity/1","doc":"iex> SnmpKit.SnmpLib.Error.error_severity(:no_error)\n    :info\n\n    iex> SnmpKit.SnmpLib.Error.error_severity(:too_big)\n    :warning\n\n    iex> SnmpKit.SnmpLib.Error.error_severity(:no_such_name)\n    :error","ref":"SnmpKit.SnmpLib.Error.html#error_severity/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Error.error_status/0","doc":"","ref":"SnmpKit.SnmpLib.Error.html#t:error_status/0"},{"type":"function","title":"SnmpKit.SnmpLib.Error.format_error/3","doc":"Formats an SNMP error for human-readable display.","ref":"SnmpKit.SnmpLib.Error.html#format_error/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Error.format_error/3","doc":"- `error_status`: Error status code (integer or atom)\n- `error_index`: Index of the varbind that caused the error (1-based)\n- `varbinds`: List of varbinds from the request (optional)","ref":"SnmpKit.SnmpLib.Error.html#format_error/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Error.format_error/3","doc":"Formatted error string suitable for logging or display.","ref":"SnmpKit.SnmpLib.Error.html#format_error/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.format_error/3","doc":"iex> SnmpKit.SnmpLib.Error.format_error(2, 1, [])\n    \"SNMP Error: no_such_name (2) at index 1\"\n\n    iex> varbinds = [{[1,3,6,1,2,1,1,1,0], \"test\"}]\n    iex> SnmpKit.SnmpLib.Error.format_error(:bad_value, 1, varbinds)\n    \"SNMP Error: bad_value (3) at index 1 - OID: 1.3.6.1.2.1.1.1.0\"","ref":"SnmpKit.SnmpLib.Error.html#format_error/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.gen_err/0","doc":"Returns the numeric code for 'general error' status.\n\nA general error occurred that doesn't fit other categories.","ref":"SnmpKit.SnmpLib.Error.html#gen_err/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.gen_err/0","doc":"iex> SnmpKit.SnmpLib.Error.gen_err()\n    5","ref":"SnmpKit.SnmpLib.Error.html#gen_err/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.no_error/0","doc":"Returns the numeric code for 'no error' status.","ref":"SnmpKit.SnmpLib.Error.html#no_error/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.no_error/0","doc":"iex> SnmpKit.SnmpLib.Error.no_error()\n    0","ref":"SnmpKit.SnmpLib.Error.html#no_error/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.no_such_name/0","doc":"Returns the numeric code for 'no such name' error status.\n\nThe requested OID does not exist on the agent.","ref":"SnmpKit.SnmpLib.Error.html#no_such_name/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.no_such_name/0","doc":"iex> SnmpKit.SnmpLib.Error.no_such_name()\n    2","ref":"SnmpKit.SnmpLib.Error.html#no_such_name/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.read_only/0","doc":"Returns the numeric code for 'read only' error status.\n\nAttempted to set a read-only variable.","ref":"SnmpKit.SnmpLib.Error.html#read_only/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.read_only/0","doc":"iex> SnmpKit.SnmpLib.Error.read_only()\n    4","ref":"SnmpKit.SnmpLib.Error.html#read_only/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.retriable_error?/1","doc":"Determines if an error status indicates a retriable condition.\n\nSome SNMP errors are temporary and operations can be retried, while others\nindicate permanent failures.","ref":"SnmpKit.SnmpLib.Error.html#retriable_error?/1"},{"type":"function","title":"Retriable Errors - SnmpKit.SnmpLib.Error.retriable_error?/1","doc":"- `too_big` - Can retry with smaller request\n- `gen_err` - General error, may be temporary\n- `resource_unavailable` - Temporary resource constraint","ref":"SnmpKit.SnmpLib.Error.html#retriable_error?/1-retriable-errors"},{"type":"function","title":"Non-Retriable Errors - SnmpKit.SnmpLib.Error.retriable_error?/1","doc":"- `no_such_name` - OID doesn't exist\n- `bad_value` - Invalid value provided\n- `read_only` - Attempted to write read-only variable\n- `no_access` - Access denied\n- Most SNMPv2c specific errors","ref":"SnmpKit.SnmpLib.Error.html#retriable_error?/1-non-retriable-errors"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.retriable_error?/1","doc":"iex> SnmpKit.SnmpLib.Error.retriable_error?(:too_big)\n    true\n\n    iex> SnmpKit.SnmpLib.Error.retriable_error?(:no_such_name)\n    false","ref":"SnmpKit.SnmpLib.Error.html#retriable_error?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.too_big/0","doc":"Returns the numeric code for 'too big' error status.\n\nThe response message would be too large to fit in a single SNMP message.","ref":"SnmpKit.SnmpLib.Error.html#too_big/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.too_big/0","doc":"iex> SnmpKit.SnmpLib.Error.too_big()\n    1","ref":"SnmpKit.SnmpLib.Error.html#too_big/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Error.valid_error_status?/1","doc":"Validates an error status code.","ref":"SnmpKit.SnmpLib.Error.html#valid_error_status?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Error.valid_error_status?/1","doc":"iex> SnmpKit.SnmpLib.Error.valid_error_status?(2)\n    true\n\n    iex> SnmpKit.SnmpLib.Error.valid_error_status?(:no_such_name)\n    true\n\n    iex> SnmpKit.SnmpLib.Error.valid_error_status?(999)\n    false","ref":"SnmpKit.SnmpLib.Error.html#valid_error_status?/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Error.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.Error.html#t:varbind/0"},{"type":"type","title":"SnmpKit.SnmpLib.Error.varbinds/0","doc":"","ref":"SnmpKit.SnmpLib.Error.html#t:varbinds/0"},{"type":"module","title":"SnmpKit.SnmpLib.ErrorHandler","doc":"Intelligent error handling with retry logic, circuit breakers, and adaptive recovery.\n\nThis module provides sophisticated error handling capabilities designed to improve\nreliability and performance in production SNMP environments. Based on patterns\nproven in high-scale network monitoring systems handling thousands of devices.","ref":"SnmpKit.SnmpLib.ErrorHandler.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.ErrorHandler","doc":"- **Exponential Backoff**: Intelligent retry timing to avoid overwhelming failing devices\n- **Circuit Breakers**: Automatic failure detection and recovery for unhealthy devices\n- **Error Classification**: Smart categorization of errors for appropriate handling\n- **Adaptive Timeouts**: Dynamic timeout adjustment based on device performance\n- **Quarantine Management**: Temporary isolation of problematic devices\n- **Recovery Strategies**: Multiple approaches for bringing devices back online","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-features"},{"type":"module","title":"Error Classification - SnmpKit.SnmpLib.ErrorHandler","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-error-classification"},{"type":"module","title":"Transient Errors (Retryable) - SnmpKit.SnmpLib.ErrorHandler","doc":"- Network timeouts\n- Temporary device overload\n- UDP packet loss\n- DNS resolution delays","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-transient-errors-retryable"},{"type":"module","title":"Permanent Errors (Non-retryable) - SnmpKit.SnmpLib.ErrorHandler","doc":"- Authentication failures\n- Unsupported SNMP versions\n- Invalid OIDs\n- Device configuration errors","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-permanent-errors-non-retryable"},{"type":"module","title":"Degraded Performance - SnmpKit.SnmpLib.ErrorHandler","doc":"- Slow response times\n- Partial failures\n- High error rates\n- Resource exhaustion","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-degraded-performance"},{"type":"module","title":"Circuit Breaker States - SnmpKit.SnmpLib.ErrorHandler","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-circuit-breaker-states"},{"type":"module","title":"Closed (Normal Operation) - SnmpKit.SnmpLib.ErrorHandler","doc":"Device is healthy, all operations proceed normally.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-closed-normal-operation"},{"type":"module","title":"Open (Failing) - SnmpKit.SnmpLib.ErrorHandler","doc":"Device has exceeded failure threshold, operations are blocked.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-open-failing"},{"type":"module","title":"Half-Open (Testing) - SnmpKit.SnmpLib.ErrorHandler","doc":"Limited operations allowed to test device recovery.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-half-open-testing"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.ErrorHandler","doc":"# Basic retry with exponential backoff\n    result = SnmpKit.SnmpLib.ErrorHandler.with_retry(fn ->\n      SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1,3,6,1,2,1,1,1,0])\n    end, max_attempts: 3)\n\n    # Circuit breaker for device management\n    {:ok, breaker} = SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker(\"192.168.1.1\")\n\n    result = SnmpKit.SnmpLib.ErrorHandler.call_through_breaker(breaker, fn ->\n      SnmpKit.SnmpLib.Manager.get_bulk(\"192.168.1.1\", [1,3,6,1,2,1,2,2])\n    end)\n\n    # Adaptive timeout based on device history\n    timeout = SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout(\"192.168.1.1\", base_timeout: 5000)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#module-usage-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout/2","doc":"Calculates an adaptive timeout based on device performance history.\n\nDynamically adjusts timeouts based on historical response times,\ndevice health, and current network conditions.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#adaptive_timeout/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout/2","doc":"- `device_id`: Device identifier\n- `opts`: Timeout calculation options","ref":"SnmpKit.SnmpLib.ErrorHandler.html#adaptive_timeout/2-parameters"},{"type":"function","title":"Options - SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout/2","doc":"- `base_timeout`: Minimum timeout value (default: 5000ms)\n- `max_timeout`: Maximum timeout value (default: 60000ms)\n- `percentile`: Response time percentile to use (default: 95)\n- `safety_factor`: Multiplier for calculated timeout (default: 2.0)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#adaptive_timeout/2-options"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout/2","doc":"Calculated timeout in milliseconds","ref":"SnmpKit.SnmpLib.ErrorHandler.html#adaptive_timeout/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout/2","doc":"# Basic adaptive timeout\n    timeout = SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout(\"192.168.1.1\")\n\n    # Custom timeout parameters\n    timeout = SnmpKit.SnmpLib.ErrorHandler.adaptive_timeout(\"slow.device.local\",\n      base_timeout: 10_000,\n      max_timeout: 120_000,\n      percentile: 99,\n      safety_factor: 3.0\n    )","ref":"SnmpKit.SnmpLib.ErrorHandler.html#adaptive_timeout/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.call_through_breaker/3","doc":"Executes a function through a circuit breaker.\n\nThe circuit breaker monitors the operation and may block future calls\nif the device is experiencing failures.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#call_through_breaker/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ErrorHandler.call_through_breaker/3","doc":"- `breaker_pid`: PID of the circuit breaker process\n- `fun`: Function to execute\n- `timeout`: Maximum execution time (optional)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#call_through_breaker/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ErrorHandler.call_through_breaker/3","doc":"- `{:ok, result}`: Operation succeeded\n- `{:error, reason}`: Operation failed\n- `{:error, :circuit_open}`: Circuit breaker is open (device unhealthy)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#call_through_breaker/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.call_through_breaker/3","doc":"result = SnmpKit.SnmpLib.ErrorHandler.call_through_breaker(breaker, fn ->\n      SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1,3,6,1,2,1,1,1,0])\n    end)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#call_through_breaker/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#child_spec/1"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.circuit_breaker_opts/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:circuit_breaker_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.circuit_state/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:circuit_state/0"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.classify_error/1","doc":"Classifies an error to determine appropriate handling strategy.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#classify_error/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ErrorHandler.classify_error/1","doc":"- `error`: The error to classify","ref":"SnmpKit.SnmpLib.ErrorHandler.html#classify_error/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ErrorHandler.classify_error/1","doc":"- `:transient`: Error is likely temporary, retry recommended\n- `:permanent`: Error is permanent, retry not recommended\n- `:degraded`: Performance issue, may benefit from backoff\n- `:unknown`: Unable to classify, use conservative approach","ref":"SnmpKit.SnmpLib.ErrorHandler.html#classify_error/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.classify_error/1","doc":":transient = SnmpKit.SnmpLib.ErrorHandler.classify_error(:timeout)\n    :permanent = SnmpKit.SnmpLib.ErrorHandler.classify_error(:authentication_failed)\n    :degraded = SnmpKit.SnmpLib.ErrorHandler.classify_error(:slow_response)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#classify_error/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.device_id/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:device_id/0"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.device_stats/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:device_stats/0"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.error_class/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:error_class/0"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.get_device_stats/1","doc":"Gets comprehensive error statistics for a device.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#get_device_stats/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.get_device_stats/1","doc":"{:ok, stats} = SnmpKit.SnmpLib.ErrorHandler.get_device_stats(\"192.168.1.1\")\n    IO.inspect(stats.failure_count)","ref":"SnmpKit.SnmpLib.ErrorHandler.html#get_device_stats/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.quarantine_device/2","doc":"Puts a device into quarantine for a specified duration.\n\nQuarantined devices have operations blocked to allow recovery.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#quarantine_device/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.quarantine_device/2","doc":":ok = SnmpKit.SnmpLib.ErrorHandler.quarantine_device(\"192.168.1.1\", 300_000)  # 5 minutes","ref":"SnmpKit.SnmpLib.ErrorHandler.html#quarantine_device/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.quarantined?/1","doc":"Checks if a device is currently quarantined.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#quarantined?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.quarantined?/1","doc":"false = SnmpKit.SnmpLib.ErrorHandler.quarantined?(\"192.168.1.1\")","ref":"SnmpKit.SnmpLib.ErrorHandler.html#quarantined?/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.retry_opts/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:retry_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.ErrorHandler.retry_strategy/0","doc":"","ref":"SnmpKit.SnmpLib.ErrorHandler.html#t:retry_strategy/0"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker/2","doc":"Starts a circuit breaker for a specific device.\n\nCircuit breakers automatically detect failing devices and prevent\ncascading failures by temporarily blocking operations.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#start_circuit_breaker/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker/2","doc":"- `device_id`: Unique identifier for the device\n- `opts`: Circuit breaker configuration options","ref":"SnmpKit.SnmpLib.ErrorHandler.html#start_circuit_breaker/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker/2","doc":"- `{:ok, pid}`: Circuit breaker started successfully\n- `{:error, reason}`: Failed to start circuit breaker","ref":"SnmpKit.SnmpLib.ErrorHandler.html#start_circuit_breaker/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker/2","doc":"{:ok, breaker} = SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker(\"192.168.1.1\")\n\n    {:ok, breaker} = SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker(\"core-switch-01\",\n      failure_threshold: 10,\n      recovery_timeout: 120_000\n    )","ref":"SnmpKit.SnmpLib.ErrorHandler.html#start_circuit_breaker/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.ErrorHandler.with_retry/2","doc":"Executes a function with intelligent retry logic and exponential backoff.\n\nAutomatically retries transient failures while avoiding permanent errors.\nUses exponential backoff with jitter to prevent thundering herd problems.","ref":"SnmpKit.SnmpLib.ErrorHandler.html#with_retry/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.ErrorHandler.with_retry/2","doc":"- `fun`: Function to execute (should return `{:ok, result}` or `{:error, reason}`)\n- `opts`: Retry configuration options","ref":"SnmpKit.SnmpLib.ErrorHandler.html#with_retry/2-parameters"},{"type":"function","title":"Options - SnmpKit.SnmpLib.ErrorHandler.with_retry/2","doc":"- `max_attempts`: Maximum retry attempts (default: 3)\n- `strategy`: Backoff strategy (:exponential, :linear, :fixed)\n- `base_delay`: Initial delay in milliseconds (default: 1000)\n- `max_delay`: Maximum delay between retries (default: 30000)\n- `jitter_factor`: Random variation factor (default: 0.1)\n- `retry_condition`: Custom function to determine if error is retryable","ref":"SnmpKit.SnmpLib.ErrorHandler.html#with_retry/2-options"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.ErrorHandler.with_retry/2","doc":"- `{:ok, result}`: Operation succeeded (possibly after retries)\n- `{:error, reason}`: Operation failed after all attempts\n- `{:error, {:max_retries_exceeded, last_error}}`: All retries exhausted","ref":"SnmpKit.SnmpLib.ErrorHandler.html#with_retry/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.ErrorHandler.with_retry/2","doc":"# Basic retry with defaults\n    result = SnmpKit.SnmpLib.ErrorHandler.with_retry(fn ->\n      SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1,3,6,1,2,1,1,1,0])\n    end)\n\n    # Custom retry configuration\n    result = SnmpKit.SnmpLib.ErrorHandler.with_retry(fn ->\n      SnmpKit.SnmpLib.Manager.get_bulk(\"slow.device.local\", [1,3,6,1,2,1,2,2])\n    end,\n    max_attempts: 5,\n    base_delay: 2000,\n    max_delay: 60000,\n    strategy: :exponential\n    )","ref":"SnmpKit.SnmpLib.ErrorHandler.html#with_retry/2-examples"},{"type":"module","title":"SnmpKit.SnmpLib.HostParser","doc":"Comprehensive host and port parsing for all possible input formats.\n\nThis module handles parsing of host and port information from any conceivable\ninput format and returns exactly what gen_udp needs: an IP tuple and integer port.","ref":"SnmpKit.SnmpLib.HostParser.html"},{"type":"module","title":"Supported Input Formats - SnmpKit.SnmpLib.HostParser","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#module-supported-input-formats"},{"type":"module","title":"IPv4 - SnmpKit.SnmpLib.HostParser","doc":"- Tuples: `{192, 168, 1, 1}`, `{{192, 168, 1, 1}, 161}`\n- Strings: `\"192.168.1.1\"`, `\"192.168.1.1:161\"`\n- Charlists: `'192.168.1.1'`, `'192.168.1.1:161'`\n- Hostnames: `\"router.local\"`, `\"router.local:161\"`","ref":"SnmpKit.SnmpLib.HostParser.html#module-ipv4"},{"type":"module","title":"IPv6 - SnmpKit.SnmpLib.HostParser","doc":"- Tuples: `{0x2001, 0xdb8, 0, 0, 0, 0, 0, 1}`, `{{0x2001, 0xdb8, 0, 0, 0, 0, 0, 1}, 161}`\n- Strings: `\"2001:db8::1\"`, `\"[2001:db8::1]:161\"`\n- Charlists: `'2001:db8::1'`, `'[2001:db8::1]:161'`\n- Compressed: `\"::1\"`, `\"[::1]:161\"`","ref":"SnmpKit.SnmpLib.HostParser.html#module-ipv6"},{"type":"module","title":"Mixed Formats - SnmpKit.SnmpLib.HostParser","doc":"- Maps: `%{host: \"192.168.1.1\", port: 161}`\n- Keyword lists: `[host: \"192.168.1.1\", port: 161]`","ref":"SnmpKit.SnmpLib.HostParser.html#module-mixed-formats"},{"type":"module","title":"Returns - SnmpKit.SnmpLib.HostParser","doc":"`{:ok, {ip_tuple, port}}` where:\n- `ip_tuple` is a 4-tuple for IPv4 or 8-tuple for IPv6\n- `port` is an integer between 1-65535\n\n`{:error, reason}` for invalid input","ref":"SnmpKit.SnmpLib.HostParser.html#module-returns"},{"type":"function","title":"SnmpKit.SnmpLib.HostParser.format/1","doc":"Format an IP tuple and port back to string representation.","ref":"SnmpKit.SnmpLib.HostParser.html#format/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.HostParser.format/1","doc":"iex> SnmpKit.SnmpLib.HostParser.format({{192, 168, 1, 1}, 161})\n    \"192.168.1.1:161\"\n\n    iex> SnmpKit.SnmpLib.HostParser.format({{0, 0, 0, 0, 0, 0, 0, 1}, 161})\n    \"[::1]:161\"","ref":"SnmpKit.SnmpLib.HostParser.html#format/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.HostParser.ip4_tuple/0","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#t:ip4_tuple/0"},{"type":"type","title":"SnmpKit.SnmpLib.HostParser.ip6_tuple/0","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#t:ip6_tuple/0"},{"type":"type","title":"SnmpKit.SnmpLib.HostParser.ip_tuple/0","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#t:ip_tuple/0"},{"type":"function","title":"SnmpKit.SnmpLib.HostParser.parse/2","doc":"Parse host and port from any input format.","ref":"SnmpKit.SnmpLib.HostParser.html#parse/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.HostParser.parse/2","doc":"# IPv4 tuples\n    iex> SnmpKit.SnmpLib.HostParser.parse({192, 168, 1, 1})\n    {:ok, {{192, 168, 1, 1}, 161}}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse({{192, 168, 1, 1}, 8161})\n    {:ok, {{192, 168, 1, 1}, 8161}}\n\n    # IPv4 strings\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"192.168.1.1\")\n    {:ok, {{192, 168, 1, 1}, 161}}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"192.168.1.1:8161\")\n    {:ok, {{192, 168, 1, 1}, 8161}}\n\n    # IPv4 charlists\n    iex> SnmpKit.SnmpLib.HostParser.parse('192.168.1.1')\n    {:ok, {{192, 168, 1, 1}, 161}}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse('192.168.1.1:8161')\n    {:ok, {{192, 168, 1, 1}, 8161}}\n\n    # IPv6 strings\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"::1\")\n    {:ok, {{0, 0, 0, 0, 0, 0, 0, 1}, 161}}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"[::1]:8161\")\n    {:ok, {{0, 0, 0, 0, 0, 0, 0, 1}, 8161}}\n\n    # Error cases\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"invalid\")\n    {:error, :invalid_host}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse(\"192.168.1.1:99999\")\n    {:error, :invalid_port}","ref":"SnmpKit.SnmpLib.HostParser.html#parse/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.HostParser.parse_ip/1","doc":"Parse and return only the IP tuple, using default port.","ref":"SnmpKit.SnmpLib.HostParser.html#parse_ip/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.HostParser.parse_ip/1","doc":"iex> SnmpKit.SnmpLib.HostParser.parse_ip(\"192.168.1.1\")\n    {:ok, {192, 168, 1, 1}}","ref":"SnmpKit.SnmpLib.HostParser.html#parse_ip/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.HostParser.parse_port/1","doc":"Parse and return only the port, using 161 as default.","ref":"SnmpKit.SnmpLib.HostParser.html#parse_port/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.HostParser.parse_port/1","doc":"iex> SnmpKit.SnmpLib.HostParser.parse_port(\"192.168.1.1:8161\")\n    {:ok, 8161}\n\n    iex> SnmpKit.SnmpLib.HostParser.parse_port(\"192.168.1.1\")\n    {:ok, 161}","ref":"SnmpKit.SnmpLib.HostParser.html#parse_port/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.HostParser.parse_result/0","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#t:parse_result/0"},{"type":"type","title":"SnmpKit.SnmpLib.HostParser.port_number/0","doc":"","ref":"SnmpKit.SnmpLib.HostParser.html#t:port_number/0"},{"type":"function","title":"SnmpKit.SnmpLib.HostParser.valid?/1","doc":"Quick validation function to check if input can be parsed.","ref":"SnmpKit.SnmpLib.HostParser.html#valid?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.HostParser.valid?/1","doc":"iex> SnmpKit.SnmpLib.HostParser.valid?(\"192.168.1.1\")\n    true\n\n    iex> SnmpKit.SnmpLib.HostParser.valid?(\"invalid\")\n    false","ref":"SnmpKit.SnmpLib.HostParser.html#valid?/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.AST","doc":"Abstract Syntax Tree definitions for MIB compilation.\n\nFaithfully ported from Erlang OTP snmpc_mib_gram.yrl and related structures.\nProvides the complete AST representation needed for SNMP MIB compilation.","ref":"SnmpKit.SnmpLib.MIB.AST.html"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.access_level/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:access_level/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.agent_capabilities/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:agent_capabilities/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.build_oid_tree/1","doc":"Build an OID tree from definitions for fast lookups.","ref":"SnmpKit.SnmpLib.MIB.AST.html#build_oid_tree/1"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.capability_module/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:capability_module/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.choice_element/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:choice_element/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.compliance_module/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:compliance_module/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.compliance_object/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:compliance_object/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.constraint/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:constraint/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.constraints/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:constraints/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.constructed_syntax/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:constructed_syntax/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.default_value/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:default_value/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.definition/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:definition/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.determine_snmp_version/1","doc":"Determine SNMP version based on MIB content.\n\nPer Erlang implementation: presence of MODULE-IDENTITY indicates SNMPv2.","ref":"SnmpKit.SnmpLib.MIB.AST.html#determine_snmp_version/1"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.import/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:import/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.index_element/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:index_element/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.index_spec/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:index_spec/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.line_number/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:line_number/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.metadata/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:metadata/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.mib/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:mib/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.module_compliance/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:module_compliance/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.module_identity/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:module_identity/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.named_bit/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:named_bit/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.named_syntax/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:named_syntax/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.named_value/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:named_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.new_import/3","doc":"Create a new import statement.","ref":"SnmpKit.SnmpLib.MIB.AST.html#new_import/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.new_mib/2","doc":"Create a new MIB AST node.","ref":"SnmpKit.SnmpLib.MIB.AST.html#new_mib/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.new_object_identity/2","doc":"Create a new object identity definition.","ref":"SnmpKit.SnmpLib.MIB.AST.html#new_object_identity/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.new_object_type/2","doc":"Create a new object type definition.","ref":"SnmpKit.SnmpLib.MIB.AST.html#new_object_type/2"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.notification_group/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:notification_group/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.notification_type/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:notification_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.object_group/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:object_group/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.object_identifier_assignment/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:object_identifier_assignment/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.object_identity/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:object_identity/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.object_type/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:object_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.object_variation/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:object_variation/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.oid/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.oid_element/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:oid_element/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.oid_tree/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:oid_tree/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.pretty_print/1","doc":"Pretty print AST node for debugging.","ref":"SnmpKit.SnmpLib.MIB.AST.html#pretty_print/1"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.primitive_syntax/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:primitive_syntax/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.range_constraint/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:range_constraint/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.range_spec/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:range_spec/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.revision/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:revision/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.sequence_element/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:sequence_element/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.size_constraint/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:size_constraint/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.size_range/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:size_range/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.status/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:status/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.syntax/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:syntax/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.tag/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:tag/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.textual_convention/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:textual_convention/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.AST.trap_type/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.AST.html#t:trap_type/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.AST.validate_node/1","doc":"Validate AST node structure.","ref":"SnmpKit.SnmpLib.MIB.AST.html#validate_node/1"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Compiler","doc":"Main MIB compiler that orchestrates the entire compilation process.\n\nThis module provides the main interface for compiling MIB files from source\nto executable format. It coordinates between the lexer, parser, semantic\nanalyzer, and code generator to produce optimized compiled MIBs.","ref":"SnmpKit.SnmpLib.MIB.Compiler.html"},{"type":"module","title":"Compilation Process - SnmpKit.SnmpLib.MIB.Compiler","doc":"1. **Lexical Analysis** - Tokenize MIB source\n2. **Parsing** - Build Abstract Syntax Tree (AST)\n3. **Semantic Analysis** - Validate and resolve symbols\n4. **Code Generation** - Generate optimized runtime format\n5. **Persistence** - Save compiled MIB for loading","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#module-compilation-process"},{"type":"module","title":"Usage - SnmpKit.SnmpLib.MIB.Compiler","doc":"# Compile a single MIB file\n    {:ok, compiled} = SnmpKit.SnmpLib.MIB.Compiler.compile(\"MY-MIB.mib\")\n\n    # Compile with options\n    {:ok, compiled} = SnmpKit.SnmpLib.MIB.Compiler.compile(\"MY-MIB.mib\",\n      output_dir: \"/tmp/mibs\",\n      format: :binary,\n      optimize: true\n    )\n\n    # Compile multiple MIBs\n    {:ok, results} = SnmpKit.SnmpLib.MIB.Compiler.compile_all([\n      \"SNMPv2-SMI.mib\",\n      \"MY-MIB.mib\"\n    ])","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#module-usage"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Compiler.compile/2","doc":"Compile a MIB file from filesystem path.","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Compiler.compile/2","doc":"iex> SnmpKit.SnmpLib.MIB.Compiler.compile(\"test/fixtures/TEST-MIB.mib\")\n    {:ok, %{name: \"TEST-MIB\", ...}}\n\n    iex> SnmpKit.SnmpLib.MIB.Compiler.compile(\"missing.mib\")\n    {:error, [%SnmpKit.SnmpLib.MIB.Error{type: :file_not_found}]}","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Compiler.compile_all/2","doc":"Compile multiple MIB files in dependency order.","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile_all/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Compiler.compile_all/2","doc":"iex> SnmpKit.SnmpLib.MIB.Compiler.compile_all([\"SNMPv2-SMI.mib\", \"MY-MIB.mib\"])\n    {:ok, [%{name: \"SNMPv2-SMI\"}, %{name: \"MY-MIB\"}]}","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile_all/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Compiler.compile_opts/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#t:compile_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Compiler.compile_result/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#t:compile_result/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Compiler.compile_string/2","doc":"Compile a MIB from string content.\n\nThis delegates to the Parser module which implements the full compilation\npipeline using YACC-based parsing.","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile_string/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Compiler.compile_string/2","doc":"iex> SnmpKit.SnmpLib.MIB.Compiler.compile_string(mib_content)\n    {:ok, %{name: \"TEST-MIB\", ...}}","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#compile_string/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Compiler.compiled_mib/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#t:compiled_mib/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Compiler.load_compiled/1","doc":"Load a previously compiled MIB.","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#load_compiled/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Compiler.load_compiled/1","doc":"iex> SnmpKit.SnmpLib.MIB.Compiler.load_compiled(\"priv/mibs/TEST-MIB.mib\")\n    {:ok, %{name: \"TEST-MIB\", ...}}","ref":"SnmpKit.SnmpLib.MIB.Compiler.html#load_compiled/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Error","doc":"Enhanced error handling with recovery and detailed diagnostics.\n\nProvides structured error reporting with context, suggestions, and\nprecise location information for MIB compilation errors.","ref":"SnmpKit.SnmpLib.MIB.Error.html"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Error.error_type/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Error.html#t:error_type/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Error.format/2","doc":"Format an error for display with optional color coding.","ref":"SnmpKit.SnmpLib.MIB.Error.html#format/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Error.format/2","doc":"iex> error = SnmpKit.SnmpLib.MIB.Error.new(:syntax_error, line: 42, column: 10)\n    iex> SnmpKit.SnmpLib.MIB.Error.format(error)\n    \"Error at line 42, column 10: Syntax error\"","ref":"SnmpKit.SnmpLib.MIB.Error.html#format/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Error.new/2","doc":"Create a new error with detailed context and suggestions.","ref":"SnmpKit.SnmpLib.MIB.Error.html#new/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Error.new/2","doc":"iex> SnmpKit.SnmpLib.MIB.Error.new(:unexpected_token,\n    ...>   expected: :max_access,\n    ...>   actual: :access,\n    ...>   line: 42,\n    ...>   column: 10\n    ...> )\n    %SnmpKit.SnmpLib.MIB.Error{\n      type: :unexpected_token,\n      message: \"Expected max_access, but found access\",\n      suggestions: [\"Did you mean 'MAX-ACCESS' instead of 'ACCESS'?\"]\n    }","ref":"SnmpKit.SnmpLib.MIB.Error.html#new/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Error.t/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Error.html#t:t/0"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Logger","doc":"Structured logging for MIB compilation with proper log levels.\n\nProvides detailed logging throughout the compilation process with\nstructured metadata for debugging and monitoring.","ref":"SnmpKit.SnmpLib.MIB.Logger.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_batch_compilation_error/2","doc":"Log batch compilation with errors.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_batch_compilation_error/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_batch_compilation_start/1","doc":"Log start of batch compilation.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_batch_compilation_start/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_batch_compilation_success/1","doc":"Log successful batch compilation.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_batch_compilation_success/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_batch_progress/2","doc":"Log batch compilation progress.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_batch_progress/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_codegen/3","doc":"Log code generation progress and results.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_codegen/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_complete/2","doc":"Log compilation completion with results.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_complete/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_error/2","doc":"Log compilation errors.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_error/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_failed/2","doc":"Log compilation failure with error details.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_failed/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_start/2","doc":"Log the start of MIB compilation with context.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_start/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_success/2","doc":"Log successful MIB compilation.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_success/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_compilation_warning/3","doc":"Log compilation with warnings.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_compilation_warning/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_dependency_order/1","doc":"Log dependency resolution order.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_dependency_order/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_import_resolution/2","doc":"Log import resolution with dependency information.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_import_resolution/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_imports_resolved/3","doc":"Log successful import resolution.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_imports_resolved/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_parse_progress/2","doc":"Log parsing progress with token/object counts.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_parse_progress/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_performance/2","doc":"Log performance metrics.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_performance/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_tokenization/3","doc":"Log tokenization statistics.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_tokenization/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_vendor_quirk/3","doc":"Log vendor-specific quirk handling.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_vendor_quirk/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Logger.log_warning/2","doc":"Log warning with context.","ref":"SnmpKit.SnmpLib.MIB.Logger.html#log_warning/2"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Parser","doc":"Pure native SNMP MIB parser using custom Elixir grammar.\n\nThis module provides complete native parsing of SNMP MIB files with 100%\ncompatibility. All MIBs are parsed natively ensuring complete data integrity.","ref":"SnmpKit.SnmpLib.MIB.Parser.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Parser.init_parser/0","doc":"Initialize the parser by compiling the grammar file.\nThis creates a proper yacc-generated parser for native MIB parsing.","ref":"SnmpKit.SnmpLib.MIB.Parser.html#init_parser/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Parser.mibdirs/1","doc":"Parse all MIB files in a list of directories using pure native parsing.\n\nReturns a map with directory paths as keys and results as values.\nEach result contains successful compilations and failures.","ref":"SnmpKit.SnmpLib.MIB.Parser.html#mibdirs/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Parser.mibdirs/1","doc":"# Parse MIBs in multiple directories\n    dirs = [\n      \"/path/to/mibs/working\",\n      \"/path/to/mibs/docsis\"\n    ]\n    results = SnmpKit.SnmpLib.MIB.Parser.mibdirs(dirs)\n\n    # Access results by directory\n    working_results = results[\"/path/to/mibs/working\"]\n    IO.puts(\"Success: #{length(working_results.success)}/#{working_results.total}\")\n\n    # Get all successful MIBs across directories\n    all_mibs = Enum.flat_map(results, fn {_dir, result} -> result.success end)","ref":"SnmpKit.SnmpLib.MIB.Parser.html#mibdirs/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Parser.parse/1","doc":"Parse a MIB file using pure native grammar parsing.\nThis is the production MIB parser with 100% native compatibility.","ref":"SnmpKit.SnmpLib.MIB.Parser.html#parse/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Parser.parse_tokens/1","doc":"Parse pre-tokenized MIB tokens using native grammar parsing.\nThis function takes tokens directly without tokenizing.","ref":"SnmpKit.SnmpLib.MIB.Parser.html#parse_tokens/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Parser.tokenize/1","doc":"Tokenize MIB content using the native SNMP tokenizer.\n\nUses the SnmpKit.SnmpLib.MIB.SnmpTokenizer module for complete MIB tokenization.","ref":"SnmpKit.SnmpLib.MIB.Parser.html#tokenize/1"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Preprocessor","doc":"Preprocessor for MIB files to handle problematic constructs before parsing.\n\nThis module addresses issues with very large TEXTUAL-CONVENTION enumerations\nthat cause parser state corruption in certain edge cases.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Preprocessor.analyze_enumerations/1","doc":"Get statistics about enumeration complexity in a MIB file.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html#analyze_enumerations/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Preprocessor.has_problematic_constructs?/1","doc":"Check if a MIB file contains problematic constructs.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html#has_problematic_constructs?/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Preprocessor.normalize_whitespace/1","doc":"Normalize excessive whitespace that might cause tokenizer issues.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html#normalize_whitespace/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Preprocessor.preprocess/1","doc":"Preprocess MIB content to handle problematic constructs.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html#preprocess/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Preprocessor.simplify_large_enumerations/1","doc":"Simplify very large TEXTUAL-CONVENTION enumerations that cause parser issues.\n\nThis replaces complex enumerations with simplified versions that preserve\nthe essential structure while avoiding parser state corruption.","ref":"SnmpKit.SnmpLib.MIB.Preprocessor.html#simplify_large_enumerations/1"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Registry","doc":"Standard SNMP MIB registry with name/OID resolution functions.\n\nProvides the standard SNMP MIB objects and core resolution functionality\nthat can be used by any SNMP application (managers, simulators, etc.).","ref":"SnmpKit.SnmpLib.MIB.Registry.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.children/1","doc":"Find direct children of a parent OID.","ref":"SnmpKit.SnmpLib.MIB.Registry.html#children/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Registry.children/1","doc":"iex> SnmpKit.SnmpLib.MIB.Registry.children([1, 3, 6, 1, 2, 1, 1])\n    {:ok, [\"sysContact\", \"sysDescr\", \"sysLocation\", \"sysName\", \"sysObjectID\", \"sysServices\", \"sysUpTime\"]}","ref":"SnmpKit.SnmpLib.MIB.Registry.html#children/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.resolve_name/1","doc":"Resolve a MIB name to an OID list.\nHandles instance notation like \"sysDescr.0\".","ref":"SnmpKit.SnmpLib.MIB.Registry.html#resolve_name/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Registry.resolve_name/1","doc":"iex> SnmpKit.SnmpLib.MIB.Registry.resolve_name(\"sysDescr.0\")\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 1, 0]}\n\n    iex> SnmpKit.SnmpLib.MIB.Registry.resolve_name(\"sysDescr\")\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 1]}\n\n    iex> SnmpKit.SnmpLib.MIB.Registry.resolve_name(\"unknownName\")\n    {:error, :not_found}","ref":"SnmpKit.SnmpLib.MIB.Registry.html#resolve_name/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.reverse_lookup/1","doc":"Reverse lookup an OID to get the MIB name.\nHandles partial matches and instances.","ref":"SnmpKit.SnmpLib.MIB.Registry.html#reverse_lookup/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Registry.reverse_lookup/1","doc":"iex> SnmpKit.SnmpLib.MIB.Registry.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    {:ok, \"sysDescr.0\"}\n\n    iex> SnmpKit.SnmpLib.MIB.Registry.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1])\n    {:ok, \"sysDescr\"}","ref":"SnmpKit.SnmpLib.MIB.Registry.html#reverse_lookup/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.standard_mibs/0","doc":"Get the standard MIB registry map.","ref":"SnmpKit.SnmpLib.MIB.Registry.html#standard_mibs/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.standard_mibs_reverse/0","doc":"Get the reverse lookup map (OID -> name).","ref":"SnmpKit.SnmpLib.MIB.Registry.html#standard_mibs_reverse/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Registry.walk_tree/1","doc":"Walk the MIB tree from a root OID.","ref":"SnmpKit.SnmpLib.MIB.Registry.html#walk_tree/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.Registry.walk_tree/1","doc":"iex> SnmpKit.SnmpLib.MIB.Registry.walk_tree([1, 3, 6, 1, 2, 1, 1])\n    {:ok, [{\"sysDescr\", [1, 3, 6, 1, 2, 1, 1, 1]}, {\"sysObjectID\", [1, 3, 6, 1, 2, 1, 1, 2]}, ...]}","ref":"SnmpKit.SnmpLib.MIB.Registry.html#walk_tree/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer","doc":"True 1:1 Elixir port of Erlang SNMP tokenizer (snmpc_tok.erl).\n\nThis is a direct translation of the official Erlang SNMP tokenizer\nfrom OTP lib/snmp/src/compile/snmpc_tok.erl\n\nOriginal copyright: Ericsson AB 1996-2025 (Apache License 2.0)","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.format_error/1","doc":"Format error message.\nEquivalent to snmpc_tok:format_error/1","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#format_error/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.get_all_tokens/1","doc":"Get all remaining tokens.\nEquivalent to snmpc_tok:get_all_tokens/1","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#get_all_tokens/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.get_token/1","doc":"Get next token from tokenizer.\nEquivalent to snmpc_tok:get_token/1","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#get_token/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.null_get_line/0","doc":"Null get_line function.\nEquivalent to snmpc_tok:null_get_line/0","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#null_get_line/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.start_link/2","doc":"Start tokenizer gen_server.\nEquivalent to snmpc_tok:start_link/2","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#start_link/2"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.state/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#t:state/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.stop/1","doc":"Stop tokenizer.\nEquivalent to snmpc_tok:stop/1","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#stop/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.test/0","doc":"Test function.\nEquivalent to snmpc_tok:test/0","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#test/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.token/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#t:token/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.tokenize/2","doc":"Tokenize a string directly.\nEquivalent to snmpc_tok:tokenize/2","ref":"SnmpKit.SnmpLib.MIB.SnmpTokenizer.html#tokenize/2"},{"type":"module","title":"SnmpKit.SnmpLib.MIB.Utilities","doc":"Direct port of Erlang snmpc_lib.erl utility functions to Elixir.\n\nThis is a 1:1 port of the utility functions from the official Erlang SNMP compiler\nfrom OTP lib/snmp/src/compile/snmpc_lib.erl\n\nOriginal copyright: Ericsson AB 1996-2025 (Apache License 2.0)","ref":"SnmpKit.SnmpLib.MIB.Utilities.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.allow_size_rfc1902/1","doc":"Check if size constraint is allowed for given type (RFC 1902 compliance).\nPort of allow_size_rfc1902/1 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#allow_size_rfc1902/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.check_sub_ids/3","doc":"Validate sub-identifier ranges.\nPort of check_sub_ids/3 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#check_sub_ids/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.compilation_error/1","doc":"Terminate compilation with error.\nPort of error/2 and error/3 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#compilation_error/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.compilation_error/2","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#compilation_error/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.key1search/2","doc":"Safe key lookup in list of tuples/maps.\nPort of key1search/2 and key1search/3 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#key1search/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.key1search/3","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#key1search/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.lookup/2","doc":"Generic lookup function with multiple criteria.\nPort of lookup/2 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#lookup/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.make_asn1_type/1","doc":"Validate and transform ASN.1 type definition.\nPort of make_ASN1type/1 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#make_asn1_type/1"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Utilities.oid/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Utilities.oid_status/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#t:oid_status/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.print_error/2","doc":"Print error message with formatting.\nPort of print_error/2 and print_error/3 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#print_error/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.print_error/3","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#print_error/3"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.printable?/2","doc":"Determine if message should be printed based on verbosity.\nPort of printable/2 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#printable?/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.register_oid/4","doc":"Register an OID entry in the OID table.\nPort of register_oid/4 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#register_oid/4"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.resolve_oids/1","doc":"Resolve symbolic OID references to numeric OIDs.\nPort of resolve_oids/1 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#resolve_oids/1"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.set_dir/2","doc":"Set directory path helper.\nPort of set_dir/2 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#set_dir/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.test_kibbles/2","doc":"Validate bit definitions for BITS syntax.\nPort of test_kibbles/2 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#test_kibbles/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.tr_oid/2","doc":"Translate symbolic name to numeric OID.\nPort of tr_oid/2 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#tr_oid/2"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.update_me_oids/3","doc":"Update MIB entries with resolved OIDs.\nPort of update_me_oids/3 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#update_me_oids/3"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.Utilities.verbosity/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#t:verbosity/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.vprint/6","doc":"Configurable verbose printing.\nPort of vprint/6 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#vprint/6"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.Utilities.vvalidate/1","doc":"Validate verbosity level.\nPort of vvalidate/1 from snmpc_lib.erl","ref":"SnmpKit.SnmpLib.MIB.Utilities.html#vvalidate/1"},{"type":"module","title":"SnmpKit.SnmpLib.Manager","doc":"High-level SNMP management operations providing a simplified interface for common SNMP tasks.\n\nThis module builds on the core SnmpLib functionality to provide production-ready SNMP\nmanagement capabilities including GET, GETBULK, SET operations with intelligent error\nhandling, connection reuse, and performance optimizations.","ref":"SnmpKit.SnmpLib.Manager.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Manager","doc":"- **Simple API**: High-level functions for common SNMP operations\n- **Connection Reuse**: Efficient socket management for multiple operations\n- **Error Handling**: Comprehensive error handling with meaningful messages\n- **Performance**: Optimized for bulk operations and large-scale polling\n- **Timeout Management**: Configurable timeouts with sensible defaults\n- **Community Support**: Support for different community strings per device","ref":"SnmpKit.SnmpLib.Manager.html#module-features"},{"type":"module","title":"Quick Start - SnmpKit.SnmpLib.Manager","doc":"# Simple GET operation\n    {:ok, {type, value}} = SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1, 1, 0])\n\n    # GET with custom community and timeout\n    {:ok, {type, value}} = SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", \"1.3.6.1.2.1.1.1.0\",\n                                       community: \"private\", timeout: 10_000)\n\n    # Bulk operations for efficiency\n    {:ok, results} = SnmpKit.SnmpLib.Manager.get_bulk(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 2, 2],\n                                               max_repetitions: 20)\n\n    # SET operation\n    {:ok, :success} = SnmpKit.SnmpLib.Manager.set(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1, 5, 0],\n                                          {:string, \"New System Name\"})","ref":"SnmpKit.SnmpLib.Manager.html#module-quick-start"},{"type":"module","title":"Configuration Options - SnmpKit.SnmpLib.Manager","doc":"- `community`: SNMP community string (default: \"public\")\n- `version`: SNMP version (:v1, :v2c) (default: :v2c)\n- `timeout`: Operation timeout in milliseconds (default: 5000)\n- `retries`: Number of retry attempts (default: 3)\n- `port`: SNMP port (default: 161)\n- `local_port`: Local source port (default: 0 for random)","ref":"SnmpKit.SnmpLib.Manager.html#module-configuration-options"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.bulk_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:bulk_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.bulk_result/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:bulk_result/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.community/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:community/0"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.get/3","doc":"Performs an SNMP GET operation to retrieve a single value.","ref":"SnmpKit.SnmpLib.Manager.html#get/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.get/3","doc":"- `host`: Target device IP address or hostname\n- `oid`: Object identifier as list or string (e.g., [1,3,6,1,2,1,1,1,0] or \"1.3.6.1.2.1.1.1.0\")\n- `opts`: Configuration options (see module docs for available options)","ref":"SnmpKit.SnmpLib.Manager.html#get/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.get/3","doc":"- `{:ok, {type, value}}`: Successfully retrieved the value with its SNMP type\n- `{:error, reason}`: Operation failed with reason","ref":"SnmpKit.SnmpLib.Manager.html#get/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.get/3","doc":"# Basic GET operation (would succeed with real device)\n    # SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1, 1, 0])\n    # {:ok, {:octet_string, \"Cisco IOS Software\"}}\n\n    # GET with custom community and timeout (would succeed with real device)\n    # SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", \"1.3.6.1.2.1.1.1.0\", community: \"private\", timeout: 10_000)\n    # {:ok, {:octet_string, \"Private System Description\"}}\n\n    # Test that function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Manager.get(\"invalid.host\", [1, 3, 6, 1, 2, 1, 1, 3, 0], timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Manager.html#get/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.get_bulk/3","doc":"Performs an SNMP GETBULK operation for efficient bulk data retrieval.\n\nGETBULK is more efficient than multiple GET operations when retrieving\nmultiple consecutive values, especially for table walking operations.","ref":"SnmpKit.SnmpLib.Manager.html#get_bulk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.get_bulk/3","doc":"- `host`: Target device IP address or hostname\n- `base_oid`: Base OID to start the bulk operation\n- `opts`: Configuration options including bulk-specific options","ref":"SnmpKit.SnmpLib.Manager.html#get_bulk/3-parameters"},{"type":"function","title":"Bulk-Specific Options - SnmpKit.SnmpLib.Manager.get_bulk/3","doc":"- `max_repetitions`: Maximum number of repetitions (default: 10)\n- `non_repeaters`: Number of non-repeating variables (default: 0)","ref":"SnmpKit.SnmpLib.Manager.html#get_bulk/3-bulk-specific-options"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.get_bulk/3","doc":"- `{:ok, varbinds}`: List of {oid, type, value} tuples\n- `{:error, reason}`: Operation failed with reason","ref":"SnmpKit.SnmpLib.Manager.html#get_bulk/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.get_bulk/3","doc":"# Test that get_bulk function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Manager.get_bulk(\"invalid.host\", [1, 3, 6, 1, 2, 1, 2, 2], timeout: 100))\n    true\n\n    # High-repetition bulk for large tables\n    # SnmpKit.SnmpLib.Manager.get_bulk(\"192.168.1.1\", \"1.3.6.1.2.1.2.2\", max_repetitions: 50)\n    # {:ok, [...]} Returns up to 50 interface entries","ref":"SnmpKit.SnmpLib.Manager.html#get_bulk/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.get_multi/3","doc":"Performs multiple GET operations efficiently with connection reuse.\n\nMore efficient than individual get/3 calls when retrieving multiple values\nfrom the same device by reusing the same socket connection.","ref":"SnmpKit.SnmpLib.Manager.html#get_multi/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.get_multi/3","doc":"- `host`: Target device IP address or hostname\n- `oids`: List of OIDs to retrieve\n- `opts`: Configuration options","ref":"SnmpKit.SnmpLib.Manager.html#get_multi/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.get_multi/3","doc":"- `{:ok, results}`: List of {oid, type, value} or {oid, {:error, reason}} tuples\n- `{:error, reason}`: Connection or overall operation failed","ref":"SnmpKit.SnmpLib.Manager.html#get_multi/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.get_multi/3","doc":"# Test that get_multi function exists and handles invalid input properly\n    iex> oids = [\"1.3.6.1.2.1.1.1.0\", \"1.3.6.1.2.1.1.3.0\", \"1.3.6.1.2.1.1.5.0\"]\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Manager.get_multi(\"invalid.host\", oids, timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Manager.html#get_multi/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.get_next/3","doc":"Performs an SNMP GETNEXT operation to retrieve the next value in the MIB tree.\n\nGETNEXT is used to traverse the MIB tree by retrieving the next available\nobject after the specified OID. This is essential for MIB walking operations\nand discovering available objects on SNMP devices.","ref":"SnmpKit.SnmpLib.Manager.html#get_next/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.get_next/3","doc":"- `host`: Target device IP address or hostname\n- `oid`: Object identifier to get the next value after\n- `opts`: Configuration options","ref":"SnmpKit.SnmpLib.Manager.html#get_next/3-parameters"},{"type":"function","title":"Implementation Details - SnmpKit.SnmpLib.Manager.get_next/3","doc":"- **SNMP v1**: Uses proper GETNEXT PDU for compatibility\n- **SNMP v2c+**: Uses optimized GETBULK with max_repetitions=1","ref":"SnmpKit.SnmpLib.Manager.html#get_next/3-implementation-details"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.get_next/3","doc":"- `{:ok, {next_oid, type, value}}`: Next OID and its value as a tuple\n- `{:error, reason}`: Operation failed with reason","ref":"SnmpKit.SnmpLib.Manager.html#get_next/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.get_next/3","doc":"# Get next OID after system description\n    {:ok, {next_oid, type, value}} = SnmpKit.SnmpLib.Manager.get_next(\"192.168.1.1\", \"1.3.6.1.2.1.1.1.0\")\n\n    # SNMP v1 compatibility\n    {:ok, {next_oid, type, value}} = SnmpKit.SnmpLib.Manager.get_next(\"192.168.1.1\", \"1.3.6.1.2.1.1.1.0\", version: :v1)\n\n    # With custom community\n    {:ok, {next_oid, type, value}} = SnmpKit.SnmpLib.Manager.get_next(\"192.168.1.1\", \"1.3.6.1.2.1.1.1.0\",\n                                                         community: \"private\", timeout: 10_000)","ref":"SnmpKit.SnmpLib.Manager.html#get_next/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.host/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:host/0"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.interpret_error/3","doc":"Interprets SNMP errors with enhanced semantics for common cases.\n\nProvides more specific error interpretation when generic errors like `:gen_err`\nare returned by devices that should return more specific SNMP error codes.","ref":"SnmpKit.SnmpLib.Manager.html#interpret_error/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.interpret_error/3","doc":"- `error`: The original error returned by SNMP operations\n- `operation`: The SNMP operation type (`:get`, `:set`, `:get_bulk`)\n- `version`: SNMP version (`:v1`, `:v2c`, `:v3`)","ref":"SnmpKit.SnmpLib.Manager.html#interpret_error/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.interpret_error/3","doc":"More specific error atom when possible, otherwise the original error.","ref":"SnmpKit.SnmpLib.Manager.html#interpret_error/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.interpret_error/3","doc":"# Interpret genErr for GET operations\n    iex> SnmpKit.SnmpLib.Manager.interpret_error(:gen_err, :get, :v2c)\n    :no_such_object\n\n    iex> SnmpKit.SnmpLib.Manager.interpret_error(:gen_err, :get, :v1)\n    :no_such_name\n\n    iex> SnmpKit.SnmpLib.Manager.interpret_error(:too_big, :get, :v2c)\n    :too_big","ref":"SnmpKit.SnmpLib.Manager.html#interpret_error/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.manager_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:manager_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.oid/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.operation_result/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:operation_result/0"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.ping/2","doc":"Checks if a host is reachable via SNMP by performing a basic GET operation.\n\nUseful for device discovery and health checking. Attempts to retrieve\nsysUpTime (1.3.6.1.2.1.1.3.0) which should be available on all SNMP devices.","ref":"SnmpKit.SnmpLib.Manager.html#ping/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.ping/2","doc":"- `host`: Target device IP address or hostname\n- `opts`: Configuration options (typically just community and timeout)","ref":"SnmpKit.SnmpLib.Manager.html#ping/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.ping/2","doc":"- `{:ok, :reachable}`: Device responded to SNMP request\n- `{:error, reason}`: Device not reachable or SNMP not available","ref":"SnmpKit.SnmpLib.Manager.html#ping/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.ping/2","doc":"# Test that ping function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Manager.ping(\"invalid.host\", timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Manager.html#ping/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Manager.set/4","doc":"Performs an SNMP SET operation to modify a value on the target device.","ref":"SnmpKit.SnmpLib.Manager.html#set/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Manager.set/4","doc":"- `host`: Target device IP address or hostname\n- `oid`: Object identifier to modify\n- `value`: New value as {type, data} tuple (e.g., {:string, \"new name\"})\n- `opts`: Configuration options","ref":"SnmpKit.SnmpLib.Manager.html#set/4-parameters"},{"type":"function","title":"Supported Value Types - SnmpKit.SnmpLib.Manager.set/4","doc":"- `{:string, binary()}`: OCTET STRING\n- `{:integer, integer()}`: INTEGER\n- `{:counter32, non_neg_integer()}`: Counter32\n- `{:gauge32, non_neg_integer()}`: Gauge32\n- `{:timeticks, non_neg_integer()}`: TimeTicks\n- `{:ip_address, binary()}`: IpAddress (4 bytes)","ref":"SnmpKit.SnmpLib.Manager.html#set/4-supported-value-types"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Manager.set/4","doc":"- `{:ok, :success}`: SET operation completed successfully\n- `{:error, reason}`: Operation failed with reason","ref":"SnmpKit.SnmpLib.Manager.html#set/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Manager.set/4","doc":"# Test that SET function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Manager.set(\"invalid.host\", [1, 3, 6, 1, 2, 1, 1, 5, 0], {:string, \"test\"}, timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Manager.html#set/4-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:snmp_value/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:varbind/0"},{"type":"type","title":"SnmpKit.SnmpLib.Manager.version/0","doc":"","ref":"SnmpKit.SnmpLib.Manager.html#t:version/0"},{"type":"module","title":"SnmpKit.SnmpLib.Monitor","doc":"Performance monitoring and metrics collection for SNMP operations.\n\nThis module provides comprehensive monitoring capabilities for SNMP applications,\nincluding real-time metrics, performance analytics, and health monitoring.\nBased on monitoring patterns proven in large-scale network management systems.","ref":"SnmpKit.SnmpLib.Monitor.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Monitor","doc":"- **Real-time Metrics**: Live performance data collection and analysis\n- **Historical Analytics**: Trend analysis and capacity planning data\n- **Health Monitoring**: Automatic detection of performance degradation\n- **Alerting**: Configurable thresholds and notification system\n- **Device Profiling**: Per-device performance characteristics\n- **Operation Tracking**: Detailed metrics for all SNMP operation types","ref":"SnmpKit.SnmpLib.Monitor.html#module-features"},{"type":"module","title":"Metric Categories - SnmpKit.SnmpLib.Monitor","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#module-metric-categories"},{"type":"module","title":"Operation Metrics - SnmpKit.SnmpLib.Monitor","doc":"- Request/response times\n- Success/failure rates\n- Throughput measurements\n- Error classifications","ref":"SnmpKit.SnmpLib.Monitor.html#module-operation-metrics"},{"type":"module","title":"Device Metrics - SnmpKit.SnmpLib.Monitor","doc":"- Per-device response characteristics\n- Availability percentages\n- Performance trends\n- Health scores","ref":"SnmpKit.SnmpLib.Monitor.html#module-device-metrics"},{"type":"module","title":"System Metrics - SnmpKit.SnmpLib.Monitor","doc":"- Connection pool utilization\n- Memory usage patterns\n- Resource consumption\n- Concurrent operation counts","ref":"SnmpKit.SnmpLib.Monitor.html#module-system-metrics"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Monitor","doc":"# Start monitoring system\n    {:ok, _pid} = SnmpKit.SnmpLib.Monitor.start_link()\n\n    # Record SNMP operation\n    SnmpKit.SnmpLib.Monitor.record_operation(\n      device: \"192.168.1.1\",\n      operation: :get,\n      duration: 245,\n      result: :success\n    )\n\n    # Get real-time stats\n    stats = SnmpKit.SnmpLib.Monitor.get_stats(\"192.168.1.1\")\n    IO.puts(\"Average response time: \" <> to_string(stats.avg_response_time) <> \"ms\")\n\n    # Set up alerting\n    SnmpKit.SnmpLib.Monitor.set_alert_threshold(\"192.168.1.1\", :response_time, 5000)","ref":"SnmpKit.SnmpLib.Monitor.html#module-usage-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.alert_threshold/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:alert_threshold/0"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.Monitor.html#child_spec/1"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.device_id/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:device_id/0"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.device_stats/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:device_stats/0"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.export_data/2","doc":"Exports monitoring data for external analysis.","ref":"SnmpKit.SnmpLib.Monitor.html#export_data/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Monitor.export_data/2","doc":"- `format`: Export format (`:json`, `:csv`, `:prometheus`)\n- `timeframe`: Time range for export","ref":"SnmpKit.SnmpLib.Monitor.html#export_data/2-parameters"},{"type":"function","title":"JSON Export - SnmpKit.SnmpLib.Monitor.export_data/2","doc":"JSON export uses Elixir's built-in JSON module (requires Elixir 1.18+).","ref":"SnmpKit.SnmpLib.Monitor.html#export_data/2-json-export"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.export_data/2","doc":"data = SnmpKit.SnmpLib.Monitor.export_data(:json, :last_hour)\n    case data do\n      \"JSON export unavailable\" <> _ -> IO.puts(\"JSON not available\")\n      json -> File.write!(\"snmp_metrics.json\", json)\n    end","ref":"SnmpKit.SnmpLib.Monitor.html#export_data/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.get_active_alerts/0","doc":"Gets currently active alerts.","ref":"SnmpKit.SnmpLib.Monitor.html#get_active_alerts/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.get_active_alerts/0","doc":"alerts = SnmpKit.SnmpLib.Monitor.get_active_alerts()\n    Enum.each(alerts, fn alert ->\n      IO.puts(\"Alert: \" <> alert.device_id <> \" \" <> to_string(alert.metric) <> \" \" <> to_string(alert.current_value))\n    end)","ref":"SnmpKit.SnmpLib.Monitor.html#get_active_alerts/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.get_device_stats/2","doc":"Gets comprehensive statistics for a specific device.","ref":"SnmpKit.SnmpLib.Monitor.html#get_device_stats/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Monitor.get_device_stats/2","doc":"- `device_id`: Device identifier\n- `timeframe`: Optional timeframe (:last_hour, :last_day, :all_time)","ref":"SnmpKit.SnmpLib.Monitor.html#get_device_stats/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Monitor.get_device_stats/2","doc":"Device statistics map or `{:error, :not_found}` if device has no recorded operations.","ref":"SnmpKit.SnmpLib.Monitor.html#get_device_stats/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.get_device_stats/2","doc":"# Get current device stats\n    stats = SnmpKit.SnmpLib.Monitor.get_device_stats(\"192.168.1.1\")\n    IO.puts(\"Error rate: \" <> to_string(stats.error_rate) <> \"%\")\n\n    # Get stats for specific timeframe\n    stats = SnmpKit.SnmpLib.Monitor.get_device_stats(\"192.168.1.1\", :last_hour)","ref":"SnmpKit.SnmpLib.Monitor.html#get_device_stats/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.get_operation_metrics/2","doc":"Gets performance metrics for a specific operation type.","ref":"SnmpKit.SnmpLib.Monitor.html#get_operation_metrics/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Monitor.get_operation_metrics/2","doc":"- `operation`: SNMP operation type\n- `timeframe`: Optional timeframe for analysis","ref":"SnmpKit.SnmpLib.Monitor.html#get_operation_metrics/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.get_operation_metrics/2","doc":"metrics = SnmpKit.SnmpLib.Monitor.get_operation_metrics(:get_bulk)\n    IO.puts(\"Average GETBULK time: \" <> to_string(metrics.avg_duration) <> \"ms\")","ref":"SnmpKit.SnmpLib.Monitor.html#get_operation_metrics/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.get_system_stats/0","doc":"Gets system-wide statistics and performance metrics.","ref":"SnmpKit.SnmpLib.Monitor.html#get_system_stats/0"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Monitor.get_system_stats/0","doc":"Comprehensive system statistics including global performance metrics,\ndevice counts, and resource utilization.","ref":"SnmpKit.SnmpLib.Monitor.html#get_system_stats/0-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.get_system_stats/0","doc":"stats = SnmpKit.SnmpLib.Monitor.get_system_stats()\n    IO.puts(\"Total devices monitored: \" <> to_string(stats.total_devices))\n    IO.puts(\"Operations per second: \" <> to_string(stats.operations_per_second))","ref":"SnmpKit.SnmpLib.Monitor.html#get_system_stats/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.health_check/0","doc":"Forces a health check of all monitored devices.\n\nUseful for immediate assessment of system health.","ref":"SnmpKit.SnmpLib.Monitor.html#health_check/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.health_check/0","doc":":ok = SnmpKit.SnmpLib.Monitor.health_check()","ref":"SnmpKit.SnmpLib.Monitor.html#health_check/0-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.metric_type/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:metric_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.operation_metric/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:operation_metric/0"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.operation_result/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:operation_result/0"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.operation_type/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:operation_type/0"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.record_operation/1","doc":"Records an SNMP operation for monitoring and analysis.\n\nThis is the primary interface for feeding operation data into the monitoring system.\nShould be called after every SNMP operation for comprehensive monitoring.","ref":"SnmpKit.SnmpLib.Monitor.html#record_operation/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Monitor.record_operation/1","doc":"- `metric`: Operation metric map with required fields","ref":"SnmpKit.SnmpLib.Monitor.html#record_operation/1-parameters"},{"type":"function","title":"Required Fields - SnmpKit.SnmpLib.Monitor.record_operation/1","doc":"- `device`: Target device identifier\n- `operation`: Type of SNMP operation\n- `duration`: Operation duration in milliseconds\n- `result`: Operation result status","ref":"SnmpKit.SnmpLib.Monitor.html#record_operation/1-required-fields"},{"type":"function","title":"Optional Fields - SnmpKit.SnmpLib.Monitor.record_operation/1","doc":"- `error_type`: Specific error classification (if result is :error)\n- `bytes_sent`: Number of bytes sent\n- `bytes_received`: Number of bytes received\n- `timestamp`: Override timestamp (defaults to current time)","ref":"SnmpKit.SnmpLib.Monitor.html#record_operation/1-optional-fields"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.record_operation/1","doc":"# Basic operation recording\n    SnmpKit.SnmpLib.Monitor.record_operation(%{\n      device: \"192.168.1.1\",\n      operation: :get,\n      duration: 245,\n      result: :success\n    })\n\n    # Detailed operation recording\n    SnmpKit.SnmpLib.Monitor.record_operation(%{\n      device: \"192.168.1.1\",\n      operation: :get_bulk,\n      duration: 1250,\n      result: :error,\n      error_type: :timeout,\n      bytes_sent: 64,\n      bytes_received: 0\n    })","ref":"SnmpKit.SnmpLib.Monitor.html#record_operation/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.remove_alert_threshold/2","doc":"Removes an alert threshold.","ref":"SnmpKit.SnmpLib.Monitor.html#remove_alert_threshold/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.remove_alert_threshold/2","doc":":ok = SnmpKit.SnmpLib.Monitor.remove_alert_threshold(\"192.168.1.1\", :response_time)","ref":"SnmpKit.SnmpLib.Monitor.html#remove_alert_threshold/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.set_alert_threshold/4","doc":"Sets an alert threshold for automated monitoring.\n\nAlerts fire when the specified metric exceeds the threshold for the given duration.","ref":"SnmpKit.SnmpLib.Monitor.html#set_alert_threshold/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Monitor.set_alert_threshold/4","doc":"- `device_id`: Device to monitor (use \":global\" for system-wide alerts)\n- `metric`: Metric type to monitor\n- `threshold`: Threshold value\n- `opts`: Alert configuration options","ref":"SnmpKit.SnmpLib.Monitor.html#set_alert_threshold/4-parameters"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Monitor.set_alert_threshold/4","doc":"- `condition`: `:above` or `:below` (default: `:above`)\n- `duration`: How long threshold must be exceeded (default: 60000ms)\n- `callback`: Function to call when alert fires","ref":"SnmpKit.SnmpLib.Monitor.html#set_alert_threshold/4-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.set_alert_threshold/4","doc":"# Alert on high response times\n    SnmpKit.SnmpLib.Monitor.set_alert_threshold(\"192.168.1.1\", :response_time, 5000)\n\n    # Alert on low availability with custom callback\n    SnmpKit.SnmpLib.Monitor.set_alert_threshold(\"core-router\", :availability, 95.0,\n      condition: :below,\n      duration: 300_000,\n      callback: &MyApp.Alerts.device_down/1\n    )","ref":"SnmpKit.SnmpLib.Monitor.html#set_alert_threshold/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Monitor.start_link/1","doc":"Starts the monitoring system.","ref":"SnmpKit.SnmpLib.Monitor.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Monitor.start_link/1","doc":"- `retention_period`: How long to keep historical data (default: 1 hour)\n- `bucket_size`: Time bucket size for aggregation (default: 1 minute)\n- `cleanup_interval`: How often to clean old data (default: 5 minutes)\n- `health_check_interval`: How often to check device health (default: 1 minute)","ref":"SnmpKit.SnmpLib.Monitor.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Monitor.start_link/1","doc":"{:ok, pid} = SnmpKit.SnmpLib.Monitor.start_link()\n\n    {:ok, pid} = SnmpKit.SnmpLib.Monitor.start_link(\n      retention_period: 7200_000,  # 2 hours\n      bucket_size: 30_000          # 30 second buckets\n    )","ref":"SnmpKit.SnmpLib.Monitor.html#start_link/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Monitor.system_stats/0","doc":"","ref":"SnmpKit.SnmpLib.Monitor.html#t:system_stats/0"},{"type":"module","title":"SnmpKit.SnmpLib.OID","doc":"Comprehensive OID (Object Identifier) manipulation utilities for SNMP operations.\n\nProvides string/list conversions, tree operations, table utilities, and validation\nfunctions needed by both SNMP managers and simulators.","ref":"SnmpKit.SnmpLib.OID.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.OID","doc":"- String/list format conversions with validation\n- OID tree operations (parent/child relationships)\n- SNMP table index parsing and construction\n- OID comparison and sorting\n- Enterprise OID utilities\n- Performance-optimized operations","ref":"SnmpKit.SnmpLib.OID.html#module-features"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.OID","doc":"# Basic conversions\n    {:ok, oid_list} = SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6.1.2.1.1.1.0\")\n    {:ok, oid_string} = SnmpKit.SnmpLib.OID.list_to_string([1, 3, 6, 1, 2, 1, 1, 1, 0])\n\n    # Tree operations\n    true = SnmpKit.SnmpLib.OID.is_child_of?([1, 3, 6, 1, 2, 1, 1, 1, 0], [1, 3, 6, 1, 2, 1])\n    {:ok, parent} = SnmpKit.SnmpLib.OID.get_parent([1, 3, 6, 1, 2, 1, 1, 1, 0])\n\n    # Comparison\n    :lt = SnmpKit.SnmpLib.OID.compare([1, 3, 6, 1], [1, 3, 6, 2])\n\n    # Table operations\n    {:ok, index} = SnmpKit.SnmpLib.OID.extract_table_index([1, 3, 6, 1, 2, 1, 2, 2, 1, 1], [1, 3, 6, 1, 2, 1, 2, 2, 1, 1, 1])","ref":"SnmpKit.SnmpLib.OID.html#module-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.build_table_index/2","doc":"Builds a table index from parsed components according to syntax.","ref":"SnmpKit.SnmpLib.OID.html#build_table_index/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.build_table_index/2","doc":"{:ok, [42]} = SnmpKit.SnmpLib.OID.build_table_index(42, :integer)\n    {:ok, [4, 116, 101, 115, 116]} = SnmpKit.SnmpLib.OID.build_table_index(\"test\", {:variable_string})","ref":"SnmpKit.SnmpLib.OID.html#build_table_index/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.build_table_instance/2","doc":"Builds a table instance OID from table OID and index.","ref":"SnmpKit.SnmpLib.OID.html#build_table_instance/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.build_table_instance/2","doc":"- `table_oid`: Base table column OID\n- `index`: Table index as list of integers","ref":"SnmpKit.SnmpLib.OID.html#build_table_instance/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.build_table_instance/2","doc":"- `{:ok, instance_oid}` if successful\n- `{:error, reason}` if construction fails","ref":"SnmpKit.SnmpLib.OID.html#build_table_instance/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.build_table_instance/2","doc":"table_oid = [1, 3, 6, 1, 2, 1, 2, 2, 1, 1]\n    index = [1]\n    {:ok, [1, 3, 6, 1, 2, 1, 2, 2, 1, 1, 1]} = SnmpKit.SnmpLib.OID.build_table_instance(table_oid, index)","ref":"SnmpKit.SnmpLib.OID.html#build_table_instance/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.compare/2","doc":"Compares two OIDs lexicographically.","ref":"SnmpKit.SnmpLib.OID.html#compare/2"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.compare/2","doc":"- `:lt` if oid1   oid2","ref":"SnmpKit.SnmpLib.OID.html#compare/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.compare/2","doc":":lt = SnmpKit.SnmpLib.OID.compare([1, 3, 6, 1], [1, 3, 6, 2])\n    :eq = SnmpKit.SnmpLib.OID.compare([1, 3, 6, 1], [1, 3, 6, 1])\n    :gt = SnmpKit.SnmpLib.OID.compare([1, 3, 6, 2], [1, 3, 6, 1])","ref":"SnmpKit.SnmpLib.OID.html#compare/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.enterprises/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#enterprises/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.experimental/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#experimental/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.extract_table_index/2","doc":"Extracts table index from an instance OID given the table column OID.","ref":"SnmpKit.SnmpLib.OID.html#extract_table_index/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.extract_table_index/2","doc":"- `table_oid`: Base table column OID\n- `instance_oid`: Full instance OID including index","ref":"SnmpKit.SnmpLib.OID.html#extract_table_index/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.extract_table_index/2","doc":"- `{:ok, index}` if successful\n- `{:error, reason}` if extraction fails","ref":"SnmpKit.SnmpLib.OID.html#extract_table_index/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.extract_table_index/2","doc":"table_oid = [1, 3, 6, 1, 2, 1, 2, 2, 1, 1]\n    instance_oid = [1, 3, 6, 1, 2, 1, 2, 2, 1, 1, 1]\n    {:ok, [1]} = SnmpKit.SnmpLib.OID.extract_table_index(table_oid, instance_oid)","ref":"SnmpKit.SnmpLib.OID.html#extract_table_index/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.get_children/2","doc":"Gets the immediate children prefix for an OID in a given set.","ref":"SnmpKit.SnmpLib.OID.html#get_children/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.get_children/2","doc":"- `parent_oid`: The parent OID\n- `oid_set`: Set of OIDs to search","ref":"SnmpKit.SnmpLib.OID.html#get_children/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.get_children/2","doc":"- List of immediate child OIDs","ref":"SnmpKit.SnmpLib.OID.html#get_children/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.get_children/2","doc":"children = SnmpKit.SnmpLib.OID.get_children([1, 3, 6, 1], [[1, 3, 6, 1, 2], [1, 3, 6, 1, 4], [1, 3, 6, 1, 2, 1]])\n    # Returns [[1, 3, 6, 1, 2], [1, 3, 6, 1, 4]]","ref":"SnmpKit.SnmpLib.OID.html#get_children/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.get_enterprise_number/1","doc":"Gets the enterprise number from an enterprise OID.","ref":"SnmpKit.SnmpLib.OID.html#get_enterprise_number/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.get_enterprise_number/1","doc":"{:ok, 9} = SnmpKit.SnmpLib.OID.get_enterprise_number([1, 3, 6, 1, 4, 1, 9, 1, 1])\n    {:error, :not_enterprise_oid} = SnmpKit.SnmpLib.OID.get_enterprise_number([1, 3, 6, 1, 2, 1])","ref":"SnmpKit.SnmpLib.OID.html#get_enterprise_number/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.get_next_oid/2","doc":"Gets the next OID in lexicographic order from a given set.","ref":"SnmpKit.SnmpLib.OID.html#get_next_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.get_next_oid/2","doc":"- `current_oid`: The current OID\n- `oid_set`: Set of OIDs to search (must be sorted)","ref":"SnmpKit.SnmpLib.OID.html#get_next_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.get_next_oid/2","doc":"- `{:ok, next_oid}` if found\n- `{:error, :end_of_mib}` if no next OID exists","ref":"SnmpKit.SnmpLib.OID.html#get_next_oid/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.get_next_oid/2","doc":"oid_set = [[1, 3, 6, 1, 2, 1, 1, 1, 0], [1, 3, 6, 1, 2, 1, 1, 2, 0], [1, 3, 6, 1, 2, 1, 1, 3, 0]]\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 2, 0]} = SnmpKit.SnmpLib.OID.get_next_oid([1, 3, 6, 1, 2, 1, 1, 1, 0], oid_set)","ref":"SnmpKit.SnmpLib.OID.html#get_next_oid/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.get_parent/1","doc":"Gets the parent OID by removing the last component.","ref":"SnmpKit.SnmpLib.OID.html#get_parent/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.get_parent/1","doc":"{:ok, [1, 3, 6, 1, 2, 1, 1, 1]} = SnmpKit.SnmpLib.OID.get_parent([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    {:error, :root_oid} = SnmpKit.SnmpLib.OID.get_parent([])\n    {:error, :root_oid} = SnmpKit.SnmpLib.OID.get_parent([1])","ref":"SnmpKit.SnmpLib.OID.html#get_parent/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.OID.index/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#t:index/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_child_of?/2","doc":"Checks if one OID is a child of another.","ref":"SnmpKit.SnmpLib.OID.html#is_child_of?/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.is_child_of?/2","doc":"- `child_oid`: Potential child OID\n- `parent_oid`: Potential parent OID","ref":"SnmpKit.SnmpLib.OID.html#is_child_of?/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.is_child_of?/2","doc":"- `true` if child_oid is a child of parent_oid\n- `false` otherwise","ref":"SnmpKit.SnmpLib.OID.html#is_child_of?/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.is_child_of?/2","doc":"true = SnmpKit.SnmpLib.OID.is_child_of?([1, 3, 6, 1, 2, 1, 1, 1, 0], [1, 3, 6, 1, 2, 1])\n    false = SnmpKit.SnmpLib.OID.is_child_of?([1, 3, 6, 1], [1, 3, 6, 1, 2, 1])\n    false = SnmpKit.SnmpLib.OID.is_child_of?([1, 3, 6, 2], [1, 3, 6, 1])","ref":"SnmpKit.SnmpLib.OID.html#is_child_of?/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_enterprise?/1","doc":"","ref":"SnmpKit.SnmpLib.OID.html#is_enterprise?/1"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_experimental?/1","doc":"","ref":"SnmpKit.SnmpLib.OID.html#is_experimental?/1"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_mib_2?/1","doc":"Checks if an OID is under a specific standard tree.","ref":"SnmpKit.SnmpLib.OID.html#is_mib_2?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.is_mib_2?/1","doc":"true = SnmpKit.SnmpLib.OID.is_mib_2?([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    true = SnmpKit.SnmpLib.OID.is_enterprise?([1, 3, 6, 1, 4, 1, 9, 1, 1])","ref":"SnmpKit.SnmpLib.OID.html#is_mib_2?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_parent_of?/2","doc":"Checks if one OID is a parent of another.","ref":"SnmpKit.SnmpLib.OID.html#is_parent_of?/2"},{"type":"function","title":"SnmpKit.SnmpLib.OID.is_private?/1","doc":"","ref":"SnmpKit.SnmpLib.OID.html#is_private?/1"},{"type":"function","title":"SnmpKit.SnmpLib.OID.list_to_string/1","doc":"Converts an OID list to a dot-separated string.","ref":"SnmpKit.SnmpLib.OID.html#list_to_string/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.list_to_string/1","doc":"- `oid_list`: List of non-negative integers","ref":"SnmpKit.SnmpLib.OID.html#list_to_string/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.list_to_string/1","doc":"- `{:ok, oid_string}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.OID.html#list_to_string/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.list_to_string/1","doc":"{:ok, \"1.3.6.1.2.1.1.1.0\"} = SnmpKit.SnmpLib.OID.list_to_string([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    {:error, :invalid_oid_list} = SnmpKit.SnmpLib.OID.list_to_string([1, 3, -1, 4])\n    {:error, :empty_oid} = SnmpKit.SnmpLib.OID.list_to_string([])","ref":"SnmpKit.SnmpLib.OID.html#list_to_string/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.mib_2/0","doc":"Returns standard SNMP OID prefixes.","ref":"SnmpKit.SnmpLib.OID.html#mib_2/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.normalize/1","doc":"Normalizes an OID to a consistent format.\n\nAccepts either string or list format and returns a list.","ref":"SnmpKit.SnmpLib.OID.html#normalize/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.normalize/1","doc":"{:ok, [1, 3, 6, 1]} = SnmpKit.SnmpLib.OID.normalize(\"1.3.6.1\")\n    {:ok, [1, 3, 6, 1]} = SnmpKit.SnmpLib.OID.normalize([1, 3, 6, 1])","ref":"SnmpKit.SnmpLib.OID.html#normalize/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.OID.oid/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.OID.oid_string/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#t:oid_string/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.parse_table_index/2","doc":"Parses a table index according to index syntax definition.","ref":"SnmpKit.SnmpLib.OID.html#parse_table_index/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.parse_table_index/2","doc":"- `index`: Raw index from OID\n- `syntax`: Index syntax specification","ref":"SnmpKit.SnmpLib.OID.html#parse_table_index/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.parse_table_index/2","doc":"- `{:ok, parsed_index}` if successful\n- `{:error, reason}` if parsing fails","ref":"SnmpKit.SnmpLib.OID.html#parse_table_index/2-returns"},{"type":"function","title":"Index Syntax Examples - SnmpKit.SnmpLib.OID.parse_table_index/2","doc":"- `:integer` - Single integer index\n- `{:string, length}` - Fixed-length string\n- `{:variable_string}` - Length-prefixed string\n- `[:integer, :integer]` - Multiple integer indices","ref":"SnmpKit.SnmpLib.OID.html#parse_table_index/2-index-syntax-examples"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.parse_table_index/2","doc":"{:ok, 42} = SnmpKit.SnmpLib.OID.parse_table_index([42], :integer)\n    {:ok, \"test\"} = SnmpKit.SnmpLib.OID.parse_table_index([4, 116, 101, 115, 116], {:variable_string})","ref":"SnmpKit.SnmpLib.OID.html#parse_table_index/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.private/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#private/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.sort/1","doc":"Sorts a list of OIDs in lexicographic order.","ref":"SnmpKit.SnmpLib.OID.html#sort/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.sort/1","doc":"sorted = SnmpKit.SnmpLib.OID.sort([[1, 3, 6, 2], [1, 3, 6, 1, 2], [1, 3, 6, 1]])\n    # Returns [[1, 3, 6, 1], [1, 3, 6, 1, 2], [1, 3, 6, 2]]","ref":"SnmpKit.SnmpLib.OID.html#sort/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.standard_prefix/1","doc":"Get standard SNMP OID prefixes.","ref":"SnmpKit.SnmpLib.OID.html#standard_prefix/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.standard_prefix/1","doc":"iex> SnmpKit.SnmpLib.OID.standard_prefix(:internet)\n    [1, 3, 6, 1]\n\n    iex> SnmpKit.SnmpLib.OID.standard_prefix(:mgmt)\n    [1, 3, 6, 1, 2]","ref":"SnmpKit.SnmpLib.OID.html#standard_prefix/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.OID.string_to_list/1","doc":"Converts an OID string to a list of integers.\n\nParses a dot-separated OID string into a list of non-negative integers.\nValidates each component and ensures the OID format is correct.","ref":"SnmpKit.SnmpLib.OID.html#string_to_list/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.OID.string_to_list/1","doc":"- `oid_string`: Dot-separated OID string (e.g., \"1.3.6.1.2.1.1.1.0\")","ref":"SnmpKit.SnmpLib.OID.html#string_to_list/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.string_to_list/1","doc":"- `{:ok, oid_list}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.OID.html#string_to_list/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.string_to_list/1","doc":"# Standard SNMP OIDs\n    iex> SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6.1.2.1.1.1.0\")\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 1, 0]}\n\n    # Short OIDs\n    iex> SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6\")\n    {:ok, [1, 3, 6]}\n\n    # Error cases\n    iex> SnmpKit.SnmpLib.OID.string_to_list(\"\")\n    {:error, :empty_oid}\n\n    iex> SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6.1.a.2\")\n    {:error, :invalid_oid_string}\n\n    iex> SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6.1.2.-1\")\n    {:error, :invalid_oid_string}","ref":"SnmpKit.SnmpLib.OID.html#string_to_list/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.OID.table_oid/0","doc":"","ref":"SnmpKit.SnmpLib.OID.html#t:table_oid/0"},{"type":"function","title":"SnmpKit.SnmpLib.OID.valid_oid?/1","doc":"Validates an OID list for correctness.","ref":"SnmpKit.SnmpLib.OID.html#valid_oid?/1"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.OID.valid_oid?/1","doc":"- `:ok` if valid\n- `{:error, reason}` if invalid","ref":"SnmpKit.SnmpLib.OID.html#valid_oid?/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.OID.valid_oid?/1","doc":":ok = SnmpKit.SnmpLib.OID.valid_oid?([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    {:error, :empty_oid} = SnmpKit.SnmpLib.OID.valid_oid?([])\n    {:error, :invalid_component} = SnmpKit.SnmpLib.OID.valid_oid?([1, 3, -1, 4])","ref":"SnmpKit.SnmpLib.OID.html#valid_oid?/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.PDU","doc":"SNMP PDU (Protocol Data Unit) encoding and decoding with RFC compliance.\n\nProvides comprehensive SNMP PDU functionality combining the best features from\nmultiple SNMP implementations. Supports SNMPv1 and SNMPv2c protocols with\nhigh-performance encoding/decoding, robust error handling, and full RFC compliance.","ref":"SnmpKit.SnmpLib.PDU.html"},{"type":"module","title":"API Documentation - SnmpKit.SnmpLib.PDU","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#module-api-documentation"},{"type":"module","title":"PDU Structure - SnmpKit.SnmpLib.PDU","doc":"All PDU functions in this library use a **consistent map structure** with these fields:\n\n```elixir\n%{\n  type: :get_request | :get_next_request | :get_response | :set_request | :get_bulk_request,\n  request_id: non_neg_integer(),\n  error_status: 0..5,\n  error_index: non_neg_integer(),\n  varbinds: [varbind()],\n  # GETBULK only:\n  non_repeaters: non_neg_integer(),      # Optional, GETBULK requests only\n  max_repetitions: non_neg_integer()     # Optional, GETBULK requests only\n}\n```\n\n**IMPORTANT**: Always use the `:type` field (not `:pdu_type`) with atom values.","ref":"SnmpKit.SnmpLib.PDU.html#module-pdu-structure"},{"type":"module","title":"Variable Bindings Format - SnmpKit.SnmpLib.PDU","doc":"Variable bindings (`varbinds`) support two formats:\n\n- **2-tuple format**: `{oid, value}` - Used for responses and simple cases\n- **3-tuple format**: `{oid, type, value}` - Used for requests with explicit type info\n\n```elixir\n# Request varbinds (3-tuple with type information)\n[{[1, 3, 6, 1, 2, 1, 1, 1, 0], :null, :null}]\n\n# Response varbinds (2-tuple format)\n[{[1, 3, 6, 1, 2, 1, 1, 1, 0], \"Linux server\"}]\n\n# Response varbinds (3-tuple format also supported)\n[{[1, 3, 6, 1, 2, 1, 1, 1, 0], :octet_string, \"Linux server\"}]\n```","ref":"SnmpKit.SnmpLib.PDU.html#module-variable-bindings-format"},{"type":"module","title":"Message Structure - SnmpKit.SnmpLib.PDU","doc":"SNMP messages have this structure:\n\n```elixir\n%{\n  version: 0 | 1,           # 0 = SNMPv1, 1 = SNMPv2c\n  community: binary(),      # Community string\n  pdu: pdu()               # PDU map as described above\n}\n```","ref":"SnmpKit.SnmpLib.PDU.html#module-message-structure"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.PDU","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#module-examples"},{"type":"module","title":"Building PDUs - SnmpKit.SnmpLib.PDU","doc":"```elixir\n# GET request for system description\n{:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_request([1, 3, 6, 1, 2, 1, 1, 1, 0], 123)\n\n# GETBULK request for interface table\n{:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_bulk_request([1, 3, 6, 1, 2, 1, 2, 2, 1], 124, 0, 10)\n\n# SET request\n{:ok, pdu} = SnmpKit.SnmpLib.PDU.build_set_request([1, 3, 6, 1, 2, 1, 1, 5, 0], {:octet_string, \"New Name\"}, 125)\n```","ref":"SnmpKit.SnmpLib.PDU.html#module-building-pdus"},{"type":"module","title":"Building Messages - SnmpKit.SnmpLib.PDU","doc":"```elixir\n# Complete SNMP message\n{:ok, message} = SnmpKit.SnmpLib.PDU.build_message(pdu, \"public\", :v2c)\n\n# Encode to binary\n{:ok, packet} = SnmpKit.SnmpLib.PDU.encode_message(message)\n\n# Decode from binary\n{:ok, decoded_message} = SnmpKit.SnmpLib.PDU.decode_message(packet)\n```","ref":"SnmpKit.SnmpLib.PDU.html#module-building-messages"},{"type":"module","title":"Error Responses - SnmpKit.SnmpLib.PDU","doc":"```elixir\n# Create error response\nerror_pdu = SnmpKit.SnmpLib.PDU.create_error_response(original_pdu, :no_such_name, 1)\n```","ref":"SnmpKit.SnmpLib.PDU.html#module-error-responses"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_get_bulk_request/4","doc":"Builds a GETBULK request PDU (SNMPv2c only).","ref":"SnmpKit.SnmpLib.PDU.html#build_get_bulk_request/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_get_bulk_request/4","doc":"- `oid_list`: Single OID list or list of OID lists to request\n- `request_id`: Unique request identifier\n- `non_repeaters`: Number of non-repeating variables (default: 0)\n- `max_repetitions`: Maximum repetitions for repeating variables (default: 10)","ref":"SnmpKit.SnmpLib.PDU.html#build_get_bulk_request/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_get_bulk_request/4","doc":"iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_bulk_request([1, 3, 6, 1, 2, 1, 2, 2, 1], 125, 0, 10)\n    iex> pdu.type\n    :get_bulk_request","ref":"SnmpKit.SnmpLib.PDU.html#build_get_bulk_request/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_get_next_request/2","doc":"Builds a GETNEXT request PDU.","ref":"SnmpKit.SnmpLib.PDU.html#build_get_next_request/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_get_next_request/2","doc":"- `oid_or_oids`: Single OID list or list of OID lists to request\n- `request_id`: Unique request identifier","ref":"SnmpKit.SnmpLib.PDU.html#build_get_next_request/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_get_next_request/2","doc":"iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_next_request([1, 3, 6, 1, 2, 1, 1], 124)\n    iex> pdu.type\n    :get_next_request","ref":"SnmpKit.SnmpLib.PDU.html#build_get_next_request/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_get_request/2","doc":"Builds a GET request PDU.","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_get_request/2","doc":"- `oid`: Single OID as list of integers\n- `request_id`: Unique request identifier","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_get_request/2","doc":"iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_request([1, 3, 6, 1, 2, 1, 1, 1, 0], 123)\n    iex> pdu.type\n    :get_request","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_get_request_multi/2","doc":"Builds a GET request PDU with multiple varbinds.","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_get_request_multi/2","doc":"- `varbinds`: List of variable bindings in format `{oid, type, value}`\n- `request_id`: Unique request identifier","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_get_request_multi/2","doc":"iex> varbinds = [{[1, 3, 6, 1, 2, 1, 1, 1, 0], :null, :null}]\n    iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_request_multi(varbinds, 123)\n    iex> pdu.type\n    :get_request","ref":"SnmpKit.SnmpLib.PDU.html#build_get_request_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_message/3","doc":"Builds an SNMP message with version, community, and PDU.","ref":"SnmpKit.SnmpLib.PDU.html#build_message/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_message/3","doc":"- `pdu`: PDU structure to include in the message\n- `community`: Community string for authentication\n- `version`: SNMP version","ref":"SnmpKit.SnmpLib.PDU.html#build_message/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_message/3","doc":"iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_get_request([1, 3, 6, 1, 2, 1, 1, 1, 0], 123)\n    iex> message = SnmpKit.SnmpLib.PDU.build_message(pdu, \"public\", :v2c)\n    iex> message.version\n    1","ref":"SnmpKit.SnmpLib.PDU.html#build_message/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_response/4","doc":"Builds a response PDU.","ref":"SnmpKit.SnmpLib.PDU.html#build_response/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_response/4","doc":"- `request_pdu`: Original request PDU to respond to\n- `varbinds`: List of variable bindings for the response\n- `error_status`: Error status code (default: 0 for no error)\n- `error_index`: Error index (default: 0)","ref":"SnmpKit.SnmpLib.PDU.html#build_response/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_response/4","doc":"iex> varbinds = [{[1, 3, 6, 1, 2, 1, 1, 1, 0], :octet_string, \"Linux server\"}]\n    iex> pdu = SnmpKit.SnmpLib.PDU.build_response(123, 0, 0, varbinds)\n    iex> pdu.type\n    :get_response","ref":"SnmpKit.SnmpLib.PDU.html#build_response/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.build_set_request/3","doc":"Builds a SET request PDU.","ref":"SnmpKit.SnmpLib.PDU.html#build_set_request/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.build_set_request/3","doc":"- `oid_list`: Single OID as list of integers\n- `type_value`: Tuple of `{type, value}` for the SET operation\n- `request_id`: Unique request identifier","ref":"SnmpKit.SnmpLib.PDU.html#build_set_request/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.build_set_request/3","doc":"iex> {:ok, pdu} = SnmpKit.SnmpLib.PDU.build_set_request([1, 3, 6, 1, 2, 1, 1, 5, 0], {:octet_string, \"New Name\"}, 126)\n    iex> pdu.type\n    :set_request","ref":"SnmpKit.SnmpLib.PDU.html#build_set_request/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.create_error_response/2","doc":"Creates an error response PDU.","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.create_error_response/2","doc":"- `request_pdu`: Original request PDU\n- `error_status`: Error status atom or code","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.create_error_response/2","doc":"iex> request_pdu = %{type: :get_request, request_id: 123, error_status: 0, error_index: 0, varbinds: []}\n    iex> error_pdu = SnmpKit.SnmpLib.PDU.create_error_response(request_pdu, :no_such_name)\n    iex> error_pdu.error_status\n    2","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.create_error_response/3","doc":"Creates an error response PDU.","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.create_error_response/3","doc":"- `request_pdu`: Original request PDU\n- `error_status`: Error status atom or code\n- `error_index`: Index of the variable that caused the error","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.create_error_response/3","doc":"iex> request_pdu = %{type: :get_request, request_id: 123, error_status: 0, error_index: 0, varbinds: []}\n    iex> error_pdu = SnmpKit.SnmpLib.PDU.create_error_response(request_pdu, :no_such_name, 1)\n    iex> error_pdu.error_status\n    2","ref":"SnmpKit.SnmpLib.PDU.html#create_error_response/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.decode/1","doc":"Alias for decode_message/1.","ref":"SnmpKit.SnmpLib.PDU.html#decode/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.decode_message/1","doc":"Decodes an SNMP message from binary format.","ref":"SnmpKit.SnmpLib.PDU.html#decode_message/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.decode_message/1","doc":"iex> {:ok, binary} = SnmpKit.SnmpLib.PDU.encode_message(%{version: 1, community: \"public\", pdu: %{type: :get_request, request_id: 123, error_status: 0, error_index: 0, varbinds: []}})\n    iex> {:ok, _message} = SnmpKit.SnmpLib.PDU.decode_message(binary)","ref":"SnmpKit.SnmpLib.PDU.html#decode_message/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.decode_snmp_packet/1","doc":"Legacy alias for decode/1.","ref":"SnmpKit.SnmpLib.PDU.html#decode_snmp_packet/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.encode/1","doc":"Alias for encode_message/1.","ref":"SnmpKit.SnmpLib.PDU.html#encode/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.encode_message/1","doc":"Encodes an SNMP message to binary format.","ref":"SnmpKit.SnmpLib.PDU.html#encode_message/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.encode_message/1","doc":"iex> message = %{version: 1, community: \"public\", pdu: %{type: :get_request, request_id: 123, error_status: 0, error_index: 0, varbinds: []}}\n    iex> {:ok, _binary} = SnmpKit.SnmpLib.PDU.encode_message(message)","ref":"SnmpKit.SnmpLib.PDU.html#encode_message/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.encode_snmp_packet/1","doc":"Legacy alias for encode/1.","ref":"SnmpKit.SnmpLib.PDU.html#encode_snmp_packet/1"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.error_status/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:error_status/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.error_status_to_atom/1","doc":"Converts an error status code to its atom representation.","ref":"SnmpKit.SnmpLib.PDU.html#error_status_to_atom/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.error_status_to_atom/1","doc":"iex> SnmpKit.SnmpLib.PDU.error_status_to_atom(2)\n    :no_such_name","ref":"SnmpKit.SnmpLib.PDU.html#error_status_to_atom/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.error_status_to_code/1","doc":"Converts an error status atom to its numeric code.","ref":"SnmpKit.SnmpLib.PDU.html#error_status_to_code/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.error_status_to_code/1","doc":"iex> SnmpKit.SnmpLib.PDU.error_status_to_code(:no_such_name)\n    2","ref":"SnmpKit.SnmpLib.PDU.html#error_status_to_code/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.message/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:message/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.normalize_oid/1","doc":"Normalizes an OID to a list of integers.","ref":"SnmpKit.SnmpLib.PDU.html#normalize_oid/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.normalize_oid/1","doc":"iex> SnmpKit.SnmpLib.PDU.normalize_oid([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    [1, 3, 6, 1, 2, 1, 1, 1, 0]","ref":"SnmpKit.SnmpLib.PDU.html#normalize_oid/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.normalize_type/1","doc":"Normalizes an SNMP type atom.","ref":"SnmpKit.SnmpLib.PDU.html#normalize_type/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.normalize_type/1","doc":"iex> SnmpKit.SnmpLib.PDU.normalize_type(:string)\n    :octet_string","ref":"SnmpKit.SnmpLib.PDU.html#normalize_type/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.oid/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:pdu/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.snmp_type/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:snmp_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:snmp_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.validate/1","doc":"Validates a PDU structure.","ref":"SnmpKit.SnmpLib.PDU.html#validate/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.validate/1","doc":"iex> pdu = %{type: :get_request, request_id: 123, error_status: 0, error_index: 0, varbinds: []}\n    iex> {:ok, ^pdu} = SnmpKit.SnmpLib.PDU.validate(pdu)","ref":"SnmpKit.SnmpLib.PDU.html#validate/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.validate_community/2","doc":"Validates a community string.","ref":"SnmpKit.SnmpLib.PDU.html#validate_community/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.validate_community/2","doc":"- `encoded_message`: Encoded SNMP message\n- `expected_community`: Expected community string","ref":"SnmpKit.SnmpLib.PDU.html#validate_community/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.validate_community/2","doc":"iex> :ok = SnmpKit.SnmpLib.PDU.validate_community(encoded_msg, \"public\")","ref":"SnmpKit.SnmpLib.PDU.html#validate_community/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.html#t:varbind/0"},{"type":"module","title":"SnmpKit.SnmpLib.PDU.Builder","doc":"High-level PDU and message building functions for SNMP operations.\n\nThis module provides functions to build various types of SNMP PDUs and messages,\nincluding GET, GETNEXT, SET, GETBULK requests, and responses.","ref":"SnmpKit.SnmpLib.PDU.Builder.html"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_get_bulk_request/4","doc":"Builds a GETBULK request PDU for SNMPv2c.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_get_bulk_request/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.Builder.build_get_bulk_request/4","doc":"- `oid_list`: Starting OID\n- `request_id`: Request identifier\n- `non_repeaters`: Number of non-repeating variables (default: 0)\n- `max_repetitions`: Maximum repetitions (default: 10)","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_get_bulk_request/4-parameters"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_get_next_request/2","doc":"Builds a GETNEXT request PDU.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_get_next_request/2"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_get_request/2","doc":"Builds a GET request PDU.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_get_request/2"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_get_request_multi/2","doc":"Builds a GET request PDU with multiple varbinds.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_get_request_multi/2"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_message/3","doc":"Builds an SNMP message structure.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_message/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.PDU.Builder.build_message/3","doc":"- `pdu`: The PDU to include in the message\n- `community`: Community string\n- `version`: SNMP version (:v1, :v2c, etc.)","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_message/3-parameters"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_response/4","doc":"Builds a response PDU.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_response/4"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.build_set_request/3","doc":"Builds a SET request PDU.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#build_set_request/3"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.create_error_response/3","doc":"Creates an error response PDU from a request PDU.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#create_error_response/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.PDU.Builder.create_error_response/3","doc":"error_pdu = SnmpKit.SnmpLib.PDU.Builder.create_error_response(request_pdu, 2, 1)","ref":"SnmpKit.SnmpLib.PDU.Builder.html#create_error_response/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.error_status/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:error_status/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.message/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:message/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.oid/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:pdu/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.pdu_type/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:pdu_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:snmp_value/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.snmp_version/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:snmp_version/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.validate/1","doc":"Validates a PDU structure.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#validate/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Builder.validate_community/2","doc":"Validates a community string against an encoded SNMP message.","ref":"SnmpKit.SnmpLib.PDU.Builder.html#validate_community/2"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Builder.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Builder.html#t:varbind/0"},{"type":"module","title":"SnmpKit.SnmpLib.PDU.Constants","doc":"Constants and type definitions for SNMP PDU operations.\n\nThis module contains all ASN.1 tags, error codes, type definitions,\nand utility functions used throughout the SNMP PDU system.","ref":"SnmpKit.SnmpLib.PDU.Constants.html"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.bad_value/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#bad_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.counter32/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#counter32/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.counter64/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#counter64/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.data_type_to_tag/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#data_type_to_tag/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.end_of_mib_view/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#end_of_mib_view/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.error_status/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:error_status/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.error_status_to_atom/1","doc":"Converts an error status code to its atom representation.","ref":"SnmpKit.SnmpLib.PDU.Constants.html#error_status_to_atom/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.error_status_to_code/1","doc":"Converts an error status atom to its numeric code.","ref":"SnmpKit.SnmpLib.PDU.Constants.html#error_status_to_code/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.gauge32/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#gauge32/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.gen_err/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#gen_err/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.get_request/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#get_request/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.get_response/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#get_response/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.getbulk_request/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#getbulk_request/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.getnext_request/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#getnext_request/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.integer/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#integer/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.ip_address/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#ip_address/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.message/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:message/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.no_error/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#no_error/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.no_such_instance/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#no_such_instance/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.no_such_name/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#no_such_name/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.no_such_object/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#no_such_object/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.normalize_oid/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#normalize_oid/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.normalize_type/1","doc":"Normalizes an SNMP type atom.","ref":"SnmpKit.SnmpLib.PDU.Constants.html#normalize_type/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.normalize_version/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#normalize_version/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.null/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#null/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.object_identifier/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#object_identifier/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.octet_string/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#octet_string/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.oid/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:oid/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.opaque_type/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#opaque_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:pdu/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.pdu_type/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:pdu_type/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.pdu_type_to_tag/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#pdu_type_to_tag/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.read_only/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#read_only/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.set_request/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#set_request/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.snmp_type/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:snmp_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:snmp_value/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.snmp_version/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:snmp_version/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.tag_to_data_type/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#tag_to_data_type/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.tag_to_pdu_type/1","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#tag_to_pdu_type/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.timeticks/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#timeticks/0"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Constants.too_big/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#too_big/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Constants.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Constants.html#t:varbind/0"},{"type":"module","title":"SnmpKit.SnmpLib.PDU.Decoder","doc":"ASN.1 BER decoding functions for SNMP PDUs and messages.\n\nThis module handles the conversion of binary ASN.1 BER format to Elixir data structures\nfor SNMP protocol communication.","ref":"SnmpKit.SnmpLib.PDU.Decoder.html"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Decoder.decode/1","doc":"Decodes an SNMP message from binary format (alias for decode_message/1).","ref":"SnmpKit.SnmpLib.PDU.Decoder.html#decode/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Decoder.decode_message/1","doc":"Decodes an SNMP message from binary format.","ref":"SnmpKit.SnmpLib.PDU.Decoder.html#decode_message/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Decoder.decode_snmp_packet/1","doc":"Alias for decode/1.","ref":"SnmpKit.SnmpLib.PDU.Decoder.html#decode_snmp_packet/1"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Decoder.message/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Decoder.html#t:message/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Decoder.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Decoder.html#t:pdu/0"},{"type":"module","title":"SnmpKit.SnmpLib.PDU.Encoder","doc":"ASN.1 BER encoding functions for SNMP PDUs and messages.\n\nThis module handles the conversion of Elixir data structures to binary ASN.1 BER format\nfor SNMP protocol communication.","ref":"SnmpKit.SnmpLib.PDU.Encoder.html"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Encoder.encode/1","doc":"Encodes an SNMP message to binary format (alias for encode_message/1).","ref":"SnmpKit.SnmpLib.PDU.Encoder.html#encode/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Encoder.encode_message/1","doc":"Encodes an SNMP message to binary format.","ref":"SnmpKit.SnmpLib.PDU.Encoder.html#encode_message/1"},{"type":"function","title":"SnmpKit.SnmpLib.PDU.Encoder.encode_snmp_packet/1","doc":"Alias for encode/1.","ref":"SnmpKit.SnmpLib.PDU.Encoder.html#encode_snmp_packet/1"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Encoder.message/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Encoder.html#t:message/0"},{"type":"type","title":"SnmpKit.SnmpLib.PDU.Encoder.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.PDU.Encoder.html#t:pdu/0"},{"type":"module","title":"SnmpKit.SnmpLib.Pool","doc":"Connection pooling and session management for high-performance SNMP operations.\n\nThis module provides sophisticated connection pooling capabilities designed for\napplications that need to manage many concurrent SNMP operations efficiently.\nBased on patterns proven in the DDumb project for handling 100+ concurrent device polls.","ref":"SnmpKit.SnmpLib.Pool.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Pool","doc":"- **Connection Pooling**: Efficient reuse of UDP sockets across operations\n- **Session Management**: Per-device session tracking with state management\n- **Load Balancing**: Intelligent distribution of operations across pool workers\n- **Health Monitoring**: Automatic detection and handling of unhealthy connections\n- **Performance Metrics**: Built-in monitoring and performance tracking\n- **Graceful Degradation**: Circuit breaker patterns for failing devices","ref":"SnmpKit.SnmpLib.Pool.html#module-features"},{"type":"module","title":"Pool Strategies - SnmpKit.SnmpLib.Pool","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#module-pool-strategies"},{"type":"module","title":"FIFO Pool (Default) - SnmpKit.SnmpLib.Pool","doc":"Best for general-purpose SNMP operations with mixed device types.","ref":"SnmpKit.SnmpLib.Pool.html#module-fifo-pool-default"},{"type":"module","title":"Round-Robin Pool - SnmpKit.SnmpLib.Pool","doc":"Optimal for polling multiple devices with similar characteristics.","ref":"SnmpKit.SnmpLib.Pool.html#module-round-robin-pool"},{"type":"module","title":"Device-Affinity Pool - SnmpKit.SnmpLib.Pool","doc":"Routes operations for the same device to the same worker for session consistency.","ref":"SnmpKit.SnmpLib.Pool.html#module-device-affinity-pool"},{"type":"module","title":"Usage Patterns - SnmpKit.SnmpLib.Pool","doc":"# Start a pool for network monitoring\n    {:ok, pool_pid} = SnmpKit.SnmpLib.Pool.start_pool(:network_monitor,\n      strategy: :device_affinity,\n      size: 20,\n      max_overflow: 10\n    )\n\n    # Perform operations through the pool\n    SnmpKit.SnmpLib.Pool.with_connection(:network_monitor, \"192.168.1.1\", fn conn ->\n      SnmpKit.SnmpLib.Manager.get_multi(conn.socket, \"192.168.1.1\", oids, conn.opts)\n    end)\n\n    # Get pool statistics\n    stats = SnmpKit.SnmpLib.Pool.get_stats(:network_monitor)\n    IO.inspect(stats.active_connections)","ref":"SnmpKit.SnmpLib.Pool.html#module-usage-patterns"},{"type":"module","title":"Performance Benefits - SnmpKit.SnmpLib.Pool","doc":"- **60-80% reduction** in socket creation overhead\n- **Improved throughput** for high-frequency polling\n- **Lower memory usage** through connection reuse\n- **Better resource utilization** with intelligent load balancing","ref":"SnmpKit.SnmpLib.Pool.html#module-performance-benefits"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.checkin_connection/2","doc":"Returns a connection to the pool after manual checkout.","ref":"SnmpKit.SnmpLib.Pool.html#checkin_connection/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.checkin_connection/2","doc":":ok = SnmpKit.SnmpLib.Pool.checkin_connection(:snmp_pool, connection)","ref":"SnmpKit.SnmpLib.Pool.html#checkin_connection/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.checkout_connection/3","doc":"Checks out a connection from the pool for manual management.\n\nUse this when you need more control over connection lifecycle.\nYou must call `checkin_connection/2` when done.","ref":"SnmpKit.SnmpLib.Pool.html#checkout_connection/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.checkout_connection/3","doc":"{:ok, conn} = SnmpKit.SnmpLib.Pool.checkout_connection(:snmp_pool, \"192.168.1.1\")\n    # ... perform operations with conn\n    :ok = SnmpKit.SnmpLib.Pool.checkin_connection(:snmp_pool, conn)","ref":"SnmpKit.SnmpLib.Pool.html#checkout_connection/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpLib.Pool.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.cleanup_unhealthy/1","doc":"Removes unhealthy connections from the pool and replaces them.","ref":"SnmpKit.SnmpLib.Pool.html#cleanup_unhealthy/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.cleanup_unhealthy/1","doc":":ok = SnmpKit.SnmpLib.Pool.cleanup_unhealthy(:snmp_pool)","ref":"SnmpKit.SnmpLib.Pool.html#cleanup_unhealthy/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Pool.connection/0","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#t:connection/0"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.get_stats/1","doc":"Gets comprehensive statistics about the pool's performance and health.","ref":"SnmpKit.SnmpLib.Pool.html#get_stats/1"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Pool.get_stats/1","doc":"A map containing:\n- Connection counts (active, idle, overflow)\n- Operation statistics (checkouts, checkins, response times)\n- Health status for connections\n- Performance metrics","ref":"SnmpKit.SnmpLib.Pool.html#get_stats/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.get_stats/1","doc":"stats = SnmpKit.SnmpLib.Pool.get_stats(:snmp_pool)\n    IO.puts(\"Active connections: \" <> to_string(stats.active_connections))\n    IO.puts(\"Average response time: \" <> to_string(stats.average_response_time) <> \"ms\")","ref":"SnmpKit.SnmpLib.Pool.html#get_stats/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.health_check/1","doc":"Forces a health check on all connections in the pool.\n\nUseful for proactive monitoring and debugging connection issues.","ref":"SnmpKit.SnmpLib.Pool.html#health_check/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.health_check/1","doc":":ok = SnmpKit.SnmpLib.Pool.health_check(:snmp_pool)","ref":"SnmpKit.SnmpLib.Pool.html#health_check/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Pool.pool_name/0","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#t:pool_name/0"},{"type":"type","title":"SnmpKit.SnmpLib.Pool.pool_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#t:pool_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.Pool.pool_stats/0","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#t:pool_stats/0"},{"type":"type","title":"SnmpKit.SnmpLib.Pool.pool_strategy/0","doc":"","ref":"SnmpKit.SnmpLib.Pool.html#t:pool_strategy/0"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.start_pool/2","doc":"Starts a new connection pool with the specified configuration.","ref":"SnmpKit.SnmpLib.Pool.html#start_pool/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Pool.start_pool/2","doc":"- `pool_name`: Unique atom identifier for the pool\n- `opts`: Pool configuration options","ref":"SnmpKit.SnmpLib.Pool.html#start_pool/2-parameters"},{"type":"function","title":"Options - SnmpKit.SnmpLib.Pool.start_pool/2","doc":"- `strategy`: Pool strategy (:fifo, :round_robin, :device_affinity)\n- `size`: Base number of connections in the pool (default: 10)\n- `max_overflow`: Maximum overflow connections allowed (default: 5)\n- `checkout_timeout`: Maximum time to wait for connection (default: 5000ms)\n- `max_idle_time`: Maximum idle time before connection cleanup (default: 300000ms)\n- `health_check_interval`: Interval for health checks (default: 30000ms)\n- `worker_opts`: Options passed to individual workers","ref":"SnmpKit.SnmpLib.Pool.html#start_pool/2-options"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Pool.start_pool/2","doc":"- `{:ok, pid}`: Pool started successfully\n- `{:error, reason}`: Failed to start pool","ref":"SnmpKit.SnmpLib.Pool.html#start_pool/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.start_pool/2","doc":"# Basic pool for general SNMP operations\n    {:ok, _pid} = SnmpKit.SnmpLib.Pool.start_pool(:snmp_pool)\n\n    # High-performance pool for device monitoring\n    {:ok, _pid} = SnmpKit.SnmpLib.Pool.start_pool(:monitor_pool,\n      strategy: :device_affinity,\n      size: 25,\n      max_overflow: 15,\n      health_check_interval: 60_000\n    )","ref":"SnmpKit.SnmpLib.Pool.html#start_pool/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.stop_pool/2","doc":"Stops a connection pool and cleans up all resources.","ref":"SnmpKit.SnmpLib.Pool.html#stop_pool/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Pool.stop_pool/2","doc":"- `pool_name`: Name of the pool to stop\n- `timeout`: Maximum time to wait for graceful shutdown (default: 5000ms)","ref":"SnmpKit.SnmpLib.Pool.html#stop_pool/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.stop_pool/2","doc":":ok = SnmpKit.SnmpLib.Pool.stop_pool(:snmp_pool)\n    :ok = SnmpKit.SnmpLib.Pool.stop_pool(:monitor_pool, 10_000)","ref":"SnmpKit.SnmpLib.Pool.html#stop_pool/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Pool.with_connection/4","doc":"Executes a function with a pooled connection, handling checkout/checkin automatically.\n\nThis is the primary interface for using pooled connections. The function\nautomatically handles connection lifecycle and error recovery.","ref":"SnmpKit.SnmpLib.Pool.html#with_connection/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Pool.with_connection/4","doc":"- `pool_name`: Name of the pool to use\n- `device`: Target device (used for device-affinity strategy)\n- `fun`: Function to execute with the connection\n- `opts`: Additional options for the operation","ref":"SnmpKit.SnmpLib.Pool.html#with_connection/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Pool.with_connection/4","doc":"The result of executing the function, or `{:error, reason}` if connection\ncheckout fails or the function raises an exception.","ref":"SnmpKit.SnmpLib.Pool.html#with_connection/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Pool.with_connection/4","doc":"# Perform multiple operations with connection reuse\n    result = SnmpKit.SnmpLib.Pool.with_connection(:snmp_pool, \"192.168.1.1\", fn conn ->\n      {:ok, sys_desc} = SnmpKit.SnmpLib.Manager.get_with_socket(conn.socket, \"192.168.1.1\",\n                                                         [1,3,6,1,2,1,1,1,0], conn.opts)\n      {:ok, sys_name} = SnmpKit.SnmpLib.Manager.get_with_socket(conn.socket, \"192.168.1.1\",\n                                                         [1,3,6,1,2,1,1,5,0], conn.opts)\n      {:sys_desc, sys_desc, :sys_name, sys_name}\n    end)","ref":"SnmpKit.SnmpLib.Pool.html#with_connection/4-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Security","doc":"SNMPv3 Security Framework - Phase 5.1A Implementation\n\nProvides comprehensive SNMPv3 User Security Model (USM) implementation including\nauthentication and privacy protocols for secure SNMP communications.","ref":"SnmpKit.SnmpLib.Security.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Security","doc":"- **User Security Model (USM)** with complete RFC 3414 compliance\n- **Authentication Protocols**: MD5, SHA-1, SHA-256, SHA-384, SHA-512\n- **Privacy Protocols**: DES, AES-128, AES-192, AES-256\n- **Key Derivation**: Password-based and localized key generation\n- **Security Parameters**: Boot counter, time synchronization, message validation\n- **Error Handling**: Comprehensive security error classification and recovery","ref":"SnmpKit.SnmpLib.Security.html#module-features"},{"type":"module","title":"Architecture - SnmpKit.SnmpLib.Security","doc":"The security framework is built on a modular architecture:\n\n- `SnmpKit.SnmpLib.Security.USM` - User Security Model implementation\n- `SnmpKit.SnmpLib.Security.Auth` - Authentication protocol handlers\n- `SnmpKit.SnmpLib.Security.Priv` - Privacy protocol handlers\n- `SnmpKit.SnmpLib.Security.Keys` - Key derivation and management","ref":"SnmpKit.SnmpLib.Security.html#module-architecture"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Security","doc":"","ref":"SnmpKit.SnmpLib.Security.html#module-usage-examples"},{"type":"module","title":"Basic SNMPv3 Authentication - SnmpKit.SnmpLib.Security","doc":"# Create authenticated user\n    {:ok, user} = SnmpKit.SnmpLib.Security.create_user(\"admin\",\n      auth_protocol: :sha256,\n      auth_password: \"secure_password\",\n      engine_id: \"engine123\"\n    )\n\n    # Authenticate message\n    {:ok, auth_params} = SnmpKit.SnmpLib.Security.authenticate_message(user, message)","ref":"SnmpKit.SnmpLib.Security.html#module-basic-snmpv3-authentication"},{"type":"module","title":"Privacy (Encryption) Support - SnmpKit.SnmpLib.Security","doc":"# Create user with privacy\n    {:ok, user} = SnmpKit.SnmpLib.Security.create_user(\"secure_admin\",\n      auth_protocol: :sha256,\n      auth_password: \"auth_password\",\n      priv_protocol: :aes256,\n      priv_password: \"priv_password\"\n    )\n\n    # Encrypt message\n    {:ok, encrypted} = SnmpKit.SnmpLib.Security.encrypt_message(user, message)","ref":"SnmpKit.SnmpLib.Security.html#module-privacy-encryption-support"},{"type":"module","title":"Engine ID Management - SnmpKit.SnmpLib.Security","doc":"# Generate engine ID\n    engine_id = SnmpKit.SnmpLib.Security.generate_engine_id(\"192.168.1.1\")\n\n    # Discover remote engine\n    {:ok, remote_engine} = SnmpKit.SnmpLib.Security.discover_engine(\"10.0.0.1\")","ref":"SnmpKit.SnmpLib.Security.html#module-engine-id-management"},{"type":"module","title":"Security Considerations - SnmpKit.SnmpLib.Security","doc":"- All key material is stored securely in memory\n- Authentication and privacy keys are derived using RFC-compliant algorithms\n- Time-based authentication prevents replay attacks\n- Boot counter management ensures message freshness\n- Comprehensive input validation prevents security bypasses","ref":"SnmpKit.SnmpLib.Security.html#module-security-considerations"},{"type":"type","title":"SnmpKit.SnmpLib.Security.auth_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:auth_protocol/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.authenticate_message/2","doc":"Authenticates an SNMP message using the user's authentication protocol.\n\nReturns authentication parameters that should be included in the message.","ref":"SnmpKit.SnmpLib.Security.html#authenticate_message/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.build_security_params/3","doc":"Builds security parameters for inclusion in SNMPv3 messages.","ref":"SnmpKit.SnmpLib.Security.html#build_security_params/3"},{"type":"function","title":"SnmpKit.SnmpLib.Security.create_user/2","doc":"Creates a new SNMPv3 security user with specified authentication and privacy settings.","ref":"SnmpKit.SnmpLib.Security.html#create_user/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.create_user/2","doc":"- `security_name`: Unique identifier for the user\n- `config`: User configuration including protocols and passwords","ref":"SnmpKit.SnmpLib.Security.html#create_user/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.create_user/2","doc":"- `{:ok, user}`: Successfully created security user\n- `{:error, reason}`: Creation failed","ref":"SnmpKit.SnmpLib.Security.html#create_user/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.create_user/2","doc":"# Authentication only user\n    {:ok, user} = SnmpKit.SnmpLib.Security.create_user(\"monitor_user\",\n      auth_protocol: :sha256,\n      auth_password: \"monitoring_secret\",\n      engine_id: \"local_engine\"\n    )\n\n    # Full authentication and privacy user\n    {:ok, admin} = SnmpKit.SnmpLib.Security.create_user(\"admin_user\",\n      auth_protocol: :sha512,\n      auth_password: \"admin_auth_pass\",\n      priv_protocol: :aes256,\n      priv_password: \"admin_priv_pass\",\n      engine_id: \"management_engine\"\n    )","ref":"SnmpKit.SnmpLib.Security.html#create_user/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.decrypt_message/3","doc":"Decrypts message data using the user's privacy protocol.","ref":"SnmpKit.SnmpLib.Security.html#decrypt_message/3"},{"type":"function","title":"SnmpKit.SnmpLib.Security.discover_engine/2","doc":"Discovers the engine ID of a remote SNMP agent.\n\nThis is typically done during the first communication with a remote agent\nto establish security context.","ref":"SnmpKit.SnmpLib.Security.html#discover_engine/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.encrypt_message/2","doc":"Encrypts message data using the user's privacy protocol.","ref":"SnmpKit.SnmpLib.Security.html#encrypt_message/2"},{"type":"type","title":"SnmpKit.SnmpLib.Security.engine_id/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:engine_id/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.generate_engine_id/1","doc":"Generates a unique engine ID for an SNMP entity.\n\nEngine IDs are used to uniquely identify SNMP engines and are required\nfor SNMPv3 security operations.","ref":"SnmpKit.SnmpLib.Security.html#generate_engine_id/1"},{"type":"function","title":"SnmpKit.SnmpLib.Security.get_security_level/1","doc":"Determines the security level for a message based on user configuration.","ref":"SnmpKit.SnmpLib.Security.html#get_security_level/1"},{"type":"function","title":"Security Levels - SnmpKit.SnmpLib.Security.get_security_level/1","doc":"- `:no_auth_no_priv` - No authentication, no privacy\n- `:auth_no_priv` - Authentication only\n- `:auth_priv` - Authentication and privacy","ref":"SnmpKit.SnmpLib.Security.html#get_security_level/1-security-levels"},{"type":"function","title":"SnmpKit.SnmpLib.Security.info/0","doc":"Returns comprehensive information about security capabilities and status.","ref":"SnmpKit.SnmpLib.Security.html#info/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.priv_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:priv_protocol/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.security_level/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:security_level/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.security_name/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:security_name/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.security_params/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:security_params/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.security_user/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:security_user/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.update_engine_time/3","doc":"Updates engine time and boot counter for time synchronization.","ref":"SnmpKit.SnmpLib.Security.html#update_engine_time/3"},{"type":"function","title":"SnmpKit.SnmpLib.Security.update_user/2","doc":"Updates security user credentials and regenerates keys.","ref":"SnmpKit.SnmpLib.Security.html#update_user/2"},{"type":"type","title":"SnmpKit.SnmpLib.Security.user_config/0","doc":"","ref":"SnmpKit.SnmpLib.Security.html#t:user_config/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.validate_security_params/2","doc":"Validates security parameters from received messages.","ref":"SnmpKit.SnmpLib.Security.html#validate_security_params/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.validate_user/3","doc":"Validates user credentials against stored authentication data.","ref":"SnmpKit.SnmpLib.Security.html#validate_user/3"},{"type":"function","title":"SnmpKit.SnmpLib.Security.verify_authentication/3","doc":"Verifies message authentication using provided authentication parameters.","ref":"SnmpKit.SnmpLib.Security.html#verify_authentication/3"},{"type":"module","title":"SnmpKit.SnmpLib.Security.Auth","doc":"Authentication protocols for SNMPv3 User Security Model.\n\nImplements HMAC-based authentication protocols as specified in RFC 3414 and RFC 7860,\nproviding message integrity and authentication for SNMPv3 communications.","ref":"SnmpKit.SnmpLib.Security.Auth.html"},{"type":"module","title":"Supported Protocols - SnmpKit.SnmpLib.Security.Auth","doc":"- **HMAC-MD5** (RFC 3414) - 16-byte digest, legacy support\n- **HMAC-SHA-1** (RFC 3414) - 20-byte digest, legacy support\n- **HMAC-SHA-224** (RFC 7860) - 28-byte digest\n- **HMAC-SHA-256** (RFC 7860) - 32-byte digest, recommended\n- **HMAC-SHA-384** (RFC 7860) - 48-byte digest\n- **HMAC-SHA-512** (RFC 7860) - 64-byte digest, highest security","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-supported-protocols"},{"type":"module","title":"Security Considerations - SnmpKit.SnmpLib.Security.Auth","doc":"- MD5 and SHA-1 are deprecated for new implementations\n- SHA-256 or higher is recommended for production use\n- Authentication keys must be properly derived using key derivation functions\n- Truncated MACs maintain security properties when properly implemented","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-security-considerations"},{"type":"module","title":"Protocol Selection Guidelines - SnmpKit.SnmpLib.Security.Auth","doc":"- **SHA-256**: Recommended for most deployments (good security/performance balance)\n- **SHA-512**: High security environments with adequate processing power\n- **SHA-384**: Alternative to SHA-512 with smaller digest size\n- **MD5/SHA-1**: Legacy compatibility only, not recommended for new deployments","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-protocol-selection-guidelines"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Security.Auth","doc":"","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-usage-examples"},{"type":"module","title":"Message Authentication - SnmpKit.SnmpLib.Security.Auth","doc":"# Authenticate outgoing message\n    auth_key = derived_authentication_key\n    message = snmp_message_data\n    {:ok, auth_params} = SnmpKit.SnmpLib.Security.Auth.authenticate(:sha256, auth_key, message)\n\n    # Verify incoming message\n    :ok = SnmpKit.SnmpLib.Security.Auth.verify(:sha256, auth_key, message, auth_params)","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-message-authentication"},{"type":"module","title":"Protocol Capabilities - SnmpKit.SnmpLib.Security.Auth","doc":"# Get protocol information\n    info = SnmpKit.SnmpLib.Security.Auth.protocol_info(:sha256)\n    # Returns: %{digest_size: 32, truncated_size: 12, secure: true, ...}\n\n    # List all supported protocols\n    protocols = SnmpKit.SnmpLib.Security.Auth.supported_protocols()","ref":"SnmpKit.SnmpLib.Security.Auth.html#module-protocol-capabilities"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Auth.auth_key/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Auth.html#t:auth_key/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Auth.auth_params/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Auth.html#t:auth_params/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Auth.auth_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Auth.html#t:auth_protocol/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.authenticate/3","doc":"Authenticates a message using the specified protocol and key.\n\nGenerates authentication parameters (truncated HMAC) for inclusion\nin the SNMPv3 message security parameters.","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Auth.authenticate/3","doc":"- `protocol`: Authentication protocol to use\n- `auth_key`: Localized authentication key (derived from password)\n- `message`: Complete message data to authenticate","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Auth.authenticate/3","doc":"- `{:ok, auth_params}`: Authentication parameters for message\n- `{:error, reason}`: Authentication failed","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.authenticate/3","doc":"# SHA-256 authentication (recommended)\n    {:ok, auth_params} = SnmpKit.SnmpLib.Security.Auth.authenticate(:sha256, auth_key, message)\n\n    # Legacy MD5 authentication\n    {:ok, auth_params} = SnmpKit.SnmpLib.Security.Auth.authenticate(:md5, auth_key, message)","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.authenticate_batch/3","doc":"Authenticates multiple messages using the same protocol and key.\n\nMore efficient than individual authentication calls when processing\nmultiple messages with the same authentication configuration.","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate_batch/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.authenticate_batch/3","doc":"messages = [msg1, msg2, msg3]\n    {:ok, auth_params_list} = SnmpKit.SnmpLib.Security.Auth.authenticate_batch(:sha256, auth_key, messages)","ref":"SnmpKit.SnmpLib.Security.Auth.html#authenticate_batch/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.benchmark_protocol/4","doc":"Measures authentication performance for a given protocol.\n\nUseful for performance tuning and protocol selection in high-throughput environments.","ref":"SnmpKit.SnmpLib.Security.Auth.html#benchmark_protocol/4"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.benchmark_protocol/4","doc":"stats = SnmpKit.SnmpLib.Security.Auth.benchmark_protocol(:sha256, test_key, test_message, 1000)\n    # Returns timing and throughput statistics","ref":"SnmpKit.SnmpLib.Security.Auth.html#benchmark_protocol/4-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Auth.message_data/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Auth.html#t:message_data/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.protocol_info/1","doc":"Returns information about a specific authentication protocol.","ref":"SnmpKit.SnmpLib.Security.Auth.html#protocol_info/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.protocol_info/1","doc":"iex> SnmpKit.SnmpLib.Security.Auth.protocol_info(:sha256)\n    %{algorithm: :sha256, digest_size: 32, truncated_size: 16, secure: true, rfc: \"RFC 7860\"}\n\n    iex> SnmpKit.SnmpLib.Security.Auth.protocol_info(:md5)\n    %{algorithm: :md5, digest_size: 16, truncated_size: 12, secure: false, rfc: \"RFC 3414\"}","ref":"SnmpKit.SnmpLib.Security.Auth.html#protocol_info/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.secure_protocol?/1","doc":"Checks if a protocol is considered cryptographically secure.","ref":"SnmpKit.SnmpLib.Security.Auth.html#secure_protocol?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.secure_protocol?/1","doc":"iex> SnmpKit.SnmpLib.Security.Auth.secure_protocol?(:sha256)\n    true\n\n    iex> SnmpKit.SnmpLib.Security.Auth.secure_protocol?(:md5)\n    false","ref":"SnmpKit.SnmpLib.Security.Auth.html#secure_protocol?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.secure_protocols/0","doc":"Returns list of cryptographically secure protocols (excludes deprecated ones).","ref":"SnmpKit.SnmpLib.Security.Auth.html#secure_protocols/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.secure_protocols/0","doc":"iex> SnmpKit.SnmpLib.Security.Auth.secure_protocols()\n    [:sha224, :sha256, :sha384, :sha512]","ref":"SnmpKit.SnmpLib.Security.Auth.html#secure_protocols/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.supported_protocols/0","doc":"Returns list of all supported authentication protocols.","ref":"SnmpKit.SnmpLib.Security.Auth.html#supported_protocols/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.supported_protocols/0","doc":"iex> SnmpKit.SnmpLib.Security.Auth.supported_protocols()\n    [:none, :md5, :sha1, :sha224, :sha256, :sha384, :sha512]","ref":"SnmpKit.SnmpLib.Security.Auth.html#supported_protocols/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.validate_key/2","doc":"Validates that an authentication key is appropriate for the specified protocol.\n\nChecks key length requirements and provides warnings for weak protocols.","ref":"SnmpKit.SnmpLib.Security.Auth.html#validate_key/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.validate_key/2","doc":":ok = SnmpKit.SnmpLib.Security.Auth.validate_key(:sha256, auth_key)\n    {:error, :key_too_short} = SnmpKit.SnmpLib.Security.Auth.validate_key(:sha512, short_key)","ref":"SnmpKit.SnmpLib.Security.Auth.html#validate_key/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.verify/4","doc":"Verifies message authentication using provided authentication parameters.\n\nRecomputes the expected authentication parameters and compares them\nwith the provided parameters using constant-time comparison.","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Auth.verify/4","doc":"- `protocol`: Authentication protocol used\n- `auth_key`: Localized authentication key\n- `message`: Message data that was authenticated\n- `provided_params`: Authentication parameters from received message","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Auth.verify/4","doc":"- `:ok`: Authentication verification successful\n- `{:error, reason}`: Verification failed","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.verify/4","doc":"# Verify SHA-256 authentication\n    :ok = SnmpKit.SnmpLib.Security.Auth.verify(:sha256, auth_key, message, auth_params)\n\n    # Failed verification\n    {:error, :authentication_mismatch} = SnmpKit.SnmpLib.Security.Auth.verify(:md5, wrong_key, message, auth_params)","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Auth.verify_batch/4","doc":"Verifies authentication for multiple messages in batch.","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify_batch/4"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Auth.verify_batch/4","doc":"results = SnmpKit.SnmpLib.Security.Auth.verify_batch(:sha256, auth_key, messages, auth_params_list)\n    # Returns: [:ok, :ok, {:error, :authentication_mismatch}]","ref":"SnmpKit.SnmpLib.Security.Auth.html#verify_batch/4-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Security.Keys","doc":"Key derivation and management for SNMPv3 User Security Model.\n\nImplements RFC 3414 compliant key derivation functions for converting\nuser passwords into cryptographic keys suitable for authentication\nand privacy operations.","ref":"SnmpKit.SnmpLib.Security.Keys.html"},{"type":"module","title":"Key Derivation Process - SnmpKit.SnmpLib.Security.Keys","doc":"SNMPv3 uses a two-step key derivation process:\n\n1. **Password Localization**: Transform user password into a localized key\n   using the authoritative engine ID\n2. **Key Expansion**: Derive authentication and privacy keys from the\n   localized key based on protocol requirements","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-key-derivation-process"},{"type":"module","title":"Security Properties - SnmpKit.SnmpLib.Security.Keys","doc":"- Keys are derived deterministically from passwords and engine IDs\n- Different engine IDs produce different keys for the same password\n- Key derivation uses cryptographic hash functions for security\n- Derived keys cannot be used to recover original passwords\n- Each protocol type (auth/priv) uses different key derivation parameters","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-security-properties"},{"type":"module","title":"Supported Algorithms - SnmpKit.SnmpLib.Security.Keys","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-supported-algorithms"},{"type":"module","title":"Authentication Key Derivation - SnmpKit.SnmpLib.Security.Keys","doc":"- **MD5**: RFC 3414 compliant (deprecated)\n- **SHA-1**: RFC 3414 compliant (deprecated)\n- **SHA-224**: RFC 7860 compliant\n- **SHA-256**: RFC 7860 compliant (recommended)\n- **SHA-384**: RFC 7860 compliant\n- **SHA-512**: RFC 7860 compliant","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-authentication-key-derivation"},{"type":"module","title":"Privacy Key Derivation - SnmpKit.SnmpLib.Security.Keys","doc":"- **DES**: 8-byte keys from authentication keys\n- **AES-128**: 16-byte keys with salt mixing\n- **AES-192**: 24-byte keys with salt mixing\n- **AES-256**: 32-byte keys with salt mixing","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-privacy-key-derivation"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Security.Keys","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-usage-examples"},{"type":"module","title":"Authentication Key Derivation - SnmpKit.SnmpLib.Security.Keys","doc":"# Derive SHA-256 authentication key\n    engine_id = <<0x80, 0x00, 0x1f, 0x88, 0x80, 0x01, 0x02, 0x03, 0x04>>\n    password = \"authentication_password\"\n\n    {:ok, auth_key} = SnmpKit.SnmpLib.Security.Keys.derive_auth_key(:sha256, password, engine_id)","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-authentication-key-derivation-1"},{"type":"module","title":"Privacy Key Derivation - SnmpKit.SnmpLib.Security.Keys","doc":"# Derive AES-256 privacy key\n    {:ok, priv_key} = SnmpKit.SnmpLib.Security.Keys.derive_priv_key(:aes256, password, engine_id)\n\n    # Or derive from existing authentication key\n    {:ok, priv_key} = SnmpKit.SnmpLib.Security.Keys.derive_priv_key_from_auth(:aes256, auth_key, engine_id)","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-privacy-key-derivation-1"},{"type":"module","title":"Key Validation - SnmpKit.SnmpLib.Security.Keys","doc":"# Validate key strength\n    :ok = SnmpKit.SnmpLib.Security.Keys.validate_password_strength(password)\n    {:error, :too_short} = SnmpKit.SnmpLib.Security.Keys.validate_password_strength(\"weak\")","ref":"SnmpKit.SnmpLib.Security.Keys.html#module-key-validation"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.auth_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:auth_protocol/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.derive_auth_key/3","doc":"Derives authentication key from password and engine ID.\n\nImplements RFC 3414 key localization algorithm for authentication protocols.\nThe derived key is specific to the combination of password, protocol, and engine ID.","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_key/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Keys.derive_auth_key/3","doc":"- `protocol`: Authentication protocol (:md5, :sha1, :sha256, etc.)\n- `password`: User password (minimum 8 characters recommended)\n- `engine_id`: Authoritative engine ID (5-32 bytes)","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_key/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Keys.derive_auth_key/3","doc":"- `{:ok, key}`: Successfully derived authentication key\n- `{:error, reason}`: Key derivation failed","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_key/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.derive_auth_key/3","doc":"# SHA-256 authentication key (recommended)\n    {:ok, key} = SnmpKit.SnmpLib.Security.Keys.derive_auth_key(\n      :sha256, \"my_secure_password\", engine_id\n    )\n\n    # Legacy MD5 key derivation\n    {:ok, key} = SnmpKit.SnmpLib.Security.Keys.derive_auth_key(\n      :md5, \"legacy_password\", engine_id\n    )","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_key/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.derive_auth_keys_multi/3","doc":"Derives multiple authentication keys for different protocols from the same password.\n\nUseful when supporting multiple authentication protocols simultaneously.","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_keys_multi/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.derive_auth_keys_multi/3","doc":"protocols = [:sha256, :sha384, :sha512]\n    {:ok, keys} = SnmpKit.SnmpLib.Security.Keys.derive_auth_keys_multi(protocols, password, engine_id)\n    # Returns: %{sha256: key1, sha384: key2, sha512: key3}","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_auth_keys_multi/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.derive_priv_key/3","doc":"Derives privacy key from password and engine ID.\n\nPrivacy keys are derived using a combination of authentication key derivation\nand protocol-specific key expansion techniques.","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Keys.derive_priv_key/3","doc":"- `protocol`: Privacy protocol (:des, :aes128, :aes192, :aes256)\n- `password`: User password for privacy\n- `engine_id`: Authoritative engine ID","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Keys.derive_priv_key/3","doc":"- `{:ok, key}`: Successfully derived privacy key\n- `{:error, reason}`: Key derivation failed","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.derive_priv_key/3","doc":"# AES-256 privacy key (recommended)\n    {:ok, key} = SnmpKit.SnmpLib.Security.Keys.derive_priv_key(\n      :aes256, \"privacy_password\", engine_id\n    )\n\n    # DES privacy key (legacy)\n    {:ok, key} = SnmpKit.SnmpLib.Security.Keys.derive_priv_key(\n      :des, \"legacy_privacy_password\", engine_id\n    )","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.derive_priv_key_from_auth/3","doc":"Derives privacy key from an existing authentication key.\n\nMore efficient when both authentication and privacy keys are needed,\nas it avoids repeating the expensive key localization process.","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key_from_auth/3"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.derive_priv_key_from_auth/3","doc":"# First derive authentication key\n    {:ok, auth_key} = derive_auth_key(:sha256, password, engine_id)\n\n    # Then derive privacy key from auth key\n    {:ok, priv_key} = SnmpKit.SnmpLib.Security.Keys.derive_priv_key_from_auth(\n      :aes256, auth_key, engine_id\n    )","ref":"SnmpKit.SnmpLib.Security.Keys.html#derive_priv_key_from_auth/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.derived_key/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:derived_key/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.engine_id/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:engine_id/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.export_key/3","doc":"Exports derived key in a secure format for storage or transmission.\n\nThe exported format includes metadata for proper key reconstruction\nwhile maintaining security properties.","ref":"SnmpKit.SnmpLib.Security.Keys.html#export_key/3"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.generate_secure_password/1","doc":"Generates a cryptographically secure random password.","ref":"SnmpKit.SnmpLib.Security.Keys.html#generate_secure_password/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.generate_secure_password/1","doc":"password = SnmpKit.SnmpLib.Security.Keys.generate_secure_password(16)\n    # Returns: \"K7mN9pQ2rT8vW3xZ\" (example)","ref":"SnmpKit.SnmpLib.Security.Keys.html#generate_secure_password/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.password/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:password/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.priv_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:priv_protocol/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Keys.salt/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Keys.html#t:salt/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.secure_compare/2","doc":"Securely compares two derived keys to prevent timing attacks.","ref":"SnmpKit.SnmpLib.Security.Keys.html#secure_compare/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.secure_compare/2","doc":"true = SnmpKit.SnmpLib.Security.Keys.secure_compare(key1, key1)\n    false = SnmpKit.SnmpLib.Security.Keys.secure_compare(key1, key2)","ref":"SnmpKit.SnmpLib.Security.Keys.html#secure_compare/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.secure_wipe/1","doc":"Securely wipes sensitive key material from memory.\n\nNote: This provides best-effort memory clearing but cannot guarantee\ncomplete removal due to Erlang VM memory management.","ref":"SnmpKit.SnmpLib.Security.Keys.html#secure_wipe/1"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.validate_imported_key/2","doc":"Validates imported key against expected parameters.","ref":"SnmpKit.SnmpLib.Security.Keys.html#validate_imported_key/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Keys.validate_password_strength/1","doc":"Validates password strength according to SNMPv3 security guidelines.","ref":"SnmpKit.SnmpLib.Security.Keys.html#validate_password_strength/1"},{"type":"function","title":"Requirements - SnmpKit.SnmpLib.Security.Keys.validate_password_strength/1","doc":"- Minimum 8 characters (RFC recommendation)\n- Should contain mix of character types for security\n- Should not be based on dictionary words","ref":"SnmpKit.SnmpLib.Security.Keys.html#validate_password_strength/1-requirements"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Keys.validate_password_strength/1","doc":":ok = SnmpKit.SnmpLib.Security.Keys.validate_password_strength(\"strong_password_123\")\n    {:error, :too_short} = SnmpKit.SnmpLib.Security.Keys.validate_password_strength(\"weak\")\n    {:warning, :weak_complexity} = SnmpKit.SnmpLib.Security.Keys.validate_password_strength(\"password\")","ref":"SnmpKit.SnmpLib.Security.Keys.html#validate_password_strength/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Security.Priv","doc":"Privacy (encryption) protocols for SNMPv3 User Security Model.\n\nImplements encryption protocols as specified in RFC 3414 and RFC 3826,\nproviding message confidentiality for SNMPv3 communications.","ref":"SnmpKit.SnmpLib.Security.Priv.html"},{"type":"module","title":"Supported Protocols - SnmpKit.SnmpLib.Security.Priv","doc":"- **DES-CBC** (RFC 3414) - 56-bit key, legacy support\n- **AES-128** (RFC 3826) - 128-bit key, good security\n- **AES-192** (RFC 3826) - 192-bit key, enhanced security\n- **AES-256** (RFC 3826) - 256-bit key, maximum security","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-supported-protocols"},{"type":"module","title":"Security Considerations - SnmpKit.SnmpLib.Security.Priv","doc":"- DES is deprecated and should only be used for legacy compatibility\n- AES-128 provides adequate security for most applications\n- AES-256 is recommended for high-security environments\n- All encryption uses CBC mode with random initialization vectors\n- Privacy requires authentication (cannot use privacy without authentication)","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-security-considerations"},{"type":"module","title":"Protocol Selection Guidelines - SnmpKit.SnmpLib.Security.Priv","doc":"- **AES-256**: Recommended for high-security environments\n- **AES-128**: Good balance of security and performance for most deployments\n- **DES**: Legacy compatibility only, not recommended for new deployments","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-protocol-selection-guidelines"},{"type":"module","title":"Technical Details - SnmpKit.SnmpLib.Security.Priv","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-technical-details"},{"type":"module","title":"Key Derivation - SnmpKit.SnmpLib.Security.Priv","doc":"Privacy keys are derived from privacy passwords using the same engine ID\nand algorithm as authentication keys, but with different key usage.","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-key-derivation"},{"type":"module","title":"Initialization Vectors - SnmpKit.SnmpLib.Security.Priv","doc":"Each encryption operation uses a unique initialization vector (IV) to ensure\nthat identical plaintexts produce different ciphertexts.","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-initialization-vectors"},{"type":"module","title":"Padding - SnmpKit.SnmpLib.Security.Priv","doc":"Block ciphers use PKCS#7 padding to handle messages that don't align\nwith block boundaries.","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-padding"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Security.Priv","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-usage-examples"},{"type":"module","title":"Message Encryption - SnmpKit.SnmpLib.Security.Priv","doc":"# Encrypt message with AES-256\n    priv_key = derived_privacy_key\n    auth_key = derived_authentication_key  # Required for IV generation\n    plaintext = \"confidential SNMP data\"\n\n    {:ok, {ciphertext, priv_params}} = SnmpKit.SnmpLib.Security.Priv.encrypt(\n      :aes256, priv_key, auth_key, plaintext\n    )\n\n    # Decrypt message\n    {:ok, decrypted} = SnmpKit.SnmpLib.Security.Priv.decrypt(\n      :aes256, priv_key, auth_key, ciphertext, priv_params\n    )","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-message-encryption"},{"type":"module","title":"Protocol Information - SnmpKit.SnmpLib.Security.Priv","doc":"# Get encryption protocol details\n    info = SnmpKit.SnmpLib.Security.Priv.protocol_info(:aes256)\n    # Returns: %{algorithm: :aes_256_cbc, key_size: 32, block_size: 16, ...}","ref":"SnmpKit.SnmpLib.Security.Priv.html#module-protocol-information"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.auth_key/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:auth_key/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.benchmark_protocol/5","doc":"Measures encryption/decryption performance for a given protocol.","ref":"SnmpKit.SnmpLib.Security.Priv.html#benchmark_protocol/5"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.ciphertext/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:ciphertext/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.decrypt/5","doc":"Decrypts ciphertext using the specified privacy protocol.","ref":"SnmpKit.SnmpLib.Security.Priv.html#decrypt/5"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Priv.decrypt/5","doc":"- `protocol`: Privacy protocol used for encryption\n- `priv_key`: Privacy key (same as used for encryption)\n- `auth_key`: Authentication key (used for IV validation)\n- `ciphertext`: Encrypted data\n- `priv_params`: Privacy parameters from encryption (contains IV)","ref":"SnmpKit.SnmpLib.Security.Priv.html#decrypt/5-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Priv.decrypt/5","doc":"- `{:ok, plaintext}`: Decryption successful\n- `{:error, reason}`: Decryption failed","ref":"SnmpKit.SnmpLib.Security.Priv.html#decrypt/5-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Priv.decrypt/5","doc":"# AES-256 decryption\n    {:ok, plaintext} = SnmpKit.SnmpLib.Security.Priv.decrypt(\n      :aes256, priv_key, auth_key, ciphertext, priv_params\n    )\n\n    # Handle decryption errors\n    case SnmpKit.SnmpLib.Security.Priv.decrypt(:des, priv_key, auth_key, ciphertext, priv_params) do\n      {:ok, plaintext} -> process_plaintext(plaintext)\n      {:error, :decryption_failed} -> handle_corruption()\n      {:error, :invalid_padding} -> handle_padding_error()\n    end","ref":"SnmpKit.SnmpLib.Security.Priv.html#decrypt/5-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.decrypt_batch/4","doc":"Decrypts multiple ciphertexts in batch.","ref":"SnmpKit.SnmpLib.Security.Priv.html#decrypt_batch/4"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.encrypt/4","doc":"Encrypts plaintext using the specified privacy protocol.","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.Priv.encrypt/4","doc":"- `protocol`: Privacy protocol to use (:des, :aes128, :aes192, :aes256)\n- `priv_key`: Privacy key (must be correct length for protocol)\n- `auth_key`: Authentication key (used for IV generation in some protocols)\n- `plaintext`: Data to encrypt","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.Priv.encrypt/4","doc":"- `{:ok, {ciphertext, priv_params}}`: Encryption successful\n- `{:error, reason}`: Encryption failed","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Priv.encrypt/4","doc":"# AES-256 encryption (recommended)\n    {:ok, {ciphertext, priv_params}} = SnmpKit.SnmpLib.Security.Priv.encrypt(\n      :aes256, priv_key, auth_key, \"secret data\"\n    )\n\n    # DES encryption (legacy)\n    {:ok, {ciphertext, priv_params}} = SnmpKit.SnmpLib.Security.Priv.encrypt(\n      :des, priv_key, auth_key, \"legacy data\"\n    )","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.encrypt_batch/4","doc":"Encrypts multiple plaintexts using the same protocol and key.\n\nEach plaintext gets a unique IV, ensuring security even for identical plaintexts.","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt_batch/4"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Priv.encrypt_batch/4","doc":"plaintexts = [\"data1\", \"data2\", \"data3\"]\n    {:ok, encrypted_list} = SnmpKit.SnmpLib.Security.Priv.encrypt_batch(:aes256, priv_key, auth_key, plaintexts)\n    # Returns list of {ciphertext, priv_params} tuples","ref":"SnmpKit.SnmpLib.Security.Priv.html#encrypt_batch/4-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.initialization_vector/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:initialization_vector/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.plaintext/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:plaintext/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.priv_key/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:priv_key/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.priv_params/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:priv_params/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.Priv.priv_protocol/0","doc":"","ref":"SnmpKit.SnmpLib.Security.Priv.html#t:priv_protocol/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.protocol_info/1","doc":"Returns information about a specific privacy protocol.","ref":"SnmpKit.SnmpLib.Security.Priv.html#protocol_info/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Priv.protocol_info/1","doc":"iex> SnmpKit.SnmpLib.Security.Priv.protocol_info(:aes256)\n    %{algorithm: :aes_256_cbc, key_size: 32, block_size: 16, iv_size: 16, secure: true, rfc: \"RFC 3826\"}\n\n    iex> SnmpKit.SnmpLib.Security.Priv.protocol_info(:des)\n    %{algorithm: :des_cbc, key_size: 8, block_size: 8, iv_size: 8, secure: false, rfc: \"RFC 3414\"}","ref":"SnmpKit.SnmpLib.Security.Priv.html#protocol_info/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.secure_protocol?/1","doc":"Checks if a protocol is considered cryptographically secure.","ref":"SnmpKit.SnmpLib.Security.Priv.html#secure_protocol?/1"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.secure_protocols/0","doc":"Returns list of cryptographically secure protocols (excludes deprecated ones).","ref":"SnmpKit.SnmpLib.Security.Priv.html#secure_protocols/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.supported_protocols/0","doc":"Returns list of all supported privacy protocols.","ref":"SnmpKit.SnmpLib.Security.Priv.html#supported_protocols/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.Priv.validate_key/2","doc":"Validates that a privacy key is appropriate for the specified protocol.","ref":"SnmpKit.SnmpLib.Security.Priv.html#validate_key/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.Priv.validate_key/2","doc":":ok = SnmpKit.SnmpLib.Security.Priv.validate_key(:aes256, key_32_bytes)\n    {:error, :key_wrong_size} = SnmpKit.SnmpLib.Security.Priv.validate_key(:aes128, key_32_bytes)","ref":"SnmpKit.SnmpLib.Security.Priv.html#validate_key/2-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Security.USM","doc":"User Security Model (USM) implementation for SNMPv3 - RFC 3414 compliant.\n\nThe User Security Model provides the foundation for SNMPv3 security by implementing:\n\n- **User-based authentication** with multiple protocols\n- **Privacy (encryption)** for message confidentiality\n- **Time synchronization** to prevent replay attacks\n- **Engine discovery** for secure agent communication\n- **Security parameter validation** and error handling","ref":"SnmpKit.SnmpLib.Security.USM.html"},{"type":"module","title":"RFC 3414 Compliance - SnmpKit.SnmpLib.Security.USM","doc":"This implementation fully complies with RFC 3414 \"User-based Security Model (USM)\nfor version 3 of the Simple Network Management Protocol (SNMPv3)\" including:\n\n- Message authentication using HMAC-MD5 and HMAC-SHA\n- Privacy using DES and AES encryption\n- Key derivation using password localization\n- Time window validation for message freshness\n- Engine ID discovery and management","ref":"SnmpKit.SnmpLib.Security.USM.html#module-rfc-3414-compliance"},{"type":"module","title":"Architecture - SnmpKit.SnmpLib.Security.USM","doc":"The USM coordinates with other security modules:\n\n```\nSnmpKit.SnmpLib.Security.USM\n├── Auth protocols (MD5, SHA variants)\n├── Priv protocols (DES, AES variants)\n├── Key derivation and management\n└── Engine and time management\n```","ref":"SnmpKit.SnmpLib.Security.USM.html#module-architecture"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Security.USM","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#module-usage-examples"},{"type":"module","title":"Engine Discovery - SnmpKit.SnmpLib.Security.USM","doc":"# Discover remote engine for secure communication\n    {:ok, engine_id} = SnmpKit.SnmpLib.Security.USM.discover_engine(\"192.168.1.1\")\n\n    # Time synchronization\n    {:ok, {boots, time}} = SnmpKit.SnmpLib.Security.USM.synchronize_time(\"192.168.1.1\", engine_id)","ref":"SnmpKit.SnmpLib.Security.USM.html#module-engine-discovery"},{"type":"module","title":"Message Processing - SnmpKit.SnmpLib.Security.USM","doc":"# Process outgoing secure message\n    {:ok, secure_message} = SnmpKit.SnmpLib.Security.USM.process_outgoing_message(\n      user, message, security_level\n    )\n\n    # Process incoming secure message\n    {:ok, {plain_message, user}} = SnmpKit.SnmpLib.Security.USM.process_incoming_message(\n      secure_message, user_database\n    )","ref":"SnmpKit.SnmpLib.Security.USM.html#module-message-processing"},{"type":"module","title":"Security Considerations - SnmpKit.SnmpLib.Security.USM","doc":"- Engine boot counters must be persistent across restarts\n- Time synchronization is critical for security\n- Failed authentication attempts should be logged\n- Key material should never be logged or persisted in plain text","ref":"SnmpKit.SnmpLib.Security.USM.html#module-security-considerations"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.discover_engine/2","doc":"Discovers the engine ID of a remote SNMP agent.\n\nEngine discovery is the first step in establishing secure communication\nwith a remote SNMPv3 agent. This function sends a discovery request and\nretrieves the agent's authoritative engine ID.","ref":"SnmpKit.SnmpLib.Security.USM.html#discover_engine/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Security.USM.discover_engine/2","doc":"- `host`: Target agent IP address or hostname\n- `opts`: Discovery options including port, timeout, and community","ref":"SnmpKit.SnmpLib.Security.USM.html#discover_engine/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Security.USM.discover_engine/2","doc":"- `{:ok, engine_id}`: Successfully discovered engine ID\n- `{:error, reason}`: Discovery failed","ref":"SnmpKit.SnmpLib.Security.USM.html#discover_engine/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Security.USM.discover_engine/2","doc":"{:ok, engine_id} = SnmpKit.SnmpLib.Security.USM.discover_engine(\"192.168.1.1\")\n    {:ok, engine_id} = SnmpKit.SnmpLib.Security.USM.discover_engine(\"10.0.0.1\", port: 1161, timeout: 5000)","ref":"SnmpKit.SnmpLib.Security.USM.html#discover_engine/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.engine_boots/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:engine_boots/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.engine_id/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:engine_id/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.engine_time/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:engine_time/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.generate_error_report/2","doc":"Generates security error reports for invalid messages.\n\nUSM error reports are sent back to the originator to indicate\nsecurity violations or configuration issues.","ref":"SnmpKit.SnmpLib.Security.USM.html#generate_error_report/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.get_engine_time/1","doc":"Calculates current engine time since boot.","ref":"SnmpKit.SnmpLib.Security.USM.html#get_engine_time/1"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.increment_engine_boots/1","doc":"Updates engine boot counter, handling rollover at maximum value.","ref":"SnmpKit.SnmpLib.Security.USM.html#increment_engine_boots/1"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.message_flags/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:message_flags/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.process_incoming_message/2","doc":"Processes an incoming SNMP message with USM security.\n\nThis function validates and decrypts an incoming secure message, returning\nthe plain message content and validated user information.","ref":"SnmpKit.SnmpLib.Security.USM.html#process_incoming_message/2"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.process_outgoing_message/3","doc":"Processes an outgoing SNMP message with USM security.\n\nThis function applies authentication and/or privacy protection to an outgoing\nmessage based on the user's security level configuration.","ref":"SnmpKit.SnmpLib.Security.USM.html#process_outgoing_message/3"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.security_level/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:security_level/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.security_name/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:security_name/0"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.security_parameters/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:security_parameters/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.synchronize_time/3","doc":"Synchronizes time with a remote SNMP agent.\n\nTime synchronization is required for authenticated communication to prevent\nreplay attacks. This function retrieves the agent's current boot counter\nand engine time.","ref":"SnmpKit.SnmpLib.Security.USM.html#synchronize_time/3"},{"type":"type","title":"SnmpKit.SnmpLib.Security.USM.user_entry/0","doc":"","ref":"SnmpKit.SnmpLib.Security.USM.html#t:user_entry/0"},{"type":"function","title":"SnmpKit.SnmpLib.Security.USM.validate_time_window/4","doc":"Validates time-based security parameters to prevent replay attacks.\n\nPer RFC 3414, messages are considered fresh if:\n- Engine boots match (within 1)\n- Engine time is within 150 seconds","ref":"SnmpKit.SnmpLib.Security.USM.html#validate_time_window/4"},{"type":"module","title":"SnmpKit.SnmpLib.Transport","doc":"UDP transport layer for SNMP communications.\n\nProvides socket management, connection utilities, and network operations\nfor both SNMP managers and agents/simulators.","ref":"SnmpKit.SnmpLib.Transport.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Transport","doc":"- UDP socket creation and management\n- Address resolution and validation\n- Connection pooling and reuse\n- Timeout handling\n- Error recovery\n- Performance optimizations","ref":"SnmpKit.SnmpLib.Transport.html#module-features"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.Transport","doc":"# Create and use a socket\n    {:ok, socket} = SnmpKit.SnmpLib.Transport.create_socket(\"0.0.0.0\", 161)\n    {:ok, data} = SnmpKit.SnmpLib.Transport.receive_packet(socket, 5000)\n    :ok = SnmpKit.SnmpLib.Transport.send_packet(socket, \"192.168.1.100\", 161, packet_data)\n    :ok = SnmpKit.SnmpLib.Transport.close_socket(socket)\n\n    # Address utilities\n    {:ok, {192, 168, 1, 100}} = SnmpKit.SnmpLib.Transport.resolve_address(\"192.168.1.100\")\n    true = SnmpKit.SnmpLib.Transport.validate_port(161)","ref":"SnmpKit.SnmpLib.Transport.html#module-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Transport.address/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#t:address/0"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.close_socket/1","doc":"Closes a UDP socket.","ref":"SnmpKit.SnmpLib.Transport.html#close_socket/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.close_socket/1","doc":":ok = SnmpKit.SnmpLib.Transport.close_socket(socket)","ref":"SnmpKit.SnmpLib.Transport.html#close_socket/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.create_client_socket/1","doc":"Creates a client socket for outgoing SNMP requests.\n\nUses an ephemeral port and optimizes settings for client use.","ref":"SnmpKit.SnmpLib.Transport.html#create_client_socket/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.create_client_socket/1","doc":"{:ok, socket} = SnmpKit.SnmpLib.Transport.create_client_socket()\n    {:ok, socket} = SnmpKit.SnmpLib.Transport.create_client_socket([{:recbuf, 65536}])","ref":"SnmpKit.SnmpLib.Transport.html#create_client_socket/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.create_server_socket/2","doc":"Creates a server socket for incoming SNMP requests.\n\nOptimizes settings for server use with proper buffer sizes.","ref":"SnmpKit.SnmpLib.Transport.html#create_server_socket/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.create_server_socket/2","doc":"{:ok, socket} = SnmpKit.SnmpLib.Transport.create_server_socket(161)\n    {:ok, socket} = SnmpKit.SnmpLib.Transport.create_server_socket(161, \"192.168.1.10\")","ref":"SnmpKit.SnmpLib.Transport.html#create_server_socket/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.create_socket/3","doc":"Creates a UDP socket bound to the specified address and port.","ref":"SnmpKit.SnmpLib.Transport.html#create_socket/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.create_socket/3","doc":"- `bind_address`: Address to bind to (use \"0.0.0.0\" for all interfaces)\n- `port`: Port number to bind to\n- `options`: Additional socket options (optional)","ref":"SnmpKit.SnmpLib.Transport.html#create_socket/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Transport.create_socket/3","doc":"- `{:ok, socket}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.Transport.html#create_socket/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.create_socket/3","doc":"{:ok, socket} = SnmpKit.SnmpLib.Transport.create_socket(\"0.0.0.0\", 161)\n    {:ok, client_socket} = SnmpKit.SnmpLib.Transport.create_socket(\"0.0.0.0\", 0, [{:active, true}])","ref":"SnmpKit.SnmpLib.Transport.html#create_socket/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.format_endpoint/2","doc":"Formats an endpoint (address and port) as a string.","ref":"SnmpKit.SnmpLib.Transport.html#format_endpoint/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.format_endpoint/2","doc":"\"192.168.1.100:161\" = SnmpKit.SnmpLib.Transport.format_endpoint({192, 168, 1, 100}, 161)\n    \"localhost:162\" = SnmpKit.SnmpLib.Transport.format_endpoint(\"localhost\", 162)","ref":"SnmpKit.SnmpLib.Transport.html#format_endpoint/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.get_socket_address/1","doc":"Gets the local address and port of a socket.","ref":"SnmpKit.SnmpLib.Transport.html#get_socket_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.get_socket_address/1","doc":"{:ok, {{127, 0, 0, 1}, 12345}} = SnmpKit.SnmpLib.Transport.get_socket_address(socket)","ref":"SnmpKit.SnmpLib.Transport.html#get_socket_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.get_socket_stats/1","doc":"Gets socket statistics and information.","ref":"SnmpKit.SnmpLib.Transport.html#get_socket_stats/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.get_socket_stats/1","doc":"{:ok, stats} = SnmpKit.SnmpLib.Transport.get_socket_stats(socket)\n    # stats contains buffer sizes, packet counts, etc.","ref":"SnmpKit.SnmpLib.Transport.html#get_socket_stats/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.is_snmp_port?/1","doc":"Checks if a port number is a standard SNMP port.","ref":"SnmpKit.SnmpLib.Transport.html#is_snmp_port?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.is_snmp_port?/1","doc":"true = SnmpKit.SnmpLib.Transport.is_snmp_port?(161)\n    true = SnmpKit.SnmpLib.Transport.is_snmp_port?(162)\n    false = SnmpKit.SnmpLib.Transport.is_snmp_port?(80)","ref":"SnmpKit.SnmpLib.Transport.html#is_snmp_port?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.max_snmp_payload_size/0","doc":"Calculates network MTU considerations for SNMP packets.\n\nReturns recommended maximum payload size to avoid fragmentation.","ref":"SnmpKit.SnmpLib.Transport.html#max_snmp_payload_size/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.max_snmp_payload_size/0","doc":"1472 = SnmpKit.SnmpLib.Transport.max_snmp_payload_size()  # Ethernet MTU - headers","ref":"SnmpKit.SnmpLib.Transport.html#max_snmp_payload_size/0-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Transport.packet_data/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#t:packet_data/0"},{"type":"type","title":"SnmpKit.SnmpLib.Transport.port_number/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#t:port_number/0"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.receive_packet/2","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#receive_packet/2"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.receive_packet_filtered/4","doc":"Receives a packet with a custom filter function.\n\nContinues receiving until a packet matches the filter or timeout occurs.","ref":"SnmpKit.SnmpLib.Transport.html#receive_packet_filtered/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.receive_packet_filtered/4","doc":"- `socket`: UDP socket to receive from\n- `filter_fn`: Function that returns true for desired packets\n- `timeout`: Total timeout in milliseconds\n- `per_recv_timeout`: Timeout per receive attempt (default: 1000ms)","ref":"SnmpKit.SnmpLib.Transport.html#receive_packet_filtered/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.receive_packet_filtered/4","doc":"# Wait for packet from specific address\n    filter_fn = fn {_data, from_addr, _from_port} -> from_addr == {192, 168, 1, 100} end\n    {:ok, {data, addr, port}} = SnmpKit.SnmpLib.Transport.receive_packet_filtered(socket, filter_fn, 5000)","ref":"SnmpKit.SnmpLib.Transport.html#receive_packet_filtered/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.resolve_address/1","doc":"Resolves an address to an IP tuple.\n\nAccepts:\n- IP address strings (e.g., \"192.168.1.1\")\n- Hostnames (e.g., \"localhost\")\n- IP tuples (e.g., {192, 168, 1, 1})","ref":"SnmpKit.SnmpLib.Transport.html#resolve_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.resolve_address/1","doc":"iex> SnmpKit.SnmpLib.Transport.resolve_address(\"192.168.1.1\")\n    {:ok, {192, 168, 1, 1}}\n\n    iex> SnmpKit.SnmpLib.Transport.resolve_address({192, 168, 1, 1})\n    {:ok, {192, 168, 1, 1}}","ref":"SnmpKit.SnmpLib.Transport.html#resolve_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.send_and_receive_packet/4","doc":"Receives a packet from the socket with optional timeout.","ref":"SnmpKit.SnmpLib.Transport.html#send_and_receive_packet/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.send_and_receive_packet/4","doc":"- `socket`: UDP socket to receive from\n- `timeout`: Timeout in milliseconds (default: 5000)","ref":"SnmpKit.SnmpLib.Transport.html#send_and_receive_packet/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Transport.send_and_receive_packet/4","doc":"- `{:ok, {data, from_address, from_port}}` on success\n- `{:error, reason}` on failure or timeout","ref":"SnmpKit.SnmpLib.Transport.html#send_and_receive_packet/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.send_and_receive_packet/4","doc":"{:ok, {data, from_ip, from_port}} = SnmpKit.SnmpLib.Transport.receive_packet(socket)\n    {:ok, {data, from_ip, from_port}} = SnmpKit.SnmpLib.Transport.receive_packet(socket, 10000)","ref":"SnmpKit.SnmpLib.Transport.html#send_and_receive_packet/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.send_packet/4","doc":"Sends a packet to the specified destination.","ref":"SnmpKit.SnmpLib.Transport.html#send_packet/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.send_packet/4","doc":"- `socket`: UDP socket to send from\n- `dest_address`: Destination IP address or hostname\n- `dest_port`: Destination port number\n- `data`: Binary data to send","ref":"SnmpKit.SnmpLib.Transport.html#send_packet/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Transport.send_packet/4","doc":"- `:ok` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.Transport.html#send_packet/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.send_packet/4","doc":":ok = SnmpKit.SnmpLib.Transport.send_packet(socket, \"192.168.1.100\", 161, packet_data)\n    :ok = SnmpKit.SnmpLib.Transport.send_packet(socket, {192, 168, 1, 100}, 161, packet_data)","ref":"SnmpKit.SnmpLib.Transport.html#send_packet/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.send_request/4","doc":"Sends a request packet and waits for a response.\n\nThis creates a temporary socket, sends a packet, waits for the response,\nand returns the response data. Useful for SNMPv3 discovery and security operations.","ref":"SnmpKit.SnmpLib.Transport.html#send_request/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.send_request/4","doc":"- `dest_address`: Destination IP address or hostname\n- `dest_port`: Destination port number\n- `request_data`: Binary request data to send\n- `timeout`: Timeout in milliseconds (default: 5000)","ref":"SnmpKit.SnmpLib.Transport.html#send_request/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Transport.send_request/4","doc":"- `{:ok, response_data}` if request succeeds and response received\n- `{:error, reason}` if request fails or times out","ref":"SnmpKit.SnmpLib.Transport.html#send_request/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.send_request/4","doc":"{:ok, response} = SnmpKit.SnmpLib.Transport.send_request(\"192.168.1.100\", 161, request_packet, 5000)\n    {:error, :timeout} = SnmpKit.SnmpLib.Transport.send_request(\"10.0.0.1\", 161, request_packet, 1000)","ref":"SnmpKit.SnmpLib.Transport.html#send_request/4-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.snmp_agent_port/0","doc":"Returns standard SNMP port numbers.","ref":"SnmpKit.SnmpLib.Transport.html#snmp_agent_port/0"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.snmp_trap_port/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#snmp_trap_port/0"},{"type":"type","title":"SnmpKit.SnmpLib.Transport.socket/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#t:socket/0"},{"type":"type","title":"SnmpKit.SnmpLib.Transport.socket_options/0","doc":"","ref":"SnmpKit.SnmpLib.Transport.html#t:socket_options/0"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.test_connectivity/3","doc":"Tests connectivity to a destination by sending a test packet.\n\nThis creates a temporary socket, sends a small packet, and waits for any response\nto verify network connectivity.","ref":"SnmpKit.SnmpLib.Transport.html#test_connectivity/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Transport.test_connectivity/3","doc":"- `dest_address`: Destination IP address or hostname\n- `dest_port`: Destination port number\n- `timeout`: Timeout in milliseconds (default: 3000)","ref":"SnmpKit.SnmpLib.Transport.html#test_connectivity/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Transport.test_connectivity/3","doc":"- `:ok` if connectivity is confirmed\n- `{:error, reason}` if connectivity fails","ref":"SnmpKit.SnmpLib.Transport.html#test_connectivity/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.test_connectivity/3","doc":":ok = SnmpKit.SnmpLib.Transport.test_connectivity(\"192.168.1.100\", 161)\n    {:error, :timeout} = SnmpKit.SnmpLib.Transport.test_connectivity(\"10.0.0.1\", 161, 1000)","ref":"SnmpKit.SnmpLib.Transport.html#test_connectivity/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.valid_packet_size?/1","doc":"Validates if a packet size is suitable for SNMP transmission.","ref":"SnmpKit.SnmpLib.Transport.html#valid_packet_size?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.valid_packet_size?/1","doc":"true = SnmpKit.SnmpLib.Transport.valid_packet_size?(500)\n    false = SnmpKit.SnmpLib.Transport.valid_packet_size?(2000)","ref":"SnmpKit.SnmpLib.Transport.html#valid_packet_size?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Transport.validate_port/1","doc":"Validates a port number.","ref":"SnmpKit.SnmpLib.Transport.html#validate_port/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Transport.validate_port/1","doc":"true = SnmpKit.SnmpLib.Transport.validate_port(161)\n    true = SnmpKit.SnmpLib.Transport.validate_port(65535)\n    true = SnmpKit.SnmpLib.Transport.validate_port(0)\n    false = SnmpKit.SnmpLib.Transport.validate_port(65536)","ref":"SnmpKit.SnmpLib.Transport.html#validate_port/1-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Utils","doc":"Common utilities for SNMP operations including pretty printing, data formatting,\ntiming utilities, and validation functions.\n\nThis module provides helpful utilities for debugging, logging, monitoring, and\ngeneral SNMP data manipulation that are commonly needed across SNMP applications.","ref":"SnmpKit.SnmpLib.Utils.html"},{"type":"module","title":"Pretty Printing - SnmpKit.SnmpLib.Utils","doc":"Format SNMP data structures for human-readable display in logs, CLI tools,\nand debugging output.","ref":"SnmpKit.SnmpLib.Utils.html#module-pretty-printing"},{"type":"module","title":"Data Formatting - SnmpKit.SnmpLib.Utils","doc":"Convert between different representations of SNMP data, format numeric values,\nand handle common data transformations.","ref":"SnmpKit.SnmpLib.Utils.html#module-data-formatting"},{"type":"module","title":"Timing Utilities - SnmpKit.SnmpLib.Utils","doc":"Measure and format timing information for SNMP operations, useful for\nperformance monitoring and debugging.","ref":"SnmpKit.SnmpLib.Utils.html#module-timing-utilities"},{"type":"module","title":"Validation - SnmpKit.SnmpLib.Utils","doc":"Common validation functions for SNMP-related data.","ref":"SnmpKit.SnmpLib.Utils.html#module-validation"},{"type":"module","title":"Usage Examples - SnmpKit.SnmpLib.Utils","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#module-usage-examples"},{"type":"module","title":"Pretty Printing - SnmpKit.SnmpLib.Utils","doc":"# Format PDU for logging\n    pdu = %{type: :get_request, request_id: 123, varbinds: [...]}\n    Logger.info(SnmpKit.SnmpLib.Utils.pretty_print_pdu(pdu))\n\n    # Format individual values\n    value = {:counter32, 12345}\n    IO.puts(SnmpKit.SnmpLib.Utils.pretty_print_value(value))","ref":"SnmpKit.SnmpLib.Utils.html#module-pretty-printing-1"},{"type":"module","title":"Data Formatting - SnmpKit.SnmpLib.Utils","doc":"# Format large numbers with separators\n    SnmpKit.SnmpLib.Utils.format_bytes(1048576)\n    # => \"1.0 MB\"\n\n    # Format hex strings for MAC addresses\n    SnmpKit.SnmpLib.Utils.format_hex(<<0x00, 0x1B, 0x21, 0x3C, 0x92, 0xEB>>)\n    # => \"00:1B:21:3C:92:EB\"","ref":"SnmpKit.SnmpLib.Utils.html#module-data-formatting-1"},{"type":"module","title":"Timing - SnmpKit.SnmpLib.Utils","doc":"# Time an operation\n    {result, time_us} = SnmpKit.SnmpLib.Utils.measure_request_time(fn ->\n      SnmpKit.SnmpLib.PDU.encode_message(pdu)\n    end)\n\n    formatted_time = SnmpKit.SnmpLib.Utils.format_response_time(time_us)","ref":"SnmpKit.SnmpLib.Utils.html#module-timing"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.format_bytes/1","doc":"Formats byte counts in human-readable units.","ref":"SnmpKit.SnmpLib.Utils.html#format_bytes/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.format_bytes/1","doc":"iex> SnmpKit.SnmpLib.Utils.format_bytes(1024)\n    \"1.0 KB\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_bytes(1048576)\n    \"1.0 MB\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_bytes(512)\n    \"512 B\"","ref":"SnmpKit.SnmpLib.Utils.html#format_bytes/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.format_hex/2","doc":"Formats binary data as hexadecimal string.","ref":"SnmpKit.SnmpLib.Utils.html#format_hex/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Utils.format_hex/2","doc":"- `data`: Binary data to format\n- `separator`: String to use between hex bytes (default: \":\")","ref":"SnmpKit.SnmpLib.Utils.html#format_hex/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.format_hex/2","doc":"iex> SnmpKit.SnmpLib.Utils.format_hex(<<0x00, 0x1B, 0x21>>)\n    \"00:1B:21\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_hex(<<0xDE, 0xAD, 0xBE, 0xEF>>, \" \")\n    \"DE AD BE EF\"","ref":"SnmpKit.SnmpLib.Utils.html#format_hex/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.format_number/1","doc":"Formats large numbers with thousand separators.","ref":"SnmpKit.SnmpLib.Utils.html#format_number/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.format_number/1","doc":"iex> SnmpKit.SnmpLib.Utils.format_number(1234567)\n    \"1,234,567\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_number(42)\n    \"42\"","ref":"SnmpKit.SnmpLib.Utils.html#format_number/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.format_rate/2","doc":"Formats rates with units.","ref":"SnmpKit.SnmpLib.Utils.html#format_rate/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.format_rate/2","doc":"iex> SnmpKit.SnmpLib.Utils.format_rate(1500, \"bps\")\n    \"1.5 Kbps\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_rate(45, \"pps\")\n    \"45 pps\"","ref":"SnmpKit.SnmpLib.Utils.html#format_rate/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.format_response_time/1","doc":"Formats response time in human-readable units.","ref":"SnmpKit.SnmpLib.Utils.html#format_response_time/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Utils.format_response_time/1","doc":"- `microseconds`: Time in microseconds","ref":"SnmpKit.SnmpLib.Utils.html#format_response_time/1-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.format_response_time/1","doc":"iex> SnmpKit.SnmpLib.Utils.format_response_time(1500)\n    \"1.50ms\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_response_time(2_500_000)\n    \"2.50s\"\n\n    iex> SnmpKit.SnmpLib.Utils.format_response_time(500)\n    \"500μs\"","ref":"SnmpKit.SnmpLib.Utils.html#format_response_time/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.measure_request_time/1","doc":"Measures the execution time of a function in microseconds.","ref":"SnmpKit.SnmpLib.Utils.html#measure_request_time/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Utils.measure_request_time/1","doc":"- `fun`: Function to execute and time","ref":"SnmpKit.SnmpLib.Utils.html#measure_request_time/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Utils.measure_request_time/1","doc":"Tuple of `{result, time_microseconds}` where result is the function's\nreturn value and time_microseconds is the execution time.","ref":"SnmpKit.SnmpLib.Utils.html#measure_request_time/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.measure_request_time/1","doc":"iex> {result, time} = SnmpKit.SnmpLib.Utils.measure_request_time(fn -> :timer.sleep(100); :ok end)\n    iex> result\n    :ok\n    iex> time > 100_000\n    true","ref":"SnmpKit.SnmpLib.Utils.html#measure_request_time/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Utils.oid/0","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#t:oid/0"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.parse_target/1","doc":"Parses SNMP target specifications into standardized format.\n\nAccepts various input formats and returns a consistent target map with host and port.\nIP addresses are resolved to tuples when possible, hostnames remain as strings.\nDefault port is 161 when not specified.","ref":"SnmpKit.SnmpLib.Utils.html#parse_target/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Utils.parse_target/1","doc":"- `target`: Target specification in various formats","ref":"SnmpKit.SnmpLib.Utils.html#parse_target/1-parameters"},{"type":"function","title":"Accepted Input Formats - SnmpKit.SnmpLib.Utils.parse_target/1","doc":"- `\"192.168.1.1:161\"` - IP with port\n- `\"192.168.1.1\"` - IP without port (uses default 161)\n- `\"device.local:162\"` - hostname with port\n- `\"device.local\"` - hostname without port (uses default 161)\n- `{192, 168, 1, 1}` - IP tuple (uses default port 161)\n- `%{host: \"192.168.1.1\", port: 161}` - already parsed map","ref":"SnmpKit.SnmpLib.Utils.html#parse_target/1-accepted-input-formats"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Utils.parse_target/1","doc":"- `{:ok, %{host: host, port: port}}` - Successfully parsed target\n- `{:error, reason}` - Parse error with reason","ref":"SnmpKit.SnmpLib.Utils.html#parse_target/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.parse_target/1","doc":"iex> SnmpKit.SnmpLib.Utils.parse_target(\"192.168.1.1:161\")\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target(\"192.168.1.1\")\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target(\"device.local:162\")\n    {:ok, %{host: \"device.local\", port: 162}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target(\"device.local\")\n    {:ok, %{host: \"device.local\", port: 161}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target({192, 168, 1, 1})\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target(%{host: \"192.168.1.1\", port: 161})\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}\n\n    iex> SnmpKit.SnmpLib.Utils.parse_target(\"invalid:99999\")\n    {:error, {:invalid_port, \"99999\"}}","ref":"SnmpKit.SnmpLib.Utils.html#parse_target/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Utils.pdu/0","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#t:pdu/0"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.pretty_print_oid/1","doc":"Pretty prints an OID for display.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_oid/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.pretty_print_oid/1","doc":"iex> SnmpKit.SnmpLib.Utils.pretty_print_oid([1,3,6,1,2,1,1,1,0])\n    \"1.3.6.1.2.1.1.1.0\"\n\n    iex> SnmpKit.SnmpLib.Utils.pretty_print_oid(\"1.3.6.1.2.1.1.1.0\")\n    \"1.3.6.1.2.1.1.1.0\"","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_oid/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.pretty_print_pdu/1","doc":"Pretty prints an SNMP PDU for human-readable display.\n\nFormats the PDU structure with proper indentation and readable field names,\nsuitable for logging, debugging, or CLI display.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_pdu/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Utils.pretty_print_pdu/1","doc":"- `pdu`: SNMP PDU map containing type, request_id, error_status, etc.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_pdu/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Utils.pretty_print_pdu/1","doc":"Formatted string representation of the PDU.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_pdu/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.pretty_print_pdu/1","doc":"iex> pdu = %{type: :get_request, request_id: 123, varbinds: [{[1,3,6,1,2,1,1,1,0], :null}]}\n    iex> result = SnmpKit.SnmpLib.Utils.pretty_print_pdu(pdu)\n    iex> String.contains?(result, \"GET Request\")\n    true","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_pdu/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.pretty_print_value/1","doc":"Pretty prints an SNMP value for display.\n\nFormats SNMP values with appropriate type information and human-readable\nrepresentations.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_value/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.pretty_print_value/1","doc":"iex> SnmpKit.SnmpLib.Utils.pretty_print_value({:counter32, 12345})\n    \"Counter32: 12,345\"\n\n    iex> SnmpKit.SnmpLib.Utils.pretty_print_value({:octet_string, \"Hello\"})\n    \"OCTET STRING: \\\"Hello\\\"\"\n\n    iex> SnmpKit.SnmpLib.Utils.pretty_print_value(:null)\n    \"NULL\"","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_value/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.pretty_print_varbinds/1","doc":"Pretty prints a list of varbinds for display.","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_varbinds/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.pretty_print_varbinds/1","doc":"iex> varbinds = [{[1,3,6,1,2,1,1,1,0], \"Linux server\"}]\n    iex> result = SnmpKit.SnmpLib.Utils.pretty_print_varbinds(varbinds)\n    iex> String.contains?(result, \"1.3.6.1.2.1.1.1.0\")\n    true","ref":"SnmpKit.SnmpLib.Utils.html#pretty_print_varbinds/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.sanitize_community/1","doc":"Sanitizes a community string for safe logging.\n\nReplaces community strings with asterisks to prevent credential leakage\nin logs while preserving length information.","ref":"SnmpKit.SnmpLib.Utils.html#sanitize_community/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.sanitize_community/1","doc":"iex> SnmpKit.SnmpLib.Utils.sanitize_community(\"secret123\")\n    \"*********\"\n\n    iex> SnmpKit.SnmpLib.Utils.sanitize_community(\"\")\n    \" \"","ref":"SnmpKit.SnmpLib.Utils.html#sanitize_community/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Utils.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#t:snmp_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.truncate_string/2","doc":"Truncates a string to maximum length with ellipsis.","ref":"SnmpKit.SnmpLib.Utils.html#truncate_string/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.truncate_string/2","doc":"iex> SnmpKit.SnmpLib.Utils.truncate_string(\"Hello, World!\", 10)\n    \"Hello, ...\"\n\n    iex> SnmpKit.SnmpLib.Utils.truncate_string(\"Short\", 10)\n    \"Short\"","ref":"SnmpKit.SnmpLib.Utils.html#truncate_string/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.valid_community_string?/1","doc":"Validates an SNMP community string.\n\nCommunity strings should be non-empty and contain only printable characters.","ref":"SnmpKit.SnmpLib.Utils.html#valid_community_string?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.valid_community_string?/1","doc":"iex> SnmpKit.SnmpLib.Utils.valid_community_string?(\"public\")\n    true\n\n    iex> SnmpKit.SnmpLib.Utils.valid_community_string?(\"\")\n    false","ref":"SnmpKit.SnmpLib.Utils.html#valid_community_string?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Utils.valid_snmp_version?/1","doc":"Validates an SNMP version number.","ref":"SnmpKit.SnmpLib.Utils.html#valid_snmp_version?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Utils.valid_snmp_version?/1","doc":"iex> SnmpKit.SnmpLib.Utils.valid_snmp_version?(1)\n    true\n\n    iex> SnmpKit.SnmpLib.Utils.valid_snmp_version?(5)\n    false","ref":"SnmpKit.SnmpLib.Utils.html#valid_snmp_version?/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Utils.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#t:varbind/0"},{"type":"type","title":"SnmpKit.SnmpLib.Utils.varbinds/0","doc":"","ref":"SnmpKit.SnmpLib.Utils.html#t:varbinds/0"},{"type":"module","title":"SnmpKit.SnmpLib.Walker","doc":"Efficient SNMP table walking with bulk operations and streaming support.\n\nThis module provides high-performance table walking capabilities using GETBULK\noperations for maximum efficiency. It's designed for collecting large amounts\nof SNMP data with minimal network overhead and memory usage.","ref":"SnmpKit.SnmpLib.Walker.html"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Walker","doc":"- **GETBULK Optimization**: Uses GETBULK requests for 3-5x faster table walking\n- **Streaming Support**: Process large tables without loading all data into memory\n- **Automatic Pagination**: Handles table boundaries and end-of-mib-view conditions\n- **Error Recovery**: Graceful handling of partial responses and network issues\n- **Adaptive Bulk Size**: Automatically adjusts bulk size based on device capabilities\n- **Memory Efficient**: Lazy evaluation and streaming for large datasets","ref":"SnmpKit.SnmpLib.Walker.html#module-features"},{"type":"module","title":"Table Walking Strategies - SnmpKit.SnmpLib.Walker","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#module-table-walking-strategies"},{"type":"module","title":"Bulk Walking (Recommended) - SnmpKit.SnmpLib.Walker","doc":"Uses GETBULK operations for maximum efficiency. Best for SNMPv2c devices.","ref":"SnmpKit.SnmpLib.Walker.html#module-bulk-walking-recommended"},{"type":"module","title":"Sequential Walking - SnmpKit.SnmpLib.Walker","doc":"Falls back to GETNEXT operations for SNMPv1 devices or when GETBULK fails.","ref":"SnmpKit.SnmpLib.Walker.html#module-sequential-walking"},{"type":"module","title":"Streaming Walking - SnmpKit.SnmpLib.Walker","doc":"Processes table rows as they arrive, ideal for very large tables.","ref":"SnmpKit.SnmpLib.Walker.html#module-streaming-walking"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.Walker","doc":"# Walk entire interface table\n    {:ok, interfaces} = SnmpKit.SnmpLib.Walker.walk_table(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 2, 2])\n\n    # Stream large table to avoid memory issues\n    SnmpKit.SnmpLib.Walker.stream_table(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 2, 2, 1, 2])\n    |> Stream.each(fn {interface_oid, interface_value} ->\n         IO.puts(\"Interface: \" <> inspect(interface_oid) <> \" = \" <> inspect(interface_value))\n       end)\n    |> Stream.run()\n\n    # Walk with custom bulk size and timeout\n    {:ok, data} = SnmpKit.SnmpLib.Walker.walk_table(\"10.0.0.1\", \"1.3.6.1.2.1.4.21\",\n                                            max_repetitions: 50, timeout: 15_000)","ref":"SnmpKit.SnmpLib.Walker.html#module-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Walker.estimate_table_size/3","doc":"Estimates the size of a table by walking just the first column.\n\nUseful for determining table size before performing full table walks,\nhelping with memory planning and progress estimation.","ref":"SnmpKit.SnmpLib.Walker.html#estimate_table_size/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Walker.estimate_table_size/3","doc":"- `host`: Target device IP address or hostname\n- `table_oid`: Base OID of the table\n- `opts`: Walking options (typically with small max_repetitions)","ref":"SnmpKit.SnmpLib.Walker.html#estimate_table_size/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Walker.estimate_table_size/3","doc":"- `{:ok, count}`: Estimated number of table rows\n- `{:error, reason}`: Estimation failed","ref":"SnmpKit.SnmpLib.Walker.html#estimate_table_size/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Walker.estimate_table_size/3","doc":"# Test that estimate_table_size function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Walker.estimate_table_size(\"invalid.host\", [1, 3, 6, 1, 2, 1, 2, 2], timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Walker.html#estimate_table_size/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.host/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:host/0"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.oid/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.stream_chunk/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:stream_chunk/0"},{"type":"function","title":"SnmpKit.SnmpLib.Walker.stream_table/3","doc":"Streams table entries as they are retrieved, ideal for very large tables.\n\nReturns a Stream that yields chunks of varbinds as they are collected.\nThis is memory-efficient for large tables as it doesn't load all data at once.","ref":"SnmpKit.SnmpLib.Walker.html#stream_table/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Walker.stream_table/3","doc":"- `host`: Target device IP address or hostname\n- `table_oid`: Base OID of the table to stream\n- `opts`: Streaming options (chunk_size controls entries per chunk)","ref":"SnmpKit.SnmpLib.Walker.html#stream_table/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Walker.stream_table/3","doc":"A Stream that yields `stream_chunk()` (lists of varbinds)","ref":"SnmpKit.SnmpLib.Walker.html#stream_table/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Walker.stream_table/3","doc":"# Process large routing table in chunks\n    # SnmpKit.SnmpLib.Walker.stream_table(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 4, 21])\n    # |> Stream.flat_map(& &1)  # Flatten chunks into individual varbinds\n    # |> Stream.filter(fn {_oid, value} -> value != 0 end)  # Filter active routes\n    # |> Enum.take(100)  # Take first 100 active routes\n    # returns list of active routes\n\n    # Test that stream_table function exists and returns a stream\n    iex> stream = SnmpKit.SnmpLib.Walker.stream_table(\"invalid.host\", \"1.3.6.1.2.1.2.2.1.1\", timeout: 100)\n    iex> is_function(stream, 2)\n    true","ref":"SnmpKit.SnmpLib.Walker.html#stream_table/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.varbind/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:varbind/0"},{"type":"function","title":"SnmpKit.SnmpLib.Walker.walk_column/3","doc":"Walks a single table column efficiently.\n\nOptimized for retrieving a single column from an SNMP table by using\nthe column OID directly and stopping at table boundaries.","ref":"SnmpKit.SnmpLib.Walker.html#walk_column/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Walker.walk_column/3","doc":"- `host`: Target device IP address or hostname\n- `column_oid`: OID of the table column (e.g., [1,3,6,1,2,1,2,2,1,2] for ifDescr)\n- `opts`: Walking options","ref":"SnmpKit.SnmpLib.Walker.html#walk_column/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Walker.walk_column/3","doc":"- `{:ok, column_data}`: List of {index_oid, value} pairs for the column\n- `{:error, reason}`: Walking failed","ref":"SnmpKit.SnmpLib.Walker.html#walk_column/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Walker.walk_column/3","doc":"# Test that walk_column function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Walker.walk_column(\"invalid.host\", [1, 3, 6, 1, 2, 1, 2, 2, 1, 2], timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Walker.html#walk_column/3-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.walk_opts/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:walk_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.Walker.walk_result/0","doc":"","ref":"SnmpKit.SnmpLib.Walker.html#t:walk_result/0"},{"type":"function","title":"SnmpKit.SnmpLib.Walker.walk_subtree/3","doc":"Walks a subtree starting from the given OID.\n\nSimilar to walk_table/3 but continues until the OID prefix no longer matches,\nmaking it suitable for walking MIB subtrees that may contain multiple tables.","ref":"SnmpKit.SnmpLib.Walker.html#walk_subtree/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Walker.walk_subtree/3","doc":"- `host`: Target device IP address or hostname\n- `base_oid`: Starting OID for the subtree walk\n- `opts`: Walking options","ref":"SnmpKit.SnmpLib.Walker.html#walk_subtree/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Walker.walk_subtree/3","doc":"- `{:ok, varbinds}`: All OIDs under the base_oid with their values\n- `{:error, reason}`: Walking failed","ref":"SnmpKit.SnmpLib.Walker.html#walk_subtree/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Walker.walk_subtree/3","doc":"# Test that walk_subtree function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Walker.walk_subtree(\"invalid.host\", [1, 3, 6, 1, 2, 1, 1], timeout: 100))\n    true","ref":"SnmpKit.SnmpLib.Walker.html#walk_subtree/3-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Walker.walk_table/3","doc":"Walks an entire SNMP table efficiently using GETBULK operations.\n\nThis is the most efficient way to retrieve a complete SNMP table. Uses GETBULK\nrequests when possible (SNMPv2c) and automatically handles table boundaries.","ref":"SnmpKit.SnmpLib.Walker.html#walk_table/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Walker.walk_table/3","doc":"- `host`: Target device IP address or hostname\n- `table_oid`: Base OID of the table to walk\n- `opts`: Walking options (see module docs)","ref":"SnmpKit.SnmpLib.Walker.html#walk_table/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Walker.walk_table/3","doc":"- `{:ok, varbinds}`: List of {oid, value} pairs for all table entries\n- `{:error, reason}`: Walking failed with reason","ref":"SnmpKit.SnmpLib.Walker.html#walk_table/3-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Walker.walk_table/3","doc":"# Test that walk_table function exists and handles invalid input properly\n    iex> match?({:error, _}, SnmpKit.SnmpLib.Walker.walk_table(\"invalid.host\", [1, 3, 6, 1, 2, 1, 2, 2], timeout: 100))\n    true\n\n    # Walk with high bulk size for faster collection\n    # SnmpKit.SnmpLib.Walker.walk_table(\"10.0.0.1\", \"1.3.6.1.2.1.2.2\", max_repetitions: 50)\n    # {:ok, [...]} returns many interface entries","ref":"SnmpKit.SnmpLib.Walker.html#walk_table/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.AdaptiveWalk","doc":"Intelligent SNMP walk operations with adaptive parameter tuning.\n\nThis module provides advanced walk operations that automatically adjust\nbulk parameters based on device response characteristics for optimal performance.","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html"},{"type":"function","title":"SnmpKit.SnmpMgr.AdaptiveWalk.benchmark_device/3","doc":"Benchmarks a device to determine optimal bulk parameters.\n\nPerforms a series of test requests with different bulk sizes\nto determine the best parameters for the target device.","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#benchmark_device/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.AdaptiveWalk.benchmark_device/3","doc":"- `target` - The target device to benchmark\n- `test_oid` - OID to use for testing (should have multiple entries)\n- `opts` - Options including :test_sizes, :iterations","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#benchmark_device/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.AdaptiveWalk.benchmark_device/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, benchmark} = SnmpKit.SnmpMgr.AdaptiveWalk.benchmark_device(\"switch.local\", \"ifTable\")\n    # Returns comprehensive performance analysis:\n    # %{\n    #   optimal_bulk_size: 25,\n    #   avg_response_time: 45,\n    #   error_rate: 0.0,\n    #   all_results: [\n    #     {1, 120}, {5, 85}, {10, 52}, {15, 48},\n    #     {20, 44}, {25, 42}, {30, 45}, {40, 52}, {50, 68}\n    #   ],\n    #   recommendations: %{\n    #     max_repetitions: 25,\n    #     timeout: 3000,\n    #     adaptive_tuning: false  # Device has consistent performance\n    #   }\n    # }\n\n    # Custom benchmark with specific test sizes:\n    {:ok, results} = SnmpKit.SnmpMgr.AdaptiveWalk.benchmark_device(\"router.local\", \"ipRouteTable\",\n      test_sizes: [5, 10, 20, 50], iterations: 5)","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#benchmark_device/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.AdaptiveWalk.bulk_walk/3","doc":"Performs an adaptive bulk walk that automatically tunes parameters.\n\nStarts with a conservative bulk size and adapts based on response times\nand error rates to find the optimal parameters for the target device.","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#bulk_walk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.AdaptiveWalk.bulk_walk/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :adaptive_tuning, :performance_threshold","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#bulk_walk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.AdaptiveWalk.bulk_walk/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, results} = SnmpKit.SnmpMgr.AdaptiveWalk.bulk_walk(\"switch.local\", \"ifTable\")\n    # Automatically adjusts bulk size for optimal performance:\n    # [\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", :octet_string, \"FastEthernet0/1\"},\n    #   {\"1.3.6.1.2.1.2.2.1.2.2\", :octet_string, \"FastEthernet0/2\"},\n    #   {\"1.3.6.1.2.1.2.2.1.2.3\", :octet_string, \"GigabitEthernet0/1\"},\n    #   # ... optimally retrieved with adaptive bulk sizing\n    # ]\n\n    # With custom options:\n    {:ok, results} = SnmpKit.SnmpMgr.AdaptiveWalk.bulk_walk(\"router.local\", \"sysDescr\",\n      adaptive_tuning: true, max_entries: 100, performance_threshold: 50)\n    # [{\"1.3.6.1.2.1.1.1.0\", :octet_string, \"Cisco IOS Software, Version 15.1\"}]","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#bulk_walk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.AdaptiveWalk.get_optimal_params/2","doc":"Gets optimal parameters for a previously benchmarked device.\n\nReturns cached optimal parameters or performs a quick benchmark\nif no cached data is available.","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#get_optimal_params/2"},{"type":"function","title":"SnmpKit.SnmpMgr.AdaptiveWalk.table_walk/3","doc":"Performs an adaptive table walk optimized for large tables.\n\nAutomatically determines the optimal bulk size for table retrieval\nand handles pagination for very large tables.","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#table_walk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.AdaptiveWalk.table_walk/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to walk\n- `opts` - Options including :adaptive_tuning, :stream, :max_entries","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#table_walk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.AdaptiveWalk.table_walk/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, table_data} = SnmpKit.SnmpMgr.AdaptiveWalk.table_walk(\"switch.local\", \"ifTable\", max_entries: 1000)\n    # Efficiently walks large tables with automatic optimization:\n    # [\n    #   {\"1.3.6.1.2.1.2.2.1.1.1\", :integer, 1},           # ifIndex.1\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", :octet_string, \"Ethernet1\"}, # ifDescr.1\n    #   {\"1.3.6.1.2.1.2.2.1.3.1\", :integer, 6},           # ifType.1 (ethernetCsmacd)\n    #   {\"1.3.6.1.2.1.2.2.1.5.1\", :gauge32, 1000000000},  # ifSpeed.1 (1 Gbps)\n    #   # ... continues with adaptive pagination for large tables\n    # ]\n\n    # For very large tables with streaming:\n    {:ok, results} = SnmpKit.SnmpMgr.AdaptiveWalk.table_walk(\"core-switch\", \"ipRouteTable\",\n      max_entries: 10000, adaptive_tuning: true)","ref":"SnmpKit.SnmpMgr.AdaptiveWalk.html#table_walk/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Bulk","doc":"Advanced SNMP bulk operations using SNMPv2c GETBULK.\n\nThis module provides efficient bulk operations that are significantly faster\nthan iterative GETNEXT requests for retrieving large amounts of data.","ref":"SnmpKit.SnmpMgr.Bulk.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Bulk.get_bulk/3","doc":"Performs a single GETBULK request.","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Bulk.get_bulk/3","doc":"- `target` - The target device\n- `oids` - Single OID or list of OIDs to retrieve\n- `opts` - Options including :max_repetitions, :non_repeaters","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Bulk.get_bulk/3","doc":"iex> SnmpKit.SnmpMgr.Bulk.get_bulk(\"192.168.1.1\", \"ifTable\", max_repetitions: 20)\n    {:ok, [\n      {\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"},\n      {\"1.3.6.1.2.1.2.2.1.2.2\", \"eth1\"},\n      # ... up to 20 entries\n    ]}","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Bulk.get_bulk_multi/2","doc":"Performs multiple concurrent GETBULK operations.","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Bulk.get_bulk_multi/2","doc":"- `targets_and_oids` - List of {target, oid} tuples\n- `opts` - Options for all requests","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Bulk.get_bulk_multi/2","doc":"iex> requests = [\n    ...>   {\"device1\", \"sysDescr.0\"},\n    ...>   {\"device2\", \"sysUpTime.0\"},\n    ...>   {\"device3\", \"ifNumber.0\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Bulk.get_bulk_multi(requests)\n    [\n      {:ok, [{\"1.3.6.1.2.1.1.1.0\", \"Device 1\"}]},\n      {:ok, [{\"1.3.6.1.2.1.1.3.0\", 123456}]},\n      {:error, :timeout}\n    ]","ref":"SnmpKit.SnmpMgr.Bulk.html#get_bulk_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Bulk.get_table_bulk/3","doc":"Optimized table retrieval using GETBULK.\n\nUses GETBULK to efficiently retrieve an entire SNMP table,\nautomatically handling pagination when tables are larger than max_repetitions.","ref":"SnmpKit.SnmpMgr.Bulk.html#get_table_bulk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Bulk.get_table_bulk/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to retrieve\n- `opts` - Options including :max_repetitions, :max_entries","ref":"SnmpKit.SnmpMgr.Bulk.html#get_table_bulk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Bulk.get_table_bulk/3","doc":"iex> SnmpKit.SnmpMgr.Bulk.get_table_bulk(\"switch.local\", \"ifTable\")\n    {:ok, [\n      {\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"},\n      {\"1.3.6.1.2.1.2.2.1.3.1\", 6},\n      {\"1.3.6.1.2.1.2.2.1.2.2\", \"eth1\"},\n      {\"1.3.6.1.2.1.2.2.1.3.2\", 6}\n    ]}","ref":"SnmpKit.SnmpMgr.Bulk.html#get_table_bulk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Bulk.walk_bulk/3","doc":"Bulk walk operation using GETBULK instead of iterative GETNEXT.\n\nSignificantly more efficient than traditional walks for large subtrees.","ref":"SnmpKit.SnmpMgr.Bulk.html#walk_bulk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Bulk.walk_bulk/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :max_repetitions, :max_entries","ref":"SnmpKit.SnmpMgr.Bulk.html#walk_bulk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Bulk.walk_bulk/3","doc":"iex> SnmpKit.SnmpMgr.Bulk.walk_bulk(\"device.local\", \"system\")\n    {:ok, [\n      {\"1.3.6.1.2.1.1.1.0\", \"System Description\"},\n      {\"1.3.6.1.2.1.1.2.0\", \"1.3.6.1.4.1.9\"},\n      {\"1.3.6.1.2.1.1.3.0\", 12345}\n    ]}","ref":"SnmpKit.SnmpMgr.Bulk.html#walk_bulk/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.CircuitBreaker","doc":"Circuit breaker pattern implementation for SNMP device failure protection.\n\nThis module implements the circuit breaker pattern to prevent cascading failures\nwhen SNMP devices become unresponsive. It provides automatic failure detection,\nrecovery attempts, and configurable thresholds for different failure scenarios.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.call/4","doc":"Executes a function with circuit breaker protection.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#call/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.CircuitBreaker.call/4","doc":"- `cb` - Circuit breaker PID or name\n- `target` - Target identifier (device address/name)\n- `fun` - Function to execute\n- `timeout` - Operation timeout in ms","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#call/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.CircuitBreaker.call/4","doc":"result = SnmpKit.SnmpMgr.CircuitBreaker.call(cb, \"192.168.1.1\", fn ->\n      SnmpKit.SnmpMgr.get(\"192.168.1.1\", \"sysDescr.0\")\n    end, 5000)","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#call/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.close_circuit/2","doc":"Manually closes a circuit breaker for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#close_circuit/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.configure/2","doc":"Configures circuit breaker settings.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#configure/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.configure_target/3","doc":"Configures settings for a specific target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#configure_target/3"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.force_half_open/2","doc":"Forces a circuit breaker to half-open state.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#force_half_open/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.force_open/2","doc":"Forces a circuit breaker to open state.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#force_open/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_all_targets/1","doc":"Gets all active targets.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_all_targets/1"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_config/2","doc":"Gets configuration for a specific target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_config/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_global_stats/1","doc":"Gets global circuit breaker statistics.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_global_stats/1"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_state/2","doc":"Gets the current state of a circuit breaker for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_state/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_stats/1","doc":"Gets statistics for all circuit breakers.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_stats/1"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.get_stats/2","doc":"Gets statistics for a specific target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#get_stats/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.open_circuit/2","doc":"Manually opens a circuit breaker for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#open_circuit/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.record_failure/3","doc":"Records a failure for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#record_failure/3"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.record_success/2","doc":"Records a successful operation for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#record_success/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.remove_target/2","doc":"Removes a target from the circuit breaker.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#remove_target/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.reset/2","doc":"Resets a specific circuit breaker for a target.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#reset/2"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.reset_all/1","doc":"Resets all circuit breakers.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#reset_all/1"},{"type":"function","title":"SnmpKit.SnmpMgr.CircuitBreaker.start_link/1","doc":"Starts the circuit breaker manager.","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpMgr.CircuitBreaker.start_link/1","doc":"- `:failure_threshold` - Number of failures before opening circuit (default: 5)\n- `:recovery_timeout` - Time to wait before attempting recovery in ms (default: 30000)\n- `:timeout_threshold` - Request timeout threshold in ms (default: 10000)\n- `:half_open_max_calls` - Max calls in half-open state (default: 3)","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.CircuitBreaker.start_link/1","doc":"{:ok, cb} = SnmpKit.SnmpMgr.CircuitBreaker.start_link(\n      failure_threshold: 10,\n      recovery_timeout: 60_000\n    )","ref":"SnmpKit.SnmpMgr.CircuitBreaker.html#start_link/1-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Config","doc":"Configuration management for SnmpKit.SnmpMgr.\n\nProvides global defaults and configuration options that can be set\napplication-wide and used by all SNMP operations.","ref":"SnmpKit.SnmpMgr.Config.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.add_mib_path/1","doc":"Adds a directory to the MIB search paths.","ref":"SnmpKit.SnmpMgr.Config.html#add_mib_path/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.add_mib_path/1","doc":"iex> SnmpKit.SnmpMgr.Config.add_mib_path(\"/usr/share/snmp/mibs\")\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#add_mib_path/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.Config.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get/1","doc":"Gets configuration value with fallback to application config.\n\nThis is the main function used by other modules to get configuration values.\nIt first checks the GenServer state, then falls back to application config,\nthen to module defaults.","ref":"SnmpKit.SnmpMgr.Config.html#get/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.get/1","doc":"iex> SnmpKit.SnmpMgr.Config.get(:community)\n    \"public\"\n\n    iex> SnmpKit.SnmpMgr.Config.get(:timeout)\n    5000","ref":"SnmpKit.SnmpMgr.Config.html#get/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_all/0","doc":"Gets all current configuration as a map.","ref":"SnmpKit.SnmpMgr.Config.html#get_all/0"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.get_all/0","doc":"iex> SnmpKit.SnmpMgr.Config.get_all()\n    %{\n      community: \"public\",\n      timeout: 5000,\n      retries: 1,\n      port: 161,\n      version: :v1,\n      mib_paths: []\n    }","ref":"SnmpKit.SnmpMgr.Config.html#get_all/0-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_default_community/0","doc":"Gets the default community string.","ref":"SnmpKit.SnmpMgr.Config.html#get_default_community/0"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.get_default_community/0","doc":"iex> SnmpKit.SnmpMgr.Config.get_default_community()\n    \"public\"","ref":"SnmpKit.SnmpMgr.Config.html#get_default_community/0-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_default_port/0","doc":"Gets the default port.","ref":"SnmpKit.SnmpMgr.Config.html#get_default_port/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_default_retries/0","doc":"Gets the default number of retries.","ref":"SnmpKit.SnmpMgr.Config.html#get_default_retries/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_default_timeout/0","doc":"Gets the default timeout.","ref":"SnmpKit.SnmpMgr.Config.html#get_default_timeout/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_default_version/0","doc":"Gets the default SNMP version.","ref":"SnmpKit.SnmpMgr.Config.html#get_default_version/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.get_mib_paths/0","doc":"Gets the current MIB search paths.","ref":"SnmpKit.SnmpMgr.Config.html#get_mib_paths/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.merge_opts/1","doc":"Merges the current configuration with provided opts, giving priority to opts.\n\nThis is useful for functions that want to use global defaults but allow\nper-request overrides.","ref":"SnmpKit.SnmpMgr.Config.html#merge_opts/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.merge_opts/1","doc":"iex> SnmpKit.SnmpMgr.Config.merge_opts(community: \"private\", timeout: 10000)\n    [community: \"private\", timeout: 10000, retries: 1, port: 161]","ref":"SnmpKit.SnmpMgr.Config.html#merge_opts/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.reset/0","doc":"Resets configuration to defaults.","ref":"SnmpKit.SnmpMgr.Config.html#reset/0"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.reset/0","doc":"iex> SnmpKit.SnmpMgr.Config.reset()\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#reset/0-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_default_community/1","doc":"Sets the default community string for SNMP requests.","ref":"SnmpKit.SnmpMgr.Config.html#set_default_community/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_default_community/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_default_community(\"private\")\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_default_community/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_default_port/1","doc":"Sets the default port for SNMP requests.","ref":"SnmpKit.SnmpMgr.Config.html#set_default_port/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_default_port/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_default_port(1161)\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_default_port/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_default_retries/1","doc":"Sets the default number of retries for SNMP requests.","ref":"SnmpKit.SnmpMgr.Config.html#set_default_retries/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_default_retries/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_default_retries(3)\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_default_retries/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_default_timeout/1","doc":"Sets the default timeout for SNMP requests in milliseconds.","ref":"SnmpKit.SnmpMgr.Config.html#set_default_timeout/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_default_timeout/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_default_timeout(10000)\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_default_timeout/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_default_version/1","doc":"Sets the default SNMP version.","ref":"SnmpKit.SnmpMgr.Config.html#set_default_version/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_default_version/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_default_version(:v2c)\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_default_version/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.set_mib_paths/1","doc":"Sets the MIB search paths (replaces existing paths).","ref":"SnmpKit.SnmpMgr.Config.html#set_mib_paths/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Config.set_mib_paths/1","doc":"iex> SnmpKit.SnmpMgr.Config.set_mib_paths([\"/usr/share/snmp/mibs\", \"./mibs\"])\n    :ok","ref":"SnmpKit.SnmpMgr.Config.html#set_mib_paths/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Config.start_link/1","doc":"Starts the configuration GenServer.","ref":"SnmpKit.SnmpMgr.Config.html#start_link/1"},{"type":"module","title":"SnmpKit.SnmpMgr.Core","doc":"Core SNMP operations using Erlang's SNMP PDU functions directly.\n\nThis module handles the low-level SNMP PDU encoding/decoding and UDP communication\nwithout requiring the heavyweight :snmpm manager process.","ref":"SnmpKit.SnmpMgr.Core.html"},{"type":"type","title":"SnmpKit.SnmpMgr.Core.oid/0","doc":"","ref":"SnmpKit.SnmpMgr.Core.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpMgr.Core.opts/0","doc":"","ref":"SnmpKit.SnmpMgr.Core.html#t:opts/0"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.parse_oid/1","doc":"Parses and normalizes an OID using SnmpKit.SnmpLib.OID.normalize with MIB support.","ref":"SnmpKit.SnmpMgr.Core.html#parse_oid/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_bulk_request/3","doc":"Sends an SNMP GETBULK request (SNMPv2c only).","ref":"SnmpKit.SnmpMgr.Core.html#send_get_bulk_request/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_bulk_request_async/3","doc":"Sends an asynchronous SNMP GETBULK request.","ref":"SnmpKit.SnmpMgr.Core.html#send_get_bulk_request_async/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_next_request/3","doc":"Sends a GETNEXT request to retrieve the next OID in the MIB tree.\n\nNow uses the proper SnmpKit.SnmpLib.Manager.get_next/3 function which handles\nversion-specific logic (GETNEXT for v1, GETBULK for v2c+) correctly.","ref":"SnmpKit.SnmpMgr.Core.html#send_get_next_request/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_request/3","doc":"Sends an SNMP GET request and returns the response.","ref":"SnmpKit.SnmpMgr.Core.html#send_get_request/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_request_async/3","doc":"Sends an asynchronous SNMP GET request.","ref":"SnmpKit.SnmpMgr.Core.html#send_get_request_async/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_get_request_with_type/3","doc":"Sends a GET request and returns the result in 3-tuple format.\n\nThis function returns `{oid_string, type, value}` for consistency with\nother operations like walk, bulk, etc.","ref":"SnmpKit.SnmpMgr.Core.html#send_get_request_with_type/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Core.send_set_request/4","doc":"Sends an SNMP SET request and returns the response.","ref":"SnmpKit.SnmpMgr.Core.html#send_set_request/4"},{"type":"type","title":"SnmpKit.SnmpMgr.Core.snmp_result/0","doc":"","ref":"SnmpKit.SnmpMgr.Core.html#t:snmp_result/0"},{"type":"type","title":"SnmpKit.SnmpMgr.Core.target/0","doc":"","ref":"SnmpKit.SnmpMgr.Core.html#t:target/0"},{"type":"module","title":"SnmpKit.SnmpMgr.Engine","doc":"High-performance streaming PDU engine with request routing and connection pooling.\n\nThis module provides the core infrastructure for handling large volumes of SNMP\nrequests efficiently through connection pooling, request batching, and intelligent\nrouting strategies.","ref":"SnmpKit.SnmpMgr.Engine.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.Engine.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.get_pool_status/1","doc":"Gets connection pool status.","ref":"SnmpKit.SnmpMgr.Engine.html#get_pool_status/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.get_stats/1","doc":"Gets engine statistics and metrics.","ref":"SnmpKit.SnmpMgr.Engine.html#get_stats/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.start_link/1","doc":"Starts the streaming PDU engine.","ref":"SnmpKit.SnmpMgr.Engine.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpMgr.Engine.start_link/1","doc":"- `:pool_size` - Number of UDP socket connections to maintain (default: 10)\n- `:max_rps` - Maximum requests per second (default: 100)\n- `:request_timeout` - Individual request timeout in ms (default: 5000)\n- `:batch_size` - Maximum requests per batch (default: 50)\n- `:batch_timeout` - Maximum time to wait for batch in ms (default: 100)","ref":"SnmpKit.SnmpMgr.Engine.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Engine.start_link/1","doc":"{:ok, engine} = SnmpKit.SnmpMgr.Engine.start_link(\n      pool_size: 20,\n      max_rps: 200,\n      batch_size: 100\n    )","ref":"SnmpKit.SnmpMgr.Engine.html#start_link/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.stop/1","doc":"Gracefully shuts down the engine.","ref":"SnmpKit.SnmpMgr.Engine.html#stop/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.submit_batch/3","doc":"Submits multiple requests as a batch.","ref":"SnmpKit.SnmpMgr.Engine.html#submit_batch/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Engine.submit_batch/3","doc":"- `engine` - Engine PID or name\n- `requests` - List of request specification maps\n- `opts` - Batch options","ref":"SnmpKit.SnmpMgr.Engine.html#submit_batch/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Engine.submit_batch/3","doc":"requests = [\n      %{type: :get, target: \"device1\", oid: \"sysDescr.0\"},\n      %{type: :get, target: \"device2\", oid: \"sysDescr.0\"}\n    ]\n\n    {:ok, batch_ref} = SnmpKit.SnmpMgr.Engine.submit_batch(engine, requests)","ref":"SnmpKit.SnmpMgr.Engine.html#submit_batch/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Engine.submit_request/3","doc":"Submits a request to the engine for processing.","ref":"SnmpKit.SnmpMgr.Engine.html#submit_request/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Engine.submit_request/3","doc":"- `engine` - Engine PID or name\n- `request` - Request specification map\n- `opts` - Request options","ref":"SnmpKit.SnmpMgr.Engine.html#submit_request/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Engine.submit_request/3","doc":"request = %{\n      type: :get,\n      target: \"192.168.1.1\",\n      oid: \"1.3.6.1.2.1.1.1.0\",\n      community: \"public\"\n    }\n\n    {:ok, ref} = SnmpKit.SnmpMgr.Engine.submit_request(engine, request)","ref":"SnmpKit.SnmpMgr.Engine.html#submit_request/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Errors","doc":"SNMP error handling and error code translation.\n\nProvides functions to handle both SNMPv1 and SNMPv2c error conditions\nand translate error codes to human-readable messages.","ref":"SnmpKit.SnmpMgr.Errors.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.analyze_error/1","doc":"Enhanced error analysis using SnmpKit.SnmpLib.Error for SNMP protocol errors.\n\nProvides detailed error information including severity and RFC compliance\nfor SNMP protocol errors while preserving our comprehensive network error handling.","ref":"SnmpKit.SnmpMgr.Errors.html#analyze_error/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.analyze_error/1","doc":"iex> SnmpKit.SnmpMgr.Errors.analyze_error({:snmp_error, 2})\n    %{\n      type: :snmp_protocol,\n      atom: :no_such_name,\n      code: 2,\n      severity: :error,\n      retriable: false,\n      category: :user_error,\n      description: \"Variable name not found\"\n    }\n\n    iex> SnmpKit.SnmpMgr.Errors.analyze_error(:timeout)\n    %{\n      type: :network,\n      atom: :timeout,\n      retriable: true,\n      category: :transient_error,\n      description: \"Operation timed out\"\n    }","ref":"SnmpKit.SnmpMgr.Errors.html#analyze_error/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.classify_error/2","doc":"Classifies an error into a category for better handling.","ref":"SnmpKit.SnmpMgr.Errors.html#classify_error/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.classify_error/2","doc":"iex> SnmpKit.SnmpMgr.Errors.classify_error({:snmp_error, :no_such_name}, \"Get request\")\n    {:user_error, :invalid_oid}\n\n    iex> SnmpKit.SnmpMgr.Errors.classify_error({:network_error, :timeout}, \"Network operation\")\n    {:transient_error, :network_timeout}\n\n    iex> SnmpKit.SnmpMgr.Errors.classify_error({:snmp_error, :too_big}, \"Bulk request\")\n    {:recoverable_error, :response_too_large}","ref":"SnmpKit.SnmpMgr.Errors.html#classify_error/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.code_to_atom/1","doc":"Translates an SNMP error code to an atom.\n\nUses SnmpKit.SnmpLib.Error for validation and standardization of RFC-compliant error codes.","ref":"SnmpKit.SnmpMgr.Errors.html#code_to_atom/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.code_to_atom/1","doc":"iex> SnmpKit.SnmpMgr.Errors.code_to_atom(2)\n    :no_such_name\n\n    iex> SnmpKit.SnmpMgr.Errors.code_to_atom(0)\n    :no_error\n\n    iex> SnmpKit.SnmpMgr.Errors.code_to_atom(999)\n    :unknown_error","ref":"SnmpKit.SnmpMgr.Errors.html#code_to_atom/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.code_to_description/1","doc":"Translates an error code directly to a description.","ref":"SnmpKit.SnmpMgr.Errors.html#code_to_description/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.code_to_description/1","doc":"iex> SnmpKit.SnmpMgr.Errors.code_to_description(2)\n    \"Variable name not found\"\n\n    iex> SnmpKit.SnmpMgr.Errors.code_to_description(18)\n    \"Inconsistent name\"","ref":"SnmpKit.SnmpMgr.Errors.html#code_to_description/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.description/1","doc":"Translates an SNMP error atom to a human-readable description.","ref":"SnmpKit.SnmpMgr.Errors.html#description/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.description/1","doc":"iex> SnmpKit.SnmpMgr.Errors.description(:no_such_name)\n    \"Variable name not found\"\n\n    iex> SnmpKit.SnmpMgr.Errors.description(:too_big)\n    \"Response too big to fit in message\"\n\n    iex> SnmpKit.SnmpMgr.Errors.description(:unknown_error)\n    \"Unknown error\"","ref":"SnmpKit.SnmpMgr.Errors.html#description/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.format_error/1","doc":"Formats an SNMP error for display.","ref":"SnmpKit.SnmpMgr.Errors.html#format_error/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.format_error/1","doc":"iex> SnmpKit.SnmpMgr.Errors.format_error({:snmp_error, 2})\n    \"SNMP Error (2): Variable name not found\"\n\n    iex> SnmpKit.SnmpMgr.Errors.format_error({:snmp_error, :no_such_name})\n    \"SNMP Error: Variable name not found\"\n\n    iex> SnmpKit.SnmpMgr.Errors.format_error({:v2c_error, :no_access, oid: \"1.2.3.4\"})\n    \"SNMPv2c Error: Access denied (OID: 1.2.3.4)\"","ref":"SnmpKit.SnmpMgr.Errors.html#format_error/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.format_user_friendly_error/2","doc":"Formats an error message in a user-friendly way with context.","ref":"SnmpKit.SnmpMgr.Errors.html#format_user_friendly_error/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.format_user_friendly_error/2","doc":"iex> SnmpKit.SnmpMgr.Errors.format_user_friendly_error({:snmp_error, :no_such_name}, \"Getting system description\")\n    \"Unable to get system description: The requested OID does not exist on the device\"\n\n    iex> SnmpKit.SnmpMgr.Errors.format_user_friendly_error({:network_error, :timeout}, \"Contacting device\")\n    \"Failed contacting device: The device did not respond within the timeout period\"","ref":"SnmpKit.SnmpMgr.Errors.html#format_user_friendly_error/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.get_recovery_suggestions/1","doc":"Provides recovery suggestions for common errors.","ref":"SnmpKit.SnmpMgr.Errors.html#get_recovery_suggestions/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.get_recovery_suggestions/1","doc":"iex> SnmpKit.SnmpMgr.Errors.get_recovery_suggestions({:snmp_error, :no_such_name})\n    [\"Verify the OID is correct\", \"Check if the OID is supported by this device\", \"Try using MIB browser to explore available OIDs\"]\n\n    iex> SnmpKit.SnmpMgr.Errors.get_recovery_suggestions({:network_error, :timeout})\n    [\"Increase timeout value\", \"Check network connectivity\", \"Verify device is responding to ping\"]","ref":"SnmpKit.SnmpMgr.Errors.html#get_recovery_suggestions/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.is_v2c_error?/1","doc":"Determines if an error is version-specific.","ref":"SnmpKit.SnmpMgr.Errors.html#is_v2c_error?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.is_v2c_error?/1","doc":"iex> SnmpKit.SnmpMgr.Errors.is_v2c_error?(:no_access)\n    true\n\n    iex> SnmpKit.SnmpMgr.Errors.is_v2c_error?(:no_such_name)\n    false","ref":"SnmpKit.SnmpMgr.Errors.html#is_v2c_error?/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Errors.recoverable?/1","doc":"Checks if an error is recoverable (can be retried).","ref":"SnmpKit.SnmpMgr.Errors.html#recoverable?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Errors.recoverable?/1","doc":"iex> SnmpKit.SnmpMgr.Errors.recoverable?({:network_error, :host_unreachable})\n    false\n\n    iex> SnmpKit.SnmpMgr.Errors.recoverable?({:snmp_error, :too_big})\n    true\n\n    iex> SnmpKit.SnmpMgr.Errors.recoverable?(:timeout)\n    true","ref":"SnmpKit.SnmpMgr.Errors.html#recoverable?/1-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Format","doc":"SNMP data formatting and presentation utilities.\n\nThis module provides user-friendly formatting functions for SNMP data types,\ndelegating to the underlying SnmpKit.SnmpLib.Types functions while maintaining a\nclean SnmpMgr API surface.\n\nAll functions work with the 3-tuple format `{oid_string, type, value}` that\nSnmpMgr uses throughout the library.","ref":"SnmpKit.SnmpMgr.Format.html"},{"type":"module","title":"Examples - SnmpKit.SnmpMgr.Format","doc":"# Format uptime from SNMP result\n    {:ok, {_oid, :timeticks, ticks}} = SnmpKit.SnmpMgr.get(\"router.local\", \"sysUpTime.0\")\n    SnmpKit.SnmpMgr.Format.uptime(ticks)\n    # => \"5 days, 12 hours, 34 minutes, 56 seconds\"\n\n    # Format IP address\n    SnmpKit.SnmpMgr.Format.ip_address(<<192, 168, 1, 1>>)\n    # => \"192.168.1.1\"\n\n    # Pretty print any SNMP result\n    {:ok, result} = SnmpKit.SnmpMgr.get(\"router.local\", \"sysDescr.0\")\n    SnmpKit.SnmpMgr.Format.pretty_print(result)\n    # => {\"1.3.6.1.2.1.1.1.0\", :octet_string, \"Cisco IOS Router\"}","ref":"SnmpKit.SnmpMgr.Format.html#module-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.bytes/1","doc":"Formats byte counts into human-readable sizes.","ref":"SnmpKit.SnmpMgr.Format.html#bytes/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.bytes/1","doc":"iex> SnmpKit.SnmpMgr.Format.bytes(1024)\n    \"1.0 KB\"\n\n    iex> SnmpKit.SnmpMgr.Format.bytes(1073741824)\n    \"1.0 GB\"","ref":"SnmpKit.SnmpMgr.Format.html#bytes/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.format_by_type/2","doc":"Automatically formats a value based on its SNMP type.\n\nThis function provides a single entry point for type-aware formatting,\nautomatically choosing the appropriate formatting function based on the type.","ref":"SnmpKit.SnmpMgr.Format.html#format_by_type/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.format_by_type/2","doc":"iex> SnmpKit.SnmpMgr.Format.format_by_type(:timeticks, 126691300)\n    \"14 days 15 hours 55 minutes 13 seconds\"\n\n    iex> SnmpKit.SnmpMgr.Format.format_by_type(:gauge32, 1000000000)\n    \"1 GB\"\n\n    iex> SnmpKit.SnmpMgr.Format.format_by_type(:octet_string, \"Hello\")\n    \"Hello\"","ref":"SnmpKit.SnmpMgr.Format.html#format_by_type/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.interface_status/1","doc":"Formats SNMP interface status values into readable strings.","ref":"SnmpKit.SnmpMgr.Format.html#interface_status/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.interface_status/1","doc":"iex> SnmpKit.SnmpMgr.Format.interface_status(1)\n    \"up\"\n\n    iex> SnmpKit.SnmpMgr.Format.interface_status(2)\n    \"down\"","ref":"SnmpKit.SnmpMgr.Format.html#interface_status/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.interface_type/1","doc":"Formats SNMP interface types into readable strings.","ref":"SnmpKit.SnmpMgr.Format.html#interface_type/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.interface_type/1","doc":"iex> SnmpKit.SnmpMgr.Format.interface_type(6)\n    \"ethernetCsmacd\"\n\n    iex> SnmpKit.SnmpMgr.Format.interface_type(24)\n    \"softwareLoopback\"","ref":"SnmpKit.SnmpMgr.Format.html#interface_type/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.ip_address/1","doc":"Formats IP address bytes into dotted decimal notation.","ref":"SnmpKit.SnmpMgr.Format.html#ip_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.ip_address/1","doc":"iex> SnmpKit.SnmpMgr.Format.ip_address(<<192, 168, 1, 1>>)\n    \"192.168.1.1\"\n\n    iex> SnmpKit.SnmpMgr.Format.ip_address({10, 0, 0, 1})\n    \"10.0.0.1\"","ref":"SnmpKit.SnmpMgr.Format.html#ip_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.mac_address/1","doc":"Formats MAC addresses into standard colon-separated hex format.\n\nHandles both binary and list representations of MAC addresses commonly\nfound in SNMP responses.","ref":"SnmpKit.SnmpMgr.Format.html#mac_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.mac_address/1","doc":"iex> SnmpKit.SnmpMgr.Format.mac_address(<<0x00, 0x1B, 0x21, 0x3C, 0x4D, 0x5E>>)\n    \"00:1b:21:3c:4d:5e\"\n\n    iex> SnmpKit.SnmpMgr.Format.mac_address([0, 27, 33, 60, 77, 94])\n    \"00:1b:21:3c:4d:5e\"\n\n    iex> SnmpKit.SnmpMgr.Format.mac_address(\"\\x00\\x1B\\x21\\x3C\\x4D\\x5E\")\n    \"00:1b:21:3c:4d:5e\"","ref":"SnmpKit.SnmpMgr.Format.html#mac_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.pretty_print/1","doc":"Pretty prints an SNMP result with type-aware formatting.\n\nTakes a 3-tuple `{oid_string, type, value}` and returns a formatted version\nwith human-readable values based on the SNMP type.","ref":"SnmpKit.SnmpMgr.Format.html#pretty_print/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.pretty_print/1","doc":"iex> SnmpKit.SnmpMgr.Format.pretty_print({\"1.3.6.1.2.1.1.3.0\", :timeticks, 12345678})\n    {\"1.3.6.1.2.1.1.3.0\", :timeticks, \"1 day, 10 hours, 17 minutes, 36 seconds\"}\n\n    iex> SnmpKit.SnmpMgr.Format.pretty_print({\"1.3.6.1.2.1.4.20.1.1.192.168.1.1\", :ip_address, <<192, 168, 1, 1>>})\n    {\"1.3.6.1.2.1.4.20.1.1.192.168.1.1\", :ip_address, \"192.168.1.1\"}","ref":"SnmpKit.SnmpMgr.Format.html#pretty_print/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.pretty_print_all/1","doc":"Pretty prints a list of SNMP results.","ref":"SnmpKit.SnmpMgr.Format.html#pretty_print_all/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.pretty_print_all/1","doc":"iex> results = [\n    ...>   {\"1.3.6.1.2.1.1.3.0\", :timeticks, 12345678},\n    ...>   {\"1.3.6.1.2.1.1.1.0\", :octet_string, \"Router\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Format.pretty_print_all(results)\n    [\n      {\"1.3.6.1.2.1.1.3.0\", :timeticks, \"1 day, 10 hours, 17 minutes, 36 seconds\"},\n      {\"1.3.6.1.2.1.1.1.0\", :octet_string, \"\"Router\"\"}\n    ]","ref":"SnmpKit.SnmpMgr.Format.html#pretty_print_all/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.speed/1","doc":"Formats network speeds (bits per second) into human-readable rates.","ref":"SnmpKit.SnmpMgr.Format.html#speed/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.speed/1","doc":"iex> SnmpKit.SnmpMgr.Format.speed(100_000_000)\n    \"100.0 Mbps\"\n\n    iex> SnmpKit.SnmpMgr.Format.speed(1_000_000_000)\n    \"1.0 Gbps\"","ref":"SnmpKit.SnmpMgr.Format.html#speed/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Format.uptime/1","doc":"Formats timeticks (hundredths of seconds) into human-readable uptime.","ref":"SnmpKit.SnmpMgr.Format.html#uptime/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Format.uptime/1","doc":"iex> SnmpKit.SnmpMgr.Format.uptime(12345678)\n    \"1 day, 10 hours, 17 minutes, 36 seconds\"\n\n    iex> SnmpKit.SnmpMgr.Format.uptime(4200)\n    \"42 seconds\"","ref":"SnmpKit.SnmpMgr.Format.html#uptime/1-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Metrics","doc":"Comprehensive metrics collection and monitoring for SNMP operations.\n\nThis module provides real-time metrics collection, aggregation, and reporting\nfor all SNMP operations including request latency, success rates, throughput,\nand resource utilization across engines, pools, and circuit breakers.","ref":"SnmpKit.SnmpMgr.Metrics.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.Metrics.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.counter/4","doc":"Records a counter metric.","ref":"SnmpKit.SnmpMgr.Metrics.html#counter/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Metrics.counter/4","doc":"- `metrics` - Metrics PID or name\n- `metric_name` - Name of the metric\n- `value` - Value to add (default: 1)\n- `tags` - Optional tags for the metric","ref":"SnmpKit.SnmpMgr.Metrics.html#counter/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Metrics.counter/4","doc":"SnmpKit.SnmpMgr.Metrics.counter(metrics, :requests_total, 1, %{target: \"device1\"})","ref":"SnmpKit.SnmpMgr.Metrics.html#counter/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.gauge/4","doc":"Records a gauge metric (current value).","ref":"SnmpKit.SnmpMgr.Metrics.html#gauge/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Metrics.gauge/4","doc":"- `metrics` - Metrics PID or name\n- `metric_name` - Name of the metric\n- `value` - Current value\n- `tags` - Optional tags for the metric","ref":"SnmpKit.SnmpMgr.Metrics.html#gauge/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Metrics.gauge/4","doc":"SnmpKit.SnmpMgr.Metrics.gauge(metrics, :active_connections, 15, %{pool: \"main\"})","ref":"SnmpKit.SnmpMgr.Metrics.html#gauge/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.get_metrics/1","doc":"Gets current metrics snapshot.","ref":"SnmpKit.SnmpMgr.Metrics.html#get_metrics/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.get_summary/1","doc":"Gets aggregated metrics summary.","ref":"SnmpKit.SnmpMgr.Metrics.html#get_summary/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.get_window_metrics/3","doc":"Gets metrics for a specific time window.","ref":"SnmpKit.SnmpMgr.Metrics.html#get_window_metrics/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.histogram/4","doc":"Records a histogram metric (for latency/duration measurements).","ref":"SnmpKit.SnmpMgr.Metrics.html#histogram/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Metrics.histogram/4","doc":"- `metrics` - Metrics PID or name\n- `metric_name` - Name of the metric\n- `value` - Value to record\n- `tags` - Optional tags for the metric","ref":"SnmpKit.SnmpMgr.Metrics.html#histogram/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Metrics.histogram/4","doc":"SnmpKit.SnmpMgr.Metrics.histogram(metrics, :request_duration_ms, 150, %{operation: \"get\"})","ref":"SnmpKit.SnmpMgr.Metrics.html#histogram/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.reset/1","doc":"Resets all metrics.","ref":"SnmpKit.SnmpMgr.Metrics.html#reset/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.start_link/1","doc":"Starts the metrics collector.","ref":"SnmpKit.SnmpMgr.Metrics.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpMgr.Metrics.start_link/1","doc":"- `:window_size` - Metrics window size in seconds (default: 60)\n- `:retention_period` - How long to keep metrics in seconds (default: 3600)\n- `:collection_interval` - Collection frequency in ms (default: 1000)","ref":"SnmpKit.SnmpMgr.Metrics.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Metrics.start_link/1","doc":"{:ok, metrics} = SnmpKit.SnmpMgr.Metrics.start_link(\n      window_size: 120,\n      retention_period: 7200\n    )","ref":"SnmpKit.SnmpMgr.Metrics.html#start_link/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.subscribe/2","doc":"Subscribes to metrics updates.","ref":"SnmpKit.SnmpMgr.Metrics.html#subscribe/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.time/4","doc":"Records timing for a function execution.","ref":"SnmpKit.SnmpMgr.Metrics.html#time/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Metrics.time/4","doc":"- `metrics` - Metrics PID or name\n- `metric_name` - Name of the metric\n- `fun` - Function to time\n- `tags` - Optional tags for the metric","ref":"SnmpKit.SnmpMgr.Metrics.html#time/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Metrics.time/4","doc":"result = SnmpKit.SnmpMgr.Metrics.time(metrics, :snmp_get_duration, fn ->\n      SnmpKit.SnmpMgr.get(\"device1\", \"sysDescr.0\")\n    end, %{device: \"device1\"})","ref":"SnmpKit.SnmpMgr.Metrics.html#time/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Metrics.unsubscribe/2","doc":"Unsubscribes from metrics updates.","ref":"SnmpKit.SnmpMgr.Metrics.html#unsubscribe/2"},{"type":"module","title":"SnmpKit.SnmpMgr.Multi","doc":"Concurrent multi-target SNMP operations.\n\nProvides functions to perform SNMP operations against multiple targets\nconcurrently, with configurable timeouts and error handling.","ref":"SnmpKit.SnmpMgr.Multi.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.execute_mixed/2","doc":"Executes mixed SNMP operations against multiple targets concurrently.\n\nAllows different operation types per target for maximum flexibility.","ref":"SnmpKit.SnmpMgr.Multi.html#execute_mixed/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.execute_mixed/2","doc":"- `operations` - List of {operation, target, oid_or_args, opts} tuples\n- `opts` - Global options","ref":"SnmpKit.SnmpMgr.Multi.html#execute_mixed/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.execute_mixed/2","doc":"iex> operations = [\n    ...>   {:get, \"device1\", \"sysDescr.0\", []},\n    ...>   {:get_bulk, \"switch1\", \"ifTable\", [max_repetitions: 20]},\n    ...>   {:walk, \"router1\", \"system\", [version: :v2c]}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Multi.execute_mixed(operations)\n    [\n      {:ok, \"Device 1 Description\"},\n      {:ok, [{\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"}, ...]},\n      {:ok, [{\"1.3.6.1.2.1.1.1.0\", \"Router 1\"}, ...]}\n    ]","ref":"SnmpKit.SnmpMgr.Multi.html#execute_mixed/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.get_bulk_multi/2","doc":"Performs GETBULK operations against multiple targets concurrently.","ref":"SnmpKit.SnmpMgr.Multi.html#get_bulk_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.get_bulk_multi/2","doc":"- `targets_and_oids` - List of {target, oid} or {target, oid, opts} tuples\n- `opts` - Global options applied to all requests","ref":"SnmpKit.SnmpMgr.Multi.html#get_bulk_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.get_bulk_multi/2","doc":"iex> requests = [\n    ...>   {\"switch1\", \"ifTable\"},\n    ...>   {\"switch2\", \"ifTable\"},\n    ...>   {\"router1\", \"ipRouteTable\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Multi.get_bulk_multi(requests, max_repetitions: 20)\n    [\n      {:ok, [{\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"}, ...]},\n      {:ok, [{\"1.3.6.1.2.1.2.2.1.2.1\", \"GigE0/1\"}, ...]},\n      {:error, :timeout}\n    ]","ref":"SnmpKit.SnmpMgr.Multi.html#get_bulk_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.get_multi/2","doc":"Performs GET operations against multiple targets concurrently.","ref":"SnmpKit.SnmpMgr.Multi.html#get_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.get_multi/2","doc":"- `targets_and_oids` - List of {target, oid} or {target, oid, opts} tuples\n- `opts` - Global options applied to all requests","ref":"SnmpKit.SnmpMgr.Multi.html#get_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.get_multi/2","doc":"iex> requests = [\n    ...>   {\"device1\", \"sysDescr.0\"},\n    ...>   {\"device2\", \"sysUpTime.0\"},\n    ...>   {\"device3\", \"ifNumber.0\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Multi.get_multi(requests)\n    [\n      {:ok, \"Device 1 Description\"},\n      {:ok, 123456},\n      {:error, :timeout}\n    ]","ref":"SnmpKit.SnmpMgr.Multi.html#get_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.monitor/3","doc":"Monitors multiple devices for changes by polling at regular intervals.","ref":"SnmpKit.SnmpMgr.Multi.html#monitor/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.monitor/3","doc":"- `targets_and_oids` - List of {target, oid} tuples to monitor\n- `callback` - Function called with {target, oid, old_value, new_value} when changes occur\n- `opts` - Options including :interval, :initial_poll, :max_concurrent","ref":"SnmpKit.SnmpMgr.Multi.html#monitor/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.monitor/3","doc":"targets = [{\"device1\", \"sysUpTime.0\"}, {\"device2\", \"ifInOctets.1\"}]\n    callback = fn change -> IO.inspect(change) end\n    {:ok, monitor_pid} = SnmpKit.SnmpMgr.Multi.monitor(targets, callback, interval: 30_000)","ref":"SnmpKit.SnmpMgr.Multi.html#monitor/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.walk_multi/2","doc":"Performs walk operations against multiple targets concurrently.","ref":"SnmpKit.SnmpMgr.Multi.html#walk_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.walk_multi/2","doc":"- `targets_and_oids` - List of {target, root_oid} or {target, root_oid, opts} tuples\n- `opts` - Global options applied to all requests","ref":"SnmpKit.SnmpMgr.Multi.html#walk_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.walk_multi/2","doc":"iex> requests = [\n    ...>   {\"device1\", \"system\"},\n    ...>   {\"device2\", \"interfaces\"},\n    ...>   {\"device3\", [1, 3, 6, 1, 2, 1, 4]}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Multi.walk_multi(requests, version: :v2c)\n    [\n      {:ok, [{\"1.3.6.1.2.1.1.1.0\", \"Device 1\"}, ...]},\n      {:ok, [{\"1.3.6.1.2.1.2.1.0\", 24}, ...]},\n      {:error, :timeout}\n    ]","ref":"SnmpKit.SnmpMgr.Multi.html#walk_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Multi.walk_table_multi/2","doc":"Performs table walk operations against multiple targets concurrently.","ref":"SnmpKit.SnmpMgr.Multi.html#walk_table_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Multi.walk_table_multi/2","doc":"- `targets_and_tables` - List of {target, table_oid} or {target, table_oid, opts} tuples\n- `opts` - Global options applied to all requests","ref":"SnmpKit.SnmpMgr.Multi.html#walk_table_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Multi.walk_table_multi/2","doc":"iex> requests = [\n    ...>   {\"switch1\", \"ifTable\"},\n    ...>   {\"switch2\", \"ifTable\"},\n    ...>   {\"router1\", \"ipRouteTable\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Multi.walk_table_multi(requests, version: :v2c)\n    [\n      {:ok, [{\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"}, ...]},\n      {:ok, [{\"1.3.6.1.2.1.2.2.1.2.1\", \"GigE0/1\"}, ...]},\n      {:error, :host_unreachable}\n    ]","ref":"SnmpKit.SnmpMgr.Multi.html#walk_table_multi/2-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Router","doc":"Intelligent request routing and load balancing for SNMP requests.\n\nThis module provides sophisticated routing strategies to optimize request\ndistribution across multiple engines and target devices, with support for\nload balancing, affinity routing, and performance-based routing decisions.","ref":"SnmpKit.SnmpMgr.Router.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.add_engine/2","doc":"Adds an engine to the routing pool.","ref":"SnmpKit.SnmpMgr.Router.html#add_engine/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.attempt_engine_recovery/2","doc":"Attempts to recover a failed engine.","ref":"SnmpKit.SnmpMgr.Router.html#attempt_engine_recovery/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.Router.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.configure_batch_strategy/2","doc":"Configures batch processing strategy.","ref":"SnmpKit.SnmpMgr.Router.html#configure_batch_strategy/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.configure_engines/2","doc":"Configures engine settings.","ref":"SnmpKit.SnmpMgr.Router.html#configure_engines/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.configure_health_check/2","doc":"Configures health check settings.","ref":"SnmpKit.SnmpMgr.Router.html#configure_health_check/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.get_engine_health/1","doc":"Gets engine health information.","ref":"SnmpKit.SnmpMgr.Router.html#get_engine_health/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.get_stats/1","doc":"Gets routing statistics and engine health.","ref":"SnmpKit.SnmpMgr.Router.html#get_stats/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.mark_engine_healthy/2","doc":"Marks an engine as healthy.","ref":"SnmpKit.SnmpMgr.Router.html#mark_engine_healthy/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.mark_engine_unhealthy/3","doc":"Marks an engine as unhealthy.","ref":"SnmpKit.SnmpMgr.Router.html#mark_engine_unhealthy/3"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.remove_engine/2","doc":"Removes an engine from the routing pool.","ref":"SnmpKit.SnmpMgr.Router.html#remove_engine/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.route_batch/3","doc":"Routes multiple requests as a batch.","ref":"SnmpKit.SnmpMgr.Router.html#route_batch/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Router.route_batch/3","doc":"- `router` - Router PID or name\n- `requests` - List of request specifications\n- `opts` - Routing options","ref":"SnmpKit.SnmpMgr.Router.html#route_batch/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Router.route_batch/3","doc":"requests = [\n      %{type: :get, target: \"device1\", oid: \"sysDescr.0\"},\n      %{type: :get, target: \"device2\", oid: \"sysUpTime.0\"}\n    ]\n\n    {:ok, results} = SnmpKit.SnmpMgr.Router.route_batch(router, requests)","ref":"SnmpKit.SnmpMgr.Router.html#route_batch/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.route_request/3","doc":"Routes a request to the best available engine.","ref":"SnmpKit.SnmpMgr.Router.html#route_request/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Router.route_request/3","doc":"- `router` - Router PID or name\n- `request` - Request specification\n- `opts` - Routing options","ref":"SnmpKit.SnmpMgr.Router.html#route_request/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Router.route_request/3","doc":"request = %{\n      type: :get,\n      target: \"192.168.1.1\",\n      oid: \"sysDescr.0\"\n    }\n\n    {:ok, result} = SnmpKit.SnmpMgr.Router.route_request(router, request)","ref":"SnmpKit.SnmpMgr.Router.html#route_request/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.set_engine_weights/2","doc":"Sets engine weights for weighted routing.","ref":"SnmpKit.SnmpMgr.Router.html#set_engine_weights/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.set_strategy/2","doc":"Updates routing strategy.","ref":"SnmpKit.SnmpMgr.Router.html#set_strategy/2"},{"type":"function","title":"SnmpKit.SnmpMgr.Router.start_link/1","doc":"Starts the request router.","ref":"SnmpKit.SnmpMgr.Router.html#start_link/1"},{"type":"function","title":"Options - SnmpKit.SnmpMgr.Router.start_link/1","doc":"- `:strategy` - Routing strategy (:round_robin, :least_connections, :weighted, :affinity)\n- `:engines` - List of engine specifications\n- `:health_check_interval` - Health check interval in ms (default: 30000)\n- `:max_retries` - Maximum retry attempts (default: 3)","ref":"SnmpKit.SnmpMgr.Router.html#start_link/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Router.start_link/1","doc":"{:ok, router} = SnmpKit.SnmpMgr.Router.start_link(\n      strategy: :least_connections,\n      engines: [\n        %{name: :engine1, weight: 2, max_load: 100},\n        %{name: :engine2, weight: 1, max_load: 50}\n      ]\n    )","ref":"SnmpKit.SnmpMgr.Router.html#start_link/1-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Stream","doc":"High-performance streaming SNMP operations for memory-efficient processing.\n\nThis module provides streaming APIs that allow processing of large SNMP datasets\nwithout loading everything into memory at once. Perfect for large tables and\nreal-time monitoring scenarios.","ref":"SnmpKit.SnmpMgr.Stream.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Stream.concurrent_stream/2","doc":"Creates a concurrent stream that processes multiple devices in parallel.\n\nCombines results from multiple devices into a single stream with\nconfigurable concurrency and ordering.","ref":"SnmpKit.SnmpMgr.Stream.html#concurrent_stream/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Stream.concurrent_stream/2","doc":"- `device_operations` - List of {target, operation, oid, opts} tuples\n- `opts` - Options including :max_concurrent, :ordered, :timeout","ref":"SnmpKit.SnmpMgr.Stream.html#concurrent_stream/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Stream.concurrent_stream/2","doc":"# Concurrent table walks\n    operations = [\n      {\"switch1\", :walk_table, \"ifTable\", []},\n      {\"switch2\", :walk_table, \"ifTable\", []},\n      {\"router1\", :walk, \"ipRouteTable\", []}\n    ]\n\n    operations\n    |> SnmpKit.SnmpMgr.Stream.concurrent_stream(max_concurrent: 3)\n    |> Stream.each(&process_device_data/1)\n    |> Stream.run()","ref":"SnmpKit.SnmpMgr.Stream.html#concurrent_stream/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Stream.filtered_stream/4","doc":"Creates a filtered stream that applies predicates during data fetching.\n\nThis is more efficient than Stream.filter/2 for large datasets as it\ncan skip unnecessary network requests based on OID patterns.","ref":"SnmpKit.SnmpMgr.Stream.html#filtered_stream/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Stream.filtered_stream/4","doc":"- `target` - The target device\n- `root_oid` - Starting OID\n- `filter_fn` - Function to filter OIDs and values\n- `opts` - Stream options","ref":"SnmpKit.SnmpMgr.Stream.html#filtered_stream/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Stream.filtered_stream/4","doc":"# Only fetch interface names (column 2)\n    filter_fn = fn {oid, _type, _value} ->\n      case SnmpKit.SnmpLib.OID.string_to_list(oid) do\n        {:ok, oid_list} -> List.last(oid_list, 2) |> hd() == 2\n        _ -> false\n      end\n    end\n\n    \"switch.local\"\n    |> SnmpKit.SnmpMgr.Stream.filtered_stream(\"ifTable\", filter_fn)\n    |> Enum.to_list()","ref":"SnmpKit.SnmpMgr.Stream.html#filtered_stream/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Stream.monitor_stream/2","doc":"Creates a real-time monitoring stream that polls devices at intervals.\n\nProvides a continuous stream of SNMP data for real-time monitoring\nand alerting applications.","ref":"SnmpKit.SnmpMgr.Stream.html#monitor_stream/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Stream.monitor_stream/2","doc":"- `targets` - List of {target, oid} tuples to monitor\n- `opts` - Options including :interval, :buffer_size, :error_handling","ref":"SnmpKit.SnmpMgr.Stream.html#monitor_stream/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Stream.monitor_stream/2","doc":"# Monitor multiple devices\n    targets = [\n      {\"switch1\", \"ifInOctets.1\"},\n      {\"switch2\", \"ifInOctets.1\"},\n      {\"router1\", \"sysUpTime.0\"}\n    ]\n\n    targets\n    |> SnmpKit.SnmpMgr.Stream.monitor_stream(interval: 30_000)\n    |> Stream.each(&send_to_metrics_system/1)\n    |> Stream.run()\n\n    # Monitor with error handling\n    targets\n    |> SnmpKit.SnmpMgr.Stream.monitor_stream(\n         interval: 10_000,\n         error_handling: :skip_errors\n       )\n    |> Stream.filter(&is_successful_reading/1)\n    |> Stream.each(&update_dashboard/1)\n    |> Stream.run()","ref":"SnmpKit.SnmpMgr.Stream.html#monitor_stream/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Stream.table_stream/3","doc":"Creates a stream for processing large SNMP tables.\n\nOptimized for table structures with intelligent chunking based on\ntable columns and indexes.","ref":"SnmpKit.SnmpMgr.Stream.html#table_stream/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Stream.table_stream/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to stream\n- `opts` - Options including :chunk_size, :columns, :indexes","ref":"SnmpKit.SnmpMgr.Stream.html#table_stream/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Stream.table_stream/3","doc":"# Stream interface table with column filtering\n    \"switch.local\"\n    |> SnmpKit.SnmpMgr.Stream.table_stream(\"ifTable\", columns: [:ifDescr, :ifOperStatus])\n    |> Stream.filter(fn {_index, data} -> data[:ifOperStatus] == 1 end)\n    |> Stream.map(fn {index, data} -> {index, data[:ifDescr]} end)\n    |> Enum.to_list()\n\n    # Process table with custom chunk size\n    \"device.local\"\n    |> SnmpKit.SnmpMgr.Stream.table_stream(\"ipRouteTable\", chunk_size: 200)\n    |> Stream.each(&process_route_entry/1)\n    |> Stream.run()","ref":"SnmpKit.SnmpMgr.Stream.html#table_stream/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Stream.walk_stream/3","doc":"Creates a stream for walking large SNMP trees without memory overhead.\n\nThe stream lazily fetches data in chunks, allowing processing of arbitrarily\nlarge SNMP trees with constant memory usage.","ref":"SnmpKit.SnmpMgr.Stream.html#walk_stream/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Stream.walk_stream/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :chunk_size, :version, :adaptive","ref":"SnmpKit.SnmpMgr.Stream.html#walk_stream/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Stream.walk_stream/3","doc":"# Process a large table efficiently\n    \"switch.local\"\n    |> SnmpKit.SnmpMgr.Stream.walk_stream(\"ifTable\")\n    |> Stream.filter(fn {_oid, value} -> String.contains?(value, \"Gigabit\") end)\n    |> Stream.map(&extract_interface_info/1)\n    |> Enum.to_list()\n\n    # Real-time processing with backpressure\n    \"router.local\"\n    |> SnmpKit.SnmpMgr.Stream.walk_stream(\"ipRouteTable\", chunk_size: 100)\n    |> Stream.each(&update_routing_database/1)\n    |> Stream.run()","ref":"SnmpKit.SnmpMgr.Stream.html#walk_stream/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Supervisor","doc":"Main supervisor for the SnmpMgr streaming PDU engine infrastructure.\n\nThis supervisor manages all Phase 5 components including engines, routers,\nconnection pools, circuit breakers, and metrics collection.","ref":"SnmpKit.SnmpMgr.Supervisor.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.Supervisor.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Supervisor.start_link/1","doc":"","ref":"SnmpKit.SnmpMgr.Supervisor.html#start_link/1"},{"type":"module","title":"SnmpKit.SnmpMgr.Table","doc":"Table processing utilities for SNMP table data.\n\nProvides functions to convert flat OID/type/value lists into structured\ntable representations and perform table analysis operations.","ref":"SnmpKit.SnmpMgr.Table.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.analyze/2","doc":"Analyzes table structure and returns metadata about the table.\n\nProvides detailed information about table dimensions, column types,\nmissing data, and statistical analysis.","ref":"SnmpKit.SnmpMgr.Table.html#analyze/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.analyze/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `opts` - Options including :analyze_types, :find_missing","ref":"SnmpKit.SnmpMgr.Table.html#analyze/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.analyze/2","doc":"iex> table = %{1 => %{2 => \"eth0\", 3 => 100}, 2 => %{2 => \"eth1\", 3 => 200}}\n    iex> SnmpKit.SnmpMgr.Table.analyze(table)\n    {:ok, %{\n      row_count: 2,\n      column_count: 2,\n      columns: [2, 3],\n      indexes: [1, 2],\n      completeness: 1.0,\n      column_types: %{2 => :string, 3 => :integer}\n    }}","ref":"SnmpKit.SnmpMgr.Table.html#analyze/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.column_stats/2","doc":"Calculates statistics for numeric columns in the table.","ref":"SnmpKit.SnmpMgr.Table.html#column_stats/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.column_stats/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `columns` - List of column numbers to analyze (optional, analyzes all numeric columns)","ref":"SnmpKit.SnmpMgr.Table.html#column_stats/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.column_stats/2","doc":"iex> table = %{1 => %{2 => \"eth0\", 3 => 100}, 2 => %{2 => \"eth1\", 3 => 200}}\n    iex> SnmpKit.SnmpMgr.Table.column_stats(table, [3])\n    {:ok, %{3 => %{count: 2, sum: 300, avg: 150.0, min: 100, max: 200}}}","ref":"SnmpKit.SnmpMgr.Table.html#column_stats/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.filter_by_column/3","doc":"Filters table data by column values using a filter function.","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_column/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.filter_by_column/3","doc":"- `table_data` - Table data as returned by to_table/2\n- `column` - Column number to filter on\n- `filter_fn` - Function that takes a value and returns boolean","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_column/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.filter_by_column/3","doc":"iex> table = %{1 => %{2 => \"eth0\", 3 => 1}, 2 => %{2 => \"eth1\", 3 => 0}}\n    iex> SnmpKit.SnmpMgr.Table.filter_by_column(table, 3, fn val -> val == 1 end)\n    {:ok, %{1 => %{2 => \"eth0\", 3 => 1}}}","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_column/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.filter_by_index/2","doc":"Filters table data by index using a filter function.","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_index/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.filter_by_index/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `index_filter` - Function that takes an index and returns boolean","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_index/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.filter_by_index/2","doc":"iex> table = %{1 => %{2 => \"eth0\"}, 2 => %{2 => \"eth1\"}, 10 => %{2 => \"lo\"}}\n    iex> SnmpKit.SnmpMgr.Table.filter_by_index(table, fn index -> index < 10 end)\n    {:ok, %{1 => %{2 => \"eth0\"}, 2 => %{2 => \"eth1\"}}}","ref":"SnmpKit.SnmpMgr.Table.html#filter_by_index/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.find_duplicates/2","doc":"Finds duplicate rows in the table based on specified columns.","ref":"SnmpKit.SnmpMgr.Table.html#find_duplicates/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.find_duplicates/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `columns` - List of column numbers to check for duplicates","ref":"SnmpKit.SnmpMgr.Table.html#find_duplicates/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.find_duplicates/2","doc":"iex> table = %{1 => %{2 => \"eth\", 3 => 1}, 2 => %{2 => \"eth\", 3 => 1}}\n    iex> SnmpKit.SnmpMgr.Table.find_duplicates(table, [2, 3])\n    {:ok, [[{1, %{2 => \"eth\", 3 => 1}}, {2, %{2 => \"eth\", 3 => 1}}]]}","ref":"SnmpKit.SnmpMgr.Table.html#find_duplicates/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.get_columns/1","doc":"Extracts all unique column numbers from table data.","ref":"SnmpKit.SnmpMgr.Table.html#get_columns/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.get_indexes/1","doc":"Extracts all unique indexes from table data.","ref":"SnmpKit.SnmpMgr.Table.html#get_indexes/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.group_by_column/2","doc":"Groups table rows by values in a specific column.","ref":"SnmpKit.SnmpMgr.Table.html#group_by_column/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.group_by_column/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `column` - Column number to group by","ref":"SnmpKit.SnmpMgr.Table.html#group_by_column/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.group_by_column/2","doc":"iex> table = %{1 => %{2 => \"eth\", 3 => 1}, 2 => %{2 => \"lo\", 3 => 1}}\n    iex> SnmpKit.SnmpMgr.Table.group_by_column(table, 3)\n    {:ok, %{1 => [%{index: 1, 2 => \"eth\", 3 => 1}, %{index: 2, 2 => \"lo\", 3 => 1}]}}","ref":"SnmpKit.SnmpMgr.Table.html#group_by_column/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.sort_by_column/3","doc":"Sorts table data by a specific column.","ref":"SnmpKit.SnmpMgr.Table.html#sort_by_column/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.sort_by_column/3","doc":"- `table_data` - Table data as returned by to_table/2\n- `column` - Column number to sort by\n- `direction` - :asc or :desc (default: :asc)","ref":"SnmpKit.SnmpMgr.Table.html#sort_by_column/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.sort_by_column/3","doc":"iex> table = %{1 => %{2 => \"eth1\"}, 2 => %{2 => \"eth0\"}}\n    iex> SnmpKit.SnmpMgr.Table.sort_by_column(table, 2)\n    {:ok, [{2, %{2 => \"eth0\"}}, {1, %{2 => \"eth1\"}}]}","ref":"SnmpKit.SnmpMgr.Table.html#sort_by_column/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.to_list/1","doc":"Converts table data to a list format for easier processing.","ref":"SnmpKit.SnmpMgr.Table.html#to_list/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.to_list/1","doc":"iex> table = %{1 => %{2 => \"eth0\", 3 => 6}, 2 => %{2 => \"eth1\", 3 => 6}}\n    iex> SnmpKit.SnmpMgr.Table.to_list(table)\n    {:ok, [\n      %{index: 1, 2 => \"eth0\", 3 => 6},\n      %{index: 2, 2 => \"eth1\", 3 => 6}\n    ]}","ref":"SnmpKit.SnmpMgr.Table.html#to_list/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.to_map/2","doc":"Converts table data to a map keyed by a specific column.","ref":"SnmpKit.SnmpMgr.Table.html#to_map/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.to_map/2","doc":"- `oid_type_value_tuples` - List of {oid_string, type, value} tuples\n- `key_column` - Column number to use as the key","ref":"SnmpKit.SnmpMgr.Table.html#to_map/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.to_map/2","doc":"iex> tuples = [\n    ...>   {\"1.3.6.1.2.1.2.2.1.1.1\", :integer, 1},\n    ...>   {\"1.3.6.1.2.1.2.2.1.2.1\", :string, \"eth0\"},\n    ...>   {\"1.3.6.1.2.1.2.2.1.1.2\", :integer, 2},\n    ...>   {\"1.3.6.1.2.1.2.2.1.2.2\", :string, \"eth1\"}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Table.to_map(tuples, 1)\n    {:ok, %{\n      1 => %{ifIndex: 1, ifDescr: \"eth0\"},\n      2 => %{ifIndex: 2, ifDescr: \"eth1\"}\n    }}","ref":"SnmpKit.SnmpMgr.Table.html#to_map/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.to_rows/1","doc":"Converts OID/type/value tuples to a list of row maps.\n\nEach row is a map where keys are column numbers and values are the data values.","ref":"SnmpKit.SnmpMgr.Table.html#to_rows/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.to_table/2","doc":"Converts flat OID/type/value tuples to a structured table format.\n\nTakes a list of {oid_string, type, value} tuples from a table walk\nand converts them into a structured table with rows and columns.","ref":"SnmpKit.SnmpMgr.Table.html#to_table/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.to_table/2","doc":"- `oid_type_value_tuples` - List of {oid_string, type, value} tuples\n- `table_oid` - The base table OID (used to determine table structure)","ref":"SnmpKit.SnmpMgr.Table.html#to_table/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.to_table/2","doc":"iex> tuples = [\n    ...>   {\"1.3.6.1.2.1.2.2.1.2.1\", :string, \"eth0\"},\n    ...>   {\"1.3.6.1.2.1.2.2.1.2.2\", :string, \"eth1\"},\n    ...>   {\"1.3.6.1.2.1.2.2.1.3.1\", :integer, 6},\n    ...>   {\"1.3.6.1.2.1.2.2.1.3.2\", :integer, 6}\n    ...> ]\n    iex> SnmpKit.SnmpMgr.Table.to_table(tuples, [1, 3, 6, 1, 2, 1, 2, 2])\n    {:ok, %{\n      1 => %{2 => \"eth0\", 3 => 6},\n      2 => %{2 => \"eth1\", 3 => 6}\n    }}","ref":"SnmpKit.SnmpMgr.Table.html#to_table/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Table.validate/2","doc":"Validates table data integrity and consistency.","ref":"SnmpKit.SnmpMgr.Table.html#validate/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Table.validate/2","doc":"- `table_data` - Table data as returned by to_table/2\n- `opts` - Validation options","ref":"SnmpKit.SnmpMgr.Table.html#validate/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Table.validate/2","doc":"iex> table = %{1 => %{2 => \"eth0\", 3 => 100}}\n    iex> SnmpKit.SnmpMgr.Table.validate(table)\n    {:ok, %{valid: true, issues: []}}","ref":"SnmpKit.SnmpMgr.Table.html#validate/2-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Target","doc":"Target parsing and validation for SNMP requests.\n\nHandles parsing of various target formats and resolves hostnames to IP addresses.","ref":"SnmpKit.SnmpMgr.Target.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Target.parse/1","doc":"Parses a target string into a structured format.","ref":"SnmpKit.SnmpMgr.Target.html#parse/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Target.parse/1","doc":"iex> SnmpKit.SnmpMgr.Target.parse(\"192.168.1.1:161\")\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}\n\n    iex> SnmpKit.SnmpMgr.Target.parse(\"device.local\")\n    {:ok, %{host: \"device.local\", port: 161}}\n\n    iex> SnmpKit.SnmpMgr.Target.parse(\"192.168.1.1\")\n    {:ok, %{host: {192, 168, 1, 1}, port: 161}}","ref":"SnmpKit.SnmpMgr.Target.html#parse/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Target.resolve_hostname/1","doc":"Resolves a hostname to an IP address if needed.\n\nIf the host is already an IP tuple, returns it unchanged.","ref":"SnmpKit.SnmpMgr.Target.html#resolve_hostname/1"},{"type":"function","title":"SnmpKit.SnmpMgr.Target.validate_connectivity/2","doc":"Validates that a target is reachable (basic connectivity check).","ref":"SnmpKit.SnmpMgr.Target.html#validate_connectivity/2"},{"type":"module","title":"SnmpKit.SnmpMgr.Types","doc":"SNMP data type handling and conversion.\n\nHandles encoding and decoding of SNMP values, including automatic type inference\nand explicit type specification.","ref":"SnmpKit.SnmpMgr.Types.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Types.decode_value/1","doc":"Decodes an SNMP value to an Elixir term.","ref":"SnmpKit.SnmpMgr.Types.html#decode_value/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Types.decode_value/1","doc":"iex> SnmpKit.SnmpMgr.Types.decode_value({:string, \"Hello\"})\n    \"Hello\"\n\n    iex> SnmpKit.SnmpMgr.Types.decode_value({:integer, 42})\n    42","ref":"SnmpKit.SnmpMgr.Types.html#decode_value/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Types.encode_value/2","doc":"Encodes a value for SNMP with optional type specification.","ref":"SnmpKit.SnmpMgr.Types.html#encode_value/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Types.encode_value/2","doc":"- `value` - The value to encode\n- `opts` - Options including :type for explicit type specification","ref":"SnmpKit.SnmpMgr.Types.html#encode_value/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Types.encode_value/2","doc":"iex> SnmpKit.SnmpMgr.Types.encode_value(\"Hello World\")\n    {:ok, {:string, \"Hello World\"}}\n\n    iex> SnmpKit.SnmpMgr.Types.encode_value(42)\n    {:ok, {:integer, 42}}\n\n    iex> SnmpKit.SnmpMgr.Types.encode_value(\"192.168.1.1\", type: :ipAddress)\n    {:ok, {:ipAddress, {192, 168, 1, 1}}}","ref":"SnmpKit.SnmpMgr.Types.html#encode_value/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Types.infer_type/1","doc":"Automatically infers the SNMP type from an Elixir value.","ref":"SnmpKit.SnmpMgr.Types.html#infer_type/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Types.infer_type/1","doc":"iex> SnmpKit.SnmpMgr.Types.infer_type(\"hello\")\n    :string\n\n    iex> SnmpKit.SnmpMgr.Types.infer_type(42)\n    :integer\n\n    iex> SnmpKit.SnmpMgr.Types.infer_type(\"192.168.1.1\")\n    :string  # Would need explicit :ipAddress type","ref":"SnmpKit.SnmpMgr.Types.html#infer_type/1-examples"},{"type":"module","title":"SnmpKit.SnmpMgr.Walk","doc":"SNMP walk operations using iterative GETNEXT requests.\n\nThis module provides efficient walking of SNMP trees and tables\nusing the GETNEXT operation repeatedly until the end of the subtree.","ref":"SnmpKit.SnmpMgr.Walk.html"},{"type":"function","title":"SnmpKit.SnmpMgr.Walk.walk/3","doc":"Performs a walk starting from the given root OID.\n\nAutomatically chooses between GETNEXT (SNMPv1) and GETBULK (SNMPv2c)\nbased on the version specified in options.","ref":"SnmpKit.SnmpMgr.Walk.html#walk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Walk.walk/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :version, :max_repetitions, :timeout, :community","ref":"SnmpKit.SnmpMgr.Walk.html#walk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.Walk.walk/3","doc":"iex> SnmpKit.SnmpMgr.Walk.walk(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1])\n    {:ok, [\n      {\"1.3.6.1.2.1.1.1.0\", \"System description\"},\n      {\"1.3.6.1.2.1.1.2.0\", \"1.3.6.1.4.1.9.1.1\"},\n      {\"1.3.6.1.2.1.1.3.0\", 12345}\n    ]}","ref":"SnmpKit.SnmpMgr.Walk.html#walk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.Walk.walk_column/3","doc":"Walks a specific table column.","ref":"SnmpKit.SnmpMgr.Walk.html#walk_column/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Walk.walk_column/3","doc":"- `target` - The target device\n- `column_oid` - The full column OID (table + entry + column)\n- `opts` - Options","ref":"SnmpKit.SnmpMgr.Walk.html#walk_column/3-parameters"},{"type":"function","title":"SnmpKit.SnmpMgr.Walk.walk_table/3","doc":"Walks an SNMP table starting from the table OID.\n\nAutomatically chooses between GETNEXT and GETBULK based on version.\nGETBULK provides significantly better performance for large tables.","ref":"SnmpKit.SnmpMgr.Walk.html#walk_table/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.Walk.walk_table/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to walk\n- `opts` - Options including :version, :max_repetitions, :timeout, :community","ref":"SnmpKit.SnmpMgr.Walk.html#walk_table/3-parameters"},{"type":"module","title":"SnmpKit.SnmpSim.BehaviorConfig","doc":"Configuration system for enhancing walk files with realistic behaviors.\nProvides easy-to-use behavior presets and customization options.","ref":"SnmpKit.SnmpSim.BehaviorConfig.html"},{"type":"function","title":"SnmpKit.SnmpSim.BehaviorConfig.apply_behaviors/2","doc":"Apply behavior configurations to a loaded profile.","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#apply_behaviors/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BehaviorConfig.apply_behaviors/2","doc":"# Apply basic realistic behaviors\n    enhanced_profile = SnmpKit.SnmpSim.BehaviorConfig.apply_behaviors(profile, [\n      :realistic_counters,\n      :daily_patterns,\n      {:custom_utilization, peak_hours: {9, 17}}\n    ])","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#apply_behaviors/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BehaviorConfig.create_custom/1","doc":"Create custom behavior configuration.","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#create_custom/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BehaviorConfig.create_custom/1","doc":"config = SnmpKit.SnmpSim.BehaviorConfig.create_custom([\n      {:traffic_counters, %{\n        rate_multiplier: 1.5,\n        daily_pattern: true,\n        burst_probability: 0.2\n      }},\n      {:signal_quality, %{\n        base_snr: 25,\n        weather_impact: true,\n        degradation_rate: 0.1\n      }}\n    ])","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#create_custom/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BehaviorConfig.get_preset/1","doc":"Get predefined behavior configuration sets.","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#get_preset/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BehaviorConfig.get_preset/1","doc":"# Cable modem realistic simulation\n    behaviors = SnmpKit.SnmpSim.BehaviorConfig.get_preset(:cable_modem_realistic)\n\n    # High traffic simulation\n    behaviors = SnmpKit.SnmpSim.BehaviorConfig.get_preset(:high_traffic_simulation)","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#get_preset/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BehaviorConfig.list_available_behaviors/0","doc":"Get available behavior types and their configurations.","ref":"SnmpKit.SnmpSim.BehaviorConfig.html#list_available_behaviors/0"},{"type":"module","title":"SnmpKit.SnmpSim.BulkOperations","doc":"Efficient GETBULK implementation for SNMPv2c.\nHandles non-repeaters, max-repetitions, and response size management.\n\nGETBULK is a powerful SNMP operation that retrieves multiple variables in a single request.\nIt's particularly useful for retrieving large tables like interface statistics.","ref":"SnmpKit.SnmpSim.BulkOperations.html"},{"type":"module","title":"Algorithm - SnmpKit.SnmpSim.BulkOperations","doc":"1. Process first N variable bindings as non-repeaters (like GETNEXT)\n2. For remaining variable bindings, repeat up to max-repetitions times\n3. Respect UDP packet size limits (typically 1472 bytes for Ethernet)\n4. Return tooBig error if response would exceed limits","ref":"SnmpKit.SnmpSim.BulkOperations.html#module-algorithm"},{"type":"function","title":"SnmpKit.SnmpSim.BulkOperations.estimate_response_size/1","doc":"Calculate estimated response size for a list of variable bindings.\nUsed for response size management.","ref":"SnmpKit.SnmpSim.BulkOperations.html#estimate_response_size/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BulkOperations.estimate_response_size/1","doc":"size = SnmpKit.SnmpSim.BulkOperations.estimate_response_size(varbinds)","ref":"SnmpKit.SnmpSim.BulkOperations.html#estimate_response_size/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BulkOperations.handle_bulk_request/4","doc":"Handle a GETBULK request with proper non-repeaters and max-repetitions processing.","ref":"SnmpKit.SnmpSim.BulkOperations.html#handle_bulk_request/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.BulkOperations.handle_bulk_request/4","doc":"- `oid_tree`: The OID tree to query\n- `non_repeaters`: Number of variables to treat as non-repeating (GETNEXT only)\n- `max_repetitions`: Maximum number of repetitions for repeating variables\n- `varbinds`: List of starting OIDs for the bulk operation","ref":"SnmpKit.SnmpSim.BulkOperations.html#handle_bulk_request/4-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.BulkOperations.handle_bulk_request/4","doc":"- `{:ok, result_varbinds}`: Successful bulk operation results\n- `{:error, :too_big}`: Response would exceed UDP size limits\n- `{:error, reason}`: Other error conditions","ref":"SnmpKit.SnmpSim.BulkOperations.html#handle_bulk_request/4-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BulkOperations.handle_bulk_request/4","doc":"varbinds = [{\"1.3.6.1.2.1.2.2.1.1\", nil}, {\"1.3.6.1.2.1.2.2.1.2\", nil}]\n    {:ok, results} = SnmpKit.SnmpSim.BulkOperations.handle_bulk_request(\n      tree, 0, 10, varbinds\n    )","ref":"SnmpKit.SnmpSim.BulkOperations.html#handle_bulk_request/4-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BulkOperations.optimize_bulk_response/2","doc":"Optimize bulk response to fit within UDP packet size limits.\nEstimates response size and truncates if necessary.","ref":"SnmpKit.SnmpSim.BulkOperations.html#optimize_bulk_response/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.BulkOperations.optimize_bulk_response/2","doc":"- `results`: List of {oid, value, behavior} tuples\n- `max_size`: Maximum response size in bytes (default: 1400)","ref":"SnmpKit.SnmpSim.BulkOperations.html#optimize_bulk_response/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.BulkOperations.optimize_bulk_response/2","doc":"- `{:ok, optimized_results}`: Results that fit within size limit\n- `{:error, :too_big}`: Even minimal response exceeds size limit","ref":"SnmpKit.SnmpSim.BulkOperations.html#optimize_bulk_response/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BulkOperations.optimize_bulk_response/2","doc":"{:ok, optimized} = SnmpKit.SnmpSim.BulkOperations.optimize_bulk_response(results, 1400)","ref":"SnmpKit.SnmpSim.BulkOperations.html#optimize_bulk_response/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.BulkOperations.process_interface_table/3","doc":"Process an interface table bulk request efficiently.\nOptimized for common SNMP table walking operations.","ref":"SnmpKit.SnmpSim.BulkOperations.html#process_interface_table/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.BulkOperations.process_interface_table/3","doc":"{:ok, results} = SnmpKit.SnmpSim.BulkOperations.process_interface_table(\n      tree, \"1.3.6.1.2.1.2.2.1\", 10\n    )","ref":"SnmpKit.SnmpSim.BulkOperations.html#process_interface_table/3-examples"},{"type":"module","title":"SnmpKit.SnmpSim.Config","doc":"Configuration management for SnmpSim with support for JSON and YAML files.\n\nThis module provides a convenient way to configure SnmpSim using external\nconfiguration files, making it especially useful for container deployments\nand development environments.","ref":"SnmpKit.SnmpSim.Config.html"},{"type":"module","title":"Supported formats - SnmpKit.SnmpSim.Config","doc":"- JSON files\n- YAML files (requires `yaml_elixir` dependency)\n- Elixir configuration maps","ref":"SnmpKit.SnmpSim.Config.html#module-supported-formats"},{"type":"module","title":"Usage - SnmpKit.SnmpSim.Config","doc":"# Load from JSON file\n    {:ok, config} = SnmpKit.SnmpSim.Config.load_from_file(\"config/devices.json\")\n    {:ok, devices} = SnmpKit.SnmpSim.Config.start_from_config(config)\n\n    # Load from YAML file\n    {:ok, config} = SnmpKit.SnmpSim.Config.load_yaml(\"config/devices.yaml\")\n    {:ok, devices} = SnmpKit.SnmpSim.Config.start_from_config(config)\n\n    # Load from environment\n    {:ok, config} = SnmpKit.SnmpSim.Config.load_from_environment()\n    {:ok, devices} = SnmpKit.SnmpSim.Config.start_from_config(config)","ref":"SnmpKit.SnmpSim.Config.html#module-usage"},{"type":"function","title":"SnmpKit.SnmpSim.Config.load_from_environment/0","doc":"Loads configuration from environment variables.\n\nThis function reads common environment variables and creates a configuration\nmap suitable for starting devices and configuring the system.","ref":"SnmpKit.SnmpSim.Config.html#load_from_environment/0"},{"type":"function","title":"SnmpKit.SnmpSim.Config.load_from_file/1","doc":"Loads configuration from a JSON file.","ref":"SnmpKit.SnmpSim.Config.html#load_from_file/1"},{"type":"function","title":"SnmpKit.SnmpSim.Config.load_yaml/1","doc":"Loads configuration from a YAML file.\n\nRequires the `yaml_elixir` dependency to be added to your mix.exs:\n\n    {:yaml_elixir, \"~> 2.9\"}","ref":"SnmpKit.SnmpSim.Config.html#load_yaml/1"},{"type":"function","title":"SnmpKit.SnmpSim.Config.sample_config/0","doc":"Creates a sample configuration for reference.","ref":"SnmpKit.SnmpSim.Config.html#sample_config/0"},{"type":"function","title":"SnmpKit.SnmpSim.Config.start_from_config/1","doc":"Starts devices and services based on the provided configuration.","ref":"SnmpKit.SnmpSim.Config.html#start_from_config/1"},{"type":"function","title":"SnmpKit.SnmpSim.Config.validate_config/1","doc":"Validates a configuration map and provides helpful error messages.","ref":"SnmpKit.SnmpSim.Config.html#validate_config/1"},{"type":"function","title":"SnmpKit.SnmpSim.Config.write_sample_config/2","doc":"Writes a sample configuration to a file.","ref":"SnmpKit.SnmpSim.Config.html#write_sample_config/2"},{"type":"module","title":"SnmpKit.SnmpSim.Core.Server","doc":"High-performance UDP server for SNMP request handling.\nSupports concurrent packet processing with minimal latency.","ref":"SnmpKit.SnmpSim.Core.Server.html"},{"type":"function","title":"SnmpKit.SnmpSim.Core.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Core.Server.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Core.Server.get_stats/1","doc":"Get server statistics.","ref":"SnmpKit.SnmpSim.Core.Server.html#get_stats/1"},{"type":"function","title":"SnmpKit.SnmpSim.Core.Server.set_device_handler/2","doc":"Update the device handler function.","ref":"SnmpKit.SnmpSim.Core.Server.html#set_device_handler/2"},{"type":"function","title":"SnmpKit.SnmpSim.Core.Server.start_link/2","doc":"Start an SNMP UDP server on the specified port.","ref":"SnmpKit.SnmpSim.Core.Server.html#start_link/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.Core.Server.start_link/2","doc":"- `:community` - SNMP community string (default: \"public\")\n- `:device_handler` - Module or function to handle device requests\n- `:socket_opts` - Additional socket options","ref":"SnmpKit.SnmpSim.Core.Server.html#start_link/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.Core.Server.start_link/2","doc":"{:ok, server} = SnmpKit.SnmpSim.Core.Server.start_link(9001,\n      community: \"public\",\n      device_handler: &MyDevice.handle_request/2\n    )","ref":"SnmpKit.SnmpSim.Core.Server.html#start_link/2-examples"},{"type":"module","title":"SnmpKit.SnmpSim.CorrelationEngine","doc":"Implement realistic correlations between different metrics.\n\nNetwork metrics don't exist in isolation - they influence each other in predictable ways:\n- Signal quality degrades with higher utilization\n- Error rates increase with poor signal quality\n- Temperature affects equipment performance\n- Power consumption correlates with activity levels\n\nThis module provides sophisticated correlation modeling for authentic network simulation.","ref":"SnmpKit.SnmpSim.CorrelationEngine.html"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.apply_correlations/5","doc":"Apply correlations to a device's metrics based on primary metric changes.","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#apply_correlations/5"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.CorrelationEngine.apply_correlations/5","doc":"device_state = %{\n      interface_utilization: 0.8,\n      signal_quality: 85.0,\n      temperature: 45.0\n    }\n\n    correlations = [\n      {:interface_utilization, :error_rate, :positive, 0.7},\n      {:signal_quality, :throughput, :positive, 0.9},\n      {:temperature, :cpu_usage, :positive, 0.6}\n    ]\n\n    updated_state = SnmpKit.SnmpSim.CorrelationEngine.apply_correlations(\n      :interface_utilization, 0.8, device_state, correlations, DateTime.utc_now()\n    )","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#apply_correlations/5-examples"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.calculate_power_consumption_correlation/2","doc":"Model power consumption correlations with activity and temperature.\n\nPower consumption correlates with:\n- CPU utilization\n- Network activity\n- Temperature (cooling requirements)","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#calculate_power_consumption_correlation/2"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.calculate_signal_throughput_correlation/3","doc":"Calculate signal quality impact on throughput for DOCSIS devices.\n\nSignal quality (SNR, power levels) directly affects achievable throughput\nin cable modem systems.","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#calculate_signal_throughput_correlation/3"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.calculate_temperature_performance_correlation/2","doc":"Calculate temperature impact on equipment performance.\n\nHigher temperatures affect:\n- CPU performance (thermal throttling)\n- Signal quality (thermal noise)\n- Error rates (increased bit errors)","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#calculate_temperature_performance_correlation/2"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.calculate_utilization_error_correlation/2","doc":"Calculate utilization impact on error rates.\n\nHigher utilization typically leads to increased error rates due to:\n- Buffer overflows\n- Increased collision probability\n- Thermal effects","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#calculate_utilization_error_correlation/2"},{"type":"type","title":"SnmpKit.SnmpSim.CorrelationEngine.correlation_config/0","doc":"","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#t:correlation_config/0"},{"type":"type","title":"SnmpKit.SnmpSim.CorrelationEngine.correlation_type/0","doc":"","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#t:correlation_type/0"},{"type":"function","title":"SnmpKit.SnmpSim.CorrelationEngine.get_device_correlations/1","doc":"Get standard correlation configurations for common device types.","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#get_device_correlations/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.CorrelationEngine.get_device_correlations/1","doc":"correlations = SnmpKit.SnmpSim.CorrelationEngine.get_device_correlations(:cable_modem)","ref":"SnmpKit.SnmpSim.CorrelationEngine.html#get_device_correlations/1-examples"},{"type":"module","title":"SnmpKit.SnmpSim.Device.ErrorInjector","doc":"Error injection functionality for SNMP device simulation.\nHandles various error conditions like timeouts, packet loss, SNMP errors, and malformed responses.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.check_error_conditions/2","doc":"Check if any error conditions should be applied to the PDU.\nReturns appropriate error response or :continue to proceed normally.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#check_error_conditions/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.check_malformed_conditions/2","doc":"Check if malformed response conditions should be applied.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#check_malformed_conditions/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.check_packet_loss_conditions/2","doc":"Check if packet loss conditions should be applied.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#check_packet_loss_conditions/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.check_snmp_error_conditions/2","doc":"Check if SNMP error conditions should be applied.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#check_snmp_error_conditions/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.check_timeout_conditions/2","doc":"Check if timeout conditions should be applied.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#check_timeout_conditions/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.create_malformed_response/2","doc":"Create a malformed response based on the error configuration.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#create_malformed_response/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.device_failure_active?/1","doc":"Check if device failure condition is active.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#device_failure_active?/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.oid_matches_target?/2","doc":"Check if PDU OIDs match target OIDs for error injection.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#oid_matches_target?/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.ErrorInjector.should_apply_error?/1","doc":"Determine if error should be applied based on probability.","ref":"SnmpKit.SnmpSim.Device.ErrorInjector.html#should_apply_error?/1"},{"type":"module","title":"SnmpKit.SnmpSim.Device.OidHandler","doc":"OID handling and value generation for SNMP device simulation.\nHandles dynamic OID value generation, interface statistics, and MIB walking.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.build_correlation_factors/1","doc":"Builds correlation factors for related OIDs.\n\nThis can be expanded to track relationships between different metrics.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_correlation_factors/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.build_correlation_factors/1","doc":"- `_state` - Device state (currently unused)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_correlation_factors/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.build_correlation_factors/1","doc":"- Map of correlation factors (currently empty)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_correlation_factors/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.build_device_state/1","doc":"Builds comprehensive device state for monitoring and OID responses.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_device_state/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.build_device_state/1","doc":"- `state` - Current device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_device_state/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.build_device_state/1","doc":"- Map containing calculated device metrics and status information","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#build_device_state/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_gauge/1","doc":"Calculates CPU utilization gauge with realistic load patterns.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_gauge/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_gauge/1","doc":"- `state` - Device state containing device_type","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_gauge/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_gauge/1","doc":"- Integer representing CPU utilization percentage (0-100)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_gauge/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_utilization/1","doc":"Calculates CPU utilization correlated with network activity.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_utilization/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_utilization/1","doc":"- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_utilization/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_cpu_utilization/1","doc":"- Float representing CPU utilization (0.0 to 1.0)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_cpu_utilization/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_error_increment/2","doc":"Calculates error increment for error counters with environmental factors.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_increment/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_error_increment/2","doc":"- `state` - Device state\n- `counter_type` - Type of error counter (:in_errors, :out_errors, etc.)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_increment/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_error_increment/2","doc":"- Integer representing error count increment since device start","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_increment/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_error_rate/1","doc":"Calculates error rate as a percentage based on signal quality.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_rate/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_error_rate/1","doc":"- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_rate/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_error_rate/1","doc":"- Float representing error rate percentage (0.0 to 0.05)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_error_rate/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_health_score/1","doc":"Calculates overall device health score.\n\nHealth score is based on signal quality, error rate, and uptime stability.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_health_score/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_health_score/1","doc":"- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_health_score/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_health_score/1","doc":"- Float representing health score (0.0 to 1.0)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_health_score/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_interface_utilization/1","doc":"Calculates interface utilization as a percentage.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_interface_utilization/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_interface_utilization/1","doc":"- `_state` - Device state (currently unused)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_interface_utilization/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_interface_utilization/1","doc":"- Float representing interface utilization (0.1 to 0.8)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_interface_utilization/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_packet_increment/2","doc":"Calculates packet increment for packet counters.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_packet_increment/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_packet_increment/2","doc":"- `state` - Device state\n- `counter_type` - Type of packet counter (:in_ucast_pkts, :out_ucast_pkts, etc.)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_packet_increment/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_packet_increment/2","doc":"- Integer representing packet count increment since device start","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_packet_increment/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_signal_quality/1","doc":"Calculates signal quality metric.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_signal_quality/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_signal_quality/1","doc":"- `_state` - Device state (currently unused)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_signal_quality/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_signal_quality/1","doc":"- Float representing signal quality (0.0 to 1.0)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_signal_quality/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_snr_gauge/1","doc":"Calculates Signal-to-Noise Ratio (SNR) gauge value for cable modems.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_snr_gauge/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_snr_gauge/1","doc":"- `state` - Device state containing device_type","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_snr_gauge/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_snr_gauge/1","doc":"- Integer representing SNR in dB (15-45 range for cable modems)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_snr_gauge/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_storage_gauge/1","doc":"Calculates storage usage gauge in allocation units (typically KB).","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_storage_gauge/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_storage_gauge/1","doc":"- `state` - Device state containing device_type","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_storage_gauge/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_storage_gauge/1","doc":"- Integer representing storage usage in allocation units","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_storage_gauge/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_temperature/1","doc":"Calculates device temperature in Celsius.\n\nTemperature is affected by CPU load and ambient conditions.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_temperature/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_temperature/1","doc":"- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_temperature/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_temperature/1","doc":"- Float representing temperature in Celsius","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_temperature/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_traffic_increment/2","doc":"Calculates traffic increment for counters based on device type and time.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_traffic_increment/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_traffic_increment/2","doc":"- `state` - Device state\n- `counter_type` - Type of traffic counter (:in_octets, :out_octets, etc.)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_traffic_increment/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_traffic_increment/2","doc":"- Integer representing traffic increment since device start","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_traffic_increment/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime/1","doc":"Calculates device uptime in milliseconds.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime/1","doc":"- `state` - Device state containing uptime_start timestamp","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime/1","doc":"- Integer representing uptime in milliseconds","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime_ticks/1","doc":"Calculates device uptime in SNMP TimeTicks (centiseconds).","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime_ticks/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime_ticks/1","doc":"- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime_ticks/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.calculate_uptime_ticks/1","doc":"- Integer representing uptime in centiseconds (1/100th of a second)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#calculate_uptime_ticks/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.extract_oid/1","doc":"Extracts OID from SNMP PDU variable binding.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_oid/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.extract_oid/1","doc":"- `varbind` - SNMP variable binding containing OID and value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_oid/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.extract_oid/1","doc":"- `oid` - Successfully extracted OID","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_oid/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.extract_type_and_value/1","doc":"Extracts type and value from SNMP variable binding.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_type_and_value/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.extract_type_and_value/1","doc":"- `varbind` - SNMP variable binding","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_type_and_value/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.extract_type_and_value/1","doc":"- `{type, value}` - Tuple containing SNMP type and value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#extract_type_and_value/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_bulk_oid_values/3","doc":"Retrieves multiple OIDs for SNMP GetBulk operations.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_bulk_oid_values/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_bulk_oid_values/3","doc":"- `oid` - Starting OID\n- `count` - Maximum number of OIDs to retrieve\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_bulk_oid_values/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_bulk_oid_values/3","doc":"- List of `{oid, type, value}` tuples","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_bulk_oid_values/3-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_dynamic_oid_value/2","doc":"Gets dynamic OID value based on device state and OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_dynamic_oid_value/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_dynamic_oid_value/2","doc":"- `oid` - OID as string\n- `state` - Device state containing configuration and counters","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_dynamic_oid_value/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_dynamic_oid_value/2","doc":"- `{:ok, {type, value}}` - Successfully retrieved OID value with type\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_dynamic_oid_value/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_fallback_bulk_oids/3","doc":"Gets fallback bulk OIDs for SNMP GetBulk operations.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_bulk_oids/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_fallback_bulk_oids/3","doc":"- `oid` - Starting OID\n- `count` - Maximum number of OIDs to retrieve\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_bulk_oids/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_fallback_bulk_oids/3","doc":"- List of `{oid, type, value}` tuples","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_bulk_oids/3-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_fallback_next_oid/2","doc":"Gets fallback next OID for SNMP GetNext operations.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_next_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_fallback_next_oid/2","doc":"- `oid_list` - Starting OID as list of integers\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_next_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_fallback_next_oid/2","doc":"- `{next_oid, type, value}` - Next OID with its type and value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_fallback_next_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_interface_description/1","doc":"Gets interface description based on device type.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_interface_description/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_interface_description/1","doc":"- `state` - Device state containing device_type","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_interface_description/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_interface_description/1","doc":"- String description of the interface","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_interface_description/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_next_oid_value/3","doc":"Finds the next OID in lexicographic order for SNMP GetNext operations.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_next_oid_value/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_next_oid_value/3","doc":"- `oid` - Starting OID as list of integers\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_next_oid_value/3-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_next_oid_value/3","doc":"- `{:ok, {next_oid, type, value}}` - Next OID with its type and value\n- `{:error, :end_of_mib}` - No more OIDs available","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_next_oid_value/3-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_oid_value/2","doc":"Gets OID value based on device state and OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_oid_value/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.get_oid_value/2","doc":"- `oid` - OID as list of integers or binary string\n- `state` - Device state containing configuration and counters or device type","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_oid_value/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_oid_value/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_oid_value/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.get_time_factor/0","doc":"Gets time-of-day factor for simulating traffic patterns.\n\nPeak traffic occurs during evening hours (8-10 PM) with lower\nutilization during overnight and early morning hours.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_time_factor/0"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.get_time_factor/0","doc":"- Float representing traffic multiplier (0.6 to 1.5)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#get_time_factor/0-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.handle_docsis_snr_oid/2","doc":"Handles DOCSIS SNR OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_docsis_snr_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.handle_docsis_snr_oid/2","doc":"- `oid` - DOCSIS SNR OID as string\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_docsis_snr_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.handle_docsis_snr_oid/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_docsis_snr_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.handle_hc_interface_oid/2","doc":"Handles high capacity interface OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_hc_interface_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.handle_hc_interface_oid/2","doc":"- `oid` - High capacity interface OID as string\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_hc_interface_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.handle_hc_interface_oid/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_hc_interface_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.handle_host_processor_oid/2","doc":"Handles Host Resources processor OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_processor_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.handle_host_processor_oid/2","doc":"- `oid` - Host Resources processor OID as string\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_processor_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.handle_host_processor_oid/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_processor_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.handle_host_storage_oid/2","doc":"Handles Host Resources storage OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_storage_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.handle_host_storage_oid/2","doc":"- `oid` - Host Resources storage OID as string\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_storage_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.handle_host_storage_oid/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_host_storage_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.handle_interface_oid/2","doc":"Handles interface OID.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_interface_oid/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.handle_interface_oid/2","doc":"- `oid` - Interface OID as string\n- `_state` - Device state (unused)","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_interface_oid/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.handle_interface_oid/2","doc":"- `{:ok, value}` - Successfully retrieved OID value\n- `{:error, reason}` - Failed to retrieve OID value","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#handle_interface_oid/2-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.oid_to_string/1","doc":"Converts OID list to string representation.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#oid_to_string/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.oid_to_string/1","doc":"- `oid` - List of integers representing OID","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#oid_to_string/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.oid_to_string/1","doc":"- String representation of OID (e.g., \"1.3.6.1.2.1.1.1.0\")","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#oid_to_string/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.string_to_oid_list/1","doc":"Converts string OID to list of integers.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#string_to_oid_list/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.string_to_oid_list/1","doc":"- `oid_string` - String representation of OID","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#string_to_oid_list/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.string_to_oid_list/1","doc":"- List of integers representing OID","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#string_to_oid_list/1-returns"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.walk_oid_recursive/4","doc":"","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#walk_oid_recursive/4"},{"type":"function","title":"SnmpKit.SnmpSim.Device.OidHandler.walk_oid_values/2","doc":"Walks OID values for SNMP MIB walking.","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#walk_oid_values/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpSim.Device.OidHandler.walk_oid_values/2","doc":"- `oid` - Starting OID as list of integers\n- `state` - Device state","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#walk_oid_values/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpSim.Device.OidHandler.walk_oid_values/2","doc":"- `{:ok, oid_values}` - List of OID values","ref":"SnmpKit.SnmpSim.Device.OidHandler.html#walk_oid_values/2-returns"},{"type":"module","title":"SnmpKit.SnmpSim.Device.PduProcessor","doc":"Handles SNMP PDU processing for devices.","ref":"SnmpKit.SnmpSim.Device.PduProcessor.html"},{"type":"function","title":"SnmpKit.SnmpSim.Device.PduProcessor.process_getnext_request/2","doc":"","ref":"SnmpKit.SnmpSim.Device.PduProcessor.html#process_getnext_request/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.PduProcessor.process_pdu/2","doc":"","ref":"SnmpKit.SnmpSim.Device.PduProcessor.html#process_pdu/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.PduProcessor.process_snmp_pdu/2","doc":"","ref":"SnmpKit.SnmpSim.Device.PduProcessor.html#process_snmp_pdu/2"},{"type":"module","title":"SnmpKit.SnmpSim.Device.WalkPduProcessor","doc":"Simplified PDU processor for devices with walk data.\nHandles GET, GETNEXT, and GETBULK requests using walk file data\nwith support for dynamic counters and gauges.","ref":"SnmpKit.SnmpSim.Device.WalkPduProcessor.html"},{"type":"function","title":"SnmpKit.SnmpSim.Device.WalkPduProcessor.process_get_request/2","doc":"Process a GET request for walk-based devices.","ref":"SnmpKit.SnmpSim.Device.WalkPduProcessor.html#process_get_request/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.WalkPduProcessor.process_getbulk_request/2","doc":"Process a GETBULK request for walk-based devices.","ref":"SnmpKit.SnmpSim.Device.WalkPduProcessor.html#process_getbulk_request/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.WalkPduProcessor.process_getnext_request/2","doc":"Process a GETNEXT request for walk-based devices.","ref":"SnmpKit.SnmpSim.Device.WalkPduProcessor.html#process_getnext_request/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.WalkPduProcessor.process_set_request/2","doc":"Process a SET request for walk-based devices.\nSince walk files contain read-only data, all SET requests return readOnly error.","ref":"SnmpKit.SnmpSim.Device.WalkPduProcessor.html#process_set_request/2"},{"type":"module","title":"SnmpKit.SnmpSim.DeviceDistribution","doc":"Device Type Distribution for realistic device type assignment across port ranges.\nSupports mixed device populations for authentic testing environments.\n\nFeatures:\n- Realistic device type distribution patterns\n- Flexible port range assignment strategies\n- Support for custom device mixes\n- Population density calculations\n- Device type metadata and characteristics","ref":"SnmpKit.SnmpSim.DeviceDistribution.html"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.build_port_assignments/2","doc":"Build port assignments from a device mix and port range.\nDistributes devices across the port range maintaining the specified ratios.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#build_port_assignments/2"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.calculate_density_stats/1","doc":"Calculate population density statistics for device assignments.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#calculate_density_stats/1"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.default_port_assignments/0","doc":"Get the default device type port ranges for large-scale simulation.\nOptimized for 10K device populations with realistic distribution.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#default_port_assignments/0"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.determine_device_type/2","doc":"Determine device type for a given port based on port assignments.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#determine_device_type/2"},{"type":"type","title":"SnmpKit.SnmpSim.DeviceDistribution.device_mix/0","doc":"","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#t:device_mix/0"},{"type":"type","title":"SnmpKit.SnmpSim.DeviceDistribution.device_type/0","doc":"","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#t:device_type/0"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.generate_device_id/3","doc":"Generate device ID with type-specific formatting.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#generate_device_id/3"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.get_device_characteristics/1","doc":"Get device type characteristics and metadata.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#get_device_characteristics/1"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.get_device_mix/1","doc":"Get common device mix patterns for different testing scenarios.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#get_device_mix/1"},{"type":"type","title":"SnmpKit.SnmpSim.DeviceDistribution.port_assignments/0","doc":"","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#t:port_assignments/0"},{"type":"type","title":"SnmpKit.SnmpSim.DeviceDistribution.port_range/0","doc":"","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#t:port_range/0"},{"type":"function","title":"SnmpKit.SnmpSim.DeviceDistribution.validate_port_assignments/1","doc":"Validate port assignments for consistency and coverage.","ref":"SnmpKit.SnmpSim.DeviceDistribution.html#validate_port_assignments/1"},{"type":"module","title":"SnmpKit.SnmpSim.ErrorInjector","doc":"Inject realistic error conditions for comprehensive testing.\n\nSupports timeouts, packet loss, malformed responses, and device failures\nfor testing SNMP polling systems under realistic network conditions.","ref":"SnmpKit.SnmpSim.ErrorInjector.html"},{"type":"module","title":"Features - SnmpKit.SnmpSim.ErrorInjector","doc":"- Network timeouts with configurable probability and duration\n- Packet loss simulation with burst patterns\n- SNMP protocol errors (noSuchName, genErr, tooBig)\n- Malformed response corruption for robustness testing\n- Device failure and reboot simulation\n- Statistical tracking of all injected errors","ref":"SnmpKit.SnmpSim.ErrorInjector.html#module-features"},{"type":"module","title":"Usage - SnmpKit.SnmpSim.ErrorInjector","doc":"# Inject timeout condition\n    SnmpKit.SnmpSim.ErrorInjector.inject_timeout(device_pid, probability: 0.1, duration: 5000)\n\n    # Simulate packet loss\n    SnmpKit.SnmpSim.ErrorInjector.inject_packet_loss(device_pid, loss_rate: 0.05)\n\n    # Generate SNMP errors\n    SnmpKit.SnmpSim.ErrorInjector.inject_snmp_error(device_pid, :noSuchName, [\"1.3.6.1.2.1.2.2.1.99\"])","ref":"SnmpKit.SnmpSim.ErrorInjector.html#module-usage"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.clear_all_errors/1","doc":"Clear all error conditions and reset device to normal operation.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#clear_all_errors/1"},{"type":"type","title":"SnmpKit.SnmpSim.ErrorInjector.error_config/0","doc":"","ref":"SnmpKit.SnmpSim.ErrorInjector.html#t:error_config/0"},{"type":"type","title":"SnmpKit.SnmpSim.ErrorInjector.error_type/0","doc":"","ref":"SnmpKit.SnmpSim.ErrorInjector.html#t:error_type/0"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.get_error_statistics/1","doc":"Get statistics for all injected errors.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#get_error_statistics/1"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.inject_malformed_response/3","doc":"Inject malformed response packets to test client robustness.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_malformed_response/3"},{"type":"function","title":"Corruption Types - SnmpKit.SnmpSim.ErrorInjector.inject_malformed_response/3","doc":"- `:truncated` - Cut off response packets\n- `:invalid_ber` - Corrupt BER/DER encoding\n- `:wrong_community` - Incorrect community string\n- `:invalid_pdu_type` - Invalid PDU type field\n- `:corrupted_varbinds` - Corrupt variable bindings","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_malformed_response/3-corruption-types"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ErrorInjector.inject_malformed_response/3","doc":"# Randomly truncate 2% of responses\n    ErrorInjector.inject_malformed_response(device, :truncated,\n      probability: 0.02,\n      corruption_severity: 0.3\n    )\n\n    # Corrupt BER encoding occasionally\n    ErrorInjector.inject_malformed_response(device, :invalid_ber,\n      probability: 0.01\n    )","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_malformed_response/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.inject_packet_loss/2","doc":"Inject packet loss with configurable loss rates and patterns.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_packet_loss/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.ErrorInjector.inject_packet_loss/2","doc":"- `loss_rate`: Float 0.0-1.0, percentage of packets to drop\n- `burst_loss`: Enable burst loss patterns (default: false)\n- `burst_size`: Number of consecutive packets to drop in burst (default: 5)\n- `recovery_time_ms`: Time between bursts (default: 30000)","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_packet_loss/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ErrorInjector.inject_packet_loss/2","doc":"# 5% random packet loss\n    ErrorInjector.inject_packet_loss(device, loss_rate: 0.05)\n\n    # Burst packet loss - lose 10 consecutive packets occasionally\n    ErrorInjector.inject_packet_loss(device,\n      loss_rate: 0.02,\n      burst_loss: true,\n      burst_size: 10,\n      recovery_time_ms: 60000\n    )","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_packet_loss/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.inject_snmp_error/3","doc":"Inject SNMP protocol errors for specific OIDs or patterns.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_snmp_error/3"},{"type":"function","title":"Error Types - SnmpKit.SnmpSim.ErrorInjector.inject_snmp_error/3","doc":"- `:noSuchName` - OID does not exist\n- `:genErr` - General error\n- `:tooBig` - Response too large for UDP packet\n- `:badValue` - Invalid value in SET request\n- `:readOnly` - Attempt to SET read-only variable","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_snmp_error/3-error-types"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ErrorInjector.inject_snmp_error/3","doc":"# Generate noSuchName errors for specific OIDs\n    ErrorInjector.inject_snmp_error(device, :noSuchName,\n      target_oids: [\"1.3.6.1.2.1.2.2.1.99\"],\n      probability: 1.0\n    )\n\n    # Random genErr responses\n    ErrorInjector.inject_snmp_error(device, :genErr,\n      probability: 0.05,\n      target_oids: :all\n    )","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_snmp_error/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.inject_timeout/2","doc":"Inject timeout conditions with specified probability and duration.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_timeout/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.ErrorInjector.inject_timeout/2","doc":"- `probability`: Float 0.0-1.0, chance each request times out\n- `duration_ms`: Timeout duration in milliseconds\n- `burst_probability`: Chance of timeout bursts (default: 0.1)\n- `burst_duration_ms`: Duration of timeout bursts (default: 10000)","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_timeout/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ErrorInjector.inject_timeout/2","doc":"# 10% chance of 5-second timeouts\n    ErrorInjector.inject_timeout(device, probability: 0.1, duration_ms: 5000)\n\n    # Burst timeouts - 20% of requests timeout for 30 seconds when burst occurs\n    ErrorInjector.inject_timeout(device,\n      probability: 0.2,\n      duration_ms: 30000,\n      burst_probability: 0.05,\n      burst_duration_ms: 60000\n    )","ref":"SnmpKit.SnmpSim.ErrorInjector.html#inject_timeout/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.remove_error_condition/2","doc":"Remove specific error condition.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#remove_error_condition/2"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.simulate_device_failure/3","doc":"Simulate device reboot or failure scenarios.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#simulate_device_failure/3"},{"type":"function","title":"Failure Types - SnmpKit.SnmpSim.ErrorInjector.simulate_device_failure/3","doc":"- `:reboot` - Device becomes unreachable then recovers\n- `:power_failure` - Complete device failure\n- `:network_disconnect` - Network connectivity lost\n- `:firmware_crash` - Device crash with recovery\n- `:overload` - Device overloaded, slow responses","ref":"SnmpKit.SnmpSim.ErrorInjector.html#simulate_device_failure/3-failure-types"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ErrorInjector.simulate_device_failure/3","doc":"# Simulate device reboot (30 seconds downtime)\n    ErrorInjector.simulate_device_failure(device, :reboot,\n      duration_ms: 30000,\n      recovery_behavior: :reset_counters\n    )\n\n    # Network disconnect with gradual recovery\n    ErrorInjector.simulate_device_failure(device, :network_disconnect,\n      duration_ms: 60000,\n      recovery_behavior: :gradual\n    )","ref":"SnmpKit.SnmpSim.ErrorInjector.html#simulate_device_failure/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ErrorInjector.start_link/2","doc":"Start error injection monitoring for a device.","ref":"SnmpKit.SnmpSim.ErrorInjector.html#start_link/2"},{"type":"module","title":"SnmpKit.SnmpSim.LazyDevicePool","doc":"Lazy Device Pool Manager for on-demand device creation and lifecycle management.\nSupports 10K+ devices with minimal memory footprint through lazy instantiation.\n\nFeatures:\n- On-demand device creation when first accessed\n- Automatic cleanup of idle devices to conserve resources\n- Port-based device type determination\n- Efficient tracking of active devices and last access times\n- Resource monitoring and cleanup scheduling","ref":"SnmpKit.SnmpSim.LazyDevicePool.html"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.cleanup_idle_devices/0","doc":"Force cleanup of idle devices immediately.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#cleanup_idle_devices/0"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.clear_cache/0","doc":"Clear the device cache - alias for shutdown_all_devices for compatibility.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#clear_cache/0"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.configure_port_assignments/1","doc":"Configure device types for specific port ranges.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#configure_port_assignments/1"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.get_or_create_device/1","doc":"Get or create a device for the specified port.\nCreates the device on first access if it doesn't exist.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#get_or_create_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.get_stats/0","doc":"Get statistics about the device pool.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#get_stats/0"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#init/1"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.shutdown_all_devices/0","doc":"Shutdown all devices and reset the pool.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#shutdown_all_devices/0"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.shutdown_device/1","doc":"Shutdown a specific device.","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#shutdown_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.start_device_population/2","doc":"","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#start_device_population/2"},{"type":"function","title":"SnmpKit.SnmpSim.LazyDevicePool.start_link/1","doc":"","ref":"SnmpKit.SnmpSim.LazyDevicePool.html#start_link/1"},{"type":"module","title":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer","doc":"Automatically determine realistic behaviors from MIB object definitions.\nAnalyze object names, descriptions, and types to infer simulation patterns.","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_mib_behaviors/1","doc":"Analyze a complete MIB and generate behavior patterns for all objects.","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#analyze_mib_behaviors/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_mib_behaviors/1","doc":"{:ok, behaviors} = SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_mib_behaviors(compiled_mib)","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#analyze_mib_behaviors/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_object_behavior/1","doc":"Analyze a MIB object and determine its simulation behavior.","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#analyze_object_behavior/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_object_behavior/1","doc":"behavior = SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.analyze_object_behavior(%{\n      name: \"ifInOctets\",\n      oid: \"1.3.6.1.2.1.2.2.1.10\",\n      type: :counter32,\n      description: \"The total number of octets received on the interface\"\n    })\n\n    # Returns: {:traffic_counter, %{rate_range: {1000, 125_000_000}, increment_pattern: :bursty}}","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#analyze_object_behavior/1-examples"},{"type":"type","title":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.description_pattern/0","doc":"","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#t:description_pattern/0"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.enhance_walk_file_behaviors/1","doc":"Create behavior specifications from walk file data enhanced with intelligent analysis.","ref":"SnmpKit.SnmpSim.MIB.BehaviorAnalyzer.html#enhance_walk_file_behaviors/1"},{"type":"module","title":"SnmpKit.SnmpSim.MIB.Compiler","doc":"MIB Compiler for SNMP Simulator (Elixir)\n\n- Compiles MIB files using MIB parsing and compilation.\n- Handles MIB dependencies via IMPORTS parsing and topological sort.\n- Does NOT use any Erlang SNMP APIs.\n- Does NOT introspect MIB objects at runtime (for that, parse the compiled MIBs yourself).\n\nThis module provides MIB compilation functionality for SNMP simulation.","ref":"SnmpKit.SnmpSim.MIB.Compiler.html"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.Compiler.compile_mib_directory/2","doc":"Compile all .mib files in `mib_dir` (recursively resolves dependencies).\nReturns a list of {mib_file, {:ok, _} | {:error, _}}.","ref":"SnmpKit.SnmpSim.MIB.Compiler.html#compile_mib_directory/2"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.Compiler.compile_mib_files/2","doc":"Compile a list of MIB files (in dependency order).\nReturns a list of {mib_file, {:ok, _} | {:error, _}}.","ref":"SnmpKit.SnmpSim.MIB.Compiler.html#compile_mib_files/2"},{"type":"module","title":"SnmpKit.SnmpSim.MIB.SharedProfiles","doc":"Memory-efficient shared OID profiles using ETS tables.\nReduces memory from 1GB to ~10MB for 10K devices by sharing profile data.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.clear_all_profiles/0","doc":"Clear all profiles (useful for testing).","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#clear_all_profiles/0"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.compare_oids_lexicographically/2","doc":"Compare OIDs lexicographically (useful for testing).","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#compare_oids_lexicographically/2"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.get_all_oids/1","doc":"Get all OIDs for a device type.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_all_oids/1"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.get_bulk_oids/3","doc":"Get multiple OIDs for GETBULK operations.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_bulk_oids/3"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.get_memory_stats/0","doc":"Get memory usage statistics for the shared profiles.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_memory_stats/0"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.get_next_oid/2","doc":"Get the next OID in lexicographic order for GETNEXT operations.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_next_oid/2"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.get_oid_value/3","doc":"Get a value for a specific OID with device-specific state applied.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_oid_value/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.SharedProfiles.get_oid_value/3","doc":"value = SnmpKit.SnmpSim.MIB.SharedProfiles.get_oid_value(\n      :cable_modem,\n      \"1.3.6.1.2.1.2.2.1.10.1\",\n      %{device_id: \"cm_001\", uptime: 3600}\n    )","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#get_oid_value/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.init_profiles/0","doc":"Initialize shared profiles for device types.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#init_profiles/0"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.SharedProfiles.init_profiles/0","doc":":ok = SnmpKit.SnmpSim.MIB.SharedProfiles.init_profiles()","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#init_profiles/0-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.list_profiles/0","doc":"List all available device type profiles.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#list_profiles/0"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.load_mib_profile/3","doc":"Load a MIB-based profile for a device type.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#load_mib_profile/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.SharedProfiles.load_mib_profile/3","doc":":ok = SnmpKit.SnmpSim.MIB.SharedProfiles.load_mib_profile(\n      :cable_modem,\n      [\"DOCS-CABLE-DEVICE-MIB\", \"IF-MIB\"]\n    )","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#load_mib_profile/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.load_walk_profile/3","doc":"Load a walk file-based profile with enhanced behaviors.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#load_walk_profile/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MIB.SharedProfiles.load_walk_profile/3","doc":":ok = SnmpKit.SnmpSim.MIB.SharedProfiles.load_walk_profile(\n      :cable_modem,\n      \"priv/walks/cable_modem.walk\",\n      behaviors: [:realistic_counters, :daily_patterns]\n    )","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#load_walk_profile/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.start_link/1","doc":"Start the shared profiles manager.","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#start_link/1"},{"type":"function","title":"SnmpKit.SnmpSim.MIB.SharedProfiles.store_profile/3","doc":"Store profile data directly (useful for testing).","ref":"SnmpKit.SnmpSim.MIB.SharedProfiles.html#store_profile/3"},{"type":"module","title":"SnmpKit.SnmpSim.MultiDeviceStartup","doc":"Multi-Device Startup functionality for large-scale device population management.\n\nFeatures:\n- Bulk device population startup\n- Progress monitoring and reporting\n- Parallel device creation for speed\n- Failure handling and recovery\n- Integration with LazyDevicePool and DeviceDistribution","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.console_progress_callback/0","doc":"Create a simple progress callback that logs to console.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#console_progress_callback/0"},{"type":"type","title":"SnmpKit.SnmpSim.MultiDeviceStartup.device_spec/0","doc":"","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#t:device_spec/0"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.get_startup_status/0","doc":"Get startup progress and statistics.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#get_startup_status/0"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.pre_warm_devices/2","doc":"Pre-warm a specified number of devices for immediate availability.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#pre_warm_devices/2"},{"type":"type","title":"SnmpKit.SnmpSim.MultiDeviceStartup.progress_callback/0","doc":"","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#t:progress_callback/0"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.shutdown_device_population/0","doc":"Gracefully shutdown all devices in the population.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#shutdown_device_population/0"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.start_device_mix/2","doc":"Start devices using predefined device mix patterns.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#start_device_mix/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MultiDeviceStartup.start_device_mix/2","doc":"{:ok, result} = SnmpKit.SnmpSim.MultiDeviceStartup.start_device_mix(\n      :cable_network,\n      port_range: 30_000..39_999\n    )","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#start_device_mix/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.start_device_population/2","doc":"Start a large population of devices based on device specifications.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#start_device_population/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.MultiDeviceStartup.start_device_population/2","doc":"device_specs = [\n      {:cable_modem, 1000},\n      {:switch, 50},\n      {:router, 10},\n      {:cmts, 5}\n    ]\n\n    {:ok, result} = SnmpKit.SnmpSim.MultiDeviceStartup.start_device_population(\n      device_specs,\n      port_range: 30_000..31_099,\n      parallel_workers: 100\n    )","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#start_device_population/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.MultiDeviceStartup.start_with_progress/2","doc":"Start devices with console progress reporting.","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#start_with_progress/2"},{"type":"type","title":"SnmpKit.SnmpSim.MultiDeviceStartup.startup_opts/0","doc":"","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#t:startup_opts/0"},{"type":"type","title":"SnmpKit.SnmpSim.MultiDeviceStartup.startup_status/0","doc":"","ref":"SnmpKit.SnmpSim.MultiDeviceStartup.html#t:startup_status/0"},{"type":"module","title":"SnmpKit.SnmpSim.OIDTree","doc":"Optimized OID tree for fast lookups and lexicographic traversal.\nSupports GETNEXT operations and GETBULK bulk retrieval.\n\nUses a radix tree structure optimized for OID operations with:\n- Fast O(log n) lookups and insertions\n- Efficient lexicographic traversal for GETNEXT\n- Memory-efficient storage for 10K+ OIDs\n- Optimized bulk operations","ref":"SnmpKit.SnmpSim.OIDTree.html"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.bulk_walk/4","doc":"Perform a bulk walk operation starting from the given OID.\nUsed for GETBULK operations with proper handling of non-repeaters and max-repetitions.","ref":"SnmpKit.SnmpSim.OIDTree.html#bulk_walk/4"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.bulk_walk/4","doc":"results = SnmpKit.SnmpSim.OIDTree.bulk_walk(tree, \"1.3.6.1.2.1.2.2.1\", 10, 0)","ref":"SnmpKit.SnmpSim.OIDTree.html#bulk_walk/4-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.empty?/1","doc":"Check if the tree is empty.","ref":"SnmpKit.SnmpSim.OIDTree.html#empty?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.empty?/1","doc":"empty? = SnmpKit.SnmpSim.OIDTree.empty?(tree)","ref":"SnmpKit.SnmpSim.OIDTree.html#empty?/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.get/2","doc":"Get the value for an exact OID match.","ref":"SnmpKit.SnmpSim.OIDTree.html#get/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.get/2","doc":"{:ok, value, behavior} = SnmpKit.SnmpSim.OIDTree.get(tree, \"1.3.6.1.2.1.1.1.0\")\n    :not_found = SnmpKit.SnmpSim.OIDTree.get(tree, \"1.3.6.1.2.1.1.1.999\")","ref":"SnmpKit.SnmpSim.OIDTree.html#get/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.get_next/2","doc":"Get the next OID in lexicographic order (GETNEXT operation).\nReturns the next OID after the given OID, or :end_of_mib if no more OIDs exist.","ref":"SnmpKit.SnmpSim.OIDTree.html#get_next/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.get_next/2","doc":"{:ok, next_oid, value, behavior} = SnmpKit.SnmpSim.OIDTree.get_next(tree, \"1.3.6.1.2.1.1.1.0\")\n    :end_of_mib = SnmpKit.SnmpSim.OIDTree.get_next(tree, \"1.3.6.1.9.9.9.9.9\")","ref":"SnmpKit.SnmpSim.OIDTree.html#get_next/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.insert/4","doc":"Insert an OID with its value and behavior information into the tree.\nMaintains lexicographic ordering for efficient GETNEXT operations.","ref":"SnmpKit.SnmpSim.OIDTree.html#insert/4"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.insert/4","doc":"tree = SnmpKit.SnmpSim.OIDTree.new()\n    tree = SnmpKit.SnmpSim.OIDTree.insert(tree, \"1.3.6.1.2.1.1.1.0\", \"System Description\", nil)","ref":"SnmpKit.SnmpSim.OIDTree.html#insert/4-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.list_oids/1","doc":"Get all OIDs in the tree in lexicographic order.\nUseful for debugging and full tree traversal.","ref":"SnmpKit.SnmpSim.OIDTree.html#list_oids/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.list_oids/1","doc":"oids = SnmpKit.SnmpSim.OIDTree.list_oids(tree)","ref":"SnmpKit.SnmpSim.OIDTree.html#list_oids/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.new/0","doc":"Create a new empty OID tree.","ref":"SnmpKit.SnmpSim.OIDTree.html#new/0"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.new/0","doc":"tree = SnmpKit.SnmpSim.OIDTree.new()","ref":"SnmpKit.SnmpSim.OIDTree.html#new/0-examples"},{"type":"function","title":"SnmpKit.SnmpSim.OIDTree.size/1","doc":"Get the size of the tree (number of OIDs).","ref":"SnmpKit.SnmpSim.OIDTree.html#size/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.OIDTree.size/1","doc":"size = SnmpKit.SnmpSim.OIDTree.size(tree)","ref":"SnmpKit.SnmpSim.OIDTree.html#size/1-examples"},{"type":"module","title":"SnmpKit.SnmpSim.PDUHelper","doc":"Provides utility functions for PDU manipulation.","ref":"SnmpKit.SnmpSim.PDUHelper.html"},{"type":"function","title":"SnmpKit.SnmpSim.PDUHelper.pdu_version_to_int/1","doc":"Converts a PDU version atom (e.g., :v1, :v2c) to its integer representation.\nDefaults to 2 (for SNMPv2c) if the atom is not :v1.","ref":"SnmpKit.SnmpSim.PDUHelper.html#pdu_version_to_int/1"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.Benchmarks","doc":"Comprehensive benchmarking framework for SNMP simulator performance testing.\n\nFeatures:\n- Load testing with configurable request patterns\n- Throughput and latency measurement\n- Memory and CPU profiling\n- Concurrent client simulation\n- Performance regression detection\n- Automated benchmark reporting","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.Benchmarks.run_benchmark_suite/1","doc":"Run comprehensive benchmark suite.","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html#run_benchmark_suite/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.Benchmarks.run_error_resilience_benchmark/1","doc":"Benchmark response to various error conditions.","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html#run_error_resilience_benchmark/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.Benchmarks.run_memory_stress_benchmark/1","doc":"Run memory stress test to identify memory leaks and limits.","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html#run_memory_stress_benchmark/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.Benchmarks.run_scaling_benchmark/1","doc":"Run scaling benchmark to test performance at different device counts.","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html#run_scaling_benchmark/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.Benchmarks.run_single_benchmark/2","doc":"Run single benchmark with specified parameters.","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.html#run_single_benchmark/2"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.Benchmarks.BenchmarkResult","doc":"Structure for benchmark results","ref":"SnmpKit.SnmpSim.Performance.Benchmarks.BenchmarkResult.html"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool","doc":"High-performance device pool with ETS-based caching and optimization.\nDesigned for 10K+ concurrent devices with sub-millisecond lookup times.\n\nFeatures:\n- ETS-based device registry for O(1) lookups\n- Profile caching to avoid repeated profile loading\n- Connection pooling for efficient resource reuse\n- Hot/warm/cold device tiers for optimal memory usage\n- Pre-computed response caching for common OIDs","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.cache_response/4","doc":"Cache response for future requests.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#cache_response/4"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.cleanup_cold_devices/0","doc":"Force cleanup of cold tier devices.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#cleanup_cold_devices/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.configure_port_assignments/1","doc":"Configure port assignments for device types.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#configure_port_assignments/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.get_cached_response/2","doc":"Get cached response for common OID requests.\nReturns {:cache_hit, response} or :cache_miss.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#get_cached_response/2"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.get_device/1","doc":"Get device PID with optimized lookup (O(1) from ETS).\nCreates device lazily if it doesn't exist.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#get_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.get_device_profile/1","doc":"Get device profile from cache or load it.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#get_device_profile/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.get_performance_stats/0","doc":"Get performance statistics for monitoring.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#get_performance_stats/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.promote_to_hot_tier/1","doc":"Promote device to hot tier for frequent access optimization.","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#promote_to_hot_tier/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.start_link/1","doc":"","ref":"SnmpKit.SnmpSim.Performance.OptimizedDevicePool.html#start_link/1"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer","doc":"High-performance UDP server optimized for 100K+ requests/second throughput.\n\nFeatures:\n- Multi-socket architecture for load distribution\n- Worker pool for concurrent packet processing\n- Ring buffer for packet queuing\n- Socket-level optimizations for minimal latency\n- Adaptive backpressure management\n- Direct response path bypassing GenServer for hot paths","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.force_packet_processing/1","doc":"Force immediate packet processing (drain queue).","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#force_packet_processing/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.get_performance_stats/1","doc":"Get comprehensive server performance statistics.","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#get_performance_stats/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.start_link/2","doc":"","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#start_link/2"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.start_optimized/2","doc":"Start optimized UDP server with performance tuning.","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#start_optimized/2"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.update_optimization/2","doc":"Update server optimization settings at runtime.","ref":"SnmpKit.SnmpSim.Performance.OptimizedUdpServer.html#update_optimization/2"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor","doc":"Real-time performance monitoring and telemetry for SNMP simulator.\nProvides comprehensive metrics collection, alerting, and performance analytics.\nIntegrates with :telemetry for external monitoring systems.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.force_performance_analysis/0","doc":"Force immediate performance analysis and reporting.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#force_performance_analysis/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.get_current_metrics/0","doc":"Get current performance metrics snapshot.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#get_current_metrics/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.get_performance_history/1","doc":"Get performance history for trend analysis.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#get_performance_history/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.record_device_event/3","doc":"Record device lifecycle events.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#record_device_event/3"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.record_request_timing/4","doc":"Record device request timing for performance tracking.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#record_request_timing/4"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.register_alert_callback/2","doc":"Register callback for performance alerts.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#register_alert_callback/2"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.start_link/1","doc":"","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#start_link/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.update_thresholds/1","doc":"Update performance thresholds for alerts.","ref":"SnmpKit.SnmpSim.Performance.PerformanceMonitor.html#update_thresholds/1"},{"type":"module","title":"SnmpKit.SnmpSim.Performance.ResourceManager","doc":"System resource management for SNMP simulator.\nEnforces memory limits, device limits, and automatic cleanup.\nOptimizes resource utilization for high-scale scenarios.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.can_allocate_device?/0","doc":"Check if we can allocate a new device within resource limits.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#can_allocate_device?/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.check_memory_limit/0","doc":"Check if memory usage is within limits.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#check_memory_limit/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.force_cleanup/0","doc":"Force immediate cleanup of idle devices.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#force_cleanup/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.get_resource_stats/0","doc":"Get current resource usage statistics.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#get_resource_stats/0"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.register_device/2","doc":"Register a new device allocation.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#register_device/2"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.start_link/1","doc":"","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#start_link/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.unregister_device/1","doc":"Unregister a device that has been stopped.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#unregister_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.Performance.ResourceManager.update_limits/1","doc":"Update resource limits dynamically.","ref":"SnmpKit.SnmpSim.Performance.ResourceManager.html#update_limits/1"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers","doc":"Comprehensive SNMP testing utilities for SnmpSim.\n\nThis module provides a rich set of testing utilities for SNMP simulation,\nperformance testing, stability testing, and production validation.","ref":"SnmpKit.SnmpSim.TestHelpers.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.analyze_response_times/1","doc":"Analyzes response time statistics.","ref":"SnmpKit.SnmpSim.TestHelpers.html#analyze_response_times/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.calculate_error_rate/2","doc":"Calculates error rate as a percentage.","ref":"SnmpKit.SnmpSim.TestHelpers.html#calculate_error_rate/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.check_system_health/0","doc":"Monitors system health metrics.","ref":"SnmpKit.SnmpSim.TestHelpers.html#check_system_health/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.cleanup_devices/1","doc":"Cleans up test devices and resources with enhanced error handling.","ref":"SnmpKit.SnmpSim.TestHelpers.html#cleanup_devices/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.cleanup_monitored_devices/1","doc":"Cleanup monitored devices with enhanced tracking.","ref":"SnmpKit.SnmpSim.TestHelpers.html#cleanup_monitored_devices/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.cleanup_single_device/1","doc":"Cleanup a single device with robust error handling.","ref":"SnmpKit.SnmpSim.TestHelpers.html#cleanup_single_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.create_monitored_test_devices/1","doc":"Create test devices with monitoring and automatic cleanup tracking.\nReturns devices with monitor references for better cleanup handling.","ref":"SnmpKit.SnmpSim.TestHelpers.html#create_monitored_test_devices/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.create_test_devices/1","doc":"Creates test devices with various configurations.","ref":"SnmpKit.SnmpSim.TestHelpers.html#create_test_devices/1"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestHelpers.create_test_devices/1","doc":"- `:count` - Number of devices to create (default: 10)\n- `:community` - SNMP community string (default: \"public\")\n- `:host` - Host address (default: \"127.0.0.1\")\n- `:port_start` - Starting port number (default: 30000)\n- `:walk_file` - SNMP walk file to use (default: \"priv/walks/cable_modem.walk\")\n- `:batch_size` - Create devices in batches (default: 10)\n- `:delay_between_batches` - Delay between batches in ms (default: 100)","ref":"SnmpKit.SnmpSim.TestHelpers.html#create_test_devices/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestHelpers.create_test_devices/1","doc":"devices = TestHelpers.create_test_devices(count: 50)\n    devices = TestHelpers.create_test_devices(count: 100, community: \"private\")","ref":"SnmpKit.SnmpSim.TestHelpers.html#create_test_devices/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.create_test_walk_data/1","doc":"Creates test SNMP walk data for testing.","ref":"SnmpKit.SnmpSim.TestHelpers.html#create_test_walk_data/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.generate_bulk_test_data/2","doc":"Generates test data for bulk operations.","ref":"SnmpKit.SnmpSim.TestHelpers.html#generate_bulk_test_data/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.inject_delay/1","doc":"Injects artificial delays for testing timeout scenarios.","ref":"SnmpKit.SnmpSim.TestHelpers.html#inject_delay/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.measure_response_time/1","doc":"Measures the response time of an operation.","ref":"SnmpKit.SnmpSim.TestHelpers.html#measure_response_time/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.perform_test_operations/2","doc":"Performs various SNMP operations on a device for testing.","ref":"SnmpKit.SnmpSim.TestHelpers.html#perform_test_operations/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.reset_system_state/0","doc":"Resets system state for testing.","ref":"SnmpKit.SnmpSim.TestHelpers.html#reset_system_state/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.run_load_test/2","doc":"Runs a load test against a set of devices.","ref":"SnmpKit.SnmpSim.TestHelpers.html#run_load_test/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestHelpers.run_load_test/2","doc":"- `:duration_ms` - Test duration in milliseconds\n- `:requests_per_second` - Target requests per second\n- `:operation` - SNMP operation to perform (:get, :get_next, :get_bulk, :walk)\n- `:monitor_memory` - Whether to monitor memory usage\n- `:monitor_processes` - Whether to monitor process count","ref":"SnmpKit.SnmpSim.TestHelpers.html#run_load_test/2-options"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.simulate_network_conditions/1","doc":"Simulates network conditions (packet loss, delays).","ref":"SnmpKit.SnmpSim.TestHelpers.html#simulate_network_conditions/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.validate_snmp_response/1","doc":"Validates SNMP response format.","ref":"SnmpKit.SnmpSim.TestHelpers.html#validate_snmp_response/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.validate_system_invariants/0","doc":"Validates system invariants during testing.","ref":"SnmpKit.SnmpSim.TestHelpers.html#validate_system_invariants/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.wait_for_condition/2","doc":"Waits for a condition to be met within a timeout.","ref":"SnmpKit.SnmpSim.TestHelpers.html#wait_for_condition/2"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper","doc":"Performance testing utilities for SnmpSim.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.analyze_memory_patterns/2","doc":"Tests memory usage patterns under different loads.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#analyze_memory_patterns/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.analyze_response_times/1","doc":"Analyzes response time data and returns statistics.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#analyze_response_times/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.benchmark_throughput/4","doc":"Performs throughput benchmarking.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#benchmark_throughput/4"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.calculate_error_rate/2","doc":"Calculates error rate from error count and total requests.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#calculate_error_rate/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.find_breaking_point/2","doc":"Stress tests the system to find breaking points.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#find_breaking_point/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.measure_latency_under_load/2","doc":"Measures latency under various load conditions.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#measure_latency_under_load/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.measure_scalability/4","doc":"Measures scalability by testing performance at different device counts.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#measure_scalability/4"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.profile_cpu_usage/3","doc":"Profiles CPU usage under load.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#profile_cpu_usage/3"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.run_sustained_load_test/4","doc":"Runs a sustained load test with comprehensive monitoring.","ref":"SnmpKit.SnmpSim.TestHelpers.PerformanceHelper.html#run_sustained_load_test/4"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator","doc":"Simple port allocation service for tests.\n\nManages a pool of available ports and allocates them on demand.\nPorts can be reserved and released for reuse.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.allocate_port_range/2","doc":"Legacy function for backward compatibility.\nAllocates a port range for a specific test type.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#allocate_port_range/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.get_stats/0","doc":"Get allocation statistics.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#get_stats/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.release_port/1","doc":"Release a single port back to the pool.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#release_port/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.release_port_range/2","doc":"Release a port range back to the pool.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#release_port_range/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.reserve_port/0","doc":"Reserve a single port.\nReturns {:ok, port} or {:error, reason}","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#reserve_port/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.reserve_port_range/1","doc":"Reserve a range of ports.\nReturns {:ok, {start_port, end_port}} or {:error, reason}","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#reserve_port_range/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.reset/0","doc":"Reset all allocations.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#reset/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.start_link/1","doc":"Start the port allocator service.","ref":"SnmpKit.SnmpSim.TestHelpers.PortAllocator.html#start_link/1"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers.PortHelper","doc":"Simple helper functions for port allocation in tests.","ref":"SnmpKit.SnmpSim.TestHelpers.PortHelper.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortHelper.get_port/0","doc":"Get a port for testing, ensuring PortAllocator is started.\nFalls back to server port range if PortAllocator fails.","ref":"SnmpKit.SnmpSim.TestHelpers.PortHelper.html#get_port/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortHelper.get_port_range/1","doc":"Get a range of ports for testing.","ref":"SnmpKit.SnmpSim.TestHelpers.PortHelper.html#get_port_range/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortHelper.release_port/1","doc":"Release a port back to the pool.","ref":"SnmpKit.SnmpSim.TestHelpers.PortHelper.html#release_port/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.PortHelper.release_port_range/2","doc":"Release a port range back to the pool.","ref":"SnmpKit.SnmpSim.TestHelpers.PortHelper.html#release_port_range/2"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper","doc":"Specialized testing utilities for production validation and testing.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.cleanup_all/0","doc":"Cleanup all production test resources.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#cleanup_all/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.cleanup_devices/1","doc":"Cleanup specific devices.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#cleanup_devices/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.clear_monitoring_condition/1","doc":"Clears a specific monitoring condition.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#clear_monitoring_condition/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.create_devices_efficiently/2","doc":"Creates devices efficiently in batches for large-scale testing.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#create_devices_efficiently/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.get_latest_alert/1","doc":"Gets the latest alert for a specific metric.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#get_latest_alert/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.inject_monitoring_condition/2","doc":"Injects specific monitoring conditions for testing alerting systems.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#inject_monitoring_condition/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.integration_available?/1","doc":"Checks if an integration system is available for testing.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#integration_available?/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.reset_system_state/0","doc":"Resets system state for production testing.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#reset_system_state/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.run_deployment_test/2","doc":"Runs deployment tests to validate operational procedures.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#run_deployment_test/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.run_integration_test/2","doc":"Runs integration tests with external systems.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#run_integration_test/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.run_reliability_test/4","doc":"Runs a comprehensive reliability test with failure injection.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#run_reliability_test/4"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.run_security_test/2","doc":"Runs security tests to validate system resilience against attacks.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#run_security_test/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.wait_for_alert/2","doc":"Waits for a specific alert to be triggered within a timeout.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#wait_for_alert/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.wait_for_recovery_alert/2","doc":"Waits for a recovery alert indicating the condition has cleared.","ref":"SnmpKit.SnmpSim.TestHelpers.ProductionTestHelper.html#wait_for_recovery_alert/2"},{"type":"module","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper","doc":"Specialized testing utilities for stability and endurance testing.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.analyze_memory_samples/1","doc":"Analyzes memory usage patterns to detect leaks and trends.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#analyze_memory_samples/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.check_system_health/0","doc":"Checks overall system health and returns metrics.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#check_system_health/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.cleanup_all/0","doc":"Cleanup all test resources.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#cleanup_all/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.clear_failure_injection/0","doc":"Clears any active failure injections.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#clear_failure_injection/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.inject_failure/1","doc":"Injects various types of failures for testing system resilience.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#inject_failure/1"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.monitor_memory_usage/3","doc":"Monitors memory usage over a specified duration while running a test function.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#monitor_memory_usage/3"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.reset_system_state/0","doc":"Resets system state for stability testing.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#reset_system_state/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.run_endurance_test/2","doc":"Runs an endurance test with varying workload patterns.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#run_endurance_test/2"},{"type":"function","title":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.run_load_test/4","doc":"Runs a comprehensive load test with monitoring.","ref":"SnmpKit.SnmpSim.TestHelpers.StabilityTestHelper.html#run_load_test/4"},{"type":"module","title":"SnmpKit.SnmpSim.TestScenarios","doc":"Pre-built test scenarios for common network conditions.\n\nSimplify complex error injection patterns with realistic network scenarios\nthat test SNMP polling systems under various failure conditions.","ref":"SnmpKit.SnmpSim.TestScenarios.html"},{"type":"module","title":"Scenario Categories - SnmpKit.SnmpSim.TestScenarios","doc":"- **Network Outages**: Complete connectivity loss and recovery patterns\n- **Signal Degradation**: DOCSIS/wireless signal quality issues\n- **High Load**: Network congestion and overload conditions\n- **Device Failures**: Equipment failures, reboots, and recovery\n- **Intermittent Issues**: Flapping, sporadic failures, timing issues\n- **Environmental**: Weather, power, temperature-related problems","ref":"SnmpKit.SnmpSim.TestScenarios.html#module-scenario-categories"},{"type":"module","title":"Usage - SnmpKit.SnmpSim.TestScenarios","doc":"# Apply network outage scenario to all devices\n    TestScenarios.network_outage_scenario(devices, duration_seconds: 300)\n\n    # Simulate signal degradation for cable modems\n    TestScenarios.signal_degradation_scenario(cable_modems,\n      snr_degradation: 10, duration_minutes: 15\n    )\n\n    # Test high load conditions\n    TestScenarios.high_load_scenario(devices, utilization_percent: 95)","ref":"SnmpKit.SnmpSim.TestScenarios.html#module-usage"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.cascading_failure_scenario/2","doc":"Simulate cascading failure - devices fail in sequence.\n\nModels scenarios where initial failures trigger subsequent failures,\ntesting system resilience under escalating conditions.","ref":"SnmpKit.SnmpSim.TestScenarios.html#cascading_failure_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.cascading_failure_scenario/2","doc":"- `initial_failure_percentage`: Initial devices that fail (default: 0.1)\n- `cascade_delay_seconds`: Time between cascade waves (default: 60)\n- `cascade_growth_factor`: How much each wave grows (default: 1.5)\n- `max_affected_percentage`: Maximum devices affected (default: 0.8)","ref":"SnmpKit.SnmpSim.TestScenarios.html#cascading_failure_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.cascading_failure_scenario/2","doc":"# Start with 5% failure, cascade every 30 seconds\n    TestScenarios.cascading_failure_scenario(devices,\n      initial_failure_percentage: 0.05,\n      cascade_delay_seconds: 30,\n      cascade_growth_factor: 2.0\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#cascading_failure_scenario/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.device_flapping_scenario/2","doc":"Simulate device flapping - intermittent connectivity issues.\n\nModels unstable devices that repeatedly go offline and come back online.","ref":"SnmpKit.SnmpSim.TestScenarios.html#device_flapping_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.device_flapping_scenario/2","doc":"- `flap_interval_seconds`: Time between state changes (default: 30)\n- `down_duration_seconds`: How long device stays down (default: 10)\n- `total_duration_minutes`: Total scenario duration (default: 30)\n- `flap_pattern`: `:regular` | `:irregular` | `:degrading` (default: :irregular)","ref":"SnmpKit.SnmpSim.TestScenarios.html#device_flapping_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.device_flapping_scenario/2","doc":"# Regular flapping every 60 seconds\n    TestScenarios.device_flapping_scenario(devices,\n      flap_interval_seconds: 60,\n      down_duration_seconds: 15,\n      flap_pattern: :regular\n    )\n\n    # Irregular flapping with degrading stability\n    TestScenarios.device_flapping_scenario(devices,\n      flap_pattern: :degrading,\n      total_duration_minutes: 45\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#device_flapping_scenario/2-examples"},{"type":"type","title":"SnmpKit.SnmpSim.TestScenarios.device_list/0","doc":"","ref":"SnmpKit.SnmpSim.TestScenarios.html#t:device_list/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.environmental_scenario/2","doc":"Simulate environmental conditions affecting network equipment.\n\nModels weather, power, or temperature-related issues that affect\nmultiple devices simultaneously.","ref":"SnmpKit.SnmpSim.TestScenarios.html#environmental_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.environmental_scenario/2","doc":"- `condition_type`: `:weather` | `:power` | `:temperature` | `:interference`\n- `severity`: `:mild` | `:moderate` | `:severe` (default: :moderate)\n- `duration_hours`: Condition duration (default: 2)\n- `geographic_pattern`: `:random` | `:clustered` | `:linear` (default: :clustered)","ref":"SnmpKit.SnmpSim.TestScenarios.html#environmental_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.environmental_scenario/2","doc":"# Severe weather affecting clustered devices\n    TestScenarios.environmental_scenario(devices,\n      condition_type: :weather,\n      severity: :severe,\n      duration_hours: 4,\n      geographic_pattern: :clustered\n    )\n\n    # Power instability\n    TestScenarios.environmental_scenario(devices,\n      condition_type: :power,\n      severity: :moderate,\n      duration_hours: 1\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#environmental_scenario/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.high_load_scenario/2","doc":"Simulate high network load and congestion conditions.\n\nTests poller behavior under network stress with increased latency,\npacket loss, and timeout conditions.","ref":"SnmpKit.SnmpSim.TestScenarios.html#high_load_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.high_load_scenario/2","doc":"- `utilization_percent`: Network utilization level (default: 85)\n- `duration_minutes`: Load duration (default: 60)\n- `congestion_type`: `:steady` | `:bursty` | `:cascade` (default: :bursty)\n- `error_rate_multiplier`: Error rate increase factor (default: 5.0)","ref":"SnmpKit.SnmpSim.TestScenarios.html#high_load_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.high_load_scenario/2","doc":"# Sustained high load\n    TestScenarios.high_load_scenario(devices,\n      utilization_percent: 95,\n      duration_minutes: 120,\n      congestion_type: :steady\n    )\n\n    # Bursty congestion with high error rates\n    TestScenarios.high_load_scenario(devices,\n      utilization_percent: 90,\n      congestion_type: :bursty,\n      error_rate_multiplier: 10.0\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#high_load_scenario/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.multi_scenario_test/2","doc":"Apply multiple scenarios simultaneously for complex testing.\n\nCombines different failure patterns to test system behavior under\nrealistic multi-factor conditions.","ref":"SnmpKit.SnmpSim.TestScenarios.html#multi_scenario_test/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.multi_scenario_test/2","doc":"scenarios = [\n      {:signal_degradation, [snr_degradation: 8, duration_minutes: 60]},\n      {:high_load, [utilization_percent: 90, duration_minutes: 45]},\n      {:device_flapping, [flap_interval_seconds: 120]}\n    ]\n\n    TestScenarios.multi_scenario_test(devices, scenarios)","ref":"SnmpKit.SnmpSim.TestScenarios.html#multi_scenario_test/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.network_outage_scenario/2","doc":"Simulate complete network outage affecting all devices.\n\nTests poller behavior during total connectivity loss and recovery.","ref":"SnmpKit.SnmpSim.TestScenarios.html#network_outage_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.network_outage_scenario/2","doc":"- `duration_seconds`: Outage duration (default: 300)\n- `recovery_type`: `:immediate` | `:gradual` | `:sporadic` (default: :gradual)\n- `affected_percentage`: Percentage of devices affected (default: 1.0)","ref":"SnmpKit.SnmpSim.TestScenarios.html#network_outage_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.network_outage_scenario/2","doc":"# 5-minute complete outage\n    TestScenarios.network_outage_scenario(devices, duration_seconds: 300)\n\n    # Gradual recovery affecting 80% of devices\n    TestScenarios.network_outage_scenario(devices,\n      duration_seconds: 600,\n      recovery_type: :gradual,\n      affected_percentage: 0.8\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#network_outage_scenario/2-examples"},{"type":"type","title":"SnmpKit.SnmpSim.TestScenarios.scenario_result/0","doc":"","ref":"SnmpKit.SnmpSim.TestScenarios.html#t:scenario_result/0"},{"type":"function","title":"SnmpKit.SnmpSim.TestScenarios.signal_degradation_scenario/2","doc":"Simulate signal degradation for wireless/cable devices.\n\nModels weather-related signal issues, interference, or equipment problems.","ref":"SnmpKit.SnmpSim.TestScenarios.html#signal_degradation_scenario/2"},{"type":"function","title":"Options - SnmpKit.SnmpSim.TestScenarios.signal_degradation_scenario/2","doc":"- `snr_degradation`: SNR reduction in dB (default: 5)\n- `power_variation`: Power level variation in dBmV (default: 3)\n- `duration_minutes`: Degradation duration (default: 30)\n- `pattern`: `:steady` | `:fluctuating` | `:progressive` (default: :fluctuating)","ref":"SnmpKit.SnmpSim.TestScenarios.html#signal_degradation_scenario/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TestScenarios.signal_degradation_scenario/2","doc":"# Weather-related signal degradation\n    TestScenarios.signal_degradation_scenario(cable_modems,\n      snr_degradation: 10,\n      power_variation: 5,\n      duration_minutes: 45,\n      pattern: :progressive\n    )","ref":"SnmpKit.SnmpSim.TestScenarios.html#signal_degradation_scenario/2-examples"},{"type":"module","title":"SnmpKit.SnmpSim.TimePatterns","doc":"Realistic time-based variations for network metrics.\nImplements daily, weekly, and seasonal patterns for authentic simulation.","ref":"SnmpKit.SnmpSim.TimePatterns.html"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.apply_seasonal_variation/2","doc":"Apply seasonal variations to any metric.\n\nGeneric seasonal pattern that can be applied to various metrics.\nUseful for metrics that have yearly cycles.","ref":"SnmpKit.SnmpSim.TimePatterns.html#apply_seasonal_variation/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.apply_seasonal_variation/2","doc":"# Apply to equipment failure rates (higher in summer heat)\n    factor = SnmpKit.SnmpSim.TimePatterns.apply_seasonal_variation(datetime, :equipment_stress)\n\n    # Apply to power consumption (higher in winter/summer for heating/cooling)\n    factor = SnmpKit.SnmpSim.TimePatterns.apply_seasonal_variation(datetime, :power_consumption)","ref":"SnmpKit.SnmpSim.TimePatterns.html#apply_seasonal_variation/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.apply_weather_variation/1","doc":"Apply weather-related variations to signal quality metrics.\n\nSimulates weather patterns that affect signal strength:\n- Rain/snow: Reduces signal quality\n- Clear weather: Optimal signal quality\n- Seasonal patterns for different weather probabilities","ref":"SnmpKit.SnmpSim.TimePatterns.html#apply_weather_variation/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.apply_weather_variation/1","doc":"factor = SnmpKit.SnmpSim.TimePatterns.apply_weather_variation(~U[2024-01-15 14:00:00Z])\n    # Returns: 0.85 (some weather impact)","ref":"SnmpKit.SnmpSim.TimePatterns.html#apply_weather_variation/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_burst_pattern/2","doc":"Get burst patterns for specific device types and times.\n\nDifferent devices have different burst characteristics:\n- Servers: Application-driven bursts\n- Routers: Protocol-driven bursts\n- Cable modems: User-activity bursts","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_burst_pattern/2"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_correlation_pattern/4","doc":"Get correlation patterns for linked metrics.\n\nMany network metrics are correlated and should move together:\n- Traffic volume vs packet count\n- Utilization vs error rates\n- Signal quality vs throughput","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_correlation_pattern/4"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_daily_temperature_pattern/1","doc":"Get daily temperature variation pattern.\n\nReturns temperature offset based on time of day:\n- Coldest: ~6 AM\n- Warmest: ~3 PM\n- Smooth sinusoidal pattern","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_daily_temperature_pattern/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.get_daily_temperature_pattern/1","doc":"# 6 AM (coldest)\n    offset = SnmpKit.SnmpSim.TimePatterns.get_daily_temperature_pattern(~U[2024-01-15 06:00:00Z])\n    # Returns: -3.2\n\n    # 3 PM (warmest)\n    offset = SnmpKit.SnmpSim.TimePatterns.get_daily_temperature_pattern(~U[2024-01-15 15:00:00Z])\n    # Returns: 4.1","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_daily_temperature_pattern/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_daily_utilization_pattern/1","doc":"Get daily utilization pattern factor (0.0 to 1.5).\n\nReturns a multiplier based on time of day:\n- 0-5 AM: Low usage (0.3)\n- 6-8 AM: Morning ramp (0.7)\n- 9-17 PM: Business hours (0.9-1.2)\n- 18-20 PM: Evening peak (1.5)\n- 21-23 PM: Late evening (0.8)","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_daily_utilization_pattern/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.get_daily_utilization_pattern/1","doc":"# 2 PM business hours\n    factor = SnmpKit.SnmpSim.TimePatterns.get_daily_utilization_pattern(~U[2024-01-15 14:00:00Z])\n    # Returns: ~1.1\n\n    # 7 PM evening peak\n    factor = SnmpKit.SnmpSim.TimePatterns.get_daily_utilization_pattern(~U[2024-01-15 19:00:00Z])\n    # Returns: ~1.5","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_daily_utilization_pattern/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_interface_traffic_rate/2","doc":"Get interface traffic rate based on interface type and time patterns.\n\nReturns expected traffic rate ranges for different interface types\nwith time-based adjustments.","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_interface_traffic_rate/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.get_interface_traffic_rate/2","doc":"rate = SnmpKit.SnmpSim.TimePatterns.get_interface_traffic_rate(:ethernet_gigabit, datetime)\n    # Returns: {min_rate, max_rate, current_factor}","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_interface_traffic_rate/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_maintenance_window_factor/1","doc":"Get maintenance window patterns.\n\nNetwork maintenance typically happens during low-usage periods:\n- 2-6 AM local time\n- Weekend mornings\n- Holiday periods","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_maintenance_window_factor/1"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_monthly_pattern/1","doc":"Get monthly pattern for maintenance windows and operational changes.\n\nSome months have different operational characteristics:\n- End of quarters: Higher activity\n- Summer months: Maintenance windows\n- Holiday months: Lower activity","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_monthly_pattern/1"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_seasonal_temperature_pattern/1","doc":"Get seasonal temperature variation.\n\nReturns temperature offset in Celsius based on month and location patterns.\nSimulates realistic seasonal temperature changes.","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_seasonal_temperature_pattern/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.get_seasonal_temperature_pattern/1","doc":"# January (winter)\n    offset = SnmpKit.SnmpSim.TimePatterns.get_seasonal_temperature_pattern(~U[2024-01-15 14:00:00Z])\n    # Returns: -8.5\n\n    # July (summer)\n    offset = SnmpKit.SnmpSim.TimePatterns.get_seasonal_temperature_pattern(~U[2024-07-15 14:00:00Z])\n    # Returns: 12.3","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_seasonal_temperature_pattern/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.TimePatterns.get_weekly_pattern/1","doc":"Get weekly pattern factor based on day of week.\n\nReturns multiplier for weekday vs weekend patterns:\n- Monday-Friday: 1.0 (full pattern)\n- Saturday: 0.7 (reduced business, increased residential)\n- Sunday: 0.5 (lowest overall usage)","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_weekly_pattern/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.TimePatterns.get_weekly_pattern/1","doc":"# Tuesday\n    factor = SnmpKit.SnmpSim.TimePatterns.get_weekly_pattern(~U[2024-01-16 14:00:00Z])\n    # Returns: 1.0\n\n    # Saturday\n    factor = SnmpKit.SnmpSim.TimePatterns.get_weekly_pattern(~U[2024-01-20 14:00:00Z])\n    # Returns: 0.7","ref":"SnmpKit.SnmpSim.TimePatterns.html#get_weekly_pattern/1-examples"},{"type":"module","title":"SnmpKit.SnmpSim.ValueSimulator","doc":"Generate realistic values based on MIB-derived behavior patterns.\nSupports counters, gauges, enums, and correlated metrics with time-based variations.","ref":"SnmpKit.SnmpSim.ValueSimulator.html"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.apply_configurable_jitter/4","doc":"Apply configurable jitter to gauge values based on device and metric type.\nDifferent metrics have different jitter characteristics.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#apply_configurable_jitter/4"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.apply_device_specific_counter_behavior/4","doc":"Simulate realistic counter wrap behavior with device-specific patterns.\nDifferent device types may handle wrap differently.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#apply_device_specific_counter_behavior/4"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.counter_approaching_wrap?/3","doc":"Check if a counter value is approaching its maximum and likely to wrap soon.\nUsed to predict and prepare for counter wrap events.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#counter_approaching_wrap?/3"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.handle_counter_discontinuity/3","doc":"Generate counter discontinuity events that occur during counter wraps.\nSome devices increment discontinuity counters when main counters wrap.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#handle_counter_discontinuity/3"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.simulate_value/3","doc":"Simulate a value based on profile data, behavior configuration, and device state.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#simulate_value/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ValueSimulator.simulate_value/3","doc":"# Traffic counter simulation\n    value = SnmpKit.SnmpSim.ValueSimulator.simulate_value(\n      %{type: \"Counter32\", value: 1000000},\n      {:traffic_counter, %{rate_range: {1000, 125_000_000}}},\n      %{device_id: \"cm_001\", uptime: 3600, interface_utilization: 0.3}\n    )","ref":"SnmpKit.SnmpSim.ValueSimulator.html#simulate_value/3-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ValueSimulator.time_until_counter_wrap/3","doc":"Calculate the time until counter wrap based on current increment rate.\nReturns estimated seconds until wrap occurs.","ref":"SnmpKit.SnmpSim.ValueSimulator.html#time_until_counter_wrap/3"},{"type":"module","title":"SnmpKit.SnmpSim.WalkParser","doc":"Parse both named MIB and numeric OID walk file formats.\nHandle different snmpwalk output variations automatically.","ref":"SnmpKit.SnmpSim.WalkParser.html"},{"type":"function","title":"SnmpKit.SnmpSim.WalkParser.parse_walk_file/1","doc":"Parse a walk file and return a map of OID -> value mappings.\n\nSupports both named MIB format and numeric OID format:\n- Named: \"IF-MIB::ifInOctets.2 = Counter32: 1234567890\"\n- Numeric: \".1.3.6.1.2.1.2.2.1.10.2 = Counter32: 1234567890\"","ref":"SnmpKit.SnmpSim.WalkParser.html#parse_walk_file/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.WalkParser.parse_walk_file/1","doc":"{:ok, oid_map} = SnmpKit.SnmpSim.WalkParser.parse_walk_file(\"priv/walks/cable_modem.walk\")","ref":"SnmpKit.SnmpSim.WalkParser.html#parse_walk_file/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.WalkParser.parse_walk_line/1","doc":"Parse a single line from a walk file.","ref":"SnmpKit.SnmpSim.WalkParser.html#parse_walk_line/1"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.WalkParser.parse_walk_line/1","doc":"# Named MIB format\n    result = SnmpKit.SnmpSim.WalkParser.parse_walk_line(\"IF-MIB::ifInOctets.2 = Counter32: 1234567890\")\n    # => {\"1.3.6.1.2.1.2.2.1.10.2\", %{type: \"Counter32\", value: 1234567890, mib_name: \"IF-MIB::ifInOctets.2\"}}\n\n    # Numeric OID format\n    result = SnmpKit.SnmpSim.WalkParser.parse_walk_line(\".1.3.6.1.2.1.2.2.1.10.2 = Counter32: 1234567890\")\n    # => {\"1.3.6.1.2.1.2.2.1.10.2\", %{type: \"Counter32\", value: 1234567890}}","ref":"SnmpKit.SnmpSim.WalkParser.html#parse_walk_line/1-examples"},{"type":"module","title":"SnmpKit","doc":"Unified API for SnmpKit - A comprehensive SNMP toolkit for Elixir.\n\nThis module provides a clean, organized interface to all SnmpKit functionality\nthrough context-based sub-modules:\n\n- `SnmpKit.SNMP` - SNMP operations (get, walk, bulk, etc.)\n- `SnmpKit.MIB` - MIB compilation, loading, and resolution\n- `SnmpKit.Sim` - SNMP device simulation and testing","ref":"SnmpKit.html"},{"type":"module","title":"Quick Examples - SnmpKit","doc":"# SNMP Operations\n    {:ok, value} = SnmpKit.SNMP.get(\"192.168.1.1\", \"sysDescr.0\")\n    {:ok, results} = SnmpKit.SNMP.walk(\"192.168.1.1\", \"system\")\n\n    # MIB Operations\n    {:ok, oid} = SnmpKit.MIB.resolve(\"sysDescr.0\")\n    {:ok, compiled} = SnmpKit.MIB.compile(\"MY-MIB.mib\")\n\n    # Simulation\n    {:ok, device} = SnmpKit.Sim.start_device(profile, port: 1161)\n\nFor backward compatibility, many common operations are also available\ndirectly on the main SnmpKit module.","ref":"SnmpKit.html#module-quick-examples"},{"type":"function","title":"SnmpKit.get/2","doc":"See  SnmpKit.SnmpMgr.get/2 .","ref":"SnmpKit.html#get/2"},{"type":"function","title":"SnmpKit.get/3","doc":"See  SnmpKit.SnmpMgr.get/3 .","ref":"SnmpKit.html#get/3"},{"type":"function","title":"SnmpKit.resolve/1","doc":"See  SnmpKit.SnmpMgr.MIB.resolve/1 .","ref":"SnmpKit.html#resolve/1"},{"type":"function","title":"SnmpKit.set/3","doc":"See  SnmpKit.SnmpMgr.set/3 .","ref":"SnmpKit.html#set/3"},{"type":"function","title":"SnmpKit.set/4","doc":"See  SnmpKit.SnmpMgr.set/4 .","ref":"SnmpKit.html#set/4"},{"type":"function","title":"SnmpKit.walk/2","doc":"See  SnmpKit.SnmpMgr.walk/2 .","ref":"SnmpKit.html#walk/2"},{"type":"function","title":"SnmpKit.walk/3","doc":"See  SnmpKit.SnmpMgr.walk/3 .","ref":"SnmpKit.html#walk/3"},{"type":"module","title":"SnmpKit.MIB","doc":"MIB (Management Information Base) operations.\n\nThis module provides comprehensive MIB support including:\n- MIB compilation from source files\n- Loading and managing compiled MIBs\n- OID name resolution and reverse lookup\n- MIB tree navigation and analysis","ref":"SnmpKit.MIB.html"},{"type":"function","title":"SnmpKit.MIB.children/1","doc":"See  SnmpKit.SnmpMgr.MIB.children/1 .","ref":"SnmpKit.MIB.html#children/1"},{"type":"function","title":"SnmpKit.MIB.compile/1","doc":"See  SnmpKit.SnmpMgr.MIB.compile/1 .","ref":"SnmpKit.MIB.html#compile/1"},{"type":"function","title":"SnmpKit.MIB.compile/2","doc":"See  SnmpKit.SnmpMgr.MIB.compile/2 .","ref":"SnmpKit.MIB.html#compile/2"},{"type":"function","title":"SnmpKit.MIB.compile_all/1","doc":"See  SnmpKit.SnmpLib.MIB.compile_all/1 .","ref":"SnmpKit.MIB.html#compile_all/1"},{"type":"function","title":"SnmpKit.MIB.compile_all/2","doc":"See  SnmpKit.SnmpLib.MIB.compile_all/2 .","ref":"SnmpKit.MIB.html#compile_all/2"},{"type":"function","title":"SnmpKit.MIB.compile_dir/1","doc":"See  SnmpKit.SnmpMgr.MIB.compile_dir/1 .","ref":"SnmpKit.MIB.html#compile_dir/1"},{"type":"function","title":"SnmpKit.MIB.compile_dir/2","doc":"See  SnmpKit.SnmpMgr.MIB.compile_dir/2 .","ref":"SnmpKit.MIB.html#compile_dir/2"},{"type":"function","title":"SnmpKit.MIB.compile_raw/1","doc":"See  SnmpKit.SnmpLib.MIB.compile/1 .","ref":"SnmpKit.MIB.html#compile_raw/1"},{"type":"function","title":"SnmpKit.MIB.compile_raw/2","doc":"See  SnmpKit.SnmpLib.MIB.compile/2 .","ref":"SnmpKit.MIB.html#compile_raw/2"},{"type":"function","title":"SnmpKit.MIB.compile_string/1","doc":"See  SnmpKit.SnmpLib.MIB.compile_string/1 .","ref":"SnmpKit.MIB.html#compile_string/1"},{"type":"function","title":"SnmpKit.MIB.compile_string/2","doc":"See  SnmpKit.SnmpLib.MIB.compile_string/2 .","ref":"SnmpKit.MIB.html#compile_string/2"},{"type":"function","title":"SnmpKit.MIB.load/1","doc":"See  SnmpKit.SnmpMgr.MIB.load/1 .","ref":"SnmpKit.MIB.html#load/1"},{"type":"function","title":"SnmpKit.MIB.load_and_integrate_mib/1","doc":"See  SnmpKit.SnmpMgr.MIB.load_and_integrate_mib/1 .","ref":"SnmpKit.MIB.html#load_and_integrate_mib/1"},{"type":"function","title":"SnmpKit.MIB.load_and_integrate_mib/2","doc":"See  SnmpKit.SnmpMgr.MIB.load_and_integrate_mib/2 .","ref":"SnmpKit.MIB.html#load_and_integrate_mib/2"},{"type":"function","title":"SnmpKit.MIB.load_compiled/1","doc":"See  SnmpKit.SnmpLib.MIB.load_compiled/1 .","ref":"SnmpKit.MIB.html#load_compiled/1"},{"type":"function","title":"SnmpKit.MIB.load_standard_mibs/0","doc":"See  SnmpKit.SnmpMgr.MIB.load_standard_mibs/0 .","ref":"SnmpKit.MIB.html#load_standard_mibs/0"},{"type":"function","title":"SnmpKit.MIB.parent/1","doc":"See  SnmpKit.SnmpMgr.MIB.parent/1 .","ref":"SnmpKit.MIB.html#parent/1"},{"type":"function","title":"SnmpKit.MIB.parse_mib_content/1","doc":"See  SnmpKit.SnmpMgr.MIB.parse_mib_content/1 .","ref":"SnmpKit.MIB.html#parse_mib_content/1"},{"type":"function","title":"SnmpKit.MIB.parse_mib_content/2","doc":"See  SnmpKit.SnmpMgr.MIB.parse_mib_content/2 .","ref":"SnmpKit.MIB.html#parse_mib_content/2"},{"type":"function","title":"SnmpKit.MIB.parse_mib_file/1","doc":"See  SnmpKit.SnmpMgr.MIB.parse_mib_file/1 .","ref":"SnmpKit.MIB.html#parse_mib_file/1"},{"type":"function","title":"SnmpKit.MIB.parse_mib_file/2","doc":"See  SnmpKit.SnmpMgr.MIB.parse_mib_file/2 .","ref":"SnmpKit.MIB.html#parse_mib_file/2"},{"type":"function","title":"SnmpKit.MIB.resolve/1","doc":"See  SnmpKit.SnmpMgr.MIB.resolve/1 .","ref":"SnmpKit.MIB.html#resolve/1"},{"type":"function","title":"SnmpKit.MIB.resolve_enhanced/1","doc":"See  SnmpKit.SnmpMgr.MIB.resolve_enhanced/1 .","ref":"SnmpKit.MIB.html#resolve_enhanced/1"},{"type":"function","title":"SnmpKit.MIB.resolve_enhanced/2","doc":"See  SnmpKit.SnmpMgr.MIB.resolve_enhanced/2 .","ref":"SnmpKit.MIB.html#resolve_enhanced/2"},{"type":"function","title":"SnmpKit.MIB.reverse_lookup/1","doc":"See  SnmpKit.SnmpMgr.MIB.reverse_lookup/1 .","ref":"SnmpKit.MIB.html#reverse_lookup/1"},{"type":"function","title":"SnmpKit.MIB.start_link/0","doc":"See  SnmpKit.SnmpMgr.MIB.start_link/0 .","ref":"SnmpKit.MIB.html#start_link/0"},{"type":"function","title":"SnmpKit.MIB.start_link/1","doc":"See  SnmpKit.SnmpMgr.MIB.start_link/1 .","ref":"SnmpKit.MIB.html#start_link/1"},{"type":"function","title":"SnmpKit.MIB.walk_tree/1","doc":"See  SnmpKit.SnmpMgr.MIB.walk_tree/1 .","ref":"SnmpKit.MIB.html#walk_tree/1"},{"type":"function","title":"SnmpKit.MIB.walk_tree/2","doc":"See  SnmpKit.SnmpMgr.MIB.walk_tree/2 .","ref":"SnmpKit.MIB.html#walk_tree/2"},{"type":"module","title":"SnmpKit.SNMP","doc":"SNMP client operations for querying and managing SNMP devices.\n\nThis module provides all SNMP protocol operations including:\n- Basic operations: get, set, get_next\n- Bulk operations: get_bulk, walk, table operations\n- Advanced features: streaming, async operations, multi-target\n- Pretty formatting and analysis tools","ref":"SnmpKit.SNMP.html"},{"type":"function","title":"SnmpKit.SNMP.adaptive_walk/2","doc":"See  SnmpKit.SnmpMgr.adaptive_walk/2 .","ref":"SnmpKit.SNMP.html#adaptive_walk/2"},{"type":"function","title":"SnmpKit.SNMP.adaptive_walk/3","doc":"See  SnmpKit.SnmpMgr.adaptive_walk/3 .","ref":"SnmpKit.SNMP.html#adaptive_walk/3"},{"type":"function","title":"SnmpKit.SNMP.analyze_table/1","doc":"See  SnmpKit.SnmpMgr.analyze_table/1 .","ref":"SnmpKit.SNMP.html#analyze_table/1"},{"type":"function","title":"SnmpKit.SNMP.analyze_table/2","doc":"See  SnmpKit.SnmpMgr.analyze_table/2 .","ref":"SnmpKit.SNMP.html#analyze_table/2"},{"type":"function","title":"SnmpKit.SNMP.benchmark_device/2","doc":"See  SnmpKit.SnmpMgr.benchmark_device/2 .","ref":"SnmpKit.SNMP.html#benchmark_device/2"},{"type":"function","title":"SnmpKit.SNMP.benchmark_device/3","doc":"See  SnmpKit.SnmpMgr.benchmark_device/3 .","ref":"SnmpKit.SNMP.html#benchmark_device/3"},{"type":"function","title":"SnmpKit.SNMP.bulk_pretty/2","doc":"See  SnmpKit.SnmpMgr.bulk_pretty/2 .","ref":"SnmpKit.SNMP.html#bulk_pretty/2"},{"type":"function","title":"SnmpKit.SNMP.bulk_pretty/3","doc":"See  SnmpKit.SnmpMgr.bulk_pretty/3 .","ref":"SnmpKit.SNMP.html#bulk_pretty/3"},{"type":"function","title":"SnmpKit.SNMP.bulk_walk/2","doc":"See  SnmpKit.SnmpMgr.bulk_walk/2 .","ref":"SnmpKit.SNMP.html#bulk_walk/2"},{"type":"function","title":"SnmpKit.SNMP.bulk_walk/3","doc":"See  SnmpKit.SnmpMgr.bulk_walk/3 .","ref":"SnmpKit.SNMP.html#bulk_walk/3"},{"type":"function","title":"SnmpKit.SNMP.bulk_walk_pretty/2","doc":"See  SnmpKit.SnmpMgr.bulk_walk_pretty/2 .","ref":"SnmpKit.SNMP.html#bulk_walk_pretty/2"},{"type":"function","title":"SnmpKit.SNMP.bulk_walk_pretty/3","doc":"See  SnmpKit.SnmpMgr.bulk_walk_pretty/3 .","ref":"SnmpKit.SNMP.html#bulk_walk_pretty/3"},{"type":"function","title":"SnmpKit.SNMP.engine_batch/1","doc":"See  SnmpKit.SnmpMgr.engine_batch/1 .","ref":"SnmpKit.SNMP.html#engine_batch/1"},{"type":"function","title":"SnmpKit.SNMP.engine_batch/2","doc":"See  SnmpKit.SnmpMgr.engine_batch/2 .","ref":"SnmpKit.SNMP.html#engine_batch/2"},{"type":"function","title":"SnmpKit.SNMP.engine_request/1","doc":"See  SnmpKit.SnmpMgr.engine_request/1 .","ref":"SnmpKit.SNMP.html#engine_request/1"},{"type":"function","title":"SnmpKit.SNMP.engine_request/2","doc":"See  SnmpKit.SnmpMgr.engine_request/2 .","ref":"SnmpKit.SNMP.html#engine_request/2"},{"type":"function","title":"SnmpKit.SNMP.get/2","doc":"See  SnmpKit.SnmpMgr.get/2 .","ref":"SnmpKit.SNMP.html#get/2"},{"type":"function","title":"SnmpKit.SNMP.get/3","doc":"See  SnmpKit.SnmpMgr.get/3 .","ref":"SnmpKit.SNMP.html#get/3"},{"type":"function","title":"SnmpKit.SNMP.get_async/2","doc":"See  SnmpKit.SnmpMgr.get_async/2 .","ref":"SnmpKit.SNMP.html#get_async/2"},{"type":"function","title":"SnmpKit.SNMP.get_async/3","doc":"See  SnmpKit.SnmpMgr.get_async/3 .","ref":"SnmpKit.SNMP.html#get_async/3"},{"type":"function","title":"SnmpKit.SNMP.get_bulk/2","doc":"See  SnmpKit.SnmpMgr.get_bulk/2 .","ref":"SnmpKit.SNMP.html#get_bulk/2"},{"type":"function","title":"SnmpKit.SNMP.get_bulk/3","doc":"See  SnmpKit.SnmpMgr.get_bulk/3 .","ref":"SnmpKit.SNMP.html#get_bulk/3"},{"type":"function","title":"SnmpKit.SNMP.get_bulk_async/2","doc":"See  SnmpKit.SnmpMgr.get_bulk_async/2 .","ref":"SnmpKit.SNMP.html#get_bulk_async/2"},{"type":"function","title":"SnmpKit.SNMP.get_bulk_async/3","doc":"See  SnmpKit.SnmpMgr.get_bulk_async/3 .","ref":"SnmpKit.SNMP.html#get_bulk_async/3"},{"type":"function","title":"SnmpKit.SNMP.get_bulk_multi/1","doc":"See  SnmpKit.SnmpMgr.get_bulk_multi/1 .","ref":"SnmpKit.SNMP.html#get_bulk_multi/1"},{"type":"function","title":"SnmpKit.SNMP.get_bulk_multi/2","doc":"See  SnmpKit.SnmpMgr.get_bulk_multi/2 .","ref":"SnmpKit.SNMP.html#get_bulk_multi/2"},{"type":"function","title":"SnmpKit.SNMP.get_column/3","doc":"See  SnmpKit.SnmpMgr.get_column/3 .","ref":"SnmpKit.SNMP.html#get_column/3"},{"type":"function","title":"SnmpKit.SNMP.get_column/4","doc":"See  SnmpKit.SnmpMgr.get_column/4 .","ref":"SnmpKit.SNMP.html#get_column/4"},{"type":"function","title":"SnmpKit.SNMP.get_engine_stats/0","doc":"See  SnmpKit.SnmpMgr.get_engine_stats/0 .","ref":"SnmpKit.SNMP.html#get_engine_stats/0"},{"type":"function","title":"SnmpKit.SNMP.get_engine_stats/1","doc":"See  SnmpKit.SnmpMgr.get_engine_stats/1 .","ref":"SnmpKit.SNMP.html#get_engine_stats/1"},{"type":"function","title":"SnmpKit.SNMP.get_multi/1","doc":"See  SnmpKit.SnmpMgr.get_multi/1 .","ref":"SnmpKit.SNMP.html#get_multi/1"},{"type":"function","title":"SnmpKit.SNMP.get_multi/2","doc":"See  SnmpKit.SnmpMgr.get_multi/2 .","ref":"SnmpKit.SNMP.html#get_multi/2"},{"type":"function","title":"SnmpKit.SNMP.get_next/2","doc":"See  SnmpKit.SnmpMgr.get_next/2 .","ref":"SnmpKit.SNMP.html#get_next/2"},{"type":"function","title":"SnmpKit.SNMP.get_next/3","doc":"See  SnmpKit.SnmpMgr.get_next/3 .","ref":"SnmpKit.SNMP.html#get_next/3"},{"type":"function","title":"SnmpKit.SNMP.get_pretty/2","doc":"See  SnmpKit.SnmpMgr.get_pretty/2 .","ref":"SnmpKit.SNMP.html#get_pretty/2"},{"type":"function","title":"SnmpKit.SNMP.get_pretty/3","doc":"See  SnmpKit.SnmpMgr.get_pretty/3 .","ref":"SnmpKit.SNMP.html#get_pretty/3"},{"type":"function","title":"SnmpKit.SNMP.get_table/2","doc":"See  SnmpKit.SnmpMgr.get_table/2 .","ref":"SnmpKit.SNMP.html#get_table/2"},{"type":"function","title":"SnmpKit.SNMP.get_table/3","doc":"See  SnmpKit.SnmpMgr.get_table/3 .","ref":"SnmpKit.SNMP.html#get_table/3"},{"type":"function","title":"SnmpKit.SNMP.get_with_type/2","doc":"See  SnmpKit.SnmpMgr.get_with_type/2 .","ref":"SnmpKit.SNMP.html#get_with_type/2"},{"type":"function","title":"SnmpKit.SNMP.get_with_type/3","doc":"See  SnmpKit.SnmpMgr.get_with_type/3 .","ref":"SnmpKit.SNMP.html#get_with_type/3"},{"type":"function","title":"SnmpKit.SNMP.record_metric/3","doc":"See  SnmpKit.SnmpMgr.record_metric/3 .","ref":"SnmpKit.SNMP.html#record_metric/3"},{"type":"function","title":"SnmpKit.SNMP.record_metric/4","doc":"See  SnmpKit.SnmpMgr.record_metric/4 .","ref":"SnmpKit.SNMP.html#record_metric/4"},{"type":"function","title":"SnmpKit.SNMP.set/3","doc":"See  SnmpKit.SnmpMgr.set/3 .","ref":"SnmpKit.SNMP.html#set/3"},{"type":"function","title":"SnmpKit.SNMP.set/4","doc":"See  SnmpKit.SnmpMgr.set/4 .","ref":"SnmpKit.SNMP.html#set/4"},{"type":"function","title":"SnmpKit.SNMP.start_engine/0","doc":"See  SnmpKit.SnmpMgr.start_engine/0 .","ref":"SnmpKit.SNMP.html#start_engine/0"},{"type":"function","title":"SnmpKit.SNMP.start_engine/1","doc":"See  SnmpKit.SnmpMgr.start_engine/1 .","ref":"SnmpKit.SNMP.html#start_engine/1"},{"type":"function","title":"SnmpKit.SNMP.table_stream/2","doc":"See  SnmpKit.SnmpMgr.table_stream/2 .","ref":"SnmpKit.SNMP.html#table_stream/2"},{"type":"function","title":"SnmpKit.SNMP.table_stream/3","doc":"See  SnmpKit.SnmpMgr.table_stream/3 .","ref":"SnmpKit.SNMP.html#table_stream/3"},{"type":"function","title":"SnmpKit.SNMP.walk/2","doc":"See  SnmpKit.SnmpMgr.walk/2 .","ref":"SnmpKit.SNMP.html#walk/2"},{"type":"function","title":"SnmpKit.SNMP.walk/3","doc":"See  SnmpKit.SnmpMgr.walk/3 .","ref":"SnmpKit.SNMP.html#walk/3"},{"type":"function","title":"SnmpKit.SNMP.walk_multi/1","doc":"See  SnmpKit.SnmpMgr.walk_multi/1 .","ref":"SnmpKit.SNMP.html#walk_multi/1"},{"type":"function","title":"SnmpKit.SNMP.walk_multi/2","doc":"See  SnmpKit.SnmpMgr.walk_multi/2 .","ref":"SnmpKit.SNMP.html#walk_multi/2"},{"type":"function","title":"SnmpKit.SNMP.walk_pretty/2","doc":"See  SnmpKit.SnmpMgr.walk_pretty/2 .","ref":"SnmpKit.SNMP.html#walk_pretty/2"},{"type":"function","title":"SnmpKit.SNMP.walk_pretty/3","doc":"See  SnmpKit.SnmpMgr.walk_pretty/3 .","ref":"SnmpKit.SNMP.html#walk_pretty/3"},{"type":"function","title":"SnmpKit.SNMP.walk_stream/2","doc":"See  SnmpKit.SnmpMgr.walk_stream/2 .","ref":"SnmpKit.SNMP.html#walk_stream/2"},{"type":"function","title":"SnmpKit.SNMP.walk_stream/3","doc":"See  SnmpKit.SnmpMgr.walk_stream/3 .","ref":"SnmpKit.SNMP.html#walk_stream/3"},{"type":"function","title":"SnmpKit.SNMP.walk_table/2","doc":"See  SnmpKit.SnmpMgr.walk_table/2 .","ref":"SnmpKit.SNMP.html#walk_table/2"},{"type":"function","title":"SnmpKit.SNMP.walk_table/3","doc":"See  SnmpKit.SnmpMgr.walk_table/3 .","ref":"SnmpKit.SNMP.html#walk_table/3"},{"type":"function","title":"SnmpKit.SNMP.with_circuit_breaker/2","doc":"See  SnmpKit.SnmpMgr.with_circuit_breaker/2 .","ref":"SnmpKit.SNMP.html#with_circuit_breaker/2"},{"type":"function","title":"SnmpKit.SNMP.with_circuit_breaker/3","doc":"See  SnmpKit.SnmpMgr.with_circuit_breaker/3 .","ref":"SnmpKit.SNMP.html#with_circuit_breaker/3"},{"type":"module","title":"SnmpKit.Sim","doc":"SNMP device simulation for testing and development.\n\nThis module provides tools for creating and managing simulated SNMP devices:\n- Start individual devices with custom profiles\n- Create device populations for testing\n- Manage device lifecycles","ref":"SnmpKit.Sim.html"},{"type":"function","title":"SnmpKit.Sim.start_device/1","doc":"See  SnmpKit.TestSupport.start_device/1 .","ref":"SnmpKit.Sim.html#start_device/1"},{"type":"function","title":"SnmpKit.Sim.start_device/2","doc":"See  SnmpKit.TestSupport.start_device/2 .","ref":"SnmpKit.Sim.html#start_device/2"},{"type":"function","title":"SnmpKit.Sim.start_device_population/1","doc":"See  SnmpKit.TestSupport.start_device_population/1 .","ref":"SnmpKit.Sim.html#start_device_population/1"},{"type":"function","title":"SnmpKit.Sim.start_device_population/2","doc":"See  SnmpKit.TestSupport.start_device_population/2 .","ref":"SnmpKit.Sim.html#start_device_population/2"},{"type":"module","title":"SnmpKit.SnmpLib","doc":"Unified SNMP library providing RFC-compliant PDU encoding/decoding, OID manipulation, and SNMP utilities.\n\nThis library consolidates common SNMP functionality from multiple projects into a single,\nwell-tested, and performant library suitable for both SNMP managers and simulators.\nPhase 2 provides complete RFC compliance including SNMPv2c exception values and\nproper multibyte OID encoding.","ref":"SnmpKit.SnmpLib.html"},{"type":"module","title":"Phase 2 Core Modules - SnmpKit.SnmpLib","doc":"- **`SnmpKit.SnmpLib.PDU`** - SNMP PDU encoding/decoding with full RFC compliance\n  - SNMPv1 and SNMPv2c protocol support\n  - SNMPv2c exception values (noSuchObject, noSuchInstance, endOfMibView)\n  - High-performance encoding/decoding\n  - Comprehensive error handling\n\n- **`SnmpKit.SnmpLib.ASN1`** - Low-level ASN.1 BER encoding/decoding\n  - RFC-compliant OID multibyte encoding (values ≥ 128)\n  - Complete integer, string, null, sequence support\n  - Optimized length handling for large values\n  - Robust error handling and validation\n\n- **`SnmpKit.SnmpLib.OID`** - OID string/list conversion and manipulation utilities\n  - Fast string/list conversions with validation\n  - Tree operations and comparisons\n  - Table index parsing and construction\n  - Enterprise OID utilities\n\n- **`SnmpKit.SnmpLib.Types`** - SNMP data type validation and formatting\n  - Complete SNMP type system support\n  - SNMPv2c exception value handling\n  - Human-readable formatting\n  - Range checking and validation\n\n- **`SnmpKit.SnmpLib.Transport`** - UDP socket management for SNMP communications\n  - Socket creation and management\n  - Address resolution and validation\n  - Performance optimizations","ref":"SnmpKit.SnmpLib.html#module-phase-2-core-modules"},{"type":"module","title":"Key Features - SnmpKit.SnmpLib","doc":"- **100% RFC Compliance**: Passes comprehensive RFC test suite (30/30 tests)\n- **SNMPv2c Exception Values**: Proper encoding/decoding of special response values\n- **Multibyte OID Support**: Correct handling of OID components ≥ 128\n- **High Performance**: Optimized encoding/decoding with fast paths\n- **Comprehensive Testing**: Extensive test coverage with edge cases\n- **Production Ready**: Used in real SNMP management systems","ref":"SnmpKit.SnmpLib.html#module-key-features"},{"type":"module","title":"Phase 3B: Advanced Features - SnmpKit.SnmpLib","doc":"Phase 3B adds enterprise-grade capabilities for high-scale SNMP deployments:\n\n- **`SnmpKit.SnmpLib.Pool`** - Connection pooling and session management\n  - FIFO, round-robin, and device-affinity strategies\n  - Automatic overflow handling and health monitoring\n  - 60-80% reduction in socket creation overhead\n  - Support for 100+ concurrent device operations\n\n- **`SnmpKit.SnmpLib.ErrorHandler`** - Intelligent error handling and recovery\n  - Exponential backoff with jitter for retry operations\n  - Circuit breaker patterns for failing device management\n  - Error classification (transient, permanent, degraded)\n  - Adaptive timeout calculation based on device performance\n\n- **`SnmpKit.SnmpLib.Monitor`** - Performance monitoring and analytics\n  - Real-time operation metrics and device statistics\n  - Configurable alerting system with callback support\n  - Data export in JSON, CSV, and Prometheus formats\n  - Health scoring and trend analysis\n\n- **`SnmpKit.SnmpLib.Manager`** - High-level SNMP management operations\n  - Simple API for GET, GETBULK, SET operations\n  - Connection reuse and performance optimizations\n  - Comprehensive error handling with meaningful messages\n  - Timeout management and community support","ref":"SnmpKit.SnmpLib.html#module-phase-3b-advanced-features"},{"type":"module","title":"Phase 4: Real-World Integration & Optimization - SnmpKit.SnmpLib","doc":"Phase 4 provides production-ready integration and optimization features:\n\n- **`SnmpKit.SnmpLib.Config`** - Configuration management system\n  - Environment-aware configuration (dev/test/prod)\n  - Hot-reload capabilities and validation\n  - Multi-tenant deployment support\n  - Secrets management and security\n\n- **`SnmpKit.SnmpLib.Dashboard`** - Real-time monitoring and visualization\n  - Live performance dashboards and metrics\n  - Alert management and notification routing\n  - Prometheus/Grafana integration\n  - Historical analytics and capacity planning\n\n- **`SnmpKit.SnmpLib.Cache`** - Intelligent caching system\n  - Multi-level caching (L1/L2/L3) with compression\n  - Adaptive TTL based on data volatility\n  - Smart invalidation and cache warming\n  - 50-80% reduction in redundant queries","ref":"SnmpKit.SnmpLib.html#module-phase-4-real-world-integration-optimization"},{"type":"module","title":"Quick Start - SnmpKit.SnmpLib","doc":"","ref":"SnmpKit.SnmpLib.html#module-quick-start"},{"type":"module","title":"Basic SNMP Operations - SnmpKit.SnmpLib","doc":"# Simple SNMP GET operation\n    {:ok, {type, value}} = SnmpKit.SnmpLib.Manager.get(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1, 1, 0])\n\n    # SNMP GETBULK for efficient bulk retrieval\n    {:ok, results} = SnmpKit.SnmpLib.Manager.get_bulk(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 2, 2],\n                                               max_repetitions: 20)\n\n    # SNMP SET operation\n    {:ok, :success} = SnmpKit.SnmpLib.Manager.set(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 1, 5, 0],\n                                          {:string, \"New System Name\"})","ref":"SnmpKit.SnmpLib.html#module-basic-snmp-operations"},{"type":"module","title":"High-Performance Connection Pooling - SnmpKit.SnmpLib","doc":"# Start a connection pool for network monitoring\n    {:ok, _pid} = SnmpKit.SnmpLib.Pool.start_pool(:network_monitor,\n      strategy: :device_affinity,\n      size: 20,\n      max_overflow: 10\n    )\n\n    # Use pooled connections for improved performance\n    SnmpKit.SnmpLib.Pool.with_connection(:network_monitor, \"192.168.1.1\", fn conn ->\n      SnmpKit.SnmpLib.Manager.get_multi(conn.socket, \"192.168.1.1\", oids, conn.opts)\n    end)","ref":"SnmpKit.SnmpLib.html#module-high-performance-connection-pooling"},{"type":"module","title":"Intelligent Error Handling - SnmpKit.SnmpLib","doc":"# Retry operations with exponential backoff\n    result = SnmpKit.SnmpLib.ErrorHandler.with_retry(fn ->\n      SnmpKit.SnmpLib.Manager.get(\"unreliable.device.local\", [1, 3, 6, 1, 2, 1, 1, 1, 0])\n    end, max_attempts: 5, base_delay: 2000)\n\n    # Circuit breaker for problematic devices\n    {:ok, breaker} = SnmpKit.SnmpLib.ErrorHandler.start_circuit_breaker(\"192.168.1.1\")\n    result = SnmpKit.SnmpLib.ErrorHandler.call_through_breaker(breaker, fn ->\n      SnmpKit.SnmpLib.Manager.get_bulk(\"192.168.1.1\", [1, 3, 6, 1, 2, 1, 2, 2])\n    end)","ref":"SnmpKit.SnmpLib.html#module-intelligent-error-handling"},{"type":"module","title":"Performance Monitoring and Analytics - SnmpKit.SnmpLib","doc":"# Start monitoring system\n    {:ok, _pid} = SnmpKit.SnmpLib.Monitor.start_link()\n\n    # Record operation metrics\n    SnmpKit.SnmpLib.Monitor.record_operation(%{\n      device: \"192.168.1.1\",\n      operation: :get,\n      duration: 245,\n      result: :success\n    })\n\n    # Get device statistics and health scores\n    stats = SnmpKit.SnmpLib.Monitor.get_device_stats(\"192.168.1.1\")\n    IO.puts(\"Device health score: \" <> to_string(stats.health_score))\n\n    # Set up automated alerting\n    SnmpKit.SnmpLib.Monitor.set_alert_threshold(\"192.168.1.1\", :response_time, 5000)","ref":"SnmpKit.SnmpLib.html#module-performance-monitoring-and-analytics"},{"type":"module","title":"Configuration Management - SnmpKit.SnmpLib","doc":"# Load production configuration\n    {:ok, _pid} = SnmpKit.SnmpLib.Config.start_link(\n      config_file: \"/etc/snmp_lib/production.exs\",\n      environment: :prod\n    )\n\n    # Get configuration values with fallbacks\n    timeout = SnmpKit.SnmpLib.Config.get(:snmp, :default_timeout, 5000)\n    pool_size = SnmpKit.SnmpLib.Config.get(:pool, :default_size, 10)\n\n    # Hot-reload configuration\n    :ok = SnmpKit.SnmpLib.Config.reload()","ref":"SnmpKit.SnmpLib.html#module-configuration-management"},{"type":"module","title":"Real-Time Dashboard and Monitoring - SnmpKit.SnmpLib","doc":"# Start dashboard with Prometheus integration\n    {:ok, _pid} = SnmpKit.SnmpLib.Dashboard.start_link(\n      port: 4000,\n      prometheus_enabled: true,\n      retention_days: 14\n    )\n\n    # Record custom metrics\n    SnmpKit.SnmpLib.Dashboard.record_metric(:snmp_response_time, 125, %{\n      device: \"192.168.1.1\",\n      operation: \"get\"\n    })\n\n    # Create alerts for monitoring\n    SnmpKit.SnmpLib.Dashboard.create_alert(:device_unreachable, :critical, %{\n      device: \"192.168.1.1\",\n      consecutive_failures: 5\n    })\n\n    # Export metrics for external systems\n    prometheus_data = SnmpKit.SnmpLib.Dashboard.export_prometheus()","ref":"SnmpKit.SnmpLib.html#module-real-time-dashboard-and-monitoring"},{"type":"module","title":"Intelligent Caching - SnmpKit.SnmpLib","doc":"# Start cache with compression and adaptive TTL\n    {:ok, _pid} = SnmpKit.SnmpLib.Cache.start_link(\n      max_size: 50_000,\n      compression_enabled: true,\n      adaptive_ttl_enabled: true\n    )\n\n    # Cache SNMP responses with adaptive TTL\n    SnmpKit.SnmpLib.Cache.put_adaptive(\"device_1:sysDescr\", description,\n      base_ttl: 3_600_000,\n      volatility: :low\n    )\n\n    # Retrieve from cache with fallback\n    device_desc = case SnmpKit.SnmpLib.Cache.get(\"device_1:sysDescr\") do\n      {:ok, cached_desc} -> cached_desc\n      :miss ->\n        {:ok, desc} = SnmpKit.SnmpLib.Manager.get(\"device_1\", [1,3,6,1,2,1,1,1,0])\n        SnmpKit.SnmpLib.Cache.put(\"device_1:sysDescr\", desc, ttl: 3_600_000)\n        desc\n    end\n\n    # Warm cache for predictable access patterns\n    SnmpKit.SnmpLib.Cache.warm_cache(\"device_1\", :auto, strategy: :predictive)","ref":"SnmpKit.SnmpLib.html#module-intelligent-caching"},{"type":"module","title":"Low-Level PDU Operations - SnmpKit.SnmpLib","doc":"# Encode a GET request PDU\n    iex> pdu = SnmpKit.SnmpLib.PDU.build_get_request([1, 3, 6, 1, 2, 1, 1, 1, 0], 12345)\n    iex> message = SnmpKit.SnmpLib.PDU.build_message(pdu, \"public\", :v2c)\n    iex> {:ok, encoded} = SnmpKit.SnmpLib.PDU.encode_message(message)\n    iex> is_binary(encoded)\n    true\n\n    # Build GETBULK request (SNMPv2c)\n    iex> bulk_pdu = SnmpKit.SnmpLib.PDU.build_get_bulk_request([1, 3, 6, 1, 2, 1, 2, 2], 456, 0, 10)\n    iex> bulk_pdu.type\n    :get_bulk_request\n\n    # OID manipulation with multibyte values\n    iex> {:ok, oid_list} = SnmpKit.SnmpLib.OID.string_to_list(\"1.3.6.1.4.1.200.1\")\n    iex> oid_list\n    [1, 3, 6, 1, 4, 1, 200, 1]\n    iex> {:ok, oid_string} = SnmpKit.SnmpLib.OID.list_to_string([1, 3, 6, 1, 4, 1, 200, 1])\n    iex> oid_string\n    \"1.3.6.1.4.1.200.1\"\n\n    # Handle SNMPv2c exception values\n    iex> {:ok, exception_val} = SnmpKit.SnmpLib.Types.coerce_value(:no_such_object, nil)\n    iex> exception_val\n    {:no_such_object, nil}","ref":"SnmpKit.SnmpLib.html#module-low-level-pdu-operations"},{"type":"module","title":"Real-World Integration Examples - SnmpKit.SnmpLib","doc":"","ref":"SnmpKit.SnmpLib.html#module-real-world-integration-examples"},{"type":"module","title":"Network Monitoring System - SnmpKit.SnmpLib","doc":"# Monitor multiple devices with error handling\n    defmodule NetworkMonitor do\n      def poll_devices(device_list, community \\ \"public\") do\n        device_list\n        |> Task.async_stream(fn device ->\n          case SnmpKit.SnmpLib.Manager.get(device, \"1.3.6.1.2.1.1.3.0\",\n                                   community: community, timeout: 5000) do\n            {:ok, uptime} -> {device, :ok, uptime}\n            {:error, reason} -> {device, :error, reason}\n          end\n        end, max_concurrency: 10, timeout: 10_000)\n        |> Enum.map(fn {:ok, result} -> result end)\n      end\n\n      def get_interface_stats(device, community \\ \"public\") do\n        base_oid = [1, 3, 6, 1, 2, 1, 2, 2, 1]\n\n        # Get interface table using GETBULK\n        case SnmpKit.SnmpLib.Manager.get_bulk(device, base_oid,\n                                      community: community,\n                                      max_repetitions: 50) do\n          {:ok, varbinds} ->\n            varbinds\n            |> Enum.group_by(fn {oid, _value} ->\n              # Group by interface index (last component)\n              List.last(oid)\n            end)\n            |> Enum.map(fn {if_index, binds} ->\n              %{\n                interface: if_index,\n                stats: parse_interface_binds(binds)\n              }\n            end)\n\n          {:error, reason} -> {:error, reason}\n        end\n      end\n\n      defp parse_interface_binds(binds) do\n        Enum.reduce(binds, %{}, fn {oid, value}, acc ->\n          case oid do\n            [1, 3, 6, 1, 2, 1, 2, 2, 1, 10, _] -> Map.put(acc, :in_octets, value)\n            [1, 3, 6, 1, 2, 1, 2, 2, 1, 16, _] -> Map.put(acc, :out_octets, value)\n            [1, 3, 6, 1, 2, 1, 2, 2, 1, 2, _] -> Map.put(acc, :description, value)\n            _ -> acc\n          end\n        end)\n      end\n    end\n\n    # Usage example\n    devices = [\"192.168.1.1\", \"192.168.1.2\", \"192.168.1.3\"]\n    results = NetworkMonitor.poll_devices(devices, \"monitoring\")","ref":"SnmpKit.SnmpLib.html#module-network-monitoring-system"},{"type":"module","title":"SNMP Agent Simulator - SnmpKit.SnmpLib","doc":"# Build custom SNMP responses for testing\n    defmodule SnmpSimulator do\n      def create_system_response(request_id, community) do\n        # Build response with system information\n        varbinds = [\n          {[1, 3, 6, 1, 2, 1, 1, 1, 0], \"Linux Test Server\"},\n          {[1, 3, 6, 1, 2, 1, 1, 2, 0], [1, 3, 6, 1, 4, 1, 8072]},\n          {[1, 3, 6, 1, 2, 1, 1, 3, 0], 123456789}\n        ]\n\n        response_pdu = SnmpKit.SnmpLib.PDU.build_response(request_id, 0, 0, varbinds)\n        message = SnmpKit.SnmpLib.PDU.build_message(response_pdu, community, :v2c)\n\n        case SnmpKit.SnmpLib.PDU.encode_message(message) do\n          {:ok, encoded} -> {:ok, encoded}\n          {:error, reason} -> {:error, reason}\n        end\n      end\n\n      def handle_bulk_request(request_pdu, community) do\n        # Simulate interface table response\n        base_oid = [1, 3, 6, 1, 2, 1, 2, 2, 1]\n        max_reps = request_pdu.max_repetitions\n\n        varbinds = for i <- 1..max_reps do\n          [\n            {base_oid ++ [2, i], \"eth\" <> Integer.to_string(i)},           # ifDescr\n            {base_oid ++ [10, i], :rand.uniform(1000000)}, # ifInOctets\n            {base_oid ++ [16, i], :rand.uniform(1000000)}  # ifOutOctets\n          ]\n        end |> List.flatten()\n\n        response_pdu = SnmpKit.SnmpLib.PDU.build_response(\n          request_pdu.request_id, 0, 0, varbinds\n        )\n\n        message = SnmpKit.SnmpLib.PDU.build_message(response_pdu, community, :v2c)\n        SnmpKit.SnmpLib.PDU.encode_message(message)\n      end\n    end","ref":"SnmpKit.SnmpLib.html#module-snmp-agent-simulator"},{"type":"module","title":"High-Performance Data Collection - SnmpKit.SnmpLib","doc":"# Efficient bulk data collection with connection reuse\n    defmodule PerformanceCollector do\n      def collect_interface_data(devices, opts \\ []) do\n        concurrency = Keyword.get(opts, :concurrency, 20)\n        timeout = Keyword.get(opts, :timeout, 5000)\n        community = Keyword.get(opts, :community, \"public\")\n\n        start_time = System.monotonic_time(:microsecond)\n\n        results = devices\n        |> Task.async_stream(fn device ->\n          collect_device_interfaces(device, community, timeout)\n        end, max_concurrency: concurrency, timeout: timeout + 1000)\n        |> Enum.map(fn\n          {:ok, result} -> result\n          {:exit, reason} -> {:error, {:timeout, reason}}\n        end)\n\n        end_time = System.monotonic_time(:microsecond)\n        duration_ms = (end_time - start_time) / 1000\n\n        %{\n          results: results,\n          performance: %{\n            total_devices: length(devices),\n            duration_ms: duration_ms,\n            devices_per_second: length(devices) / (duration_ms / 1000),\n            success_rate: calculate_success_rate(results)\n          }\n        }\n      end\n\n      defp collect_device_interfaces(device, community, timeout) do\n        # Use GETBULK for efficient table walking\n        case SnmpKit.SnmpLib.Manager.get_bulk(\n          device,\n          [1, 3, 6, 1, 2, 1, 2, 2, 1, 2], # ifDescr table\n          community: community,\n          timeout: timeout,\n          max_repetitions: 100\n        ) do\n          {:ok, varbinds} ->\n            {:ok, %{device: device, interface_count: length(varbinds), data: varbinds}}\n          {:error, reason} ->\n            {:error, %{device: device, reason: reason}}\n        end\n      end\n\n      defp calculate_success_rate(results) do\n        total = length(results)\n        successes = Enum.count(results, fn\n          {:ok, _} -> true\n          _ -> false\n        end)\n\n        if total > 0, do: (successes / total) * 100, else: 0\n      end\n    end","ref":"SnmpKit.SnmpLib.html#module-high-performance-data-collection"},{"type":"module","title":"Performance Benchmarking Examples - SnmpKit.SnmpLib","doc":"","ref":"SnmpKit.SnmpLib.html#module-performance-benchmarking-examples"},{"type":"module","title":"Encoding/Decoding Performance - SnmpKit.SnmpLib","doc":"# Benchmark PDU encoding performance\n    defmodule SnmpBenchmark do\n      def benchmark_encoding(iterations \\ 10_000) do\n        # Prepare test data\n        pdu = SnmpKit.SnmpLib.PDU.build_get_request([1, 3, 6, 1, 2, 1, 1, 1, 0], 12345)\n        message = SnmpKit.SnmpLib.PDU.build_message(pdu, \"public\", :v2c)\n\n        # Benchmark encoding\n        {encode_time, _} = :timer.tc(fn ->\n          for _ <- 1..iterations do\n            {:ok, _encoded} = SnmpKit.SnmpLib.PDU.encode_message(message)\n          end\n        end)\n\n        # Encode once for decoding benchmark\n        {:ok, encoded} = SnmpKit.SnmpLib.PDU.encode_message(message)\n\n        # Benchmark decoding\n        {decode_time, _} = :timer.tc(fn ->\n          for _ <- 1..iterations do\n            {:ok, _decoded} = SnmpKit.SnmpLib.PDU.decode_message(encoded)\n          end\n        end)\n\n        %{\n          iterations: iterations,\n          encode_time_ms: encode_time / 1000,\n          decode_time_ms: decode_time / 1000,\n          encode_ops_per_sec: iterations / (encode_time / 1_000_000),\n          decode_ops_per_sec: iterations / (decode_time / 1_000_000),\n          encode_time_per_op_us: encode_time / iterations,\n          decode_time_per_op_us: decode_time / iterations\n        }\n      end\n\n      def benchmark_bulk_operations(device_count \\ 100) do\n        devices = for i <- 1..device_count, do: \"192.168.1.\" <> Integer.to_string(i)\n\n        # Benchmark sequential operations\n        {seq_time, seq_results} = :timer.tc(fn ->\n          Enum.map(devices, fn device ->\n            SnmpKit.SnmpLib.Manager.get(device, [1, 3, 6, 1, 2, 1, 1, 3, 0], timeout: 100)\n          end)\n        end)\n\n        # Benchmark concurrent operations\n        {conc_time, conc_results} = :timer.tc(fn ->\n          devices\n          |> Task.async_stream(fn device ->\n            SnmpKit.SnmpLib.Manager.get(device, [1, 3, 6, 1, 2, 1, 1, 3, 0], timeout: 100)\n          end, max_concurrency: 50, timeout: 1000)\n          |> Enum.map(fn {:ok, result} -> result end)\n        end)\n\n        %{\n          device_count: device_count,\n          sequential: %{\n            time_ms: seq_time / 1000,\n            ops_per_sec: device_count / (seq_time / 1_000_000),\n            success_count: count_successes(seq_results)\n          },\n          concurrent: %{\n            time_ms: conc_time / 1000,\n            ops_per_sec: device_count / (conc_time / 1_000_000),\n            success_count: count_successes(conc_results),\n            speedup: seq_time / conc_time\n          }\n        }\n      end\n\n      def benchmark_oid_operations(iterations \\ 100_000) do\n        test_oids = [\n          \"1.3.6.1.2.1.1.1.0\",\n          \"1.3.6.1.4.1.8072.1.3.2.3.1.2.8.110.101.116.45.115.110.109.112\",\n          \"1.3.6.1.2.1.2.2.1.10.1000\"\n        ]\n\n        results = for oid_string <- test_oids do\n          # Benchmark string to list conversion\n          {str_to_list_time, _} = :timer.tc(fn ->\n            for _ <- 1..iterations do\n              {:ok, _list} = SnmpKit.SnmpLib.OID.string_to_list(oid_string)\n            end\n          end)\n\n          # Convert once for reverse benchmark\n          {:ok, oid_list} = SnmpKit.SnmpLib.OID.string_to_list(oid_string)\n\n          # Benchmark list to string conversion\n          {list_to_str_time, _} = :timer.tc(fn ->\n            for _ <- 1..iterations do\n              {:ok, _string} = SnmpKit.SnmpLib.OID.list_to_string(oid_list)\n            end\n          end)\n\n          %{\n            oid: oid_string,\n            oid_length: length(oid_list),\n            str_to_list_us_per_op: str_to_list_time / iterations,\n            list_to_str_us_per_op: list_to_str_time / iterations,\n            str_to_list_ops_per_sec: iterations / (str_to_list_time / 1_000_000),\n            list_to_str_ops_per_sec: iterations / (list_to_str_time / 1_000_000)\n          }\n        end\n\n        %{\n          iterations: iterations,\n          oid_benchmarks: results,\n          average_str_to_list_us: Enum.reduce(results, 0, &(&1.str_to_list_us_per_op + &2)) / length(results),\n          average_list_to_str_us: Enum.reduce(results, 0, &(&1.list_to_str_us_per_op + &2)) / length(results)\n        }\n      end\n\n      defp count_successes(results) do\n        Enum.count(results, fn\n          {:ok, _} -> true\n          _ -> false\n        end)\n      end\n    end\n\n    # Example usage:\n    # encoding_perf = SnmpBenchmark.benchmark_encoding(50_000)\n    # IO.puts(\"Encoding: \" <> Integer.to_string(trunc(encoding_perf.encode_ops_per_sec)) <> \" ops/sec\")\n    # IO.puts(\"Decoding: \" <> Integer.to_string(trunc(encoding_perf.decode_ops_per_sec)) <> \" ops/sec\")\n\n    # bulk_perf = SnmpBenchmark.benchmark_bulk_operations(200)\n    # IO.puts(\"Sequential: \" <> Float.to_string(bulk_perf.sequential.time_ms) <> \"ms\")\n    # IO.puts(\"Concurrent: \" <> Float.to_string(bulk_perf.concurrent.time_ms) <> \"ms (\" <> Float.to_string(bulk_perf.concurrent.speedup) <> \"x faster)\")\n\n    # oid_perf = SnmpBenchmark.benchmark_oid_operations(100_000)\n    # IO.puts(\"Average OID conversion: \" <> Float.to_string(oid_perf.average_str_to_list_us) <> \"μs per operation\")","ref":"SnmpKit.SnmpLib.html#module-encoding-decoding-performance"},{"type":"module","title":"RFC Compliance - SnmpKit.SnmpLib","doc":"This library achieves 100% compliance with:\n- RFC 1157 (SNMPv1)\n- RFC 1905 (SNMPv2c Protocol Operations)\n- RFC 3416 (SNMPv2c Enhanced Operations)\n- ITU-T X.690 (ASN.1 BER Encoding Rules)","ref":"SnmpKit.SnmpLib.html#module-rfc-compliance"},{"type":"function","title":"SnmpKit.SnmpLib.info/0","doc":"Returns comprehensive information about the SnmpLib library capabilities.\n\nUseful for debugging, configuration validation, and feature discovery.","ref":"SnmpKit.SnmpLib.html#info/0"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.info/0","doc":"A map containing:\n- `:version`: Library version\n- `:features`: Available features and capabilities\n- `:modules`: Core modules and their descriptions\n- `:compliance`: RFC compliance information","ref":"SnmpKit.SnmpLib.html#info/0-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.info/0","doc":"info = SnmpLib.info()\n    IO.puts(\"SNMP Library v\" <> info.version)\n    IO.puts(\"Features: \" <> Enum.join(info.features, \", \"))","ref":"SnmpKit.SnmpLib.html#info/0-examples"},{"type":"function","title":"SnmpKit.SnmpLib.version/0","doc":"Returns the version of the SnmpLib library.","ref":"SnmpKit.SnmpLib.html#version/0"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.version/0","doc":"iex> is_binary(SnmpLib.version())\n    true\n\n    iex> SnmpLib.version() |> String.contains?(\".\")\n    true","ref":"SnmpKit.SnmpLib.html#version/0-examples"},{"type":"module","title":"SnmpKit.SnmpLib.Types","doc":"SNMP data type validation, formatting, and coercion utilities.\n\nProvides comprehensive support for all SNMP data types including validation,\nformatting for display, and type coercion between different representations.\nIncludes full support for SNMPv2c exception values.","ref":"SnmpKit.SnmpLib.Types.html"},{"type":"module","title":"Supported SNMP Types - SnmpKit.SnmpLib.Types","doc":"- **Basic Types**: INTEGER, OCTET STRING, NULL, OBJECT IDENTIFIER\n- **Application Types**: Counter32, Gauge32, TimeTicks, Counter64, IpAddress, Opaque\n- **SNMPv2c Exception Types**: NoSuchObject, NoSuchInstance, EndOfMibView\n- **Constructed Types**: SEQUENCE (for complex structures)","ref":"SnmpKit.SnmpLib.Types.html#module-supported-snmp-types"},{"type":"module","title":"SNMPv2c Exception Values - SnmpKit.SnmpLib.Types","doc":"These special values are used in SNMPv2c responses to indicate specific conditions:\n\n- **`:no_such_object`** (0x80): The requested object does not exist in the MIB\n- **`:no_such_instance`** (0x81): The object exists but the specific instance does not\n- **`:end_of_mib_view`** (0x82): End of MIB tree reached during GETBULK/walk operations","ref":"SnmpKit.SnmpLib.Types.html#module-snmpv2c-exception-values"},{"type":"module","title":"Features - SnmpKit.SnmpLib.Types","doc":"- Type validation with detailed error reporting\n- Human-readable formatting for logging and display\n- Type coercion and normalization\n- Range checking and constraint validation\n- Performance-optimized operations\n- RFC-compliant exception value handling","ref":"SnmpKit.SnmpLib.Types.html#module-features"},{"type":"module","title":"Examples - SnmpKit.SnmpLib.Types","doc":"# Basic type validation\n    iex> SnmpKit.SnmpLib.Types.validate_counter32(42)\n    :ok\n    iex> SnmpKit.SnmpLib.Types.validate_counter32(-1)\n    {:error, :out_of_range}\n\n    # Formatting for display\n    iex> SnmpKit.SnmpLib.Types.format_timeticks_uptime(4200)\n    \"42 seconds\"\n    iex> SnmpKit.SnmpLib.Types.format_ip_address(<<192, 168, 1, 1>>)\n    \"192.168.1.1\"\n\n    # Type coercion\n    iex> SnmpKit.SnmpLib.Types.coerce_value(:counter32, 42)\n    {:ok, {:counter32, 42}}\n    iex> SnmpKit.SnmpLib.Types.coerce_value(:string, \"test\")\n    {:ok, {:string, \"test\"}}\n\n    # SNMPv2c exception values\n    iex> SnmpKit.SnmpLib.Types.coerce_value(:no_such_object, nil)\n    {:ok, {:no_such_object, nil}}\n    iex> SnmpKit.SnmpLib.Types.coerce_value(:end_of_mib_view, nil)\n    {:ok, {:end_of_mib_view, nil}}","ref":"SnmpKit.SnmpLib.Types.html#module-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.coerce_value/2","doc":"Coerces a value to the specified SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#coerce_value/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.coerce_value/2","doc":"- `type`: Target SNMP type\n- `raw_value`: Value to coerce","ref":"SnmpKit.SnmpLib.Types.html#coerce_value/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.coerce_value/2","doc":"- `{:ok, typed_value}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.Types.html#coerce_value/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.coerce_value/2","doc":"{:ok, {:counter32, 42}} = SnmpKit.SnmpLib.Types.coerce_value(:counter32, 42)\n    {:ok, {:string, \"test\"}} = SnmpKit.SnmpLib.Types.coerce_value(:string, \"test\")\n    {:ok, {:ip_address, <<192, 168, 1, 1>>}} = SnmpKit.SnmpLib.Types.coerce_value(:ip_address, {192, 168, 1, 1})","ref":"SnmpKit.SnmpLib.Types.html#coerce_value/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.decode_value/1","doc":"Decodes an SNMP typed value back to a native Elixir value.\n\nConverts SNMP-encoded values back to their most natural Elixir representation,\nwith consistent handling of strings (always returns binaries, not charlists).","ref":"SnmpKit.SnmpLib.Types.html#decode_value/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.decode_value/1","doc":"- `typed_value`: A tuple of `{type, value}` or just a value","ref":"SnmpKit.SnmpLib.Types.html#decode_value/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.decode_value/1","doc":"The decoded Elixir value in its most natural form","ref":"SnmpKit.SnmpLib.Types.html#decode_value/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.decode_value/1","doc":"\"hello\" = SnmpKit.SnmpLib.Types.decode_value({:string, \"hello\"})\n    \"192.168.1.1\" = SnmpKit.SnmpLib.Types.decode_value({:ip_address, {192, 168, 1, 1}})\n    42 = SnmpKit.SnmpLib.Types.decode_value({:counter32, 42})\n    [1, 3, 6, 1] = SnmpKit.SnmpLib.Types.decode_value({:object_identifier, [1, 3, 6, 1]})","ref":"SnmpKit.SnmpLib.Types.html#decode_value/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.encode_value/2","doc":"Encodes a value with automatic type inference or explicit type specification.\n\nThis is the main entry point for encoding values into SNMP types. It supports\nboth automatic type inference based on the value and explicit type specification.","ref":"SnmpKit.SnmpLib.Types.html#encode_value/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.encode_value/2","doc":"- `value`: The value to encode\n- `opts`: Options including:\n  - `:type` - Explicit type specification (overrides inference)\n  - `:validate` - Whether to validate the encoded value (default: true)","ref":"SnmpKit.SnmpLib.Types.html#encode_value/2-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.encode_value/2","doc":"- `{:ok, {type, encoded_value}}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.Types.html#encode_value/2-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.encode_value/2","doc":"# Automatic type inference\n    {:ok, {:string, \"hello\"}} = SnmpKit.SnmpLib.Types.encode_value(\"hello\")\n    {:ok, {:integer, 42}} = SnmpKit.SnmpLib.Types.encode_value(42)\n\n    # Explicit type specification\n    {:ok, {:ip_address, {192, 168, 1, 1}}} = SnmpKit.SnmpLib.Types.encode_value(\"192.168.1.1\", type: :ip_address)\n    {:ok, {:counter32, 100}} = SnmpKit.SnmpLib.Types.encode_value(100, type: :counter32)","ref":"SnmpKit.SnmpLib.Types.html#encode_value/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_bytes/1","doc":"Formats bytes as human-readable size.","ref":"SnmpKit.SnmpLib.Types.html#format_bytes/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_bytes/1","doc":"\"1.5 KB\" = SnmpKit.SnmpLib.Types.format_bytes(1536)\n    \"2.3 MB\" = SnmpKit.SnmpLib.Types.format_bytes(2400000)","ref":"SnmpKit.SnmpLib.Types.html#format_bytes/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_counter64/1","doc":"Formats Counter64 value with appropriate units.","ref":"SnmpKit.SnmpLib.Types.html#format_counter64/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_counter64/1","doc":"\"42\" = SnmpKit.SnmpLib.Types.format_counter64(42)\n    \"18,446,744,073,709,551,615\" = SnmpKit.SnmpLib.Types.format_counter64(18446744073709551615)","ref":"SnmpKit.SnmpLib.Types.html#format_counter64/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_hex/1","doc":"Formats binary data as hexadecimal string.","ref":"SnmpKit.SnmpLib.Types.html#format_hex/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_hex/1","doc":"\"48656C6C6F\" = SnmpKit.SnmpLib.Types.format_hex(<<\"Hello\">>)\n    \"DEADBEEF\" = SnmpKit.SnmpLib.Types.format_hex(<<0xDE, 0xAD, 0xBE, 0xEF>>)","ref":"SnmpKit.SnmpLib.Types.html#format_hex/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_ip_address/1","doc":"Formats an IP address from binary format.","ref":"SnmpKit.SnmpLib.Types.html#format_ip_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_ip_address/1","doc":"\"192.168.1.1\" = SnmpKit.SnmpLib.Types.format_ip_address(<<192, 168, 1, 1>>)\n    \"0.0.0.0\" = SnmpKit.SnmpLib.Types.format_ip_address(<<0, 0, 0, 0>>)","ref":"SnmpKit.SnmpLib.Types.html#format_ip_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_rate/2","doc":"Formats a rate value with units.","ref":"SnmpKit.SnmpLib.Types.html#format_rate/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_rate/2","doc":"\"100 bps\" = SnmpKit.SnmpLib.Types.format_rate(100, \"bps\")\n    \"1.5 Mbps\" = SnmpKit.SnmpLib.Types.format_rate(1500000, \"bps\")","ref":"SnmpKit.SnmpLib.Types.html#format_rate/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.format_timeticks_uptime/1","doc":"Formats TimeTicks as human-readable uptime string.","ref":"SnmpKit.SnmpLib.Types.html#format_timeticks_uptime/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.format_timeticks_uptime/1","doc":"- `centiseconds`: Time in centiseconds (hundredths of a second)","ref":"SnmpKit.SnmpLib.Types.html#format_timeticks_uptime/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.format_timeticks_uptime/1","doc":"- Human-readable uptime string","ref":"SnmpKit.SnmpLib.Types.html#format_timeticks_uptime/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.format_timeticks_uptime/1","doc":"\"42 centiseconds\" = SnmpKit.SnmpLib.Types.format_timeticks_uptime(42)\n    \"1 second 50 centiseconds\" = SnmpKit.SnmpLib.Types.format_timeticks_uptime(150)\n    \"1 minute 30 seconds\" = SnmpKit.SnmpLib.Types.format_timeticks_uptime(9000)\n    \"2 hours 15 minutes 30 seconds\" = SnmpKit.SnmpLib.Types.format_timeticks_uptime(81300)","ref":"SnmpKit.SnmpLib.Types.html#format_timeticks_uptime/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.infer_type/1","doc":"Automatically infers the SNMP type from an Elixir value.\n\nUses intelligent heuristics to determine the most appropriate SNMP type\nfor a given Elixir value.","ref":"SnmpKit.SnmpLib.Types.html#infer_type/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.infer_type/1","doc":":string = SnmpKit.SnmpLib.Types.infer_type(\"hello\")\n    :integer = SnmpKit.SnmpLib.Types.infer_type(42)\n    :ip_address = SnmpKit.SnmpLib.Types.infer_type(\"192.168.1.1\")\n    :object_identifier = SnmpKit.SnmpLib.Types.infer_type([1, 3, 6, 1, 2, 1])\n    :boolean = SnmpKit.SnmpLib.Types.infer_type(true)","ref":"SnmpKit.SnmpLib.Types.html#infer_type/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.is_binary_type?/1","doc":"Checks if a type is a binary SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#is_binary_type?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.is_binary_type?/1","doc":"true = SnmpKit.SnmpLib.Types.is_binary_type?(:string)\n    true = SnmpKit.SnmpLib.Types.is_binary_type?(:opaque)\n    false = SnmpKit.SnmpLib.Types.is_binary_type?(:integer)","ref":"SnmpKit.SnmpLib.Types.html#is_binary_type?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.is_exception_type?/1","doc":"Checks if a type is an exception SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#is_exception_type?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.is_exception_type?/1","doc":"true = SnmpKit.SnmpLib.Types.is_exception_type?(:no_such_object)\n    false = SnmpKit.SnmpLib.Types.is_exception_type?(:integer)","ref":"SnmpKit.SnmpLib.Types.html#is_exception_type?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.is_numeric_type?/1","doc":"Checks if a type is a numeric SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#is_numeric_type?/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.is_numeric_type?/1","doc":"true = SnmpKit.SnmpLib.Types.is_numeric_type?(:counter32)\n    true = SnmpKit.SnmpLib.Types.is_numeric_type?(:integer)\n    false = SnmpKit.SnmpLib.Types.is_numeric_type?(:string)","ref":"SnmpKit.SnmpLib.Types.html#is_numeric_type?/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.max_value/1","doc":"Returns the maximum value for a numeric SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#max_value/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.max_value/1","doc":"4294967295 = SnmpKit.SnmpLib.Types.max_value(:counter32)\n    2147483647 = SnmpKit.SnmpLib.Types.max_value(:integer)","ref":"SnmpKit.SnmpLib.Types.html#max_value/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.min_value/1","doc":"Returns the minimum value for a numeric SNMP type.","ref":"SnmpKit.SnmpLib.Types.html#min_value/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.min_value/1","doc":"-2147483648 = SnmpKit.SnmpLib.Types.min_value(:integer)\n    0 = SnmpKit.SnmpLib.Types.min_value(:counter32)","ref":"SnmpKit.SnmpLib.Types.html#min_value/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.normalize_type/1","doc":"Normalizes a type identifier to a consistent format.","ref":"SnmpKit.SnmpLib.Types.html#normalize_type/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.normalize_type/1","doc":":counter32 = SnmpKit.SnmpLib.Types.normalize_type(\"counter32\")\n    :integer = SnmpKit.SnmpLib.Types.normalize_type(:integer)\n    :string = SnmpKit.SnmpLib.Types.normalize_type(\"octet_string\")","ref":"SnmpKit.SnmpLib.Types.html#normalize_type/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.parse_hex_string/1","doc":"Parses a hexadecimal string to binary.","ref":"SnmpKit.SnmpLib.Types.html#parse_hex_string/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.parse_hex_string/1","doc":"{:ok, <<\"Hello\">>} = SnmpKit.SnmpLib.Types.parse_hex_string(\"48656C6C6F\")\n    {:error, :invalid_hex} = SnmpKit.SnmpLib.Types.parse_hex_string(\"XYZ\")","ref":"SnmpKit.SnmpLib.Types.html#parse_hex_string/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.parse_ip_address/1","doc":"Parses an IP address string into a 4-tuple of integers.","ref":"SnmpKit.SnmpLib.Types.html#parse_ip_address/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.parse_ip_address/1","doc":"- `ip_string`: IP address as a string like \"192.168.1.1\"","ref":"SnmpKit.SnmpLib.Types.html#parse_ip_address/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.parse_ip_address/1","doc":"- `{:ok, {a, b, c, d}}` on success\n- `{:error, reason}` on failure","ref":"SnmpKit.SnmpLib.Types.html#parse_ip_address/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.parse_ip_address/1","doc":"{:ok, {192, 168, 1, 1}} = SnmpKit.SnmpLib.Types.parse_ip_address(\"192.168.1.1\")\n    {:ok, {127, 0, 0, 1}} = SnmpKit.SnmpLib.Types.parse_ip_address(\"127.0.0.1\")\n    {:error, :invalid_format} = SnmpKit.SnmpLib.Types.parse_ip_address(\"invalid\")","ref":"SnmpKit.SnmpLib.Types.html#parse_ip_address/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.Types.snmp_type/0","doc":"","ref":"SnmpKit.SnmpLib.Types.html#t:snmp_type/0"},{"type":"type","title":"SnmpKit.SnmpLib.Types.snmp_value/0","doc":"","ref":"SnmpKit.SnmpLib.Types.html#t:snmp_value/0"},{"type":"function","title":"SnmpKit.SnmpLib.Types.truncate_string/2","doc":"Truncates a string to a maximum length with ellipsis.","ref":"SnmpKit.SnmpLib.Types.html#truncate_string/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.truncate_string/2","doc":"\"hello\" = SnmpKit.SnmpLib.Types.truncate_string(\"hello\", 10)\n    \"hello...\" = SnmpKit.SnmpLib.Types.truncate_string(\"hello world\", 8)","ref":"SnmpKit.SnmpLib.Types.html#truncate_string/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_counter32/1","doc":"Validates a Counter32 value.\n\nCounter32 is a 32-bit unsigned integer that wraps around when it reaches its maximum value.","ref":"SnmpKit.SnmpLib.Types.html#validate_counter32/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpLib.Types.validate_counter32/1","doc":"- `value`: Value to validate","ref":"SnmpKit.SnmpLib.Types.html#validate_counter32/1-parameters"},{"type":"function","title":"Returns - SnmpKit.SnmpLib.Types.validate_counter32/1","doc":"- `:ok` if valid\n- `{:error, reason}` if invalid","ref":"SnmpKit.SnmpLib.Types.html#validate_counter32/1-returns"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.validate_counter32/1","doc":":ok = SnmpKit.SnmpLib.Types.validate_counter32(42)\n    :ok = SnmpKit.SnmpLib.Types.validate_counter32(4294967295)\n    {:error, :out_of_range} = SnmpKit.SnmpLib.Types.validate_counter32(-1)\n    {:error, :not_integer} = SnmpKit.SnmpLib.Types.validate_counter32(\"42\")","ref":"SnmpKit.SnmpLib.Types.html#validate_counter32/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_counter64/1","doc":"Validates a Counter64 value.\n\nCounter64 is a 64-bit unsigned integer for high-speed interfaces.","ref":"SnmpKit.SnmpLib.Types.html#validate_counter64/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_gauge32/1","doc":"Validates a Gauge32 value.\n\nGauge32 is a 32-bit unsigned integer that represents a non-negative integer value.\nUnlike Counter32, it does not wrap around.","ref":"SnmpKit.SnmpLib.Types.html#validate_gauge32/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_integer/1","doc":"Validates an SNMP integer value.\n\nSNMP INTEGER is a signed 32-bit integer.","ref":"SnmpKit.SnmpLib.Types.html#validate_integer/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_ip_address/1","doc":"Validates an IP address value.\n\nIP address should be a 4-byte binary or a tuple of 4 integers.","ref":"SnmpKit.SnmpLib.Types.html#validate_ip_address/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.Types.validate_ip_address/1","doc":":ok = SnmpKit.SnmpLib.Types.validate_ip_address(<<192, 168, 1, 1>>)\n    :ok = SnmpKit.SnmpLib.Types.validate_ip_address({192, 168, 1, 1})\n    {:error, :invalid_length} = SnmpKit.SnmpLib.Types.validate_ip_address(<<192, 168, 1>>)","ref":"SnmpKit.SnmpLib.Types.html#validate_ip_address/1-examples"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_octet_string/1","doc":"Validates an OCTET STRING value.\n\nOCTET STRING should be a binary with reasonable length limits.","ref":"SnmpKit.SnmpLib.Types.html#validate_octet_string/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_oid/1","doc":"Validates an OBJECT IDENTIFIER value.\n\nOID should be a list of non-negative integers.","ref":"SnmpKit.SnmpLib.Types.html#validate_oid/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_opaque/1","doc":"Validates an Opaque value.\n\nOpaque is used for arbitrary binary data.","ref":"SnmpKit.SnmpLib.Types.html#validate_opaque/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_timeticks/1","doc":"Validates a TimeTicks value.\n\nTimeTicks represents time in hundredths of a second (centiseconds).","ref":"SnmpKit.SnmpLib.Types.html#validate_timeticks/1"},{"type":"function","title":"SnmpKit.SnmpLib.Types.validate_unsigned32/1","doc":"Validates an Unsigned32 value.\n\nUnsigned32 is a 32-bit unsigned integer.","ref":"SnmpKit.SnmpLib.Types.html#validate_unsigned32/1"},{"type":"module","title":"SnmpKit.SnmpLib.MIB","doc":"SNMP MIB compiler with enhanced Elixir ergonomics.\n\nProvides a clean, functional API for compiling MIB files with proper\nerror handling, logging, and performance optimizations.\n\nThis module serves as the main entry point for MIB compilation operations.","ref":"SnmpKit.SnmpLib.MIB.html"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.compile/2","doc":"Compile a MIB file to Elixir code.","ref":"SnmpKit.SnmpLib.MIB.html#compile/2"},{"type":"function","title":"Options - SnmpKit.SnmpLib.MIB.compile/2","doc":"- `:output_dir` - Directory for generated files (default: \"./lib/generated\")\n- `:include_dirs` - Directories to search for imported MIBs (default: [\"./priv/mibs\"])\n- `:log_level` - Logging verbosity (default: `:info`)\n- `:format` - Output format `:elixir`, `:erlang`, or `:both` (default: `:elixir`)\n- `:optimize` - Enable performance optimizations (default: `true`)\n- `:warnings_as_errors` - Treat warnings as compilation errors (default: `false`)\n- `:vendor_quirks` - Enable vendor-specific MIB compatibility (default: `true`)","ref":"SnmpKit.SnmpLib.MIB.html#compile/2-options"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.compile/2","doc":"iex> SnmpKit.SnmpLib.MIB.compile(\"priv/mibs/RFC1213-MIB.txt\")\n    {:error, [%SnmpKit.SnmpLib.MIB.Error{type: :file_not_found, ...}]}","ref":"SnmpKit.SnmpLib.MIB.html#compile/2-examples"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.compile_all/2","doc":"Compile multiple MIB files in dependency order.\n\nAutomatically resolves import dependencies and compiles MIBs in the correct order.","ref":"SnmpKit.SnmpLib.MIB.html#compile_all/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.compile_all/2","doc":"iex> mibs = [\"SNMPv2-SMI.txt\", \"SNMPv2-TC.txt\", \"RFC1213-MIB.txt\"]\n    iex> SnmpKit.SnmpLib.MIB.compile_all(mibs)\n    {:ok, [%{name: \"SNMPv2-SMI\", ...}, %{name: \"SNMPv2-TC\", ...}, ...]}","ref":"SnmpKit.SnmpLib.MIB.html#compile_all/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.compile_opts/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.html#t:compile_opts/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.compile_result/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.html#t:compile_result/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.compile_string/2","doc":"Compile MIB content from a string.\n\nUseful when the MIB content is already loaded or generated dynamically.","ref":"SnmpKit.SnmpLib.MIB.html#compile_string/2"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.compile_string/2","doc":"iex> mib_content = File.read!(\"RFC1213-MIB.txt\")\n    iex> SnmpKit.SnmpLib.MIB.compile_string(mib_content)\n    {:ok, %{name: \"RFC1213-MIB\", ...}}","ref":"SnmpKit.SnmpLib.MIB.html#compile_string/2-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.compiled_mib/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.html#t:compiled_mib/0"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.error/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.html#t:error/0"},{"type":"function","title":"SnmpKit.SnmpLib.MIB.load_compiled/1","doc":"Load a previously compiled MIB module.","ref":"SnmpKit.SnmpLib.MIB.html#load_compiled/1"},{"type":"function","title":"Examples - SnmpKit.SnmpLib.MIB.load_compiled/1","doc":"iex> SnmpKit.SnmpLib.MIB.load_compiled(\"lib/generated/rfc1213_mib.ex\")\n    {:ok, %{name: \"RFC1213-MIB\", ...}}","ref":"SnmpKit.SnmpLib.MIB.html#load_compiled/1-examples"},{"type":"type","title":"SnmpKit.SnmpLib.MIB.warning/0","doc":"","ref":"SnmpKit.SnmpLib.MIB.html#t:warning/0"},{"type":"module","title":"SnmpKit.SnmpMgr.MIB","doc":"MIB compilation and symbolic name resolution.\n\nThis module provides MIB compilation using Erlang's :snmpc when available,\nand includes a built-in registry of standard MIB objects for basic operations.","ref":"SnmpKit.SnmpMgr.MIB.html"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpMgr.MIB.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.children/1","doc":"Gets the children of an OID node.","ref":"SnmpKit.SnmpMgr.MIB.html#children/1"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.compile/2","doc":"Compiles a MIB file using SnmpKit.SnmpLib.MIB pure Elixir implementation.\n\nEnhanced to use SnmpKit.SnmpLib.MIB for improved compilation with better error handling.","ref":"SnmpKit.SnmpMgr.MIB.html#compile/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.MIB.compile/2","doc":"iex> SnmpKit.SnmpMgr.MIB.compile(\"SNMPv2-MIB.mib\")\n    {:ok, \"SNMPv2-MIB.bin\"}\n\n    iex> SnmpKit.SnmpMgr.MIB.compile(\"nonexistent.mib\")\n    {:error, :file_not_found}","ref":"SnmpKit.SnmpMgr.MIB.html#compile/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.compile_dir/2","doc":"Compiles all MIB files in a directory using enhanced SnmpKit.SnmpLib.MIB capabilities.","ref":"SnmpKit.SnmpMgr.MIB.html#compile_dir/2"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.load/1","doc":"Loads a compiled MIB file using SnmpKit.SnmpLib.MIB.load_compiled with fallback.","ref":"SnmpKit.SnmpMgr.MIB.html#load/1"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.load_and_integrate_mib/2","doc":"Loads and parses a MIB file, integrating it into the name resolution system.\n\nThis combines compilation/loading with parsing for comprehensive MIB support.","ref":"SnmpKit.SnmpMgr.MIB.html#load_and_integrate_mib/2"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.load_standard_mibs/0","doc":"Loads standard MIBs that are built into the library.","ref":"SnmpKit.SnmpMgr.MIB.html#load_standard_mibs/0"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.parent/1","doc":"Gets the parent of an OID node.","ref":"SnmpKit.SnmpMgr.MIB.html#parent/1"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.parse_mib_content/2","doc":"Parses MIB content string using SnmpKit.SnmpLib.MIB.Parser.","ref":"SnmpKit.SnmpMgr.MIB.html#parse_mib_content/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.MIB.parse_mib_content/2","doc":"iex> content = \"sysDescr OBJECT-TYPE SYNTAX DisplayString ACCESS read-only STATUS mandatory\"\n    iex> SnmpKit.SnmpMgr.MIB.parse_mib_content(content)\n    {:ok, %{tokens: [...], parsed_objects: [...]}}","ref":"SnmpKit.SnmpMgr.MIB.html#parse_mib_content/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.parse_mib_file/2","doc":"Parses a MIB file to extract object definitions using SnmpKit.SnmpLib.MIB.Parser.\n\nThis provides enhanced MIB analysis without requiring compilation.","ref":"SnmpKit.SnmpMgr.MIB.html#parse_mib_file/2"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.MIB.parse_mib_file/2","doc":"iex> SnmpKit.SnmpMgr.MIB.parse_mib_file(\"SNMPv2-MIB.mib\")\n    {:ok, %{objects: [...], imports: [...], exports: [...]}}","ref":"SnmpKit.SnmpMgr.MIB.html#parse_mib_file/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.resolve/1","doc":"Resolves a symbolic name to an OID.","ref":"SnmpKit.SnmpMgr.MIB.html#resolve/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.MIB.resolve/1","doc":"iex> SnmpKit.SnmpMgr.MIB.resolve(\"sysDescr.0\")\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 1, 0]}\n\n    iex> SnmpKit.SnmpMgr.MIB.resolve(\"sysDescr\")\n    {:ok, [1, 3, 6, 1, 2, 1, 1, 1]}\n\n    iex> SnmpKit.SnmpMgr.MIB.resolve(\"unknownName\")\n    {:error, :not_found}","ref":"SnmpKit.SnmpMgr.MIB.html#resolve/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.resolve_enhanced/2","doc":"Enhanced MIB object resolution with parsed MIB data integration.\n\nLeverages both standard MIBs and any loaded/parsed MIB files for comprehensive name resolution.","ref":"SnmpKit.SnmpMgr.MIB.html#resolve_enhanced/2"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.reverse_lookup/1","doc":"Performs reverse lookup from OID to symbolic name.","ref":"SnmpKit.SnmpMgr.MIB.html#reverse_lookup/1"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.MIB.reverse_lookup/1","doc":"iex> SnmpKit.SnmpMgr.MIB.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1, 0])\n    {:ok, \"sysDescr.0\"}\n\n    iex> SnmpKit.SnmpMgr.MIB.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1])\n    {:ok, \"sysDescr\"}","ref":"SnmpKit.SnmpMgr.MIB.html#reverse_lookup/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.start_link/1","doc":"Starts the MIB registry GenServer.","ref":"SnmpKit.SnmpMgr.MIB.html#start_link/1"},{"type":"function","title":"SnmpKit.SnmpMgr.MIB.walk_tree/2","doc":"Walks the MIB tree starting from a root OID.","ref":"SnmpKit.SnmpMgr.MIB.html#walk_tree/2"},{"type":"module","title":"SnmpKit.SnmpSim.Device","doc":"Lightweight Device GenServer for handling SNMP requests.\nUses shared profiles and minimal device-specific state for scalability.\n\nFeatures:\n- Dynamic value generation with realistic patterns\n- Shared profile system for memory efficiency\n- Counter and gauge simulation with proper incrementing\n- Comprehensive error handling and fallback mechanisms\n- Support for SNMP walk operations","ref":"SnmpKit.SnmpSim.Device.html"},{"type":"function","title":"SnmpKit.SnmpSim.Device.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SnmpKit.SnmpSim.Device.html#child_spec/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.cleanup_all_devices/0","doc":"Cleanup all orphaned SNMP simulator device processes.\nUseful for test cleanup when devices may have been left running.","ref":"SnmpKit.SnmpSim.Device.html#cleanup_all_devices/0"},{"type":"function","title":"SnmpKit.SnmpSim.Device.get/2","doc":"Get an OID value from the device (for testing).","ref":"SnmpKit.SnmpSim.Device.html#get/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.get_bulk/3","doc":"Get bulk OID values from the device (for testing).","ref":"SnmpKit.SnmpSim.Device.html#get_bulk/3"},{"type":"function","title":"SnmpKit.SnmpSim.Device.get_bulk/4","doc":"","ref":"SnmpKit.SnmpSim.Device.html#get_bulk/4"},{"type":"function","title":"SnmpKit.SnmpSim.Device.get_info/1","doc":"Get device information and statistics.","ref":"SnmpKit.SnmpSim.Device.html#get_info/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.get_next/2","doc":"Get the next OID value from the device (for testing).","ref":"SnmpKit.SnmpSim.Device.html#get_next/2"},{"type":"function","title":"SnmpKit.SnmpSim.Device.monitor_device/1","doc":"Monitor a device process and get notified when it dies.\nReturns a monitor reference that can be used with Process.demonitor/1.","ref":"SnmpKit.SnmpSim.Device.html#monitor_device/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.reboot/1","doc":"Simulate a device reboot.","ref":"SnmpKit.SnmpSim.Device.html#reboot/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.set_gauge/3","doc":"Set a gauge value manually (useful for testing).","ref":"SnmpKit.SnmpSim.Device.html#set_gauge/3"},{"type":"function","title":"SnmpKit.SnmpSim.Device.start_link/1","doc":"Start a device with the given device configuration.","ref":"SnmpKit.SnmpSim.Device.html#start_link/1"},{"type":"function","title":"Device Config - SnmpKit.SnmpSim.Device.start_link/1","doc":"Device config should contain:\n- `:port` - UDP port for the device (required)\n- `:device_type` - Type of device (:cable_modem, :switch, etc.)\n- `:device_id` - Unique device identifier\n- `:community` - SNMP community string (default: \"public\")\n- `:mac_address` - MAC address (auto-generated if not provided)","ref":"SnmpKit.SnmpSim.Device.html#start_link/1-device-config"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.Device.start_link/1","doc":"device_config = %{\n      port: 9001,\n      device_type: :cable_modem,\n      device_id: \"cable_modem_9001\",\n      community: \"public\"\n    }\n\n    {:ok, device} = SnmpKit.SnmpSim.Device.start_link(device_config)","ref":"SnmpKit.SnmpSim.Device.html#start_link/1-examples"},{"type":"function","title":"SnmpKit.SnmpSim.Device.start_link_monitored/1","doc":"Create a device with monitoring enabled.\nReturns {:ok, {device_pid, monitor_ref}} or {:error, reason}.","ref":"SnmpKit.SnmpSim.Device.html#start_link_monitored/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.stop/1","doc":"Stop a device gracefully with resilient error handling.","ref":"SnmpKit.SnmpSim.Device.html#stop/1"},{"type":"function","title":"SnmpKit.SnmpSim.Device.update_counter/3","doc":"Update device counters manually (useful for testing).","ref":"SnmpKit.SnmpSim.Device.html#update_counter/3"},{"type":"function","title":"SnmpKit.SnmpSim.Device.walk/2","doc":"Walk OID values from the device (for testing).","ref":"SnmpKit.SnmpSim.Device.html#walk/2"},{"type":"module","title":"SnmpKit.SnmpSim.ProfileLoader","doc":"Flexible profile loading supporting multiple sources and progressive enhancement.\nStart with simple walk files, upgrade to MIB-based simulation when ready.","ref":"SnmpKit.SnmpSim.ProfileLoader.html"},{"type":"function","title":"SnmpKit.SnmpSim.ProfileLoader.get_next_oid/2","doc":"Find the next OID after the given OID for GETNEXT operations.","ref":"SnmpKit.SnmpSim.ProfileLoader.html#get_next_oid/2"},{"type":"function","title":"SnmpKit.SnmpSim.ProfileLoader.get_oid_value/2","doc":"Get the value for a specific OID from a loaded profile.","ref":"SnmpKit.SnmpSim.ProfileLoader.html#get_oid_value/2"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ProfileLoader.get_oid_value/2","doc":"value = SnmpKit.SnmpSim.ProfileLoader.get_oid_value(profile, \"1.3.6.1.2.1.1.1.0\")","ref":"SnmpKit.SnmpSim.ProfileLoader.html#get_oid_value/2-examples"},{"type":"function","title":"SnmpKit.SnmpSim.ProfileLoader.get_ordered_oids/1","doc":"Get all OIDs in lexicographic order for GETNEXT operations.","ref":"SnmpKit.SnmpSim.ProfileLoader.html#get_ordered_oids/1"},{"type":"function","title":"SnmpKit.SnmpSim.ProfileLoader.load_profile/3","doc":"Load a device profile from various source types.\n\nSupported source types:\n- `{:walk_file, path}` - SNMP walk files (both named and numeric formats)\n- `{:oid_walk, path}` - Raw OID dumps (numeric OIDs only)\n- `{:json_profile, path}` - Structured JSON profiles\n- `{:manual, oid_map}` - Manual OID definitions (for testing)\n- `{:compiled_mib, mib_files}` - Advanced MIB compilation (future)","ref":"SnmpKit.SnmpSim.ProfileLoader.html#load_profile/3"},{"type":"function","title":"Examples - SnmpKit.SnmpSim.ProfileLoader.load_profile/3","doc":"# Load from SNMP walk file\n    profile = SnmpKit.SnmpSim.ProfileLoader.load_profile(\n      :cable_modem,\n      {:walk_file, \"priv/walks/cable_modem.walk\"}\n    )\n\n    # Load with behaviors\n    profile = SnmpKit.SnmpSim.ProfileLoader.load_profile(\n      :cable_modem,\n      {:walk_file, \"priv/walks/cable_modem.walk\"},\n      behaviors: [\n        {:increment_counters, rate: 1000},\n        {:vary_gauges, variance: 0.1}\n      ]\n    )","ref":"SnmpKit.SnmpSim.ProfileLoader.html#load_profile/3-examples"},{"type":"module","title":"SnmpKit.SnmpMgr","doc":"Lightweight SNMP client library for Elixir.\n\nThis library provides a simple, stateless interface for SNMP operations\nwithout requiring heavyweight management processes or configurations.","ref":"SnmpKit.SnmpMgr.html"},{"type":"function","title":"SnmpKit.SnmpMgr.adaptive_walk/3","doc":"Performs an adaptive bulk walk that automatically optimizes parameters.\n\nUses intelligent parameter tuning based on device response characteristics\nfor optimal performance.","ref":"SnmpKit.SnmpMgr.html#adaptive_walk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.adaptive_walk/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :adaptive_tuning, :max_entries","ref":"SnmpKit.SnmpMgr.html#adaptive_walk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.adaptive_walk/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, results} = SnmpMgr.adaptive_walk(\"switch.local\", \"ifTable\")\n    # Returns optimally retrieved interface table data:\n    # [\n    #   {\"1.3.6.1.2.1.2.2.1.1.1\", 1},           # ifIndex.1\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"},      # ifDescr.1\n    #   {\"1.3.6.1.2.1.2.2.1.8.1\", 1},           # ifOperStatus.1\n    #   {\"1.3.6.1.2.1.2.2.1.1.2\", 2},           # ifIndex.2\n    #   {\"1.3.6.1.2.1.2.2.1.2.2\", \"eth1\"},      # ifDescr.2\n    #   {\"1.3.6.1.2.1.2.2.1.8.2\", 1}            # ifOperStatus.2\n    # ]","ref":"SnmpKit.SnmpMgr.html#adaptive_walk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.analyze_table/2","doc":"Analyzes table structure and returns detailed metadata.","ref":"SnmpKit.SnmpMgr.html#analyze_table/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.analyze_table/2","doc":"- `table_data` - Table data as returned by get_table/3\n- `opts` - Analysis options","ref":"SnmpKit.SnmpMgr.html#analyze_table/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.analyze_table/2","doc":"{:ok, table} = SnmpMgr.get_table(\"192.0.2.1\", \"ifTable\")\n    {:ok, analysis} = SnmpMgr.analyze_table(table)\n    IO.inspect(analysis.completeness)  # Shows data completeness ratio","ref":"SnmpKit.SnmpMgr.html#analyze_table/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.benchmark_device/3","doc":"Benchmarks a device to determine optimal bulk parameters.","ref":"SnmpKit.SnmpMgr.html#benchmark_device/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.benchmark_device/3","doc":"- `target` - The target device to benchmark\n- `test_oid` - OID to use for testing\n- `opts` - Benchmark options","ref":"SnmpKit.SnmpMgr.html#benchmark_device/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.benchmark_device/3","doc":"{:ok, results} = SnmpMgr.benchmark_device(\"192.0.2.1\", \"ifTable\")\n    optimal_size = results.optimal_bulk_size","ref":"SnmpKit.SnmpMgr.html#benchmark_device/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.bulk_pretty/3","doc":"Performs an SNMP BULK operation and returns formatted results.\n\nReturns a list of {oid, formatted_value} tuples where values are automatically\nformatted based on their SNMP types.","ref":"SnmpKit.SnmpMgr.html#bulk_pretty/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.bulk_pretty/3","doc":"# Bulk operation with automatic formatting\n    {:ok, results} = SnmpMgr.bulk_pretty(\"192.168.1.1\", \"1.3.6.1.2.1.2.2\", max_repetitions: 10)\n    # Returns: [{\"1.3.6.1.2.1.2.2.1.2.1\", \"eth0\"}, ...]","ref":"SnmpKit.SnmpMgr.html#bulk_pretty/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.bulk_walk/3","doc":"Performs an SNMP BULK WALK operation and returns raw results with type information.\n\nReturns a list of {oid_string, type, raw_value} tuples where:\n- oid_string: OID formatted as dotted decimal string\n- type: SNMP type atom (:string, :integer, :gauge32, etc.)\n- raw_value: Raw unformatted value from SNMP response","ref":"SnmpKit.SnmpMgr.html#bulk_walk/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.bulk_walk/3","doc":"# Bulk walk interface table with raw values\n    {:ok, results} = SnmpMgr.bulk_walk(\"192.168.1.1\", \"1.3.6.1.2.1.2.2\")\n    # Returns: [\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", :octet_string, \"eth0\"},\n    #   {\"1.3.6.1.2.1.2.2.1.5.1\", :gauge32, 1000000000},\n    #   ...\n    # ]","ref":"SnmpKit.SnmpMgr.html#bulk_walk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.bulk_walk_pretty/3","doc":"Performs an SNMP BULK WALK operation and returns formatted results with type information.\n\nReturns a list of {oid_string, type, formatted_value} tuples where:\n- oid_string: OID formatted as dotted decimal string\n- type: SNMP type atom (:string, :integer, :gauge32, etc.)\n- formatted_value: Human-readable formatted value","ref":"SnmpKit.SnmpMgr.html#bulk_walk_pretty/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.bulk_walk_pretty/3","doc":"# Bulk walk interface table with automatic formatting\n    {:ok, results} = SnmpMgr.bulk_walk_pretty(\"192.168.1.1\", \"1.3.6.1.2.1.2.2\")\n    # Returns: [\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", :octet_string, \"eth0\"},\n    #   {\"1.3.6.1.2.1.2.2.1.5.1\", :gauge32, \"1 Gbps\"},\n    #   ...\n    # ]","ref":"SnmpKit.SnmpMgr.html#bulk_walk_pretty/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.engine_batch/2","doc":"Submits multiple requests as a batch through the streaming engine.","ref":"SnmpKit.SnmpMgr.html#engine_batch/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.engine_batch/2","doc":"- `requests` - List of request specification maps\n- `opts` - Batch options","ref":"SnmpKit.SnmpMgr.html#engine_batch/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.engine_batch/2","doc":"requests = [\n      %{type: :get, target: \"device1\", oid: \"sysDescr.0\"},\n      %{type: :get, target: \"device2\", oid: \"sysUpTime.0\"}\n    ]\n\n    {:ok, results} = SnmpMgr.engine_batch(requests)","ref":"SnmpKit.SnmpMgr.html#engine_batch/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.engine_request/2","doc":"Submits a request through the streaming engine.\n\nRoutes the request through the high-performance engine infrastructure\nwith automatic load balancing, circuit breaking, and metrics collection.","ref":"SnmpKit.SnmpMgr.html#engine_request/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.engine_request/2","doc":"- `request` - Request specification map\n- `opts` - Request options","ref":"SnmpKit.SnmpMgr.html#engine_request/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.engine_request/2","doc":"request = %{\n      type: :get,\n      target: \"192.0.2.1\",\n      oid: \"sysDescr.0\",\n      community: \"public\"\n    }\n\n    {:ok, result} = SnmpMgr.engine_request(request)","ref":"SnmpKit.SnmpMgr.html#engine_request/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get/3","doc":"Performs an SNMP GET request.","ref":"SnmpKit.SnmpMgr.html#get/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get/3","doc":"- `target` - The target device (e.g., \"192.168.1.1:161\" or \"device.local\")\n- `oid` - The OID to retrieve (string format)\n- `opts` - Options including :community, :timeout, :retries","ref":"SnmpKit.SnmpMgr.html#get/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, value} = SnmpMgr.get(\"device.local:161\", \"sysDescr.0\", community: \"public\")\n    # \"Linux server 5.4.0-42-generic #46-Ubuntu SMP Fri Jul 10 00:24:02 UTC 2020 x86_64\"\n\n    {:ok, uptime} = SnmpMgr.get(\"router.local\", \"sysUpTime.0\")\n    # {:timeticks, 123456789}  # System uptime in hundredths of seconds","ref":"SnmpKit.SnmpMgr.html#get/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_async/3","doc":"Performs an asynchronous SNMP GET request.\n\nReturns immediately with a reference. The caller will receive a message\nwith the result.","ref":"SnmpKit.SnmpMgr.html#get_async/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_async/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    ref = SnmpMgr.get_async(\"device.local\", \"sysDescr.0\")\n    receive do\n      {^ref, {:ok, description}} -> description\n      {^ref, {:error, reason}} -> {:error, reason}\n    after\n      5000 -> {:error, :timeout}\n    end\n    # \"Linux server 5.4.0-42-generic\"","ref":"SnmpKit.SnmpMgr.html#get_async/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_bulk/3","doc":"Performs an SNMP GETBULK request (SNMPv2c only).\n\nGETBULK is more efficient than multiple GETNEXT requests for retrieving\nlarge amounts of data. It can retrieve multiple variables in a single request.","ref":"SnmpKit.SnmpMgr.html#get_bulk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_bulk/3","doc":"- `target` - The target device\n- `oid` - The starting OID\n- `opts` - Options including :non_repeaters, :max_repetitions, :community, :timeout","ref":"SnmpKit.SnmpMgr.html#get_bulk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_bulk/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, results} = SnmpMgr.get_bulk(\"switch.local\", \"ifTable\", max_repetitions: 10)\n    # [\n    #   {\"1.3.6.1.2.1.2.2.1.1.1\", 1},                    # ifIndex.1\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", \"FastEthernet0/1\"},    # ifDescr.1\n    #   {\"1.3.6.1.2.1.2.2.1.8.1\", 1},                    # ifOperStatus.1 (up)\n    #   {\"1.3.6.1.2.1.2.2.1.1.2\", 2},                    # ifIndex.2\n    #   {\"1.3.6.1.2.1.2.2.1.2.2\", \"FastEthernet0/2\"},    # ifDescr.2\n    #   # ... up to max_repetitions entries\n    # ]","ref":"SnmpKit.SnmpMgr.html#get_bulk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_bulk_async/3","doc":"Performs an asynchronous SNMP GETBULK request.\n\nReturns immediately with a reference. The caller will receive a message\nwith the result.","ref":"SnmpKit.SnmpMgr.html#get_bulk_async/3"},{"type":"function","title":"SnmpKit.SnmpMgr.get_bulk_multi/2","doc":"Performs concurrent GETBULK operations against multiple targets.","ref":"SnmpKit.SnmpMgr.html#get_bulk_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_bulk_multi/2","doc":"- `targets_and_oids` - List of {target, oid} tuples\n- `opts` - Options applied to all requests including :max_repetitions","ref":"SnmpKit.SnmpMgr.html#get_bulk_multi/2-parameters"},{"type":"function","title":"SnmpKit.SnmpMgr.get_column/4","doc":"Gets a specific column from an SNMP table.","ref":"SnmpKit.SnmpMgr.html#get_column/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_column/4","doc":"- `target` - The target device\n- `table_oid` - The table OID\n- `column` - The column number or name\n- `opts` - Options including :community, :timeout","ref":"SnmpKit.SnmpMgr.html#get_column/4-parameters"},{"type":"function","title":"SnmpKit.SnmpMgr.get_engine_stats/1","doc":"Gets comprehensive system metrics and statistics.","ref":"SnmpKit.SnmpMgr.html#get_engine_stats/1"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_engine_stats/1","doc":"- `opts` - Options including which components to include","ref":"SnmpKit.SnmpMgr.html#get_engine_stats/1-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_engine_stats/1","doc":"{:ok, stats} = SnmpMgr.get_engine_stats()\n    IO.inspect(stats.router.requests_routed)\n    IO.inspect(stats.metrics.current_metrics)","ref":"SnmpKit.SnmpMgr.html#get_engine_stats/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_multi/2","doc":"Performs concurrent GET operations against multiple targets.","ref":"SnmpKit.SnmpMgr.html#get_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_multi/2","doc":"- `targets_and_oids` - List of {target, oid} tuples\n- `opts` - Options applied to all requests","ref":"SnmpKit.SnmpMgr.html#get_multi/2-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_multi/2","doc":"# Note: Network operations will fail on unreachable hosts\n    iex> SnmpMgr.get_multi([{\"device1\", [1,3,6,1,2,1,1,1,0]}, {\"device2\", [1,3,6,1,2,1,1,3,0]}])\n    [{:error, {:network_error, :hostname_resolution_failed}}, {:error, {:network_error, :hostname_resolution_failed}}]","ref":"SnmpKit.SnmpMgr.html#get_multi/2-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_next/3","doc":"Performs an SNMP GETNEXT request.","ref":"SnmpKit.SnmpMgr.html#get_next/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_next/3","doc":"- `target` - The target device\n- `oid` - The starting OID\n- `opts` - Options including :community, :timeout, :retries","ref":"SnmpKit.SnmpMgr.html#get_next/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_next/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, {next_oid, value}} = SnmpMgr.get_next(\"switch.local\", \"1.3.6.1.2.1.1\")\n    # {\"1.3.6.1.2.1.1.1.0\", \"Cisco IOS Software, C2960 Software\"}\n\n    {:ok, {oid, val}} = SnmpMgr.get_next(\"device.local\", \"sysDescr\")\n    # {\"1.3.6.1.2.1.1.1.0\", \"Linux hostname 5.4.0 #1 SMP\"}","ref":"SnmpKit.SnmpMgr.html#get_next/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_pretty/3","doc":"Performs an SNMP GET operation and returns a formatted value.\n\nThis is a convenience function that combines `get_with_type/3` and automatic\nformatting based on the SNMP type. Returns just the formatted value since\nthe OID is already known.","ref":"SnmpKit.SnmpMgr.html#get_pretty/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_pretty/3","doc":"# Get system uptime with automatic formatting\n    {:ok, formatted_uptime} = SnmpMgr.get_pretty(\"192.168.1.1\", \"1.3.6.1.2.1.1.3.0\")\n    # Returns: \"14 days 15 hours 55 minutes 13 seconds\"","ref":"SnmpKit.SnmpMgr.html#get_pretty/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_table/3","doc":"Gets all entries from an SNMP table and formats them as a structured table.","ref":"SnmpKit.SnmpMgr.html#get_table/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_table/3","doc":"- `target` - The target device\n- `table_oid` - The table OID\n- `opts` - Options including :community, :timeout","ref":"SnmpKit.SnmpMgr.html#get_table/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_table/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, table} = SnmpMgr.get_table(\"switch.local\", \"ifTable\")\n    # %{\n    #   columns: [\"ifIndex\", \"ifDescr\", \"ifType\", \"ifMtu\", \"ifSpeed\", \"ifOperStatus\"],\n    #   rows: [\n    #     %{\"ifIndex\" => 1, \"ifDescr\" => \"GigabitEthernet0/1\", \"ifType\" => 6,\n    #       \"ifMtu\" => 1500, \"ifSpeed\" => 1000000000, \"ifOperStatus\" => 1},\n    #     %{\"ifIndex\" => 2, \"ifDescr\" => \"GigabitEthernet0/2\", \"ifType\" => 6,\n    #       \"ifMtu\" => 1500, \"ifSpeed\" => 1000000000, \"ifOperStatus\" => 2}\n    #   ]\n    # }","ref":"SnmpKit.SnmpMgr.html#get_table/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.get_with_type/3","doc":"Performs an SNMP GET request and returns the result in 3-tuple format.\n\nThis function returns the same format as walk, bulk, and other operations:\n`{oid_string, type, value}` for consistency across the library.","ref":"SnmpKit.SnmpMgr.html#get_with_type/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.get_with_type/3","doc":"- `target` - The target device (e.g., \"192.168.1.1:161\" or \"device.local\")\n- `oid` - The OID to retrieve (string format)\n- `opts` - Options including :community, :timeout, :retries","ref":"SnmpKit.SnmpMgr.html#get_with_type/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.get_with_type/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, {oid, type, value}} = SnmpMgr.get_with_type(\"device.local:161\", \"sysDescr.0\", community: \"public\")\n    # {:ok, {\"1.3.6.1.2.1.1.1.0\", :octet_string, \"Linux server 5.4.0-42-generic\"}}\n\n    {:ok, {oid, type, uptime}} = SnmpMgr.get_with_type(\"router.local\", \"sysUpTime.0\")\n    # {:ok, {\"1.3.6.1.2.1.1.3.0\", :timeticks, 123456789}}","ref":"SnmpKit.SnmpMgr.html#get_with_type/3-examples"},{"type":"type","title":"SnmpKit.SnmpMgr.oid/0","doc":"","ref":"SnmpKit.SnmpMgr.html#t:oid/0"},{"type":"type","title":"SnmpKit.SnmpMgr.opts/0","doc":"","ref":"SnmpKit.SnmpMgr.html#t:opts/0"},{"type":"function","title":"SnmpKit.SnmpMgr.record_metric/4","doc":"Records a custom metric.","ref":"SnmpKit.SnmpMgr.html#record_metric/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.record_metric/4","doc":"- `metric_type` - Type of metric (:counter, :gauge, :histogram)\n- `metric_name` - Name of the metric\n- `value` - Value to record\n- `tags` - Optional tags","ref":"SnmpKit.SnmpMgr.html#record_metric/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.record_metric/4","doc":"SnmpMgr.record_metric(:counter, :custom_requests, 1, %{device: \"switch1\"})\n    SnmpMgr.record_metric(:histogram, :custom_latency, 150, %{operation: \"bulk\"})","ref":"SnmpKit.SnmpMgr.html#record_metric/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.set/4","doc":"Performs an SNMP SET request.","ref":"SnmpKit.SnmpMgr.html#set/4"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.set/4","doc":"- `target` - The target device\n- `oid` - The OID to set\n- `value` - The value to set\n- `opts` - Options including :community, :timeout, :retries","ref":"SnmpKit.SnmpMgr.html#set/4-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.set/4","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, :ok} = SnmpMgr.set(\"device.local\", \"sysLocation.0\", \"Server Room A\")\n    # :ok\n\n    {:ok, :ok} = SnmpMgr.set(\"switch.local\", \"sysContact.0\", \"admin@company.com\",\n      community: \"private\", timeout: 3000)\n    # :ok","ref":"SnmpKit.SnmpMgr.html#set/4-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.start_engine/1","doc":"Starts the streaming PDU engine infrastructure.\n\nInitializes all Phase 5 components including engines, routers, connection pools,\ncircuit breakers, and metrics collection for high-performance SNMP operations.","ref":"SnmpKit.SnmpMgr.html#start_engine/1"},{"type":"function","title":"Options - SnmpKit.SnmpMgr.start_engine/1","doc":"- `:engine` - Engine configuration options\n- `:router` - Router configuration options\n- `:pool` - Connection pool options\n- `:circuit_breaker` - Circuit breaker options\n- `:metrics` - Metrics collection options","ref":"SnmpKit.SnmpMgr.html#start_engine/1-options"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.start_engine/1","doc":"{:ok, _pid} = SnmpMgr.start_engine(\n      engine: [pool_size: 20, max_rps: 500],\n      router: [strategy: :least_connections],\n      pool: [pool_size: 50],\n      metrics: [window_size: 120]\n    )","ref":"SnmpKit.SnmpMgr.html#start_engine/1-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.table_stream/3","doc":"Creates a stream for processing large SNMP tables.","ref":"SnmpKit.SnmpMgr.html#table_stream/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.table_stream/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to stream\n- `opts` - Options including :chunk_size, :columns","ref":"SnmpKit.SnmpMgr.html#table_stream/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.table_stream/3","doc":"# Note: Requires Erlang SNMP modules for actual operation\n    stream = SnmpMgr.table_stream(\"192.0.2.1\", \"ifTable\")\n    # Process table stream...","ref":"SnmpKit.SnmpMgr.html#table_stream/3-examples"},{"type":"type","title":"SnmpKit.SnmpMgr.target/0","doc":"","ref":"SnmpKit.SnmpMgr.html#t:target/0"},{"type":"function","title":"SnmpKit.SnmpMgr.walk/3","doc":"Performs an SNMP walk operation using iterative GETNEXT requests.\n\nWalks the SNMP tree starting from the given OID and returns all OID/value\npairs found under that subtree.","ref":"SnmpKit.SnmpMgr.html#walk/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.walk/3","doc":"- `target` - The target device\n- `root_oid` - The starting OID for the walk\n- `opts` - Options including :community, :timeout, :max_repetitions","ref":"SnmpKit.SnmpMgr.html#walk/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.walk/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, results} = SnmpKit.SnmpMgr.Walk(\"device.local\", \"1.3.6.1.2.1.1\")\n    # [\n    #   {\"1.3.6.1.2.1.1.1.0\", \"Linux hostname 5.4.0-42-generic\"},  # sysDescr\n    #   {\"1.3.6.1.2.1.1.2.0\", [1,3,6,1,4,1,8072,3,2,10]},         # sysObjectID\n    #   {\"1.3.6.1.2.1.1.3.0\", {:timeticks, 12345678}},             # sysUpTime\n    #   {\"1.3.6.1.2.1.1.4.0\", \"admin@company.com\"},                # sysContact\n    #   {\"1.3.6.1.2.1.1.5.0\", \"server01.company.com\"},             # sysName\n    #   {\"1.3.6.1.2.1.1.6.0\", \"Data Center Room 42\"}               # sysLocation\n    # ]","ref":"SnmpKit.SnmpMgr.html#walk/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.walk_multi/2","doc":"Performs concurrent walk operations against multiple targets.","ref":"SnmpKit.SnmpMgr.html#walk_multi/2"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.walk_multi/2","doc":"- `targets_and_oids` - List of {target, root_oid} tuples\n- `opts` - Options applied to all requests","ref":"SnmpKit.SnmpMgr.html#walk_multi/2-parameters"},{"type":"function","title":"SnmpKit.SnmpMgr.walk_pretty/3","doc":"Performs an SNMP WALK operation and returns formatted results.\n\nReturns a list of {oid, formatted_value} tuples where values are automatically\nformatted based on their SNMP types.","ref":"SnmpKit.SnmpMgr.html#walk_pretty/3"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.walk_pretty/3","doc":"# Walk system group with automatic formatting\n    {:ok, results} = SnmpMgr.walk_pretty(\"192.168.1.1\", \"1.3.6.1.2.1.1\")\n    # Returns: [{\"1.3.6.1.2.1.1.3.0\", \"14 days 15 hours\"}, ...]","ref":"SnmpKit.SnmpMgr.html#walk_pretty/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.walk_stream/3","doc":"Creates a stream for memory-efficient processing of large SNMP data.","ref":"SnmpKit.SnmpMgr.html#walk_stream/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.walk_stream/3","doc":"- `target` - The target device\n- `root_oid` - Starting OID for the walk\n- `opts` - Options including :chunk_size, :adaptive","ref":"SnmpKit.SnmpMgr.html#walk_stream/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.walk_stream/3","doc":"# Note: Requires Erlang SNMP modules for actual operation\n    stream = SnmpMgr.walk_stream(\"192.0.2.1\", \"ifTable\")\n    # Process stream lazily...","ref":"SnmpKit.SnmpMgr.html#walk_stream/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.walk_table/3","doc":"Walks an SNMP table and returns all entries.","ref":"SnmpKit.SnmpMgr.html#walk_table/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.walk_table/3","doc":"- `target` - The target device\n- `table_oid` - The table OID to walk\n- `opts` - Options including :community, :timeout","ref":"SnmpKit.SnmpMgr.html#walk_table/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.walk_table/3","doc":"# Note: This function makes actual network calls and is not suitable for doctests\n    {:ok, entries} = SnmpMgr.walk_table(\"switch.local\", \"ifTable\")\n    # [\n    #   {\"1.3.6.1.2.1.2.2.1.1.1\", 1},\n    #   {\"1.3.6.1.2.1.2.2.1.2.1\", \"GigabitEthernet0/1\"},\n    #   {\"1.3.6.1.2.1.2.2.1.3.1\", 6},  # ethernetCsmacd\n    #   {\"1.3.6.1.2.1.2.2.1.5.1\", 1000000000},  # 1 Gbps\n    #   # ... all interface table entries\n    # ]","ref":"SnmpKit.SnmpMgr.html#walk_table/3-examples"},{"type":"function","title":"SnmpKit.SnmpMgr.with_circuit_breaker/3","doc":"Executes a function with circuit breaker protection.","ref":"SnmpKit.SnmpMgr.html#with_circuit_breaker/3"},{"type":"function","title":"Parameters - SnmpKit.SnmpMgr.with_circuit_breaker/3","doc":"- `target` - Target device identifier\n- `fun` - Function to execute with protection\n- `opts` - Circuit breaker options","ref":"SnmpKit.SnmpMgr.html#with_circuit_breaker/3-parameters"},{"type":"function","title":"Examples - SnmpKit.SnmpMgr.with_circuit_breaker/3","doc":"result = SnmpMgr.with_circuit_breaker(\"192.0.2.1\", fn ->\n      SnmpMgr.get(\"192.0.2.1\", \"sysDescr.0\")\n    end)","ref":"SnmpKit.SnmpMgr.html#with_circuit_breaker/3-examples"},{"type":"module","title":"SnmpKit.TestSupport","doc":"SNMP Simulator for Elixir - Production-ready SNMP device simulation.\n\nProvides high-performance SNMP device simulation supporting walk files,\nrealistic behaviors, and large-scale testing scenarios.","ref":"SnmpKit.TestSupport.html"},{"type":"function","title":"SnmpKit.TestSupport.start/2","doc":"","ref":"SnmpKit.TestSupport.html#start/2"},{"type":"function","title":"SnmpKit.TestSupport.start_device/2","doc":"Start a single SNMP device with the given profile.","ref":"SnmpKit.TestSupport.html#start_device/2"},{"type":"function","title":"Examples - SnmpKit.TestSupport.start_device/2","doc":"# Start device with walk file profile\n    profile = SnmpKit.SnmpSim.ProfileLoader.load_profile(\n      :cable_modem,\n      {:walk_file, \"priv/walks/cable_modem.walk\"}\n    )\n    {:ok, device} = SnmpSim.start_device(profile, port: 9001)","ref":"SnmpKit.TestSupport.html#start_device/2-examples"},{"type":"function","title":"SnmpKit.TestSupport.start_device_population/2","doc":"Start a population of devices with mixed types and configurations.","ref":"SnmpKit.TestSupport.html#start_device_population/2"},{"type":"function","title":"Examples - SnmpKit.TestSupport.start_device_population/2","doc":"device_configs = [\n      {:cable_modem, {:walk_file, \"priv/walks/cm.walk\"}, count: 1000},\n      {:switch, {:oid_walk, \"priv/walks/switch.walk\"}, count: 50}\n    ]\n\n    {:ok, devices} = SnmpSim.start_device_population(\n      device_configs,\n      port_range: 30_000..39_999\n    )","ref":"SnmpKit.TestSupport.html#start_device_population/2-examples"},{"type":"extras","title":"SnmpKit 🚀","doc":"# SnmpKit 🚀\n\n[![Hex.pm](https://img.shields.io/hexpm/v/snmpkit.svg)](https://hex.pm/packages/snmpkit)\n[![Documentation](https://img.shields.io/badge/docs-hexdocs-blue.svg)](https://hexdocs.pm/snmpkit)\n[![License](https://img.shields.io/github/license/awksedgreep/snmpkit.svg)](LICENSE)\n[![Build Status](https://img.shields.io/github/workflow/status/awksedgreep/snmpkit/CI)](https://github.com/awksedgreep/snmpkit/actions)\n\n**A modern, comprehensive SNMP toolkit for Elixir - featuring a unified API, pure Elixir implementation, and powerful device simulation.**\n\nSnmpKit is a complete SNMP (Simple Network Management Protocol) solution built from the ground up in pure Elixir. It provides a clean, organized API for SNMP operations, MIB management, and realistic device simulation - perfect for network monitoring, testing, and development.","ref":"readme.html"},{"type":"extras","title":"✨ Key Features - SnmpKit 🚀","doc":"- 🎯 **Unified API** - Clean, context-based modules (`SnmpKit.SNMP`, `SnmpKit.MIB`, `SnmpKit.Sim`)\n- 🧬 **Pure Elixir Implementation** - No Erlang SNMP dependencies\n- 📋 **Advanced MIB Support** - Native parsing, compilation, and object resolution\n- 🖥️ **Realistic Device Simulation** - Create SNMP devices for testing and development\n- ⚡ **High Performance** - Optimized for large-scale operations and concurrent requests\n- 🧪 **Testing Friendly** - Comprehensive test helpers and simulated devices\n- 🔧 **Modern Architecture** - GenServer patterns, supervision trees, circuit breakers\n- 📊 **Enterprise Ready** - DOCSIS, standard MIBs, and custom implementations\n- 🚀 **Zero Warnings** - Clean, production-ready codebase","ref":"readme.html#key-features"},{"type":"extras","title":"🚀 Quick Start - SnmpKit 🚀","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Installation - SnmpKit 🚀","doc":"Add `snmpkit` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:snmpkit, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Unified API Examples - SnmpKit 🚀","doc":"SnmpKit provides a clean, organized API through context-based modules:\n\n#### 📡 SNMP Operations (`SnmpKit.SNMP`)\n\n```elixir\n# Basic SNMP operations\n{:ok, description} = SnmpKit.SNMP.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, uptime} = SnmpKit.SNMP.get(\"192.168.1.1\", \"sysUpTime.0\")\n\n# Walk operations\n{:ok, system_info} = SnmpKit.SNMP.walk(\"192.168.1.1\", \"system\")\n{:ok, interface_table} = SnmpKit.SNMP.get_table(\"192.168.1.1\", \"ifTable\")\n\n# Bulk operations for efficiency\n{:ok, results} = SnmpKit.SNMP.bulk_walk(\"192.168.1.1\", \"interfaces\")\n\n# Multi-target operations\n{:ok, results} = SnmpKit.SNMP.get_multi([\n  {\"host1\", \"sysDescr.0\"},\n  {\"host2\", \"sysUpTime.0\"},\n  {\"host3\", \"ifInOctets.1\"}\n])\n\n# Pretty formatting\n{:ok, formatted} = SnmpKit.SNMP.get_pretty(\"192.168.1.1\", \"sysUpTime.0\")\n# Returns: \"12 days, 4:32:10.45\"\n\n# Async operations\ntask = SnmpKit.SNMP.get_async(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, result} = Task.await(task)\n```\n\n#### 📚 MIB Operations (`SnmpKit.MIB`)\n\n```elixir\n# OID name resolution\n{:ok, oid} = SnmpKit.MIB.resolve(\"sysDescr.0\")\n# Returns: [1, 3, 6, 1, 2, 1, 1, 1, 0]\n\n# Reverse lookup\n{:ok, name} = SnmpKit.MIB.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1, 0])\n# Returns: \"sysDescr.0\"\n\n# MIB compilation and loading\n{:ok, compiled} = SnmpKit.MIB.compile(\"MY-CUSTOM-MIB.mib\")\n{:ok, _} = SnmpKit.MIB.load(compiled)\n\n# Tree navigation\n{:ok, children} = SnmpKit.MIB.children([1, 3, 6, 1, 2, 1, 1])\n{:ok, parent} = SnmpKit.MIB.parent([1, 3, 6, 1, 2, 1, 1, 1, 0])\n```\n\n#### 🧪 Device Simulation (`SnmpKit.Sim`)\n\n```elixir\n# Load a device profile\n{:ok, profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(\n  :cable_modem,\n  {:walk_file, \"priv/walks/cable_modem.walk\"}\n)\n\n# Start a simulated device\n{:ok, device} = SnmpKit.Sim.start_device(profile, port: 1161)\n\n# Create a population of devices for testing\ndevice_configs = [\n  %{type: :cable_modem, port: 30001, community: \"public\"},\n  %{type: :switch, port: 30002, community: \"public\"},\n  %{type: :router, port: 30003, community: \"private\"}\n]\n\n{:ok, devices} = SnmpKit.Sim.start_device_population(device_configs)\n```\n\n#### 🎯 Direct Access (Backward Compatibility)\n\nFor convenience, common operations are also available directly:\n\n```elixir\n# These work the same as their SnmpKit.SNMP.* equivalents\n{:ok, value} = SnmpKit.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, results} = SnmpKit.walk(\"192.168.1.1\", \"system\")\n\n# MIB resolution\n{:ok, oid} = SnmpKit.resolve(\"sysDescr.0\")\n```","ref":"readme.html#unified-api-examples"},{"type":"extras","title":"Advanced Features - SnmpKit 🚀","doc":"#### Engine Management and Performance\n\n```elixir\n# Start the SNMP engine for advanced features\n{:ok, _engine} = SnmpKit.SNMP.start_engine()\n\n# Get performance statistics\n{:ok, stats} = SnmpKit.SNMP.get_engine_stats()\n\n# Batch operations for efficiency\nrequests = [\n  %{type: :get, target: \"host1\", oid: \"sysDescr.0\"},\n  %{type: :walk, target: \"host2\", oid: \"interfaces\"}\n]\n{:ok, results} = SnmpKit.SNMP.engine_batch(requests)\n\n# Circuit breaker for reliability\n{:ok, result} = SnmpKit.SNMP.with_circuit_breaker(\"unreliable.host\", fn ->\n  SnmpKit.SNMP.get(\"unreliable.host\", \"sysDescr.0\")\nend)\n```\n\n#### Streaming and Large-Scale Operations\n\n```elixir\n# Stream large walks to avoid memory issues\nstream = SnmpKit.SNMP.walk_stream(\"192.168.1.1\", \"interfaces\")\nstream\n|> Stream.take(1000)\n|> Enum.to_list()\n\n# Adaptive bulk operations that optimize themselves\n{:ok, results} = SnmpKit.SNMP.adaptive_walk(\"192.168.1.1\", \"interfaces\")\n\n# Performance benchmarking\n{:ok, benchmark} = SnmpKit.SNMP.benchmark_device(\"192.168.1.1\", \"system\")\n```","ref":"readme.html#advanced-features"},{"type":"extras","title":"🏗️ Architecture - SnmpKit 🚀","doc":"SnmpKit is organized into logical, discoverable modules:\n\n- **`SnmpKit.SNMP`** - Complete SNMP protocol operations\n  - Basic operations: get, set, walk, bulk\n  - Advanced features: streaming, async, multi-target\n  - Performance tools: engine, circuit breaker, metrics\n  - Pretty formatting and analysis\n\n- **`SnmpKit.MIB`** - Comprehensive MIB management\n  - OID resolution and reverse lookup\n  - MIB compilation and loading (both high-level and low-level)\n  - Tree navigation and analysis\n  - Standard and custom MIB support\n\n- **`SnmpKit.Sim`** - Realistic device simulation\n  - Profile-based device behavior\n  - Population management for testing\n  - Integration with test frameworks\n\n- **`SnmpKit`** - Direct access for convenience\n  - Common operations without module prefixes\n  - Backward compatibility for existing code\n  - Simple one-import access","ref":"readme.html#architecture"},{"type":"extras","title":"📊 Enterprise Features - SnmpKit 🚀","doc":"","ref":"readme.html#enterprise-features"},{"type":"extras","title":"DOCSIS and Cable Modem Support - SnmpKit 🚀","doc":"```elixir\n# DOCSIS-specific operations\n{:ok, cm_status} = SnmpKit.SNMP.get(\"10.1.1.100\", \"docsIfCmtsServiceAdminStatus.1\")\n{:ok, signal_quality} = SnmpKit.SNMP.get(\"10.1.1.100\", \"docsIfSigQSignalNoise.1\")\n\n# Load DOCSIS MIBs\n{:ok, _} = SnmpKit.MIB.compile(\"DOCS-CABLE-DEVICE-MIB.mib\")\n{:ok, _} = SnmpKit.MIB.compile(\"DOCS-IF-MIB.mib\")\n```","ref":"readme.html#docsis-and-cable-modem-support"},{"type":"extras","title":"Network Monitoring Integration - SnmpKit 🚀","doc":"```elixir\n# Monitor interface statistics\ninterface_oids = [\n  \"ifInOctets.1\", \"ifOutOctets.1\",\n  \"ifInErrors.1\", \"ifOutErrors.1\"\n]\n\n# Collect metrics from multiple devices\ndevices = [\"router1\", \"router2\", \"switch1\", \"switch2\"]\n\nresults = \n  for device <- devices,\n      oid <- interface_oids do\n    {device, oid, SnmpKit.SNMP.get(device, oid)}\n  end\n\n# Analyze and format results\nanalysis = SnmpKit.SNMP.analyze_table(results)\n```","ref":"readme.html#network-monitoring-integration"},{"type":"extras","title":"🧪 Testing and Development - SnmpKit 🚀","doc":"","ref":"readme.html#testing-and-development"},{"type":"extras","title":"Simulated Devices for Testing - SnmpKit 🚀","doc":"```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case\n  \n  setup do\n    # Start a simulated device for testing\n    {:ok, profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(:generic_router)\n    {:ok, device} = SnmpKit.Sim.start_device(profile, port: 1161)\n\n    %{device: device, target: \"127.0.0.1:1161\"}\n  end\n\n  test \"can query simulated device\", %{target: target} do\n    {:ok, description} = SnmpKit.SNMP.get(target, \"sysDescr.0\")\n    assert description =~ \"Simulated Router\"\n  end\nend\n```","ref":"readme.html#simulated-devices-for-testing"},{"type":"extras","title":"Performance Testing - SnmpKit 🚀","doc":"```elixir\n# Benchmark different devices and operations\ndevices = [\"fast.device\", \"slow.device\", \"unreliable.device\"]\n\nbenchmarks = \n  for device <- devices do\n    SnmpKit.SNMP.benchmark_device(device, \"system\")\n  end\n\n# Compare performance characteristics\nfor {device, {:ok, benchmark}} <- Enum.zip(devices, benchmarks) do\n  IO.puts \"#{device}: avg=#{benchmark.avg_response_time}ms, success_rate=#{benchmark.success_rate}%\"\nend\n```","ref":"readme.html#performance-testing"},{"type":"extras","title":"📚 Documentation - SnmpKit 🚀","doc":"- **[Full API Documentation](https://hexdocs.pm/snmpkit)** - Complete function reference\n- **[Livebook Tour](livebooks/snmpkit_tour.livemd)** - Interactive examples and tutorials\n- **[Examples Directory](examples/)** - Practical usage examples\n- **[MIB Guide](docs/mib-guide.md)** - Working with MIBs and OID resolution\n- **[Testing Guide](docs/testing-guide.md)** - Testing strategies and simulated devices","ref":"readme.html#documentation"},{"type":"extras","title":"🚀 Migration from Other Libraries - SnmpKit 🚀","doc":"","ref":"readme.html#migration-from-other-libraries"},{"type":"extras","title":"From :snmp (Erlang) - SnmpKit 🚀","doc":"```elixir\n# Before (Erlang SNMP)\n:snmp.sync_get(manager, oid, timeout)\n\n# After (SnmpKit)\nSnmpKit.SNMP.get(\"192.168.1.1\", \"sysDescr.0\", timeout: 5000)\n```","ref":"readme.html#from-snmp-erlang"},{"type":"extras","title":"From Other Elixir SNMP Libraries - SnmpKit 🚀","doc":"```elixir\n# SnmpKit provides more features with cleaner syntax\n{:ok, results} = SnmpKit.SNMP.walk_multi([\n  {\"host1\", \"interfaces\"},\n  {\"host2\", \"system\"}\n])\n\n# Built-in formatting and analysis\n{:ok, formatted} = SnmpKit.SNMP.walk_pretty(\"192.168.1.1\", \"interfaces\")\n```","ref":"readme.html#from-other-elixir-snmp-libraries"},{"type":"extras","title":"⚡ Performance - SnmpKit 🚀","doc":"SnmpKit is designed for high-performance network monitoring:\n\n- **Concurrent Operations** - Efficient handling of thousands of simultaneous requests\n- **Bulk Operations** - Optimized SNMP bulk protocols for large data sets\n- **Connection Pooling** - Managed through the underlying SnmpLib layer\n- **Circuit Breakers** - Automatic failure handling and recovery\n- **Streaming** - Memory-efficient processing of large SNMP walks\n- **Adaptive Algorithms** - Self-tuning bulk sizes and timeouts","ref":"readme.html#performance"},{"type":"extras","title":"🔧 Configuration - SnmpKit 🚀","doc":"```elixir\n# config/config.exs\nconfig :snmpkit,\n  default_community: \"public\",\n  default_timeout: 5000,\n  default_retries: 3,\n  default_version: :v2c\n\n# For simulation\nconfig :snmpkit, :simulation,\n  device_profiles_path: \"priv/device_profiles\",\n  walk_files_path: \"priv/walks\"\n\n# For MIB management\nconfig :snmpkit, :mib,\n  mib_path: [\"priv/mibs\", \"/usr/share/snmp/mibs\"],\n  auto_load_standard_mibs: true\n```","ref":"readme.html#configuration"},{"type":"extras","title":"🤝 Contributing - SnmpKit 🚀","doc":"We welcome contributions! Please see [CONTRIBUTING.md](CONTRIBUTING.md) for guidelines.","ref":"readme.html#contributing"},{"type":"extras","title":"Development Setup - SnmpKit 🚀","doc":"```bash\ngit clone https://github.com/awksedgreep/snmpkit.git\ncd snmpkit\nmix deps.get\nmix test\n```","ref":"readme.html#development-setup"},{"type":"extras","title":"Running the Test Suite - SnmpKit 🚀","doc":"```bash\n# Run all tests\nmix test\n\n# Run with coverage\nmix test --cover\n\n# Run specific test categories\nmix test --include docsis\nmix test --include integration\n```","ref":"readme.html#running-the-test-suite"},{"type":"extras","title":"📈 Roadmap - SnmpKit 🚀","doc":"- 🔐 **SNMPv3 Support** - Authentication and encryption\n- 🌐 **IPv6 Enhancement** - Full IPv6 support throughout\n- 📊 **Advanced Analytics** - Built-in network analysis tools  \n- 🔌 **Plugin System** - Custom protocol extensions\n- 🎯 **More Device Profiles** - Extended simulation library\n- 📱 **Management UI** - Web interface for monitoring","ref":"readme.html#roadmap"},{"type":"extras","title":"📄 License - SnmpKit 🚀","doc":"SnmpKit is released under the [MIT License](LICENSE).","ref":"readme.html#license"},{"type":"extras","title":"🙏 Acknowledgments - SnmpKit 🚀","doc":"- Built with ❤️ for the Elixir community\n- Inspired by the need for modern, testable SNMP tools\n- Thanks to all contributors and early adopters\n\n---\n\n**Ready to simplify your SNMP operations?** Get started with SnmpKit today! 🚀\n\nFor questions, issues, or feature requests, please visit our [GitHub repository](https://github.com/awksedgreep/snmpkit).","ref":"readme.html#acknowledgments"},{"type":"extras","title":"SnmpKit Unified API Guide","doc":"# SnmpKit Unified API Guide\n\n*A comprehensive guide to SnmpKit's new unified API design*","ref":"unified-api-guide.html"},{"type":"extras","title":"🎯 Overview - SnmpKit Unified API Guide","doc":"SnmpKit v0.2.0 introduces a **unified API** that organizes all functionality into logical, context-based modules. This design eliminates naming conflicts, improves discoverability, and provides a cleaner developer experience while maintaining 100% backward compatibility.","ref":"unified-api-guide.html#overview"},{"type":"extras","title":"🏗️ API Architecture - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#api-architecture"},{"type":"extras","title":"Context-Based Modules - SnmpKit Unified API Guide","doc":"| Module | Purpose | Functions |\n|--------|---------|-----------|\n| `SnmpKit.SNMP` | SNMP protocol operations | get, walk, bulk, set, multi-target, streaming |\n| `SnmpKit.MIB` | MIB management | resolve, compile, load, tree navigation |\n| `SnmpKit.Sim` | Device simulation | start devices, create populations, testing |\n| `SnmpKit` | Direct access | Common operations for convenience |","ref":"unified-api-guide.html#context-based-modules"},{"type":"extras","title":"Design Benefits - SnmpKit Unified API Guide","doc":"✅ **No Naming Conflicts** - Context prevents function name collisions  \n✅ **Improved Discoverability** - Related functions grouped logically  \n✅ **Clear Documentation** - Module boundaries define responsibilities  \n✅ **Backward Compatibility** - Existing code continues to work  \n✅ **Flexible Usage** - Choose namespaced or direct access as preferred","ref":"unified-api-guide.html#design-benefits"},{"type":"extras","title":"📡 SNMP Operations (SnmpKit.SNMP) - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#snmp-operations-snmpkit-snmp"},{"type":"extras","title":"Basic Operations - SnmpKit Unified API Guide","doc":"```elixir\n# GET operations\n{:ok, value} = SnmpKit.SNMP.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, {oid, type, value}} = SnmpKit.SNMP.get_with_type(\"192.168.1.1\", \"sysUpTime.0\")\n\n# SET operations (to simulation devices)\n:ok = SnmpKit.SNMP.set(\"127.0.0.1:1161\", \"sysContact.0\", \"admin@example.com\")\n\n# WALK operations\n{:ok, results} = SnmpKit.SNMP.walk(\"192.168.1.1\", \"system\")\n{:ok, table} = SnmpKit.SNMP.get_table(\"192.168.1.1\", \"ifTable\")\n```","ref":"unified-api-guide.html#basic-operations"},{"type":"extras","title":"Bulk Operations - SnmpKit Unified API Guide","doc":"```elixir\n# Efficient bulk retrieval\n{:ok, results} = SnmpKit.SNMP.get_bulk(\"192.168.1.1\", \"interfaces\", max_repetitions: 10)\n\n# Adaptive bulk walking (auto-optimizes)\n{:ok, results} = SnmpKit.SNMP.adaptive_walk(\"192.168.1.1\", \"interfaces\")\n\n# Traditional bulk walk\n{:ok, results} = SnmpKit.SNMP.bulk_walk(\"192.168.1.1\", \"system\")\n```","ref":"unified-api-guide.html#bulk-operations"},{"type":"extras","title":"Multi-Target Operations - SnmpKit Unified API Guide","doc":"```elixir\n# Query multiple devices simultaneously\ntargets_and_oids = [\n  {\"router1.example.com\", \"sysDescr.0\"},\n  {\"switch1.example.com\", \"sysUpTime.0\"},\n  {\"ap1.example.com\", \"sysLocation.0\"}\n]\n\n{:ok, results} = SnmpKit.SNMP.get_multi(targets_and_oids)\n\n# Bulk operations across multiple targets\n{:ok, results} = SnmpKit.SNMP.walk_multi([\n  {\"host1\", \"interfaces\"},\n  {\"host2\", \"system\"}\n])\n```","ref":"unified-api-guide.html#multi-target-operations"},{"type":"extras","title":"Streaming Operations - SnmpKit Unified API Guide","doc":"```elixir\n# Memory-efficient streaming for large datasets\nstream = SnmpKit.SNMP.walk_stream(\"192.168.1.1\", \"interfaces\")\nresults = stream |> Stream.take(1000) |> Enum.to_list()\n\n# Table streaming\ntable_stream = SnmpKit.SNMP.table_stream(\"192.168.1.1\", \"ifTable\")\n```","ref":"unified-api-guide.html#streaming-operations"},{"type":"extras","title":"Async Operations - SnmpKit Unified API Guide","doc":"```elixir\n# Non-blocking operations\ntask = SnmpKit.SNMP.get_async(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, result} = Task.await(task, 5000)\n\n# Bulk async operations\ntask = SnmpKit.SNMP.get_bulk_async(\"192.168.1.1\", \"interfaces\")\n```","ref":"unified-api-guide.html#async-operations"},{"type":"extras","title":"Pretty Formatting - SnmpKit Unified API Guide","doc":"```elixir\n# Human-readable output\n{:ok, formatted} = SnmpKit.SNMP.get_pretty(\"192.168.1.1\", \"sysUpTime.0\")\n# Returns: \"12 days, 4:32:10.45\"\n\n{:ok, formatted_walk} = SnmpKit.SNMP.walk_pretty(\"192.168.1.1\", \"system\")\n# Returns: [{\"sysDescr.0\", \"Linux router\"}, {\"sysUpTime.0\", \"12 days, 4:32:10.45\"}]\n\n{:ok, formatted_bulk} = SnmpKit.SNMP.bulk_walk_pretty(\"192.168.1.1\", \"interfaces\")\n```","ref":"unified-api-guide.html#pretty-formatting"},{"type":"extras","title":"Advanced Features - SnmpKit Unified API Guide","doc":"```elixir\n# Engine management for performance\n{:ok, _engine} = SnmpKit.SNMP.start_engine()\n{:ok, stats} = SnmpKit.SNMP.get_engine_stats()\n\n# Circuit breaker for reliability\n{:ok, result} = SnmpKit.SNMP.with_circuit_breaker(\"unreliable.host\", fn ->\n  SnmpKit.SNMP.get(\"unreliable.host\", \"sysDescr.0\")\nend)\n\n# Performance analysis\n{:ok, analysis} = SnmpKit.SNMP.analyze_table(table_data)\n{:ok, benchmark} = SnmpKit.SNMP.benchmark_device(\"192.168.1.1\", \"system\")\n\n# Metrics recording\nSnmpKit.SNMP.record_metric(:counter, :requests_total, 1, %{host: \"router1\"})\n```","ref":"unified-api-guide.html#advanced-features"},{"type":"extras","title":"📚 MIB Operations (SnmpKit.MIB) - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#mib-operations-snmpkit-mib"},{"type":"extras","title":"OID Resolution - SnmpKit Unified API Guide","doc":"```elixir\n# Name to OID resolution\n{:ok, oid} = SnmpKit.MIB.resolve(\"sysDescr.0\")\n# Returns: [1, 3, 6, 1, 2, 1, 1, 1, 0]\n\n{:ok, oid} = SnmpKit.MIB.resolve(\"ifInOctets.1\")\n# Returns: [1, 3, 6, 1, 2, 1, 2, 2, 1, 10, 1]\n\n# Group resolution\n{:ok, oid} = SnmpKit.MIB.resolve(\"system\")\n# Returns: [1, 3, 6, 1, 2, 1, 1]\n\n# Reverse lookup\n{:ok, name} = SnmpKit.MIB.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1, 0])\n# Returns: \"sysDescr.0\"\n```","ref":"unified-api-guide.html#oid-resolution"},{"type":"extras","title":"MIB Tree Navigation - SnmpKit Unified API Guide","doc":"```elixir\n# Get children of an OID node\n{:ok, children} = SnmpKit.MIB.children([1, 3, 6, 1, 2, 1, 1])\n# Returns: [[1, 3, 6, 1, 2, 1, 1, 1], [1, 3, 6, 1, 2, 1, 1, 2], ...]\n\n# Get parent of an OID\n{:ok, parent} = SnmpKit.MIB.parent([1, 3, 6, 1, 2, 1, 1, 1, 0])\n# Returns: [1, 3, 6, 1, 2, 1, 1, 1]\n\n# Walk MIB tree\n{:ok, tree} = SnmpKit.MIB.walk_tree([1, 3, 6, 1, 2, 1, 1])\n```","ref":"unified-api-guide.html#mib-tree-navigation"},{"type":"extras","title":"MIB Compilation and Loading - SnmpKit Unified API Guide","doc":"```elixir\n# High-level compilation (recommended)\n{:ok, compiled} = SnmpKit.MIB.compile(\"MY-ENTERPRISE-MIB.mib\")\n{:ok, _} = SnmpKit.MIB.load(compiled)\n\n# Compile entire directory\n{:ok, results} = SnmpKit.MIB.compile_dir(\"mibs/\")\n\n# Low-level compilation (advanced)\n{:ok, mib} = SnmpKit.MIB.compile_raw(\"MY-MIB.mib\")\n{:ok, _} = SnmpKit.MIB.load_compiled(\"compiled_mib.bin\")\n\n# Compile multiple files\n{:ok, compiled_mibs} = SnmpKit.MIB.compile_all([\"mib1.mib\", \"mib2.mib\"])\n```","ref":"unified-api-guide.html#mib-compilation-and-loading"},{"type":"extras","title":"MIB Parsing and Analysis - SnmpKit Unified API Guide","doc":"```elixir\n# Parse MIB file\n{:ok, parsed} = SnmpKit.MIB.parse_mib_file(\"CUSTOM-MIB.mib\")\n\n# Parse MIB content string\nmib_content = File.read!(\"MY-MIB.mib\")\n{:ok, parsed} = SnmpKit.MIB.parse_mib_content(mib_content)\n\n# Enhanced resolution with custom MIBs\n{:ok, oid} = SnmpKit.MIB.resolve_enhanced(\"customObject.0\")\n\n# Integrate compilation and parsing\n{:ok, integrated} = SnmpKit.MIB.load_and_integrate_mib(\"ENTERPRISE-MIB.mib\")\n```","ref":"unified-api-guide.html#mib-parsing-and-analysis"},{"type":"extras","title":"Standard MIBs - SnmpKit Unified API Guide","doc":"```elixir\n# Load built-in standard MIBs\n:ok = SnmpKit.MIB.load_standard_mibs()\n\n# Start MIB registry (for advanced usage)\n{:ok, _pid} = SnmpKit.MIB.start_link()\n```","ref":"unified-api-guide.html#standard-mibs"},{"type":"extras","title":"🧪 Device Simulation (SnmpKit.Sim) - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#device-simulation-snmpkit-sim"},{"type":"extras","title":"Single Device Simulation - SnmpKit Unified API Guide","doc":"```elixir\n# Load a device profile\n{:ok, profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(\n  :cable_modem,\n  {:walk_file, \"priv/walks/cable_modem.walk\"}\n)\n\n# Start simulated device\n{:ok, device} = SnmpKit.Sim.start_device(profile, [\n  port: 1161,\n  community: \"public\"\n])\n\n# Device will respond to SNMP queries on localhost:1161\n{:ok, description} = SnmpKit.SNMP.get(\"127.0.0.1:1161\", \"sysDescr.0\")\n```","ref":"unified-api-guide.html#single-device-simulation"},{"type":"extras","title":"Population Simulation - SnmpKit Unified API Guide","doc":"```elixir\n# Create multiple devices for testing\ndevice_configs = [\n  %{type: :cable_modem, port: 30001, community: \"public\"},\n  %{type: :switch, port: 30002, community: \"public\"},\n  %{type: :router, port: 30003, community: \"private\"}\n]\n\n{:ok, devices} = SnmpKit.Sim.start_device_population(device_configs)\n\n# Query the simulated devices\n{:ok, cm_desc} = SnmpKit.SNMP.get(\"127.0.0.1:30001\", \"sysDescr.0\")\n{:ok, switch_desc} = SnmpKit.SNMP.get(\"127.0.0.1:30002\", \"sysDescr.0\")\n```","ref":"unified-api-guide.html#population-simulation"},{"type":"extras","title":"Testing Integration - SnmpKit Unified API Guide","doc":"```elixir\ndefmodule MyNetworkTest do\n  use ExUnit.Case\n  \n  setup do\n    # Start simulated devices for each test\n    {:ok, cable_modem_profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(:cable_modem)\n    {:ok, router_profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(:router)\n    \n    {:ok, cm} = SnmpKit.Sim.start_device(cable_modem_profile, port: 1161)\n    {:ok, router} = SnmpKit.Sim.start_device(router_profile, port: 1162)\n    \n    %{\n      cable_modem: \"127.0.0.1:1161\",\n      router: \"127.0.0.1:1162\"\n    }\n  end\n  \n  test \"can monitor cable modem\", %{cable_modem: cm_target} do\n    {:ok, signal_noise} = SnmpKit.SNMP.get(cm_target, \"docsIfSigQSignalNoise.1\")\n    assert is_integer(signal_noise)\n  end\n  \n  test \"can get interface statistics\", %{router: router_target} do\n    {:ok, interfaces} = SnmpKit.SNMP.get_table(router_target, \"ifTable\")\n    assert length(interfaces) > 0\n  end\nend\n```","ref":"unified-api-guide.html#testing-integration"},{"type":"extras","title":"🎯 Direct Access (SnmpKit) - SnmpKit Unified API Guide","doc":"For convenience and backward compatibility, common operations are available directly:\n\n```elixir\n# These are equivalent to their SnmpKit.SNMP.* counterparts\n{:ok, value} = SnmpKit.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, results} = SnmpKit.walk(\"192.168.1.1\", \"system\")\n:ok = SnmpKit.set(\"127.0.0.1:1161\", \"sysContact.0\", \"admin@example.com\")\n\n# MIB resolution\n{:ok, oid} = SnmpKit.resolve(\"sysDescr.0\")\n```","ref":"unified-api-guide.html#direct-access-snmpkit"},{"type":"extras","title":"🔄 Migration Guide - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#migration-guide"},{"type":"extras","title":"From Direct Module Usage - SnmpKit Unified API Guide","doc":"```elixir\n# Before (still works)\n{:ok, value} = SnmpKit.SnmpMgr.get(\"host\", \"oid\")\n{:ok, oid} = SnmpKit.SnmpMgr.MIB.resolve(\"name\")\n\n# After (recommended)\n{:ok, value} = SnmpKit.SNMP.get(\"host\", \"oid\")\n{:ok, oid} = SnmpKit.MIB.resolve(\"name\")\n\n# Or use direct access\n{:ok, value} = SnmpKit.get(\"host\", \"oid\")\n{:ok, oid} = SnmpKit.resolve(\"name\")\n```","ref":"unified-api-guide.html#from-direct-module-usage"},{"type":"extras","title":"Gradual Migration Strategy - SnmpKit Unified API Guide","doc":"1. **Phase 1**: Start using unified API for new code\n2. **Phase 2**: Gradually update existing code module by module\n3. **Phase 3**: Adopt consistent style across codebase","ref":"unified-api-guide.html#gradual-migration-strategy"},{"type":"extras","title":"Import Strategy - SnmpKit Unified API Guide","doc":"```elixir\n# Option 1: Import specific modules\nalias SnmpKit.{SNMP, MIB, Sim}\n\n{:ok, value} = SNMP.get(\"host\", \"oid\")\n{:ok, oid} = MIB.resolve(\"name\")\n\n# Option 2: Use fully qualified names\n{:ok, value} = SnmpKit.SNMP.get(\"host\", \"oid\")\n{:ok, oid} = SnmpKit.MIB.resolve(\"name\")\n\n# Option 3: Direct access for simple operations\n{:ok, value} = SnmpKit.get(\"host\", \"oid\")\n{:ok, oid} = SnmpKit.resolve(\"name\")\n```","ref":"unified-api-guide.html#import-strategy"},{"type":"extras","title":"📚 Function Reference Quick Guide - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#function-reference-quick-guide"},{"type":"extras","title":"Most Common Operations - SnmpKit Unified API Guide","doc":"| Task | Function | Example |\n|------|----------|---------|\n| Get single value | `SnmpKit.SNMP.get/3` | `get(\"host\", \"sysDescr.0\")` |\n| Walk OID tree | `SnmpKit.SNMP.walk/3` | `walk(\"host\", \"system\")` |\n| Get table | `SnmpKit.SNMP.get_table/3` | `get_table(\"host\", \"ifTable\")` |\n| Resolve OID name | `SnmpKit.MIB.resolve/1` | `resolve(\"sysDescr.0\")` |\n| Start simulated device | `SnmpKit.Sim.start_device/2` | `start_device(profile, port: 1161)` |","ref":"unified-api-guide.html#most-common-operations"},{"type":"extras","title":"Performance Operations - SnmpKit Unified API Guide","doc":"| Task | Function | Example |\n|------|----------|---------|\n| Bulk retrieval | `SnmpKit.SNMP.get_bulk/3` | `get_bulk(\"host\", \"interfaces\")` |\n| Multi-target query | `SnmpKit.SNMP.get_multi/2` | `get_multi([{\"h1\", \"oid1\"}, {\"h2\", \"oid2\"}])` |\n| Streaming walk | `SnmpKit.SNMP.walk_stream/3` | `walk_stream(\"host\", \"large_table\")` |\n| Adaptive walk | `SnmpKit.SNMP.adaptive_walk/3` | `adaptive_walk(\"host\", \"interfaces\")` |","ref":"unified-api-guide.html#performance-operations"},{"type":"extras","title":"Advanced Operations - SnmpKit Unified API Guide","doc":"| Task | Function | Example |\n|------|----------|---------|\n| Circuit breaker | `SnmpKit.SNMP.with_circuit_breaker/3` | `with_circuit_breaker(\"host\", fn -> ... end)` |\n| Engine stats | `SnmpKit.SNMP.get_engine_stats/1` | `get_engine_stats()` |\n| MIB compilation | `SnmpKit.MIB.compile/2` | `compile(\"MY-MIB.mib\")` |\n| Tree navigation | `SnmpKit.MIB.children/1` | `children([1,3,6,1,2,1,1])` |","ref":"unified-api-guide.html#advanced-operations"},{"type":"extras","title":"🚀 Best Practices - SnmpKit Unified API Guide","doc":"","ref":"unified-api-guide.html#best-practices"},{"type":"extras","title":"1. Choose the Right API Level - SnmpKit Unified API Guide","doc":"```elixir\n# For simple scripts and convenience\n{:ok, value} = SnmpKit.get(\"host\", \"oid\")\n\n# For applications with multiple SNMP operations\nalias SnmpKit.SNMP\n{:ok, value} = SNMP.get(\"host\", \"oid\")\n{:ok, table} = SNMP.get_table(\"host\", \"ifTable\")\n\n# For complex applications\ndefmodule MyMonitor do\n  alias SnmpKit.{SNMP, MIB, Sim}\n  \n  def monitor_device(host) do\n    with {:ok, description} <- SNMP.get(host, \"sysDescr.0\"),\n         {:ok, interfaces} <- SNMP.get_table(host, \"ifTable\") do\n      %{description: description, interfaces: interfaces}\n    end\n  end\nend\n```","ref":"unified-api-guide.html#1-choose-the-right-api-level"},{"type":"extras","title":"2. Use Appropriate Operations for Scale - SnmpKit Unified API Guide","doc":"```elixir\n# For single values\n{:ok, value} = SnmpKit.SNMP.get(\"host\", \"oid\")\n\n# For multiple values from same host\n{:ok, results} = SnmpKit.SNMP.walk(\"host\", \"system\")\n\n# For multiple hosts\n{:ok, results} = SnmpKit.SNMP.get_multi([{\"h1\", \"oid\"}, {\"h2\", \"oid\"}])\n\n# For large datasets\nstream = SnmpKit.SNMP.walk_stream(\"host\", \"large_table\")\n```","ref":"unified-api-guide.html#2-use-appropriate-operations-for-scale"},{"type":"extras","title":"3. Handle Errors Gracefully - SnmpKit Unified API Guide","doc":"```elixir\ncase SnmpKit.SNMP.get(\"host\", \"oid\", timeout: 1000) do\n  {:ok, value} -> \n    process_value(value)\n  {:error, :timeout} -> \n    log_timeout_error(\"host\")\n  {:error, reason} -> \n    log_snmp_error(\"host\", reason)\nend\n```","ref":"unified-api-guide.html#3-handle-errors-gracefully"},{"type":"extras","title":"4. Use Simulated Devices for Testing - SnmpKit Unified API Guide","doc":"```elixir\n# In test setup\nsetup do\n  {:ok, profile} = SnmpKit.SnmpSim.ProfileLoader.load_profile(:generic_device)\n  {:ok, _device} = SnmpKit.Sim.start_device(profile, port: 1161)\n  %{target: \"127.0.0.1:1161\"}\nend\n\ntest \"my network function\", %{target: target} do\n  # Test against simulated device\n  result = my_network_function(target)\n  assert result.status == :ok\nend\n```","ref":"unified-api-guide.html#4-use-simulated-devices-for-testing"},{"type":"extras","title":"🎉 Conclusion - SnmpKit Unified API Guide","doc":"The unified API makes SnmpKit more approachable for new users while maintaining all the power and flexibility for advanced use cases. Choose the approach that best fits your needs:\n\n- **Direct access** (`SnmpKit.*`) for simple operations\n- **Namespaced modules** (`SnmpKit.SNMP.*`) for organized applications  \n- **Mixed approach** based on context and preference\n\nAll approaches provide the same functionality with 100% backward compatibility.\n\nHappy SNMP monitoring! 🚀","ref":"unified-api-guide.html#conclusion"},{"type":"extras","title":"SnmpKit v0.3.0 Release Notes","doc":"# SnmpKit v0.3.0 Release Notes\n\n**Release Date:** 2024-12-19  \n**Version:** 0.3.0  \n**Git Tag:** v0.3.0","ref":"v0-3-0-release-notes.html"},{"type":"extras","title":"🎯 Major Features - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#major-features"},{"type":"extras","title":"Unified API Architecture - SnmpKit v0.3.0 Release Notes","doc":"The biggest change in v0.3.0 is the introduction of a **unified API architecture** that organizes all SnmpKit functionality into logical, discoverable modules:\n\n- **`SnmpKit.SNMP`** - Complete SNMP protocol operations\n- **`SnmpKit.MIB`** - Comprehensive MIB management  \n- **`SnmpKit.Sim`** - Realistic device simulation\n\nThis replaces the previous fragmented approach where functions were scattered across multiple modules, making the library much more intuitive and easier to use.","ref":"v0-3-0-release-notes.html#unified-api-architecture"},{"type":"extras","title":"Enhanced Module Organization - SnmpKit v0.3.0 Release Notes","doc":"#### SnmpKit.SNMP\nAll SNMP operations are now organized under a single, comprehensive module:\n\n```elixir\n# Basic operations\n{:ok, value} = SnmpKit.SNMP.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, results} = SnmpKit.SNMP.walk(\"192.168.1.1\", \"system\")\n\n# Bulk operations\n{:ok, data} = SnmpKit.SNMP.bulk_walk(\"192.168.1.1\", \"interfaces\")\n\n# Multi-target operations\n{:ok, results} = SnmpKit.SNMP.get_multi([\n  {\"host1\", \"sysDescr.0\"},\n  {\"host2\", \"sysUpTime.0\"}\n])\n\n# Pretty formatting\n{:ok, formatted} = SnmpKit.SNMP.get_pretty(\"192.168.1.1\", \"sysUpTime.0\")\n\n# Async operations\ntask = SnmpKit.SNMP.get_async(\"192.168.1.1\", \"sysDescr.0\")\n```\n\n#### SnmpKit.MIB\nComprehensive MIB management with both high-level and low-level operations:\n\n```elixir\n# OID resolution\n{:ok, oid} = SnmpKit.MIB.resolve(\"sysDescr.0\")\n{:ok, name} = SnmpKit.MIB.reverse_lookup([1, 3, 6, 1, 2, 1, 1, 1, 0])\n\n# High-level MIB compilation\n{:ok, compiled} = SnmpKit.MIB.compile(\"MY-MIB.mib\")\n{:ok, _} = SnmpKit.MIB.load(compiled)\n\n# Low-level MIB operations\n{:ok, raw_compiled} = SnmpKit.MIB.compile_raw(\"MY-MIB.mib\")\n```\n\n#### SnmpKit.Sim\nSimplified device simulation interface:\n\n```elixir\n# Start individual devices\n{:ok, device} = SnmpKit.Sim.start_device(profile, port: 1161)\n\n# Create device populations\n{:ok, devices} = SnmpKit.Sim.start_device_population(device_configs)\n```","ref":"v0-3-0-release-notes.html#enhanced-module-organization"},{"type":"extras","title":"Backward Compatibility - SnmpKit v0.3.0 Release Notes","doc":"All existing code continues to work! Common operations are still available directly on the main `SnmpKit` module:\n\n```elixir\n# These still work exactly as before\n{:ok, value} = SnmpKit.get(\"192.168.1.1\", \"sysDescr.0\")\n{:ok, results} = SnmpKit.walk(\"192.168.1.1\", \"system\")\n{:ok, oid} = SnmpKit.resolve(\"sysDescr.0\")\n```","ref":"v0-3-0-release-notes.html#backward-compatibility"},{"type":"extras","title":"📚 Documentation Improvements - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#documentation-improvements"},{"type":"extras","title":"Updated README - SnmpKit v0.3.0 Release Notes","doc":"- Complete rewrite showcasing the unified API\n- Clear examples for each module\n- Migration guide from other libraries\n- Enterprise features section\n- Performance optimization tips","ref":"v0-3-0-release-notes.html#updated-readme"},{"type":"extras","title":"Interactive Livebook Tour - SnmpKit v0.3.0 Release Notes","doc":"- Completely rewritten `snmpkit_tour.livemd`\n- Now uses simulated devices for all examples\n- No external network dependencies\n- Comprehensive coverage of all new API features\n- Step-by-step guided experience","ref":"v0-3-0-release-notes.html#interactive-livebook-tour"},{"type":"extras","title":"New Documentation Files - SnmpKit v0.3.0 Release Notes","doc":"- `docs/unified-api-guide.md` - Deep dive into the new API structure\n- `docs/v0.3.0-release-notes.md` - These release notes\n- `examples/unified_api_demo.exs` - Practical usage examples","ref":"v0-3-0-release-notes.html#new-documentation-files"},{"type":"extras","title":"🔧 Technical Improvements - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#technical-improvements"},{"type":"extras","title":"Code Quality - SnmpKit v0.3.0 Release Notes","doc":"- **Zero compiler warnings** - Maintains the clean codebase from v0.2.0\n- **1,140 tests passing** - Comprehensive test coverage\n- **76 doctests** - Extensive documentation examples\n- **Clean module structure** - Logical organization with `defdelegate`","ref":"v0-3-0-release-notes.html#code-quality"},{"type":"extras","title":"Performance - SnmpKit v0.3.0 Release Notes","doc":"- Maintained all performance optimizations from previous versions\n- Added streaming operations for large datasets\n- Improved memory usage for bulk operations","ref":"v0-3-0-release-notes.html#performance"},{"type":"extras","title":"Developer Experience - SnmpKit v0.3.0 Release Notes","doc":"- **Discoverability** - Functions are now easy to find in logical modules\n- **Consistency** - Similar operations follow consistent patterns\n- **IDE Support** - Better autocomplete and documentation in editors","ref":"v0-3-0-release-notes.html#developer-experience"},{"type":"extras","title":"🚀 New Features - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#new-features"},{"type":"extras","title":"Enhanced SNMP Operations - SnmpKit v0.3.0 Release Notes","doc":"- **Streaming operations** - `walk_stream/3`, `table_stream/3`\n- **Performance benchmarking** - `benchmark_device/3`\n- **Circuit breaker pattern** - `with_circuit_breaker/3`\n- **Engine management** - Advanced SNMP engine features","ref":"v0-3-0-release-notes.html#enhanced-snmp-operations"},{"type":"extras","title":"Advanced MIB Features - SnmpKit v0.3.0 Release Notes","doc":"- **Tree navigation** - `children/1`, `parent/1`, `walk_tree/2`\n- **Enhanced resolution** - `resolve_enhanced/2`\n- **Dual compilation paths** - High-level and low-level MIB compilation\n- **Standard MIB loading** - `load_standard_mibs/0`","ref":"v0-3-0-release-notes.html#advanced-mib-features"},{"type":"extras","title":"Simulation Enhancements - SnmpKit v0.3.0 Release Notes","doc":"- **Simplified device creation** - Easier device simulation setup\n- **Population management** - Better handling of device groups\n- **Profile integration** - Seamless integration with existing profiles","ref":"v0-3-0-release-notes.html#simulation-enhancements"},{"type":"extras","title":"📊 Migration Guide - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#migration-guide"},{"type":"extras","title":"From v0.2.x - SnmpKit v0.3.0 Release Notes","doc":"**No breaking changes!** Your existing code will continue to work exactly as before.\n\n**Optional migrations** for better discoverability:\n\n```elixir\n# Old way (still works)\n{:ok, value} = SnmpKit.get(\"host\", \"oid\")\n\n# New way (recommended)\n{:ok, value} = SnmpKit.SNMP.get(\"host\", \"oid\")\n```","ref":"v0-3-0-release-notes.html#from-v0-2-x"},{"type":"extras","title":"From Other Libraries - SnmpKit v0.3.0 Release Notes","doc":"```elixir\n# From :snmp (Erlang)\n:snmp.sync_get(manager, oid, timeout)\n# Becomes:\nSnmpKit.SNMP.get(\"host\", \"oid\", timeout: timeout)\n\n# From other Elixir SNMP libraries\nOtherLib.snmp_get(host, oid)\n# Becomes:\nSnmpKit.SNMP.get(host, oid)\n```","ref":"v0-3-0-release-notes.html#from-other-libraries"},{"type":"extras","title":"🎉 What's Next - SnmpKit v0.3.0 Release Notes","doc":"","ref":"v0-3-0-release-notes.html#what-s-next"},{"type":"extras","title":"v0.4.0 Roadmap - SnmpKit v0.3.0 Release Notes","doc":"- Enhanced metrics and monitoring\n- More simulation scenarios\n- Performance optimization tools\n- Extended MIB analysis capabilities","ref":"v0-3-0-release-notes.html#v0-4-0-roadmap"},{"type":"extras","title":"Community - SnmpKit v0.3.0 Release Notes","doc":"- Join the discussion on GitHub\n- Contribute examples and use cases\n- Share feedback on the unified API","ref":"v0-3-0-release-notes.html#community"},{"type":"extras","title":"📈 Statistics - SnmpKit v0.3.0 Release Notes","doc":"- **Total Functions**: 50+ organized functions across 3 main modules\n- **Lines of Code**: Maintained clean, focused codebase\n- **Test Coverage**: 1,140 tests, 76 doctests, 0 failures\n- **Documentation**: Complete rewrite with 3 new guides","ref":"v0-3-0-release-notes.html#statistics"},{"type":"extras","title":"🙏 Acknowledgments - SnmpKit v0.3.0 Release Notes","doc":"This release represents a significant step forward in making SnmpKit the most user-friendly and powerful SNMP library for Elixir. Thank you to all users who provided feedback on the API design and helped shape this unified architecture.\n\n---\n\n**Upgrade today:** `{:snmpkit, \"~> 0.3.0\"}`\n\nFor questions or support, please visit the [GitHub repository](https://github.com/awksedgreep/snmpkit).","ref":"v0-3-0-release-notes.html#acknowledgments"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}